﻿// https://akelpad.sourceforge.net/forum/viewtopic.php?p=11246#p11246
// https://infocatcher.ucoz.net/js/akelpad_scripts/Include/jsBeautifier_tests.js
// https://github.com/Infocatcher/AkelPad_scripts/blob/master/Include/jsBeautifier_tests.js

// Tests part of jsBeautifier.js, don't use directly!
// Should be placed in AkelFiles\Plugs\Scripts\Include\jsBeautifier_tests.js
// https://infocatcher.ucoz.net/js/akelpad_scripts/jsBeautifier.js
// https://github.com/Infocatcher/AkelPad_scripts/blob/master/jsBeautifier.js

// Scripts from http://jsbeautifier.org/
// [built from https://github.com/beautify-web/js-beautify/tree/release 2022-06-20 23:53:03 UTC]


//== js/test/generated/beautify-javascript-tests.js
/*
    AUTO-GENERATED. DO NOT MODIFY.
    Script: test/generate-tests.js
    Template: test/data/javascript/node.mustache
    Data: test/data/javascript/tests.js

  The MIT License (MIT)

  Copyright (c) 2007-2018 Einar Lielmanis, Liam Newman, and contributors.

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
/*jshint unused:false */
/*jshint strict:false */

function run_javascript_tests(test_obj, Urlencoded, js_beautify, html_beautify, css_beautify)
{

    var default_opts = {
        indent_size: 4,
        indent_char: ' ',
        preserve_newlines: true,
        jslint_happy: false,
        space_before_conditional: true,
        break_chained_methods: false,
        selector_separator: '\n',
        end_with_newline: false
    };
    var opts;

    default_opts.indent_size = 4;
    default_opts.indent_char = ' ';
    default_opts.preserve_newlines = true;
    default_opts.jslint_happy = false;

    function reset_options()
    {
        opts = {}; for(var p in default_opts) opts[p] = default_opts[p];
        test_name = 'js-beautify';
    }

    function test_beautifier(input)
    {
        return js_beautify(input, opts);
    }

    var sanitytest;
    var test_name = '';

    function set_name(name)
    {
        name = (name || '').trim();
        if (name) {
            test_name = name.replace(/\r/g, '\\r').replace(/\n/g, '\\n');
        }
    }

    // test the input on beautifier with the current flag settings
    // does not check the indentation / surroundings as bt() does
    function test_fragment(input, expected)
    {
        var success = true;
        sanitytest.test_function(test_beautifier, test_name);
        expected = expected || expected === '' ? expected : input;
        success = success && sanitytest.expect(input, expected);
        // if the expected is different from input, run it again
        // expected output should be unchanged when run twice.
        if (success && expected !== input) {
            success = success && sanitytest.expect(expected, expected);
        }

        // Everywhere we do newlines, they should be replaced with opts.eol
        sanitytest.test_function(test_beautifier, 'eol ' + test_name);
        opts.eol = '\r\\n';
        expected = expected.replace(/[\n]/g, '\r\n');
        opts.disabled = true;
        success = success && sanitytest.expect(input, input || '');
        success = success && sanitytest.expect('\n\n' + expected, '\n\n' + expected);
        opts.disabled = false;
        success = success && sanitytest.expect(input, expected);
        if (success && input && input.indexOf('\n') !== -1) {
            input = input.replace(/[\n]/g, '\r\n');
            sanitytest.expect(input, expected);
            // Ensure support for auto eol detection
            opts.eol = 'auto';
            success = success && sanitytest.expect(input, expected);
        }
        opts.eol = '\n';
        return success;
    }


    // test the input on beautifier with the current flag settings
    // test both the input as well as { input } wrapping
    function bt(input, expectation)
    {
        var success = true;

        var wrapped_input, wrapped_expectation;

        expectation = expectation || expectation === '' ? expectation : input;
        success = success && test_fragment(input, expectation);

        // If we set raw, input should be unchanged
        opts.test_output_raw = true;
        if (!opts.end_with_newline) {
            success = success && test_fragment(input, input);
        }
        opts.test_output_raw = false;

        // test also the returned indentation
        // e.g if input = "asdf();"
        // then test that this remains properly formatted as well:
        // {
        //     asdf();
        //     indent;
        // }

        var current_indent_size = opts.js ? opts.js.indent_size : null;
        current_indent_size = current_indent_size ? current_indent_size : opts.indent_size;
        if (current_indent_size === 4 && input) {
            wrapped_input = '{\n' + input.replace(/^(.+)$/mg, '    $1') + '\n    foo = bar;\n}';
            wrapped_expectation = '{\n' + expectation.replace(/^(.+)$/mg, '    $1') + '\n    foo = bar;\n}';
            success = success && test_fragment(wrapped_input, wrapped_expectation);

            // If we set raw, input should be unchanged
            opts.test_output_raw = true;
            if (!opts.end_with_newline) {
                success = success && test_fragment(wrapped_input, wrapped_input);
            }
            opts.test_output_raw = false;
        }
        return success;
    }

    // run all tests for the given brace style ("collapse", "expand", "end-expand", or "none").
    // uses various whitespace combinations before and after opening and closing braces,
    // respectively, for most of the tests' inputs.
    function beautify_brace_tests(brace_style) {

        var indent_on_wrap_str = '    '; // could use Array(opts.indent_size + 1).join(' '); if we wanted to replace _all_ of the hardcoded 4-space in the test and expectation strings

        function permute_brace_tests(expect_open_white, expect_close_white) {

            // run the tests that need permutation against a specific combination of
            // pre-opening-brace and pre-closing-brace whitespace
            function run_brace_permutation(test_open_white, test_close_white) {
                var to = test_open_white,
                    tc = test_close_white,
                    eo = expect_open_white ? expect_open_white : to === '' ? ' ' : to,
                    ec = expect_close_white ? expect_close_white : tc === '' ? ' ' : tc,
                    i = eo === '\n' ? indent_on_wrap_str: '';

                bt( '//case 1\nif (a == 1)' + to + '{}\n//case 2\nelse if (a == 2)' + to + '{}',
                    '//case 1\nif (a == 1)' + eo + '{}\n//case 2\nelse if (a == 2)' + eo + '{}');
                bt( 'if(1)' + to + '{2}' + tc + 'else' + to + '{3}',
                    'if (1)' + eo + '{\n    2\n}' + ec + 'else' + eo + '{\n    3\n}');
                bt( 'try' + to + '{a();}' + tc +
                    'catch(b)' + to + '{c();}' + tc +
                    'catch(d)' + to + '{}' + tc +
                    'finally' + to + '{e();}',
                    // expected
                    'try' + eo + '{\n    a();\n}' + ec +
                    'catch (b)' + eo + '{\n    c();\n}' + ec +
                    'catch (d)' + eo + '{}' + ec +
                    'finally' + eo + '{\n    e();\n}');
                bt( 'if(a)' + to + '{b();}' + tc + 'else if(c) foo();',
                    'if (a)' + eo + '{\n    b();\n}' + ec + 'else if (c) foo();');
                // if/else statement with empty body
                bt( 'if (a)' + to + '{\n// comment\n}' + tc + 'else' + to + '{\n// comment\n}',
                    'if (a)' + eo + '{\n    // comment\n}' + ec + 'else' + eo + '{\n    // comment\n}');
                bt( 'if (x)' + to + '{y}' + tc + 'else' + to + '{ if (x)' + to + '{y}}',
                    'if (x)' + eo + '{\n    y\n}' + ec + 'else' + eo + '{\n    if (x)' + eo + i + '{\n        y\n    }\n}');
                bt( 'if (a)' + to + '{\nb;\n}' + tc + 'else' + to + '{\nc;\n}',
                    'if (a)' + eo + '{\n    b;\n}' + ec + 'else' + eo + '{\n    c;\n}');
                test_fragment('    /*\n* xx\n*/\n// xx\nif (foo)' + to + '{\n    bar();\n}',
                              '    /*\n     * xx\n     */\n    // xx\n    if (foo)' + eo + i + '{\n        bar();\n    }');
                bt( 'if (foo)' + to + '{}' + tc + 'else /regex/.test();',
                    'if (foo)' + eo + '{}' + ec + 'else /regex/.test();');
                test_fragment('if (foo)' + to + '{', 'if (foo)' + eo + '{');
                test_fragment('foo' + to + '{', 'foo' + eo + '{');
                test_fragment('return;' + to + '{', 'return;' + eo + '{');
                bt( 'function x()' + to + '{\n    foo();\n}zzz', 'function x()' + eo +'{\n    foo();\n}\nzzz');
                bt( 'var a = new function a()' + to + '{};', 'var a = new function a()' + eo + '{};');
                bt( 'var a = new function a()' + to + '    {},\n    b = new function b()' + to + '    {};',
                    'var a = new function a()' + eo + i + '{},\n    b = new function b()' + eo + i + '{};');
                bt("foo(" + to + "{\n    'a': 1\n},\n10);",
                   "foo(" + (eo === ' ' ? '' : eo) + i + "{\n        'a': 1\n    },\n    10);"); // "foo( {..." is a weird case
                bt('(["foo","bar"]).each(function(i)' + to + '{return i;});',
                   '(["foo", "bar"]).each(function(i)' + eo + '{\n    return i;\n});');
                bt('(function(i)' + to + '{return i;})();', '(function(i)' + eo + '{\n    return i;\n})();');

                bt( "test( /*Argument 1*/" + to + "{\n" +
                    "    'Value1': '1'\n" +
                    "}, /*Argument 2\n" +
                    " */ {\n" +
                    "    'Value2': '2'\n" +
                    "});",
                    // expected
                    "test( /*Argument 1*/" + eo + i + "{\n" +
                    "        'Value1': '1'\n" +
                    "    },\n" +
                    "    /*Argument 2\n" +
                    "     */\n" +
                    "    {\n" +
                    "        'Value2': '2'\n" +
                    "    });");

                bt( "test( /*Argument 1*/" + to + "{\n" +
                    "    'Value1': '1'\n" +
                    "}, /*Argument 2\n" +
                    " */\n" +
                    "{\n" +
                    "    'Value2': '2'\n" +
                    "});",
                    // expected
                    "test( /*Argument 1*/" + eo + i + "{\n" +
                    "        'Value1': '1'\n" +
                    "    },\n" +
                    "    /*Argument 2\n" +
                    "     */\n" +
                    "    {\n" +
                    "        'Value2': '2'\n" +
                    "    });");
            }

            run_brace_permutation('\n', '\n');
            run_brace_permutation('\n', ' ');
            run_brace_permutation(' ', ' ');
            run_brace_permutation(' ', '\n');
            run_brace_permutation('','');

            // brace tests that don't make sense to permutate
            test_fragment('return {'); // return needs the brace.
            test_fragment('return /* inline */ {');
            bt('throw {}');
            bt('throw {\n    foo;\n}');
            bt( 'var foo = {}');
            test_fragment('a: do {} while (); xxx', 'a: do {} while ();\nxxx');
            bt( '{a: do {} while (); xxx}', '{\n    a: do {} while ();xxx\n}');
            bt( 'var a = new function() {};');
            bt( 'var a = new function()\n{};', 'var a = new function() {};');
            bt( "test(\n" +
                "/*Argument 1*/ {\n" +
                "    'Value1': '1'\n" +
                "},\n" +
                "/*Argument 2\n" +
                " */ {\n" +
                "    'Value2': '2'\n" +
                "});",
                // expected
                "test(\n" +
                "    /*Argument 1*/\n" +
                "    {\n" +
                "        'Value1': '1'\n" +
                "    },\n" +
                "    /*Argument 2\n" +
                "     */\n" +
                "    {\n" +
                "        'Value2': '2'\n" +
                "    });");
        }

        reset_options();
        opts.brace_style = brace_style;

        switch(opts.brace_style) {
        case 'collapse':
            permute_brace_tests(' ', ' ');
            break;
        case 'expand':
            permute_brace_tests('\n', '\n');
            break;
        case 'end-expand':
            permute_brace_tests(' ', '\n');
            break;
        case 'none':
            permute_brace_tests();
            break;
        }
    }

    function unicode_char(value) {
        return String.fromCharCode(value);
    }

    function beautifier_tests()
    {
        sanitytest = test_obj;

        //============================================================
        // Line wrap test inputs
        //...---------1---------2---------3---------4---------5---------6---------7
        //...1234567890123456789012345678901234567890123456789012345678901234567890
        var wrap_input_1=(
            'foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n.but_this_can\n' +
            'return between_return_and_expression_should_never_wrap.but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            'if (wraps_can_occur && inside_an_if_block) that_is_\n.okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token + 12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap + but_this_can,\n' +
            '    propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '}');

        //...---------1---------2---------3---------4---------5---------6---------7
        //...1234567890123456789012345678901234567890123456789012345678901234567890
        var wrap_input_2=(
            '{\n' +
            '    foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n.but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap.but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            '    if (wraps_can_occur && inside_an_if_block) that_is_\n.okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token + 12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap + but_this_can,\n' +
            '        propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '    }' +
            '}');


        //============================================================
        // Unicode Support
        reset_options();
        set_name('Unicode Support');
        bt('var ' + unicode_char(3232) + '_' + unicode_char(3232) + ' = "hi";');
        bt(
            'var ' + unicode_char(228) + 'x = {\n' +
            '    ' + unicode_char(228) + 'rgerlich: true\n' +
            '};');
        bt(
            'var \\u00E4\\u0ca0\\u0cA0\\u0Ca0 = {\n' +
            '    \\u0ca0rgerlich: true\n' +
            '};');
        bt(
            'var \\u00E4add\\u0025 = {\n' +
            '    \\u0044rgerlich\\u0ca0: true\n' +
            '};');
        bt(
            'var' + unicode_char(160) + unicode_char(3232) + '_' + unicode_char(3232) + ' = "hi";',
            //  -- output --
            'var ' + unicode_char(3232) + '_' + unicode_char(3232) + ' = "hi";');


        //============================================================
        // Test template and continuation strings
        reset_options();
        set_name('Test template and continuation strings');
        bt('`This is a ${template} string.`');
        bt(
            '`This\n' +
            '  is\n' +
            '  a\n' +
            '  ${template}\n' +
            '  string.`');
        bt(
            'a = `This is a continuation\\\n' +
            'string.`');
        bt(
            'a = "This is a continuation\\\n' +
            'string."');
        bt(
            '`SELECT\n' +
            '  nextval(\'${this.options.schema ? `${this.options.schema}.` : \'\'}"${this.tableName}_${this.autoIncrementField}_seq"\'::regclass\n' +
            '  ) nextval;`');

        // Tests for #1030
        bt(
            'const composeUrl = (host) => {\n' +
            '    return `${host `test`}`;\n' +
            '};');
        bt(
            'const composeUrl = (host, api, key, data) => {\n' +
            '    switch (api) {\n' +
            '        case "Init":\n' +
            '            return `${host}/vwapi/Init?VWID=${key}&DATA=${encodeURIComponent(\n' +
            '                Object.keys(data).map((k) => `${k}=${ data[k]}` ).join(";")\n' +
            '            )}`;\n' +
            '        case "Pay":\n' +
            '            return `${host}/vwapi/Pay?SessionId=${par}`;\n' +
            '    };\n' +
            '};');


        //============================================================
        // Private Class Fields
        reset_options();
        set_name('Private Class Fields');
        bt('#foo');
        bt(
            'class X {\n' +
            '    #foo = null;\n' +
            '    get foo() {\n' +
            '        return this.#foo;\n' +
            '    }\n' +
            '}');
        bt(
            'class X {#foo=null;}',
            //  -- output --
            'class X {\n' +
            '    #foo = null;\n' +
            '}');


        //============================================================
        // ES7 Decorators
        reset_options();
        set_name('ES7 Decorators');
        bt('@foo');
        bt('@foo(bar)');
        bt(
            '@foo(function(k, v) {\n' +
            '    implementation();\n' +
            '})');


        //============================================================
        // ES7 exponential
        reset_options();
        set_name('ES7 exponential');
        bt('x ** 2');
        bt('x ** -2');


        //============================================================
        // Spread operator
        reset_options();
        set_name('Spread operator');
        opts.brace_style = "collapse,preserve-inline";
        bt('const m = { ...item, c: 3 };');
        bt(
            'const m = {\n' +
            '    ...item,\n' +
            '    c: 3\n' +
            '};');
        bt('const m = { c: 3, ...item };');
        bt('const m = [...item, 3];');
        bt('const m = [3, ...item];');


        //============================================================
        // Object literal shorthand functions
        reset_options();
        set_name('Object literal shorthand functions');
        bt(
            'return {\n' +
            '    foo() {\n' +
            '        return 42;\n' +
            '    }\n' +
            '}');
        bt(
            'var foo = {\n' +
            '    * bar() {\n' +
            '        yield 42;\n' +
            '    }\n' +
            '};');
        bt(
            'var foo = {bar(){return 42;},*barGen(){yield 42;}};',
            //  -- output --
            'var foo = {\n' +
            '    bar() {\n' +
            '        return 42;\n' +
            '    },\n' +
            '    * barGen() {\n' +
            '        yield 42;\n' +
            '    }\n' +
            '};');

        // also handle generator shorthand in class - #1013
        bt(
            'class A {\n' +
            '    fn() {\n' +
            '        return true;\n' +
            '    }\n' +
            '\n' +
            '    * gen() {\n' +
            '        return true;\n' +
            '    }\n' +
            '}');
        bt(
            'class A {\n' +
            '    * gen() {\n' +
            '        return true;\n' +
            '    }\n' +
            '\n' +
            '    fn() {\n' +
            '        return true;\n' +
            '    }\n' +
            '}');


        //============================================================
        // End With Newline - (end_with_newline = "true")
        reset_options();
        set_name('End With Newline - (end_with_newline = "true")');
        opts.end_with_newline = true;
        test_fragment('', '\n');
        test_fragment('   return .5', '   return .5\n');
        test_fragment(
            '   \n' +
            '\n' +
            'return .5\n' +
            '\n' +
            '\n' +
            '\n',
            //  -- output --
            '   return .5\n');
        test_fragment('\n');

        // End With Newline - (end_with_newline = "false")
        reset_options();
        set_name('End With Newline - (end_with_newline = "false")');
        opts.end_with_newline = false;
        test_fragment('');
        test_fragment('   return .5');
        test_fragment(
            '   \n' +
            '\n' +
            'return .5\n' +
            '\n' +
            '\n' +
            '\n',
            //  -- output --
            '   return .5');
        test_fragment('\n', '');


        //============================================================
        // Support Indent Level Options and Base Indent Autodetection - ()
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - ()');
        test_fragment('   a');
        test_fragment(
            '   function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '   function test() {\n' +
            '       console.log("this is a test");\n' +
            '   }');
        test_fragment(
            '   // This is a random comment\n' +
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '   // This is a random comment\n' +
            '   function test() {\n' +
            '       console.log("this is a test");\n' +
            '   }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")');
        opts.indent_level = 0;
        test_fragment('   a');
        test_fragment(
            '   function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '   function test() {\n' +
            '       console.log("this is a test");\n' +
            '   }');
        test_fragment(
            '   // This is a random comment\n' +
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '   // This is a random comment\n' +
            '   function test() {\n' +
            '       console.log("this is a test");\n' +
            '   }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "1")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "1")');
        opts.indent_level = 1;
        test_fragment('   a', '    a');
        test_fragment(
            '   function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '    function test() {\n' +
            '        console.log("this is a test");\n' +
            '    }');
        test_fragment(
            '   // This is a random comment\n' +
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '    // This is a random comment\n' +
            '    function test() {\n' +
            '        console.log("this is a test");\n' +
            '    }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "2")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "2")');
        opts.indent_level = 2;
        test_fragment('a', '        a');
        test_fragment(
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '        function test() {\n' +
            '            console.log("this is a test");\n' +
            '        }');
        test_fragment(
            '// This is a random comment\n' +
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '        // This is a random comment\n' +
            '        function test() {\n' +
            '            console.log("this is a test");\n' +
            '        }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_with_tabs = "true", indent_level = "2")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_with_tabs = "true", indent_level = "2")');
        opts.indent_with_tabs = true;
        opts.indent_level = 2;
        test_fragment('a', '\t\ta');
        test_fragment(
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '\t\tfunction test() {\n' +
            '\t\t\tconsole.log("this is a test");\n' +
            '\t\t}');
        test_fragment(
            '// This is a random comment\n' +
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '\t\t// This is a random comment\n' +
            '\t\tfunction test() {\n' +
            '\t\t\tconsole.log("this is a test");\n' +
            '\t\t}');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")');
        opts.indent_level = 0;
        test_fragment('\t   a');
        test_fragment(
            '\t   function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '\t   function test() {\n' +
            '\t       console.log("this is a test");\n' +
            '\t   }');
        test_fragment(
            '\t   // This is a random comment\n' +
            'function test(){\n' +
            '  console.log("this is a test");\n' +
            '}',
            //  -- output --
            '\t   // This is a random comment\n' +
            '\t   function test() {\n' +
            '\t       console.log("this is a test");\n' +
            '\t   }');


        //============================================================
        // Support simple language specific option inheritance/overriding - (js = "{ "indent_size": 3 }", css = "{ "indent_size": 5 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (js = "{ "indent_size": 3 }", css = "{ "indent_size": 5 }")');
        opts.js = { 'indent_size': 3 };
        opts.css = { 'indent_size': 5 };
        bt(
            'if (a == b) {\n' +
            '   test();\n' +
            '}');

        // Support simple language specific option inheritance/overriding - (html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 } }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 } }")');
        opts.html = { 'js': { 'indent_size': 3 }, 'css': { 'indent_size': 5 } };
        bt(
            'if (a == b) {\n' +
            '    test();\n' +
            '}');

        // Support simple language specific option inheritance/overriding - (indent_size = "9", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 }, "indent_size": 2}", js = "{ "indent_size": 4 }", css = "{ "indent_size": 3 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_size = "9", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 }, "indent_size": 2}", js = "{ "indent_size": 4 }", css = "{ "indent_size": 3 }")');
        opts.indent_size = 9;
        opts.html = { 'js': { 'indent_size': 3 }, 'css': { 'indent_size': 5 }, 'indent_size': 2};
        opts.js = { 'indent_size': 4 };
        opts.css = { 'indent_size': 3 };
        bt(
            'if (a == b) {\n' +
            '    test();\n' +
            '}');


        //============================================================
        // Brace style permutations - (brace_style = ""collapse,preserve-inline"")
        reset_options();
        set_name('Brace style permutations - (brace_style = ""collapse,preserve-inline"")');
        opts.brace_style = 'collapse,preserve-inline';
        bt(
            'var a ={a: 2};\n' +
            'var a ={a: 2};',
            //  -- output --
            'var a = { a: 2 };\n' +
            'var a = { a: 2 };');
        bt(
            '//case 1\n' +
            'if (a == 1){}\n' +
            '//case 2\n' +
            'else if (a == 2){}',
            //  -- output --
            '//case 1\n' +
            'if (a == 1) {}\n' +
            '//case 2\n' +
            'else if (a == 2) {}');
        bt('if(1){2}else{3}', 'if (1) { 2 } else { 3 }');
        bt('try{a();}catch(b){c();}catch(d){}finally{e();}', 'try { a(); } catch (b) { c(); } catch (d) {} finally { e(); }');

        // Brace style permutations - (brace_style = ""collapse,preserve-inline"")
        reset_options();
        set_name('Brace style permutations - (brace_style = ""collapse,preserve-inline"")');
        opts.brace_style = 'collapse,preserve-inline';
        bt(
            'var a =\n' +
            '{\n' +
            'a: 2\n' +
            '}\n' +
            ';\n' +
            'var a =\n' +
            '{\n' +
            'a: 2\n' +
            '}\n' +
            ';',
            //  -- output --
            'var a = {\n' +
            '    a: 2\n' +
            '};\n' +
            'var a = {\n' +
            '    a: 2\n' +
            '};');
        bt(
            '//case 1\n' +
            'if (a == 1)\n' +
            '{}\n' +
            '//case 2\n' +
            'else if (a == 2)\n' +
            '{}',
            //  -- output --
            '//case 1\n' +
            'if (a == 1) {}\n' +
            '//case 2\n' +
            'else if (a == 2) {}');
        bt(
            'if(1)\n' +
            '{\n' +
            '2\n' +
            '}\n' +
            'else\n' +
            '{\n' +
            '3\n' +
            '}',
            //  -- output --
            'if (1) {\n' +
            '    2\n' +
            '} else {\n' +
            '    3\n' +
            '}');
        bt(
            'try\n' +
            '{\n' +
            'a();\n' +
            '}\n' +
            'catch(b)\n' +
            '{\n' +
            'c();\n' +
            '}\n' +
            'catch(d)\n' +
            '{}\n' +
            'finally\n' +
            '{\n' +
            'e();\n' +
            '}',
            //  -- output --
            'try {\n' +
            '    a();\n' +
            '} catch (b) {\n' +
            '    c();\n' +
            '} catch (d) {} finally {\n' +
            '    e();\n' +
            '}');

        // Brace style permutations - ()
        reset_options();
        set_name('Brace style permutations - ()');
        bt(
            'var a ={a: 2};\n' +
            'var a ={a: 2};',
            //  -- output --
            'var a = {\n' +
            '    a: 2\n' +
            '};\n' +
            'var a = {\n' +
            '    a: 2\n' +
            '};');
        bt(
            '//case 1\n' +
            'if (a == 1){}\n' +
            '//case 2\n' +
            'else if (a == 2){}',
            //  -- output --
            '//case 1\n' +
            'if (a == 1) {}\n' +
            '//case 2\n' +
            'else if (a == 2) {}');
        bt(
            'if(1){2}else{3}',
            //  -- output --
            'if (1) {\n' +
            '    2\n' +
            '} else {\n' +
            '    3\n' +
            '}');
        bt(
            'try{a();}catch(b){c();}catch(d){}finally{e();}',
            //  -- output --
            'try {\n' +
            '    a();\n' +
            '} catch (b) {\n' +
            '    c();\n' +
            '} catch (d) {} finally {\n' +
            '    e();\n' +
            '}');

        // Brace style permutations - (brace_style = ""collapse"")
        reset_options();
        set_name('Brace style permutations - (brace_style = ""collapse"")');
        opts.brace_style = 'collapse';
        bt(
            'var a ={a: 2};\n' +
            'var a ={a: 2};',
            //  -- output --
            'var a = {\n' +
            '    a: 2\n' +
            '};\n' +
            'var a = {\n' +
            '    a: 2\n' +
            '};');
        bt(
            '//case 1\n' +
            'if (a == 1){}\n' +
            '//case 2\n' +
            'else if (a == 2){}',
            //  -- output --
            '//case 1\n' +
            'if (a == 1) {}\n' +
            '//case 2\n' +
            'else if (a == 2) {}');
        bt(
            'if(1){2}else{3}',
            //  -- output --
            'if (1) {\n' +
            '    2\n' +
            '} else {\n' +
            '    3\n' +
            '}');
        bt(
            'try{a();}catch(b){c();}catch(d){}finally{e();}',
            //  -- output --
            'try {\n' +
            '    a();\n' +
            '} catch (b) {\n' +
            '    c();\n' +
            '} catch (d) {} finally {\n' +
            '    e();\n' +
            '}');

        // Brace style permutations - (brace_style = ""collapse"")
        reset_options();
        set_name('Brace style permutations - (brace_style = ""collapse"")');
        opts.brace_style = 'collapse';
        bt(
            'var a =\n' +
            '{\n' +
            'a: 2\n' +
            '}\n' +
            ';\n' +
            'var a =\n' +
            '{\n' +
            'a: 2\n' +
            '}\n' +
            ';',
            //  -- output --
            'var a = {\n' +
            '    a: 2\n' +
            '};\n' +
            'var a = {\n' +
            '    a: 2\n' +
            '};');
        bt(
            '//case 1\n' +
            'if (a == 1)\n' +
            '{}\n' +
            '//case 2\n' +
            'else if (a == 2)\n' +
            '{}',
            //  -- output --
            '//case 1\n' +
            'if (a == 1) {}\n' +
            '//case 2\n' +
            'else if (a == 2) {}');
        bt(
            'if(1)\n' +
            '{\n' +
            '2\n' +
            '}\n' +
            'else\n' +
            '{\n' +
            '3\n' +
            '}',
            //  -- output --
            'if (1) {\n' +
            '    2\n' +
            '} else {\n' +
            '    3\n' +
            '}');
        bt(
            'try\n' +
            '{\n' +
            'a();\n' +
            '}\n' +
            'catch(b)\n' +
            '{\n' +
            'c();\n' +
            '}\n' +
            'catch(d)\n' +
            '{}\n' +
            'finally\n' +
            '{\n' +
            'e();\n' +
            '}',
            //  -- output --
            'try {\n' +
            '    a();\n' +
            '} catch (b) {\n' +
            '    c();\n' +
            '} catch (d) {} finally {\n' +
            '    e();\n' +
            '}');


        //============================================================
        // Comma-first option - (comma_first = "false")
        reset_options();
        set_name('Comma-first option - (comma_first = "false")');
        opts.comma_first = false;
        bt(
            '{a:1, b:2}',
            //  -- output --
            '{\n' +
            '    a: 1,\n' +
            '    b: 2\n' +
            '}');
        bt(
            'var a=1, b=c[d], e=6;',
            //  -- output --
            'var a = 1,\n' +
            '    b = c[d],\n' +
            '    e = 6;');
        bt(
            'for(var a=1,b=2,c=3;d<3;d++)\n' +
            'e',
            //  -- output --
            'for (var a = 1, b = 2, c = 3; d < 3; d++)\n' +
            '    e');
        bt(
            'for(var a=1,b=2,\n' +
            'c=3;d<3;d++)\n' +
            'e',
            //  -- output --
            'for (var a = 1, b = 2,\n' +
            '        c = 3; d < 3; d++)\n' +
            '    e');
        bt(
            'function foo() {\n' +
            '    return [\n' +
            '        "one",\n' +
            '        "two"\n' +
            '    ];\n' +
            '}');
        bt(
            'a=[[1,2],[4,5],[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],[7,8],]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    [7, 8],\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    function() {},\n' +
            '    [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    function() {},\n' +
            '    function() {},\n' +
            '    [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    function() {},\n' +
            '    [7, 8]\n' +
            ']');
        bt('a=[b,c,function(){},function(){},d]', 'a = [b, c, function() {}, function() {}, d]');
        bt(
            'a=[b,c,\n' +
            'function(){},function(){},d]',
            //  -- output --
            'a = [b, c,\n' +
            '    function() {},\n' +
            '    function() {},\n' +
            '    d\n' +
            ']');
        bt('a=[a[1],b[4],c[d[7]]]', 'a = [a[1], b[4], c[d[7]]]');
        bt('[1,2,[3,4,[5,6],7],8]', '[1, 2, [3, 4, [5, 6], 7], 8]');
        bt(
            '[[["1","2"],["3","4"]],[["5","6","7"],["8","9","0"]],[["1","2","3"],["4","5","6","7"],["8","9","0"]]]',
            //  -- output --
            '[\n' +
            '    [\n' +
            '        ["1", "2"],\n' +
            '        ["3", "4"]\n' +
            '    ],\n' +
            '    [\n' +
            '        ["5", "6", "7"],\n' +
            '        ["8", "9", "0"]\n' +
            '    ],\n' +
            '    [\n' +
            '        ["1", "2", "3"],\n' +
            '        ["4", "5", "6", "7"],\n' +
            '        ["8", "9", "0"]\n' +
            '    ]\n' +
            ']');
        bt(
            'changeCollection.add({\n' +
            '    name: "Jonathan" // New line inserted after this line on every save\n' +
            '    , age: 25\n' +
            '});',
            //  -- output --
            'changeCollection.add({\n' +
            '    name: "Jonathan" // New line inserted after this line on every save\n' +
            '        ,\n' +
            '    age: 25\n' +
            '});');
        bt(
            'changeCollection.add(\n' +
            '    function() {\n' +
            '        return true;\n' +
            '    },\n' +
            '    function() {\n' +
            '        return true;\n' +
            '    }\n' +
            ');');

        // Comma-first option - (comma_first = "true")
        reset_options();
        set_name('Comma-first option - (comma_first = "true")');
        opts.comma_first = true;
        bt(
            '{a:1, b:2}',
            //  -- output --
            '{\n' +
            '    a: 1\n' +
            '    , b: 2\n' +
            '}');
        bt(
            'var a=1, b=c[d], e=6;',
            //  -- output --
            'var a = 1\n' +
            '    , b = c[d]\n' +
            '    , e = 6;');
        bt(
            'for(var a=1,b=2,c=3;d<3;d++)\n' +
            'e',
            //  -- output --
            'for (var a = 1, b = 2, c = 3; d < 3; d++)\n' +
            '    e');
        bt(
            'for(var a=1,b=2,\n' +
            'c=3;d<3;d++)\n' +
            'e',
            //  -- output --
            'for (var a = 1, b = 2\n' +
            '        , c = 3; d < 3; d++)\n' +
            '    e');
        bt(
            'function foo() {\n' +
            '    return [\n' +
            '        "one"\n' +
            '        , "two"\n' +
            '    ];\n' +
            '}');
        bt(
            'a=[[1,2],[4,5],[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2]\n' +
            '    , [4, 5]\n' +
            '    , [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],[7,8],]',
            //  -- output --
            'a = [\n' +
            '    [1, 2]\n' +
            '    , [4, 5]\n' +
            '    , [7, 8]\n' +
            ', ]');
        bt(
            'a=[[1,2],[4,5],function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2]\n' +
            '    , [4, 5]\n' +
            '    , function() {}\n' +
            '    , [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2]\n' +
            '    , [4, 5]\n' +
            '    , function() {}\n' +
            '    , function() {}\n' +
            '    , [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2]\n' +
            '    , [4, 5]\n' +
            '    , function() {}\n' +
            '    , [7, 8]\n' +
            ']');
        bt('a=[b,c,function(){},function(){},d]', 'a = [b, c, function() {}, function() {}, d]');
        bt(
            'a=[b,c,\n' +
            'function(){},function(){},d]',
            //  -- output --
            'a = [b, c\n' +
            '    , function() {}\n' +
            '    , function() {}\n' +
            '    , d\n' +
            ']');
        bt('a=[a[1],b[4],c[d[7]]]', 'a = [a[1], b[4], c[d[7]]]');
        bt('[1,2,[3,4,[5,6],7],8]', '[1, 2, [3, 4, [5, 6], 7], 8]');
        bt(
            '[[["1","2"],["3","4"]],[["5","6","7"],["8","9","0"]],[["1","2","3"],["4","5","6","7"],["8","9","0"]]]',
            //  -- output --
            '[\n' +
            '    [\n' +
            '        ["1", "2"]\n' +
            '        , ["3", "4"]\n' +
            '    ]\n' +
            '    , [\n' +
            '        ["5", "6", "7"]\n' +
            '        , ["8", "9", "0"]\n' +
            '    ]\n' +
            '    , [\n' +
            '        ["1", "2", "3"]\n' +
            '        , ["4", "5", "6", "7"]\n' +
            '        , ["8", "9", "0"]\n' +
            '    ]\n' +
            ']');
        bt(
            'changeCollection.add({\n' +
            '    name: "Jonathan" // New line inserted after this line on every save\n' +
            '    , age: 25\n' +
            '});');
        bt(
            'changeCollection.add(\n' +
            '    function() {\n' +
            '        return true;\n' +
            '    },\n' +
            '    function() {\n' +
            '        return true;\n' +
            '    }\n' +
            ');',
            //  -- output --
            'changeCollection.add(\n' +
            '    function() {\n' +
            '        return true;\n' +
            '    }\n' +
            '    , function() {\n' +
            '        return true;\n' +
            '    }\n' +
            ');');


        //============================================================
        // Unindent chained functions - (unindent_chained_methods = "true")
        reset_options();
        set_name('Unindent chained functions - (unindent_chained_methods = "true")');
        opts.unindent_chained_methods = true;
        bt(
            'f().f().f()\n' +
            '    .f().f();',
            //  -- output --
            'f().f().f()\n' +
            '.f().f();');
        bt(
            'f()\n' +
            '    .f()\n' +
            '    .f();',
            //  -- output --
            'f()\n' +
            '.f()\n' +
            '.f();');
        bt(
            'f(function() {\n' +
            '    f()\n' +
            '        .f()\n' +
            '        .f();\n' +
            '});',
            //  -- output --
            'f(function() {\n' +
            '    f()\n' +
            '    .f()\n' +
            '    .f();\n' +
            '});');

        // regression test for fix #1378
        bt(
            'f(function() {\n' +
            '    if(g === 1)\n' +
            '        g = 0;\n' +
            '    else\n' +
            '        g = 1;\n' +
            '\n' +
            '    f()\n' +
            '        .f()\n' +
            '        .f();\n' +
            '});',
            //  -- output --
            'f(function() {\n' +
            '    if (g === 1)\n' +
            '        g = 0;\n' +
            '    else\n' +
            '        g = 1;\n' +
            '\n' +
            '    f()\n' +
            '    .f()\n' +
            '    .f();\n' +
            '});');

        // regression test for fix #1533
        bt(
            'angular.module("test").controller("testCtrl", function($scope) {\n' +
            '    $scope.tnew;\n' +
            '    $scope.toggle_tnew = function() {\n' +
            '        $scope.mode = 0;\n' +
            '        if (!$scope.tnew) {\n' +
            '            $scope.tnew = {};\n' +
            '        } else $scope.tnew = null;\n' +
            '    }\n' +
            '    $scope.fn = function() {\n' +
            '        return null;\n' +
            '    }\n' +
            '});');


        //============================================================
        // Space in parens tests - (space_in_paren = "false", space_in_empty_paren = "false")
        reset_options();
        set_name('Space in parens tests - (space_in_paren = "false", space_in_empty_paren = "false")');
        opts.space_in_paren = false;
        opts.space_in_empty_paren = false;
        bt('if(p) foo(a,b);', 'if (p) foo(a, b);');
        bt(
            'try{while(true){willThrow()}}catch(result)switch(result){case 1:++result }',
            //  -- output --
            'try {\n' +
            '    while (true) {\n' +
            '        willThrow()\n' +
            '    }\n' +
            '} catch (result) switch (result) {\n' +
            '    case 1:\n' +
            '        ++result\n' +
            '}');
        bt('((e/((a+(b)*c)-d))^2)*5;', '((e / ((a + (b) * c) - d)) ^ 2) * 5;');
        bt(
            'function f(a,b) {if(a) b()}function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f(a, b) {\n' +
            '    if (a) b()\n' +
            '}\n' +
            '\n' +
            'function g(a, b) {\n' +
            '    if (!a) b()\n' +
            '}');
        bt('a=[][    ](  );', 'a = [][]();');
        bt('a=()(    )[  ];', 'a = ()()[];');
        bt('a=[b,c,d];', 'a = [b, c, d];');
        bt('a= f[b];', 'a = f[b];');

        // Issue #1151 - inside class methods
        bt(
            'export default class Test extends Component {\n' +
            '    render() {\n' +
            '        someOther();\n' +
            '        return null;\n' +
            '    }\n' +
            '}');
        bt(
            '{\n' +
            '    files: a[][ {\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b(c)[ "im/design_standards/*.*" ],\n' +
            '        dest: "www/gui/build"\n' +
            '    } ]\n' +
            '}',
            //  -- output --
            '{\n' +
            '    files: a[][{\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b(c)["im/design_standards/*.*"],\n' +
            '        dest: "www/gui/build"\n' +
            '    }]\n' +
            '}');

        // Space in parens tests - (space_in_paren = "false", space_in_empty_paren = "true")
        reset_options();
        set_name('Space in parens tests - (space_in_paren = "false", space_in_empty_paren = "true")');
        opts.space_in_paren = false;
        opts.space_in_empty_paren = true;
        bt('if(p) foo(a,b);', 'if (p) foo(a, b);');
        bt(
            'try{while(true){willThrow()}}catch(result)switch(result){case 1:++result }',
            //  -- output --
            'try {\n' +
            '    while (true) {\n' +
            '        willThrow()\n' +
            '    }\n' +
            '} catch (result) switch (result) {\n' +
            '    case 1:\n' +
            '        ++result\n' +
            '}');
        bt('((e/((a+(b)*c)-d))^2)*5;', '((e / ((a + (b) * c) - d)) ^ 2) * 5;');
        bt(
            'function f(a,b) {if(a) b()}function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f(a, b) {\n' +
            '    if (a) b()\n' +
            '}\n' +
            '\n' +
            'function g(a, b) {\n' +
            '    if (!a) b()\n' +
            '}');
        bt('a=[][    ](  );', 'a = [][]();');
        bt('a=()(    )[  ];', 'a = ()()[];');
        bt('a=[b,c,d];', 'a = [b, c, d];');
        bt('a= f[b];', 'a = f[b];');

        // Issue #1151 - inside class methods
        bt(
            'export default class Test extends Component {\n' +
            '    render() {\n' +
            '        someOther();\n' +
            '        return null;\n' +
            '    }\n' +
            '}');
        bt(
            '{\n' +
            '    files: a[][ {\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b(c)[ "im/design_standards/*.*" ],\n' +
            '        dest: "www/gui/build"\n' +
            '    } ]\n' +
            '}',
            //  -- output --
            '{\n' +
            '    files: a[][{\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b(c)["im/design_standards/*.*"],\n' +
            '        dest: "www/gui/build"\n' +
            '    }]\n' +
            '}');

        // Space in parens tests - (space_in_paren = "true", space_in_empty_paren = "false")
        reset_options();
        set_name('Space in parens tests - (space_in_paren = "true", space_in_empty_paren = "false")');
        opts.space_in_paren = true;
        opts.space_in_empty_paren = false;
        bt('if(p) foo(a,b);', 'if ( p ) foo( a, b );');
        bt(
            'try{while(true){willThrow()}}catch(result)switch(result){case 1:++result }',
            //  -- output --
            'try {\n' +
            '    while ( true ) {\n' +
            '        willThrow()\n' +
            '    }\n' +
            '} catch ( result ) switch ( result ) {\n' +
            '    case 1:\n' +
            '        ++result\n' +
            '}');
        bt('((e/((a+(b)*c)-d))^2)*5;', '( ( e / ( ( a + ( b ) * c ) - d ) ) ^ 2 ) * 5;');
        bt(
            'function f(a,b) {if(a) b()}function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f( a, b ) {\n' +
            '    if ( a ) b()\n' +
            '}\n' +
            '\n' +
            'function g( a, b ) {\n' +
            '    if ( !a ) b()\n' +
            '}');
        bt('a=[][    ](  );', 'a = [][]();');
        bt('a=()(    )[  ];', 'a = ()()[];');
        bt('a=[b,c,d];', 'a = [ b, c, d ];');
        bt('a= f[b];', 'a = f[ b ];');

        // Issue #1151 - inside class methods
        bt(
            'export default class Test extends Component {\n' +
            '    render() {\n' +
            '        someOther();\n' +
            '        return null;\n' +
            '    }\n' +
            '}');
        bt(
            '{\n' +
            '    files: a[][ {\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b(c)[ "im/design_standards/*.*" ],\n' +
            '        dest: "www/gui/build"\n' +
            '    } ]\n' +
            '}',
            //  -- output --
            '{\n' +
            '    files: a[][ {\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b( c )[ "im/design_standards/*.*" ],\n' +
            '        dest: "www/gui/build"\n' +
            '    } ]\n' +
            '}');

        // Space in parens tests - (space_in_paren = "true", space_in_empty_paren = "true")
        reset_options();
        set_name('Space in parens tests - (space_in_paren = "true", space_in_empty_paren = "true")');
        opts.space_in_paren = true;
        opts.space_in_empty_paren = true;
        bt('if(p) foo(a,b);', 'if ( p ) foo( a, b );');
        bt(
            'try{while(true){willThrow()}}catch(result)switch(result){case 1:++result }',
            //  -- output --
            'try {\n' +
            '    while ( true ) {\n' +
            '        willThrow( )\n' +
            '    }\n' +
            '} catch ( result ) switch ( result ) {\n' +
            '    case 1:\n' +
            '        ++result\n' +
            '}');
        bt('((e/((a+(b)*c)-d))^2)*5;', '( ( e / ( ( a + ( b ) * c ) - d ) ) ^ 2 ) * 5;');
        bt(
            'function f(a,b) {if(a) b()}function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f( a, b ) {\n' +
            '    if ( a ) b( )\n' +
            '}\n' +
            '\n' +
            'function g( a, b ) {\n' +
            '    if ( !a ) b( )\n' +
            '}');
        bt('a=[][    ](  );', 'a = [ ][ ]( );');
        bt('a=()(    )[  ];', 'a = ( )( )[ ];');
        bt('a=[b,c,d];', 'a = [ b, c, d ];');
        bt('a= f[b];', 'a = f[ b ];');

        // Issue #1151 - inside class methods
        bt(
            'export default class Test extends Component {\n' +
            '    render() {\n' +
            '        someOther();\n' +
            '        return null;\n' +
            '    }\n' +
            '}',
            //  -- output --
            'export default class Test extends Component {\n' +
            '    render( ) {\n' +
            '        someOther( );\n' +
            '        return null;\n' +
            '    }\n' +
            '}');
        bt(
            '{\n' +
            '    files: a[][ {\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b(c)[ "im/design_standards/*.*" ],\n' +
            '        dest: "www/gui/build"\n' +
            '    } ]\n' +
            '}',
            //  -- output --
            '{\n' +
            '    files: a[ ][ {\n' +
            '        expand: true,\n' +
            '        cwd: "www/gui/",\n' +
            '        src: b( c )[ "im/design_standards/*.*" ],\n' +
            '        dest: "www/gui/build"\n' +
            '    } ]\n' +
            '}');


        //============================================================
        // general preserve_newlines tests - (preserve_newlines = "false")
        reset_options();
        set_name('general preserve_newlines tests - (preserve_newlines = "false")');
        opts.preserve_newlines = false;
        bt(
            'if (foo) // comment\n' +
            '    bar();');
        bt(
            'if (foo) // comment\n' +
            '    bar();');
        bt(
            'if (foo) // comment\n' +
            '    (bar());');
        bt(
            'if (foo) // comment\n' +
            '    (bar());');
        bt(
            'if (foo) // comment\n' +
            '    /asdf/;');
        bt(
            'this.oa = new OAuth(\n' +
            '    _requestToken,\n' +
            '    _accessToken,\n' +
            '    consumer_key\n' +
            ');',
            //  -- output --
            'this.oa = new OAuth(_requestToken, _accessToken, consumer_key);');
        bt(
            'foo = {\n' +
            '    x: y, // #44\n' +
            '    w: z // #44\n' +
            '}');
        bt(
            'switch (x) {\n' +
            '    case "a":\n' +
            '        // comment on newline\n' +
            '        break;\n' +
            '    case "b": // comment on same line\n' +
            '        break;\n' +
            '}');
        bt(
            'this.type =\n' +
            '    this.options =\n' +
            '    // comment\n' +
            '    this.enabled null;',
            //  -- output --
            'this.type = this.options =\n' +
            '    // comment\n' +
            '    this.enabled null;');
        bt(
            'someObj\n' +
            '    .someFunc1()\n' +
            '    // This comment should not break the indent\n' +
            '    .someFunc2();',
            //  -- output --
            'someObj.someFunc1()\n' +
            '    // This comment should not break the indent\n' +
            '    .someFunc2();');
        bt(
            'if (true ||\n' +
            '!true) return;',
            //  -- output --
            'if (true || !true) return;');
        bt(
            'if\n' +
            '(foo)\n' +
            'if\n' +
            '(bar)\n' +
            'if\n' +
            '(baz)\n' +
            'whee();\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz) whee();\n' +
            'a();');
        bt(
            'if\n' +
            '(foo)\n' +
            'if\n' +
            '(bar)\n' +
            'if\n' +
            '(baz)\n' +
            'whee();\n' +
            'else\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz) whee();\n' +
            '        else a();');
        bt(
            'if (foo)\n' +
            'bar();\n' +
            'else\n' +
            'car();',
            //  -- output --
            'if (foo) bar();\n' +
            'else car();');
        bt(
            'if (foo) if (bar) if (baz);\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz);\n' +
            'a();');
        bt(
            'if (foo) if (bar) if (baz) whee();\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz) whee();\n' +
            'a();');
        bt(
            'if (foo) a()\n' +
            'if (bar) if (baz) whee();\n' +
            'a();',
            //  -- output --
            'if (foo) a()\n' +
            'if (bar)\n' +
            '    if (baz) whee();\n' +
            'a();');
        bt(
            'if (foo);\n' +
            'if (bar) if (baz) whee();\n' +
            'a();',
            //  -- output --
            'if (foo);\n' +
            'if (bar)\n' +
            '    if (baz) whee();\n' +
            'a();');
        bt(
            'if (options)\n' +
            '    for (var p in options)\n' +
            '        this[p] = options[p];',
            //  -- output --
            'if (options)\n' +
            '    for (var p in options) this[p] = options[p];');
        bt(
            'if (options) for (var p in options) this[p] = options[p];',
            //  -- output --
            'if (options)\n' +
            '    for (var p in options) this[p] = options[p];');
        bt(
            'if (options) do q(); while (b());',
            //  -- output --
            'if (options)\n' +
            '    do q(); while (b());');
        bt(
            'if (options) while (b()) q();',
            //  -- output --
            'if (options)\n' +
            '    while (b()) q();');
        bt(
            'if (options) do while (b()) q(); while (a());',
            //  -- output --
            'if (options)\n' +
            '    do\n' +
            '        while (b()) q(); while (a());');
        bt(
            'function f(a, b, c,\n' +
            'd, e) {}',
            //  -- output --
            'function f(a, b, c, d, e) {}');
        bt(
            'function f(a,b) {if(a) b()}function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f(a, b) {\n' +
            '    if (a) b()\n' +
            '}\n' +
            '\n' +
            'function g(a, b) {\n' +
            '    if (!a) b()\n' +
            '}');
        bt(
            'function f(a,b) {if(a) b()}\n' +
            '\n' +
            '\n' +
            '\n' +
            'function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f(a, b) {\n' +
            '    if (a) b()\n' +
            '}\n' +
            '\n' +
            'function g(a, b) {\n' +
            '    if (!a) b()\n' +
            '}');
        bt(
            '(if(a) b())(if(a) b())',
            //  -- output --
            '(\n' +
            '    if (a) b())(\n' +
            '    if (a) b())');
        bt(
            '(if(a) b())\n' +
            '\n' +
            '\n' +
            '(if(a) b())',
            //  -- output --
            '(\n' +
            '    if (a) b())\n' +
            '(\n' +
            '    if (a) b())');
        bt(
            'if\n' +
            '(a)\n' +
            'b();',
            //  -- output --
            'if (a) b();');
        bt(
            'var a =\n' +
            'foo',
            //  -- output --
            'var a = foo');
        bt(
            'var a = {\n' +
            '"a":1,\n' +
            '"b":2}',
            //  -- output --
            'var a = {\n' +
            '    "a": 1,\n' +
            '    "b": 2\n' +
            '}');
        bt(
            'var a = {\n' +
            '\'a\':1,\n' +
            '\'b\':2}',
            //  -- output --
            'var a = {\n' +
            '    \'a\': 1,\n' +
            '    \'b\': 2\n' +
            '}');
        bt('var a = /*i*/ "b";');
        bt(
            'var a = /*i*/\n' +
            '"b";',
            //  -- output --
            'var a = /*i*/ "b";');
        bt(
            '{\n' +
            '\n' +
            '\n' +
            '"x"\n' +
            '}',
            //  -- output --
            '{\n' +
            '    "x"\n' +
            '}');
        bt(
            'if(a &&\n' +
            'b\n' +
            '||\n' +
            'c\n' +
            '||d\n' +
            '&&\n' +
            'e) e = f',
            //  -- output --
            'if (a && b || c || d && e) e = f');
        bt(
            'if(a &&\n' +
            '(b\n' +
            '||\n' +
            'c\n' +
            '||d)\n' +
            '&&\n' +
            'e) e = f',
            //  -- output --
            'if (a && (b || c || d) && e) e = f');
        test_fragment(
            '\n' +
            '\n' +
            '"x"',
            //  -- output --
            '"x"');
        test_fragment(
            '{\n' +
            '\n' +
            '"x"\n' +
            'h=5;\n' +
            '}',
            //  -- output --
            '{\n' +
            '    "x"\n' +
            '    h = 5;\n' +
            '}');
        bt(
            'var a = "foo" +\n' +
            '    "bar";',
            //  -- output --
            'var a = "foo" + "bar";');
        bt(
            'var a = 42; // foo\n' +
            '\n' +
            'var b;',
            //  -- output --
            'var a = 42; // foo\n' +
            'var b;');
        bt(
            'var a = 42; // foo\n' +
            '\n' +
            '\n' +
            'var b;',
            //  -- output --
            'var a = 42; // foo\n' +
            'var b;');
        bt(
            'a = 1;\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            'b = 2;',
            //  -- output --
            'a = 1;\n' +
            'b = 2;');

        // general preserve_newlines tests - (preserve_newlines = "true")
        reset_options();
        set_name('general preserve_newlines tests - (preserve_newlines = "true")');
        opts.preserve_newlines = true;
        bt(
            'if (foo) // comment\n' +
            '    bar();');
        bt(
            'if (foo) // comment\n' +
            '    bar();');
        bt(
            'if (foo) // comment\n' +
            '    (bar());');
        bt(
            'if (foo) // comment\n' +
            '    (bar());');
        bt(
            'if (foo) // comment\n' +
            '    /asdf/;');
        bt(
            'this.oa = new OAuth(\n' +
            '    _requestToken,\n' +
            '    _accessToken,\n' +
            '    consumer_key\n' +
            ');');
        bt(
            'foo = {\n' +
            '    x: y, // #44\n' +
            '    w: z // #44\n' +
            '}');
        bt(
            'switch (x) {\n' +
            '    case "a":\n' +
            '        // comment on newline\n' +
            '        break;\n' +
            '    case "b": // comment on same line\n' +
            '        break;\n' +
            '}');
        bt(
            'this.type =\n' +
            '    this.options =\n' +
            '    // comment\n' +
            '    this.enabled null;');
        bt(
            'someObj\n' +
            '    .someFunc1()\n' +
            '    // This comment should not break the indent\n' +
            '    .someFunc2();');
        bt(
            'if (true ||\n' +
            '!true) return;',
            //  -- output --
            'if (true ||\n' +
            '    !true) return;');
        bt(
            'if\n' +
            '(foo)\n' +
            'if\n' +
            '(bar)\n' +
            'if\n' +
            '(baz)\n' +
            'whee();\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz)\n' +
            '            whee();\n' +
            'a();');
        bt(
            'if\n' +
            '(foo)\n' +
            'if\n' +
            '(bar)\n' +
            'if\n' +
            '(baz)\n' +
            'whee();\n' +
            'else\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz)\n' +
            '            whee();\n' +
            '        else\n' +
            '            a();');
        bt(
            'if (foo)\n' +
            'bar();\n' +
            'else\n' +
            'car();',
            //  -- output --
            'if (foo)\n' +
            '    bar();\n' +
            'else\n' +
            '    car();');
        bt(
            'if (foo) if (bar) if (baz);\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz);\n' +
            'a();');
        bt(
            'if (foo) if (bar) if (baz) whee();\n' +
            'a();',
            //  -- output --
            'if (foo)\n' +
            '    if (bar)\n' +
            '        if (baz) whee();\n' +
            'a();');
        bt(
            'if (foo) a()\n' +
            'if (bar) if (baz) whee();\n' +
            'a();',
            //  -- output --
            'if (foo) a()\n' +
            'if (bar)\n' +
            '    if (baz) whee();\n' +
            'a();');
        bt(
            'if (foo);\n' +
            'if (bar) if (baz) whee();\n' +
            'a();',
            //  -- output --
            'if (foo);\n' +
            'if (bar)\n' +
            '    if (baz) whee();\n' +
            'a();');
        bt(
            'if (options)\n' +
            '    for (var p in options)\n' +
            '        this[p] = options[p];');
        bt(
            'if (options) for (var p in options) this[p] = options[p];',
            //  -- output --
            'if (options)\n' +
            '    for (var p in options) this[p] = options[p];');
        bt(
            'if (options) do q(); while (b());',
            //  -- output --
            'if (options)\n' +
            '    do q(); while (b());');
        bt(
            'if (options) while (b()) q();',
            //  -- output --
            'if (options)\n' +
            '    while (b()) q();');
        bt(
            'if (options) do while (b()) q(); while (a());',
            //  -- output --
            'if (options)\n' +
            '    do\n' +
            '        while (b()) q(); while (a());');
        bt(
            'function f(a, b, c,\n' +
            'd, e) {}',
            //  -- output --
            'function f(a, b, c,\n' +
            '    d, e) {}');
        bt(
            'function f(a,b) {if(a) b()}function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f(a, b) {\n' +
            '    if (a) b()\n' +
            '}\n' +
            '\n' +
            'function g(a, b) {\n' +
            '    if (!a) b()\n' +
            '}');
        bt(
            'function f(a,b) {if(a) b()}\n' +
            '\n' +
            '\n' +
            '\n' +
            'function g(a,b) {if(!a) b()}',
            //  -- output --
            'function f(a, b) {\n' +
            '    if (a) b()\n' +
            '}\n' +
            '\n' +
            '\n' +
            '\n' +
            'function g(a, b) {\n' +
            '    if (!a) b()\n' +
            '}');
        bt(
            '(if(a) b())(if(a) b())',
            //  -- output --
            '(\n' +
            '    if (a) b())(\n' +
            '    if (a) b())');
        bt(
            '(if(a) b())\n' +
            '\n' +
            '\n' +
            '(if(a) b())',
            //  -- output --
            '(\n' +
            '    if (a) b())\n' +
            '\n' +
            '\n' +
            '(\n' +
            '    if (a) b())');
        bt(
            'if\n' +
            '(a)\n' +
            'b();',
            //  -- output --
            'if (a)\n' +
            '    b();');
        bt(
            'var a =\n' +
            'foo',
            //  -- output --
            'var a =\n' +
            '    foo');
        bt(
            'var a = {\n' +
            '"a":1,\n' +
            '"b":2}',
            //  -- output --
            'var a = {\n' +
            '    "a": 1,\n' +
            '    "b": 2\n' +
            '}');
        bt(
            'var a = {\n' +
            '\'a\':1,\n' +
            '\'b\':2}',
            //  -- output --
            'var a = {\n' +
            '    \'a\': 1,\n' +
            '    \'b\': 2\n' +
            '}');
        bt('var a = /*i*/ "b";');
        bt(
            'var a = /*i*/\n' +
            '"b";',
            //  -- output --
            'var a = /*i*/\n' +
            '    "b";');
        bt(
            '{\n' +
            '\n' +
            '\n' +
            '"x"\n' +
            '}',
            //  -- output --
            '{\n' +
            '\n' +
            '\n' +
            '    "x"\n' +
            '}');
        bt(
            'if(a &&\n' +
            'b\n' +
            '||\n' +
            'c\n' +
            '||d\n' +
            '&&\n' +
            'e) e = f',
            //  -- output --
            'if (a &&\n' +
            '    b ||\n' +
            '    c ||\n' +
            '    d &&\n' +
            '    e) e = f');
        bt(
            'if(a &&\n' +
            '(b\n' +
            '||\n' +
            'c\n' +
            '||d)\n' +
            '&&\n' +
            'e) e = f',
            //  -- output --
            'if (a &&\n' +
            '    (b ||\n' +
            '        c ||\n' +
            '        d) &&\n' +
            '    e) e = f');
        test_fragment(
            '\n' +
            '\n' +
            '"x"',
            //  -- output --
            '"x"');
        test_fragment(
            '{\n' +
            '\n' +
            '"x"\n' +
            'h=5;\n' +
            '}',
            //  -- output --
            '{\n' +
            '\n' +
            '    "x"\n' +
            '    h = 5;\n' +
            '}');
        bt(
            'var a = "foo" +\n' +
            '    "bar";');
        bt(
            'var a = 42; // foo\n' +
            '\n' +
            'var b;');
        bt(
            'var a = 42; // foo\n' +
            '\n' +
            '\n' +
            'var b;');
        bt(
            'a = 1;\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            'b = 2;');


        //============================================================
        // break chained methods - (break_chained_methods = "false", preserve_newlines = "false")
        reset_options();
        set_name('break chained methods - (break_chained_methods = "false", preserve_newlines = "false")');
        opts.break_chained_methods = false;
        opts.preserve_newlines = false;
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'foo.bar().baz().cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat); foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo.bar().baz().cucumber(fat);\n' +
            'foo.bar().baz().cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)\n' +
            ' foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo.bar().baz().cucumber(fat)\n' +
            'foo.bar().baz().cucumber(fat)');
        bt(
            'this\n' +
            '.something = foo.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'this.something = foo.bar().baz().cucumber(fat)');
        bt('this.something.xxx = foo.moo.bar()');
        bt(
            'this\n' +
            '.something\n' +
            '.xxx = foo.moo\n' +
            '.bar()',
            //  -- output --
            'this.something.xxx = foo.moo.bar()');

        // optional chaining operator
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'foo?.bar()?.baz()?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat); foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo?.bar()?.baz()?.cucumber(fat);\n' +
            'foo?.bar()?.baz()?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)\n' +
            ' foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo?.bar()?.baz()?.cucumber(fat)\n' +
            'foo?.bar()?.baz()?.cucumber(fat)');
        bt(
            'this\n' +
            '?.something = foo?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'this?.something = foo?.bar()?.baz()?.cucumber(fat)');
        bt('this?.something?.xxx = foo?.moo?.bar()');
        bt(
            'this\n' +
            '?.something\n' +
            '?.xxx = foo?.moo\n' +
            '?.bar()',
            //  -- output --
            'this?.something?.xxx = foo?.moo?.bar()');

        // break chained methods - (break_chained_methods = "false", preserve_newlines = "true")
        reset_options();
        set_name('break chained methods - (break_chained_methods = "false", preserve_newlines = "true")');
        opts.break_chained_methods = false;
        opts.preserve_newlines = true;
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    .bar()\n' +
            '    .baz().cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat); foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    .bar()\n' +
            '    .baz().cucumber(fat);\n' +
            'foo.bar().baz().cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)\n' +
            ' foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    .bar()\n' +
            '    .baz().cucumber(fat)\n' +
            'foo.bar().baz().cucumber(fat)');
        bt(
            'this\n' +
            '.something = foo.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'this\n' +
            '    .something = foo.bar()\n' +
            '    .baz().cucumber(fat)');
        bt('this.something.xxx = foo.moo.bar()');
        bt(
            'this\n' +
            '.something\n' +
            '.xxx = foo.moo\n' +
            '.bar()',
            //  -- output --
            'this\n' +
            '    .something\n' +
            '    .xxx = foo.moo\n' +
            '    .bar()');

        // optional chaining operator
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    ?.bar()\n' +
            '    ?.baz()?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat); foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    ?.bar()\n' +
            '    ?.baz()?.cucumber(fat);\n' +
            'foo?.bar()?.baz()?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)\n' +
            ' foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    ?.bar()\n' +
            '    ?.baz()?.cucumber(fat)\n' +
            'foo?.bar()?.baz()?.cucumber(fat)');
        bt(
            'this\n' +
            '?.something = foo?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'this\n' +
            '    ?.something = foo?.bar()\n' +
            '    ?.baz()?.cucumber(fat)');
        bt('this?.something?.xxx = foo?.moo?.bar()');
        bt(
            'this\n' +
            '?.something\n' +
            '?.xxx = foo?.moo\n' +
            '?.bar()',
            //  -- output --
            'this\n' +
            '    ?.something\n' +
            '    ?.xxx = foo?.moo\n' +
            '    ?.bar()');

        // break chained methods - (break_chained_methods = "true", preserve_newlines = "false")
        reset_options();
        set_name('break chained methods - (break_chained_methods = "true", preserve_newlines = "false")');
        opts.break_chained_methods = true;
        opts.preserve_newlines = false;
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat); foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat);\n' +
            'foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)\n' +
            ' foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)\n' +
            'foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt(
            'this\n' +
            '.something = foo.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'this.something = foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt('this.something.xxx = foo.moo.bar()');
        bt(
            'this\n' +
            '.something\n' +
            '.xxx = foo.moo\n' +
            '.bar()',
            //  -- output --
            'this.something.xxx = foo.moo.bar()');

        // optional chaining operator
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat); foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat);\n' +
            'foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)\n' +
            ' foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)\n' +
            'foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt(
            'this\n' +
            '?.something = foo?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'this?.something = foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt('this?.something?.xxx = foo?.moo?.bar()');
        bt(
            'this\n' +
            '?.something\n' +
            '?.xxx = foo?.moo\n' +
            '?.bar()',
            //  -- output --
            'this?.something?.xxx = foo?.moo?.bar()');

        // break chained methods - (break_chained_methods = "true", preserve_newlines = "true")
        reset_options();
        set_name('break chained methods - (break_chained_methods = "true", preserve_newlines = "true")');
        opts.break_chained_methods = true;
        opts.preserve_newlines = true;
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    .bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat); foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    .bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat);\n' +
            'foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt(
            'foo\n' +
            '.bar()\n' +
            '.baz().cucumber(fat)\n' +
            ' foo.bar().baz().cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    .bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)\n' +
            'foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt(
            'this\n' +
            '.something = foo.bar()\n' +
            '.baz().cucumber(fat)',
            //  -- output --
            'this\n' +
            '    .something = foo.bar()\n' +
            '    .baz()\n' +
            '    .cucumber(fat)');
        bt('this.something.xxx = foo.moo.bar()');
        bt(
            'this\n' +
            '.something\n' +
            '.xxx = foo.moo\n' +
            '.bar()',
            //  -- output --
            'this\n' +
            '    .something\n' +
            '    .xxx = foo.moo\n' +
            '    .bar()');

        // optional chaining operator
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    ?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat); foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    ?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat);\n' +
            'foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt(
            'foo\n' +
            '?.bar()\n' +
            '?.baz()?.cucumber(fat)\n' +
            ' foo?.bar()?.baz()?.cucumber(fat)',
            //  -- output --
            'foo\n' +
            '    ?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)\n' +
            'foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt(
            'this\n' +
            '?.something = foo?.bar()\n' +
            '?.baz()?.cucumber(fat)',
            //  -- output --
            'this\n' +
            '    ?.something = foo?.bar()\n' +
            '    ?.baz()\n' +
            '    ?.cucumber(fat)');
        bt('this?.something?.xxx = foo?.moo?.bar()');
        bt(
            'this\n' +
            '?.something\n' +
            '?.xxx = foo?.moo\n' +
            '?.bar()',
            //  -- output --
            'this\n' +
            '    ?.something\n' +
            '    ?.xxx = foo?.moo\n' +
            '    ?.bar()');


        //============================================================
        // line wrapping 0
        reset_options();
        set_name('line wrapping 0');
        opts.preserve_newlines = false;
        opts.wrap_line_length = 0;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap.but_this_can\n' +
            'return between_return_and_expression_should_never_wrap.but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            'if (wraps_can_occur && inside_an_if_block) that_is_.okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token + 12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap + but_this_can,\n' +
            '    propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap.but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap.but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            '    if (wraps_can_occur && inside_an_if_block) that_is_.okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token + 12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap + but_this_can,\n' +
            '        propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 70
        reset_options();
        set_name('line wrapping 70');
        opts.preserve_newlines = false;
        opts.wrap_line_length = 70;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap.but_this_can\n' +
            'return between_return_and_expression_should_never_wrap.but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            'if (wraps_can_occur && inside_an_if_block) that_is_.okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token + 12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap + but_this_can,\n' +
            '    propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap.but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            '    if (wraps_can_occur && inside_an_if_block) that_is_.okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token + 12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap + but_this_can,\n' +
            '        propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 40
        reset_options();
        set_name('line wrapping 40');
        opts.preserve_newlines = false;
        opts.wrap_line_length = 40;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f &&\n' +
            '    "sass") || (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'if (wraps_can_occur &&\n' +
            '    inside_an_if_block) that_is_.okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token +\n' +
            '        12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap +\n' +
            '        but_this_can,\n' +
            '    propertz: first_token_should_never_wrap +\n' +
            '        !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" +\n' +
            '        "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f &&\n' +
            '        "sass") || (leans &&\n' +
            '        mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    if (wraps_can_occur &&\n' +
            '        inside_an_if_block) that_is_\n' +
            '        .okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token +\n' +
            '            12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap +\n' +
            '            but_this_can,\n' +
            '        propertz: first_token_should_never_wrap +\n' +
            '            !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" +\n' +
            '            "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 41
        reset_options();
        set_name('line wrapping 41');
        opts.preserve_newlines = false;
        opts.wrap_line_length = 41;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") ||\n' +
            '    (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'if (wraps_can_occur &&\n' +
            '    inside_an_if_block) that_is_.okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token +\n' +
            '        12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap +\n' +
            '        but_this_can,\n' +
            '    propertz: first_token_should_never_wrap +\n' +
            '        !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" +\n' +
            '        "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f &&\n' +
            '        "sass") || (leans &&\n' +
            '        mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    if (wraps_can_occur &&\n' +
            '        inside_an_if_block) that_is_\n' +
            '        .okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token +\n' +
            '            12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap +\n' +
            '            but_this_can,\n' +
            '        propertz: first_token_should_never_wrap +\n' +
            '            !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" +\n' +
            '            "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 45
        reset_options();
        set_name('line wrapping 45');
        opts.preserve_newlines = false;
        opts.wrap_line_length = 45;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") || (\n' +
            '    leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'if (wraps_can_occur && inside_an_if_block)\n' +
            '    that_is_.okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token +\n' +
            '        12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap +\n' +
            '        but_this_can,\n' +
            '    propertz: first_token_should_never_wrap +\n' +
            '        !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" +\n' +
            '        "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f && "sass") ||\n' +
            '        (leans && mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    if (wraps_can_occur &&\n' +
            '        inside_an_if_block) that_is_.okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token +\n' +
            '            12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap +\n' +
            '            but_this_can,\n' +
            '        propertz: first_token_should_never_wrap +\n' +
            '            !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" +\n' +
            '            "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 0
        reset_options();
        set_name('line wrapping 0');
        opts.preserve_newlines = true;
        opts.wrap_line_length = 0;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap.but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            'if (wraps_can_occur && inside_an_if_block) that_is_\n' +
            '    .okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token + 12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap + but_this_can,\n' +
            '    propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap.but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            '    if (wraps_can_occur && inside_an_if_block) that_is_\n' +
            '        .okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token + 12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap + but_this_can,\n' +
            '        propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 70
        reset_options();
        set_name('line wrapping 70');
        opts.preserve_newlines = true;
        opts.wrap_line_length = 70;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap.but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            'if (wraps_can_occur && inside_an_if_block) that_is_\n' +
            '    .okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token + 12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap + but_this_can,\n' +
            '    propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f && "sass") || (leans && mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap.but_this_can\n' +
            '    if (wraps_can_occur && inside_an_if_block) that_is_\n' +
            '        .okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token + 12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap + but_this_can,\n' +
            '        propertz: first_token_should_never_wrap + !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" + "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 40
        reset_options();
        set_name('line wrapping 40');
        opts.preserve_newlines = true;
        opts.wrap_line_length = 40;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f &&\n' +
            '    "sass") || (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'if (wraps_can_occur &&\n' +
            '    inside_an_if_block) that_is_\n' +
            '    .okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token +\n' +
            '        12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap +\n' +
            '        but_this_can,\n' +
            '    propertz: first_token_should_never_wrap +\n' +
            '        !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" +\n' +
            '        "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f &&\n' +
            '        "sass") || (leans &&\n' +
            '        mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    if (wraps_can_occur &&\n' +
            '        inside_an_if_block) that_is_\n' +
            '        .okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token +\n' +
            '            12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap +\n' +
            '            but_this_can,\n' +
            '        propertz: first_token_should_never_wrap +\n' +
            '            !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" +\n' +
            '            "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 41
        reset_options();
        set_name('line wrapping 41');
        opts.preserve_newlines = true;
        opts.wrap_line_length = 41;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") ||\n' +
            '    (leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'if (wraps_can_occur &&\n' +
            '    inside_an_if_block) that_is_\n' +
            '    .okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token +\n' +
            '        12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap +\n' +
            '        but_this_can,\n' +
            '    propertz: first_token_should_never_wrap +\n' +
            '        !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" +\n' +
            '        "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f &&\n' +
            '        "sass") || (leans &&\n' +
            '        mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    if (wraps_can_occur &&\n' +
            '        inside_an_if_block) that_is_\n' +
            '        .okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token +\n' +
            '            12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap +\n' +
            '            but_this_can,\n' +
            '        propertz: first_token_should_never_wrap +\n' +
            '            !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" +\n' +
            '            "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // line wrapping 45
        reset_options();
        set_name('line wrapping 45');
        opts.preserve_newlines = true;
        opts.wrap_line_length = 45;
        test_fragment(
            '' + wrap_input_1 + '',
            //  -- output --
            'foo.bar().baz().cucumber((f && "sass") || (\n' +
            '    leans && mean));\n' +
            'Test_very_long_variable_name_this_should_never_wrap\n' +
            '    .but_this_can\n' +
            'return between_return_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'throw between_throw_and_expression_should_never_wrap\n' +
            '    .but_this_can\n' +
            'if (wraps_can_occur && inside_an_if_block)\n' +
            '    that_is_\n' +
            '    .okay();\n' +
            'object_literal = {\n' +
            '    propertx: first_token +\n' +
            '        12345678.99999E-6,\n' +
            '    property: first_token_should_never_wrap +\n' +
            '        but_this_can,\n' +
            '    propertz: first_token_should_never_wrap +\n' +
            '        !but_this_can,\n' +
            '    proper: "first_token_should_never_wrap" +\n' +
            '        "but_this_can"\n' +
            '}');
        test_fragment(
            '' + wrap_input_2 + '',
            //  -- output --
            '{\n' +
            '    foo.bar().baz().cucumber((f && "sass") ||\n' +
            '        (leans && mean));\n' +
            '    Test_very_long_variable_name_this_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    return between_return_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    throw between_throw_and_expression_should_never_wrap\n' +
            '        .but_this_can\n' +
            '    if (wraps_can_occur &&\n' +
            '        inside_an_if_block) that_is_\n' +
            '        .okay();\n' +
            '    object_literal = {\n' +
            '        propertx: first_token +\n' +
            '            12345678.99999E-6,\n' +
            '        property: first_token_should_never_wrap +\n' +
            '            but_this_can,\n' +
            '        propertz: first_token_should_never_wrap +\n' +
            '            !but_this_can,\n' +
            '        proper: "first_token_should_never_wrap" +\n' +
            '            "but_this_can"\n' +
            '    }\n' +
            '}');


        //============================================================
        // general preserve_newlines tests preserve limit
        reset_options();
        set_name('general preserve_newlines tests preserve limit');
        opts.preserve_newlines = true;
        opts.max_preserve_newlines = 8;
        bt(
            'a = 1;\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            'b = 2;',
            //  -- output --
            'a = 1;\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            '\n' +
            'b = 2;');


        //============================================================
        // more random test
        reset_options();
        set_name('more random test');
        bt('return function();');
        bt('var a = function();');
        bt('var a = 5 + function();');

        // actionscript import
        bt('import foo.*;');

        // actionscript
        bt('function f(a: a, b: b)');
        bt(
            'function a(a) {} function b(b) {} function c(c) {}',
            //  -- output --
            'function a(a) {}\n' +
            '\n' +
            'function b(b) {}\n' +
            '\n' +
            'function c(c) {}');
        bt('foo(a, function() {})');
        bt('foo(a, /regex/)');
        bt(
            '/* foo */\n' +
            '"x"');
        test_fragment(
            'roo = {\n' +
            '    /*\n' +
            '    ****\n' +
            '      FOO\n' +
            '    ****\n' +
            '    */\n' +
            '    BAR: 0\n' +
            '};');
        test_fragment(
            'if (zz) {\n' +
            '    // ....\n' +
            '}\n' +
            '(function');
        bt(
            'a = //comment\n' +
            '    /regex/;');
        bt('var a = new function();');
        bt('new function');
        bt(
            'if (a)\n' +
            '{\n' +
            'b;\n' +
            '}\n' +
            'else\n' +
            '{\n' +
            'c;\n' +
            '}',
            //  -- output --
            'if (a) {\n' +
            '    b;\n' +
            '} else {\n' +
            '    c;\n' +
            '}');
        bt('fn`tagged`');
        bt('fn()`tagged`');
        bt('fn`${algo} ${`6string`}`');
        bt('fn`${fn2()} more text ${`${`more text`}`} banana ${fn3`test`} ${fn4()`moretest banana2`}`');
        bt('`untagged`+`untagged`', '`untagged` + `untagged`');
        bt('fun() `taggedd`');
        bt('fn[0]`tagged`', 'fn[0] `tagged`');


        //============================================================
        // operator_position option - ensure no newlines if preserve_newlines is false - (preserve_newlines = "false")
        reset_options();
        set_name('operator_position option - ensure no newlines if preserve_newlines is false - (preserve_newlines = "false")');
        opts.preserve_newlines = false;
        bt(
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');

        // operator_position option - ensure no newlines if preserve_newlines is false - (operator_position = ""before-newline"", preserve_newlines = "false")
        reset_options();
        set_name('operator_position option - ensure no newlines if preserve_newlines is false - (operator_position = ""before-newline"", preserve_newlines = "false")');
        opts.operator_position = 'before-newline';
        opts.preserve_newlines = false;
        bt(
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');

        // operator_position option - ensure no newlines if preserve_newlines is false - (operator_position = ""after-newline"", preserve_newlines = "false")
        reset_options();
        set_name('operator_position option - ensure no newlines if preserve_newlines is false - (operator_position = ""after-newline"", preserve_newlines = "false")');
        opts.operator_position = 'after-newline';
        opts.preserve_newlines = false;
        bt(
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');

        // operator_position option - ensure no newlines if preserve_newlines is false - (operator_position = ""preserve-newline"", preserve_newlines = "false")
        reset_options();
        set_name('operator_position option - ensure no newlines if preserve_newlines is false - (operator_position = ""preserve-newline"", preserve_newlines = "false")');
        opts.operator_position = 'preserve-newline';
        opts.preserve_newlines = false;
        bt(
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b - c / d * e % f;\n' +
            'var res = g & h | i ^ j |> console.log;\n' +
            'var res = (k && l || m) ? n ?? nn : o;\n' +
            'var res = p >> q << r >>> s;\n' +
            'var res = t === u !== v != w == x >= y <= z > aa < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac + -ad');


        //============================================================
        // operator_position option - set to "before-newline" (default value) - ()
        reset_options();
        set_name('operator_position option - set to "before-newline" (default value) - ()');

        // comprehensive, various newlines
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b -\n' +
            '    c /\n' +
            '    d * e %\n' +
            '    f;\n' +
            'var res = g & h |\n' +
            '    i ^\n' +
            '    j |>\n' +
            '    console.log;\n' +
            'var res = (k &&\n' +
            '        l ||\n' +
            '        m) ?\n' +
            '    n ??\n' +
            '    nn :\n' +
            '    o;\n' +
            'var res = p >>\n' +
            '    q <<\n' +
            '    r >>>\n' +
            '    s;\n' +
            'var res = t\n' +
            '\n' +
            '    ===\n' +
            '    u !== v !=\n' +
            '    w ==\n' +
            '    x >=\n' +
            '    y <= z > aa <\n' +
            '    ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac +\n' +
            '    -ad');

        // colon special case
        bt(
            'var a = {\n' +
            '    b\n' +
            ': bval,\n' +
            '    c:\n' +
            'cval\n' +
            '    ,d: dval\n' +
            '};\n' +
            'var e = f ? g\n' +
            ': h;\n' +
            'var i = j ? k :\n' +
            'l;',
            //  -- output --
            'var a = {\n' +
            '    b: bval,\n' +
            '    c: cval,\n' +
            '    d: dval\n' +
            '};\n' +
            'var e = f ? g :\n' +
            '    h;\n' +
            'var i = j ? k :\n' +
            '    l;');

        // catch-all, includes brackets and other various code
        bt(
            'var d = 1;\n' +
            'if (a === b\n' +
            '    && c) {\n' +
            '    d = (c * everything\n' +
            '            / something_else) %\n' +
            '        b;\n' +
            '    e\n' +
            '        += d;\n' +
            '\n' +
            '} else if (!(complex && simple) ||\n' +
            '    (emotion && emotion.name === "happy")) {\n' +
            '    cryTearsOfJoy(many ||\n' +
            '        anOcean\n' +
            '        || aRiver);\n' +
            '}',
            //  -- output --
            'var d = 1;\n' +
            'if (a === b &&\n' +
            '    c) {\n' +
            '    d = (c * everything /\n' +
            '            something_else) %\n' +
            '        b;\n' +
            '    e\n' +
            '        += d;\n' +
            '\n' +
            '} else if (!(complex && simple) ||\n' +
            '    (emotion && emotion.name === "happy")) {\n' +
            '    cryTearsOfJoy(many ||\n' +
            '        anOcean ||\n' +
            '        aRiver);\n' +
            '}');

        // operator_position option - set to "before-newline" (default value) - (operator_position = ""before-newline"")
        reset_options();
        set_name('operator_position option - set to "before-newline" (default value) - (operator_position = ""before-newline"")');
        opts.operator_position = 'before-newline';

        // comprehensive, various newlines
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b -\n' +
            '    c /\n' +
            '    d * e %\n' +
            '    f;\n' +
            'var res = g & h |\n' +
            '    i ^\n' +
            '    j |>\n' +
            '    console.log;\n' +
            'var res = (k &&\n' +
            '        l ||\n' +
            '        m) ?\n' +
            '    n ??\n' +
            '    nn :\n' +
            '    o;\n' +
            'var res = p >>\n' +
            '    q <<\n' +
            '    r >>>\n' +
            '    s;\n' +
            'var res = t\n' +
            '\n' +
            '    ===\n' +
            '    u !== v !=\n' +
            '    w ==\n' +
            '    x >=\n' +
            '    y <= z > aa <\n' +
            '    ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac +\n' +
            '    -ad');

        // colon special case
        bt(
            'var a = {\n' +
            '    b\n' +
            ': bval,\n' +
            '    c:\n' +
            'cval\n' +
            '    ,d: dval\n' +
            '};\n' +
            'var e = f ? g\n' +
            ': h;\n' +
            'var i = j ? k :\n' +
            'l;',
            //  -- output --
            'var a = {\n' +
            '    b: bval,\n' +
            '    c: cval,\n' +
            '    d: dval\n' +
            '};\n' +
            'var e = f ? g :\n' +
            '    h;\n' +
            'var i = j ? k :\n' +
            '    l;');

        // catch-all, includes brackets and other various code
        bt(
            'var d = 1;\n' +
            'if (a === b\n' +
            '    && c) {\n' +
            '    d = (c * everything\n' +
            '            / something_else) %\n' +
            '        b;\n' +
            '    e\n' +
            '        += d;\n' +
            '\n' +
            '} else if (!(complex && simple) ||\n' +
            '    (emotion && emotion.name === "happy")) {\n' +
            '    cryTearsOfJoy(many ||\n' +
            '        anOcean\n' +
            '        || aRiver);\n' +
            '}',
            //  -- output --
            'var d = 1;\n' +
            'if (a === b &&\n' +
            '    c) {\n' +
            '    d = (c * everything /\n' +
            '            something_else) %\n' +
            '        b;\n' +
            '    e\n' +
            '        += d;\n' +
            '\n' +
            '} else if (!(complex && simple) ||\n' +
            '    (emotion && emotion.name === "happy")) {\n' +
            '    cryTearsOfJoy(many ||\n' +
            '        anOcean ||\n' +
            '        aRiver);\n' +
            '}');


        //============================================================
        // operator_position option - set to "after_newline"
        reset_options();
        set_name('operator_position option - set to "after_newline"');
        opts.operator_position = 'after-newline';

        // comprehensive, various newlines
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b\n' +
            '    - c\n' +
            '    / d * e\n' +
            '    % f;\n' +
            'var res = g & h\n' +
            '    | i\n' +
            '    ^ j\n' +
            '    |> console.log;\n' +
            'var res = (k\n' +
            '        && l\n' +
            '        || m)\n' +
            '    ? n\n' +
            '    ?? nn\n' +
            '    : o;\n' +
            'var res = p\n' +
            '    >> q\n' +
            '    << r\n' +
            '    >>> s;\n' +
            'var res = t\n' +
            '\n' +
            '    === u !== v\n' +
            '    != w\n' +
            '    == x\n' +
            '    >= y <= z > aa\n' +
            '    < ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac\n' +
            '    + -ad');

        // colon special case
        bt(
            'var a = {\n' +
            '    b\n' +
            ': bval,\n' +
            '    c:\n' +
            'cval\n' +
            '    ,d: dval\n' +
            '};\n' +
            'var e = f ? g\n' +
            ': h;\n' +
            'var i = j ? k :\n' +
            'l;',
            //  -- output --
            'var a = {\n' +
            '    b: bval,\n' +
            '    c: cval,\n' +
            '    d: dval\n' +
            '};\n' +
            'var e = f ? g\n' +
            '    : h;\n' +
            'var i = j ? k\n' +
            '    : l;');

        // catch-all, includes brackets and other various code
        bt(
            'var d = 1;\n' +
            'if (a === b\n' +
            '    && c) {\n' +
            '    d = (c * everything\n' +
            '            / something_else) %\n' +
            '        b;\n' +
            '    e\n' +
            '        += d;\n' +
            '\n' +
            '} else if (!(complex && simple) ||\n' +
            '    (emotion && emotion.name === "happy")) {\n' +
            '    cryTearsOfJoy(many ||\n' +
            '        anOcean\n' +
            '        || aRiver);\n' +
            '}',
            //  -- output --
            'var d = 1;\n' +
            'if (a === b\n' +
            '    && c) {\n' +
            '    d = (c * everything\n' +
            '            / something_else)\n' +
            '        % b;\n' +
            '    e\n' +
            '        += d;\n' +
            '\n' +
            '} else if (!(complex && simple)\n' +
            '    || (emotion && emotion.name === "happy")) {\n' +
            '    cryTearsOfJoy(many\n' +
            '        || anOcean\n' +
            '        || aRiver);\n' +
            '}');


        //============================================================
        // operator_position option - set to "preserve-newline"
        reset_options();
        set_name('operator_position option - set to "preserve-newline"');
        opts.operator_position = 'preserve-newline';

        // comprehensive, various newlines
        bt(
            'var res = a + b\n' +
            '- c /\n' +
            'd  *     e\n' +
            '%\n' +
            'f;\n' +
            '   var res = g & h\n' +
            '| i ^\n' +
            'j\n' +
            '|> console.log;\n' +
            'var res = (k &&\n' +
            'l\n' +
            '|| m) ?\n' +
            'n\n' +
            '?? nn\n' +
            ': o\n' +
            ';\n' +
            'var res = p\n' +
            '>> q <<\n' +
            'r\n' +
            '>>> s;\n' +
            'var res\n' +
            '  = t\n' +
            '\n' +
            ' === u !== v\n' +
            ' !=\n' +
            'w\n' +
            '== x >=\n' +
            'y <= z > aa <\n' +
            'ab;\n' +
            'res??=a;res||=b;res&&=c;\n' +
            'ac +\n' +
            '-ad',
            //  -- output --
            'var res = a + b\n' +
            '    - c /\n' +
            '    d * e\n' +
            '    %\n' +
            '    f;\n' +
            'var res = g & h\n' +
            '    | i ^\n' +
            '    j\n' +
            '    |> console.log;\n' +
            'var res = (k &&\n' +
            '        l\n' +
            '        || m) ?\n' +
            '    n\n' +
            '    ?? nn\n' +
            '    : o;\n' +
            'var res = p\n' +
            '    >> q <<\n' +
            '    r\n' +
            '    >>> s;\n' +
            'var res = t\n' +
            '\n' +
            '    === u !== v\n' +
            '    !=\n' +
            '    w\n' +
            '    == x >=\n' +
            '    y <= z > aa <\n' +
            '    ab;\n' +
            'res ??= a;\n' +
            'res ||= b;\n' +
            'res &&= c;\n' +
            'ac +\n' +
            '    -ad');

        // colon special case
        bt(
            'var a = {\n' +
            '    b\n' +
            ': bval,\n' +
            '    c:\n' +
            'cval\n' +
            '    ,d: dval\n' +
            '};\n' +
            'var e = f ? g\n' +
            ': h;\n' +
            'var i = j ? k :\n' +
            'l;',
            //  -- output --
            'var a = {\n' +
            '    b: bval,\n' +
            '    c: cval,\n' +
            '    d: dval\n' +
            '};\n' +
            'var e = f ? g\n' +
            '    : h;\n' +
            'var i = j ? k :\n' +
            '    l;');

        // catch-all, includes brackets and other various code
        bt(
            'var d = 1;\n' +
            'if (a === b\n' +
            '    && c) {\n' +
            '    d = (c * everything\n' +
            '            / something_else) %\n' +
            '        b;\n' +
            '    e\n' +
            '        += d;\n' +
            '\n' +
            '} else if (!(complex && simple) ||\n' +
            '    (emotion && emotion.name === "happy")) {\n' +
            '    cryTearsOfJoy(many ||\n' +
            '        anOcean\n' +
            '        || aRiver);\n' +
            '}');


        //============================================================
        // Yield tests
        reset_options();
        set_name('Yield tests');
        bt('yield /foo\\//;');
        bt('result = yield pgClient.query_(queryString);');
        bt('yield [1, 2]');
        bt('yield function() {};');
        bt('yield* bar();');

        // yield should have no space between yield and star
        bt('yield * bar();', 'yield* bar();');

        // yield should have space between star and generator
        bt('yield *bar();', 'yield* bar();');


        //============================================================
        // Async / await tests
        reset_options();
        set_name('Async / await tests');
        bt('async function foo() {}');
        bt('let w = async function foo() {}');
        bt(
            'async function foo() {}\n' +
            'var x = await foo();');

        // async function as an input to another function
        bt('wrapper(async function foo() {})');

        // await on inline anonymous function. should have a space after await
        bt(
            'async function() {\n' +
            '    var w = await(async function() {\n' +
            '        return await foo();\n' +
            '    })();\n' +
            '}',
            //  -- output --
            'async function() {\n' +
            '    var w = await (async function() {\n' +
            '        return await foo();\n' +
            '    })();\n' +
            '}');

        // Regression test #1228
        bt('const module = await import("...")');

        // Regression test #1658
        bt('.');

        // ensure that this doesn't break anyone with the async library
        bt('async.map(function(t) {})');

        // async on arrow function. should have a space after async
        bt(
            'async() => {}',
            //  -- output --
            'async () => {}');

        // async on arrow function. should have a space after async
        bt(
            'async() => {\n' +
            '    return 5;\n' +
            '}',
            //  -- output --
            'async () => {\n' +
            '    return 5;\n' +
            '}');

        // async on arrow function returning expression. should have a space after async
        bt(
            'async() => 5;',
            //  -- output --
            'async () => 5;');

        // async on arrow function returning object literal. should have a space after async
        bt(
            'async(x) => ({\n' +
            '    foo: "5"\n' +
            '})',
            //  -- output --
            'async (x) => ({\n' +
            '    foo: "5"\n' +
            '})');
        bt(
            'async (x) => {\n' +
            '    return x * 2;\n' +
            '}');
        bt('async () => 5;');
        bt('async x => x * 2;');
        bt(
            'async function() {\n' +
            '    const obj = {\n' +
            '        a: 1,\n' +
            '        b: await fn(),\n' +
            '        c: 2\n' +
            '    };\n' +
            '}');
        bt(
            'const a = 1,\n' +
            '    b = a ? await foo() : b,\n' +
            '    c = await foo(),\n' +
            '    d = 3,\n' +
            '    e = (await foo()),\n' +
            '    f = 4;');
        bt(
            'a = {\n' +
            '    myVar: async function() {\n' +
            '        return a;\n' +
            '    },\n' +
            '    myOtherVar: async function() {\n' +
            '        yield b;\n' +
            '    }\n' +
            '}');
        bt(
            'a = {\n' +
            '    myVar: async () => {\n' +
            '        return a;\n' +
            '    },\n' +
            '    myOtherVar: async async () => {\n' +
            '        yield b;\n' +
            '    }\n' +
            '}');


        //============================================================
        // e4x - Test that e4x literals passed through when e4x-option is enabled
        reset_options();
        set_name('e4x - Test that e4x literals passed through when e4x-option is enabled');
        opts.e4x = true;
        bt(
            'xml=<a b="c"><d/><e>\n' +
            ' foo</e>x</a>;',
            //  -- output --
            'xml = <a b="c"><d/><e>\n' +
            ' foo</e>x</a>;');
        bt('<a b=\'This is a quoted "c".\'/>');
        bt('<a b="This is a quoted \'c\'."/>');
        bt('<a b="A quote \' inside string."/>');
        bt('<a b=\'A quote " inside string.\'/>');
        bt('<a b=\'Some """ quotes ""  inside string.\'/>');

        // Handles inline expressions
        bt(
            'xml=<{a} b="c"><d/><e v={z}>\n' +
            ' foo</e>x</{a}>;',
            //  -- output --
            'xml = <{a} b="c"><d/><e v={z}>\n' +
            ' foo</e>x</{a}>;');
        bt(
            'xml=<{a} b="c">\n' +
            '    <e v={z}>\n' +
            ' foo</e>x</{a}>;',
            //  -- output --
            'xml = <{a} b="c">\n' +
            '    <e v={z}>\n' +
            ' foo</e>x</{a}>;');

        // xml literals with special characters in elem names - see http://www.w3.org/TR/REC-xml/#NT-NameChar
        bt('xml = <_:.valid.xml- _:.valid.xml-="123"/>;');

        // xml literals with attributes without equal sign
        bt('xml = <elem someAttr/>;');

        // Handles CDATA
        bt(
            'xml=<![CDATA[ b="c"><d/><e v={z}>\n' +
            ' foo</e>x/]]>;',
            //  -- output --
            'xml = <![CDATA[ b="c"><d/><e v={z}>\n' +
            ' foo</e>x/]]>;');
        bt('xml=<![CDATA[]]>;', 'xml = <![CDATA[]]>;');
        bt('xml=<a b="c"><![CDATA[d/></a></{}]]></a>;', 'xml = <a b="c"><![CDATA[d/></a></{}]]></a>;');

        // JSX - working jsx from http://prettydiff.com/unit_tests/beautification_javascript_jsx.txt
        bt(
            'var ListItem = React.createClass({\n' +
            '    render: function() {\n' +
            '        return (\n' +
            '            <li className="ListItem">\n' +
            '                <a href={ "/items/" + this.props.item.id }>\n' +
            '                    this.props.item.name\n' +
            '                </a>\n' +
            '            </li>\n' +
            '        );\n' +
            '    }\n' +
            '});');
        bt(
            'var List = React.createClass({\n' +
            '    renderList: function() {\n' +
            '        return this.props.items.map(function(item) {\n' +
            '            return <ListItem item={item} key={item.id} />;\n' +
            '        });\n' +
            '    },\n' +
            '\n' +
            '    render: function() {\n' +
            '        return <ul className="List">\n' +
            '                this.renderList()\n' +
            '            </ul>\n' +
            '    }\n' +
            '});');
        bt(
            'var Mist = React.createClass({\n' +
            '    renderList: function() {\n' +
            '        return this.props.items.map(function(item) {\n' +
            '            return <ListItem item={return <tag>{item}</tag>} key={item.id} />;\n' +
            '        });\n' +
            '    }\n' +
            '});');
        bt(
            '// JSX\n' +
            'var box = <Box>\n' +
            '    {shouldShowAnswer(user) ?\n' +
            '        <Answer value={false}>no</Answer> : <Box.Comment>\n' +
            '        Text Content\n' +
            '        </Box.Comment>}\n' +
            '    </Box>;\n' +
            'var a = function() {\n' +
            '    return <tsdf>asdf</tsdf>;\n' +
            '};\n' +
            '\n' +
            'var HelloMessage = React.createClass({\n' +
            '    render: function() {\n' +
            '        return <div {someAttr}>Hello {this.props.name}</div>;\n' +
            '    }\n' +
            '});\n' +
            'React.render(<HelloMessage name="John" />, mountNode);');
        bt(
            'var Timer = React.createClass({\n' +
            '    getInitialState: function() {\n' +
            '        return {\n' +
            '            secondsElapsed: 0\n' +
            '        };\n' +
            '    },\n' +
            '    tick: function() {\n' +
            '        this.setState({\n' +
            '            secondsElapsed: this.state.secondsElapsed + 1\n' +
            '        });\n' +
            '    },\n' +
            '    componentDidMount: function() {\n' +
            '        this.interval = setInterval(this.tick, 1000);\n' +
            '    },\n' +
            '    componentWillUnmount: function() {\n' +
            '        clearInterval(this.interval);\n' +
            '    },\n' +
            '    render: function() {\n' +
            '        return (\n' +
            '            <div>Seconds Elapsed: {this.state.secondsElapsed}</div>\n' +
            '        );\n' +
            '    }\n' +
            '});\n' +
            'React.render(<Timer />, mountNode);');
        bt(
            'var TodoList = React.createClass({\n' +
            '    render: function() {\n' +
            '        var createItem = function(itemText) {\n' +
            '            return <li>{itemText}</li>;\n' +
            '        };\n' +
            '        return <ul>{this.props.items.map(createItem)}</ul>;\n' +
            '    }\n' +
            '});');
        bt(
            'var TodoApp = React.createClass({\n' +
            '    getInitialState: function() {\n' +
            '        return {\n' +
            '            items: [],\n' +
            '            text: \'\'\n' +
            '        };\n' +
            '    },\n' +
            '    onChange: function(e) {\n' +
            '        this.setState({\n' +
            '            text: e.target.value\n' +
            '        });\n' +
            '    },\n' +
            '    handleSubmit: function(e) {\n' +
            '        e.preventDefault();\n' +
            '        var nextItems = this.state.items.concat([this.state.text]);\n' +
            '        var nextText = \'\';\n' +
            '        this.setState({\n' +
            '            items: nextItems,\n' +
            '            text: nextText\n' +
            '        });\n' +
            '    },\n' +
            '    render: function() {\n' +
            '        return (\n' +
            '            <div>\n' +
            '                <h3 {someAttr}>TODO</h3>\n' +
            '                <TodoList items={this.state.items} />\n' +
            '                <form onSubmit={this.handleSubmit}>\n' +
            '                    <input onChange={this.onChange} value={this.state.text} />\n' +
            '                    <button>{\'Add #\' + (this.state.items.length + 1)}</button>\n' +
            '                </form>\n' +
            '            </div>\n' +
            '        );\n' +
            '    }\n' +
            '});\n' +
            'React.render(<TodoApp />, mountNode);');
        bt(
            'var converter = new Showdown.converter();\n' +
            'var MarkdownEditor = React.createClass({\n' +
            '    getInitialState: function() {\n' +
            '        return {value: \'Type some *markdown* here!\'};\n' +
            '    },\n' +
            '    handleChange: function() {\n' +
            '        this.setState({value: this.refs.textarea.getDOMNode().value});\n' +
            '    },\n' +
            '    render: function() {\n' +
            '        return (\n' +
            '            <div className="MarkdownEditor">\n' +
            '                <h3>Input</h3>\n' +
            '                <textarea\n' +
            '                    onChange={this.handleChange}\n' +
            '                    ref="textarea"\n' +
            '                    defaultValue={this.state.value} />\n' +
            '                <h3>Output</h3>\n' +
            '            <div\n' +
            '                className="content"\n' +
            '                dangerouslySetInnerHTML=\n' +
            '                />\n' +
            '            </div>\n' +
            '        );\n' +
            '    }\n' +
            '});\n' +
            'React.render(<MarkdownEditor />, mountNode);',
            //  -- output --
            'var converter = new Showdown.converter();\n' +
            'var MarkdownEditor = React.createClass({\n' +
            '    getInitialState: function() {\n' +
            '        return {\n' +
            '            value: \'Type some *markdown* here!\'\n' +
            '        };\n' +
            '    },\n' +
            '    handleChange: function() {\n' +
            '        this.setState({\n' +
            '            value: this.refs.textarea.getDOMNode().value\n' +
            '        });\n' +
            '    },\n' +
            '    render: function() {\n' +
            '        return (\n' +
            '            <div className="MarkdownEditor">\n' +
            '                <h3>Input</h3>\n' +
            '                <textarea\n' +
            '                    onChange={this.handleChange}\n' +
            '                    ref="textarea"\n' +
            '                    defaultValue={this.state.value} />\n' +
            '                <h3>Output</h3>\n' +
            '            <div\n' +
            '                className="content"\n' +
            '                dangerouslySetInnerHTML=\n' +
            '                />\n' +
            '            </div>\n' +
            '        );\n' +
            '    }\n' +
            '});\n' +
            'React.render(<MarkdownEditor />, mountNode);');

        // JSX - Not quite correct jsx formatting that still works
        bt(
            'var content = (\n' +
            '        <Nav>\n' +
            '            {/* child comment, put {} around */}\n' +
            '            <Person\n' +
            '                /* multi\n' +
            '         line\n' +
            '         comment */\n' +
            '         //attr="test"\n' +
            '                name={window.isLoggedIn ? window.name : \'\'} // end of line comment\n' +
            '            />\n' +
            '        </Nav>\n' +
            '    );\n' +
            'var qwer = <DropDown> A dropdown list <Menu> <MenuItem>Do Something</MenuItem> <MenuItem>Do Something Fun!</MenuItem> <MenuItem>Do Something Else</MenuItem> </Menu> </DropDown>;\n' +
            'render(dropdown);',
            //  -- output --
            'var content = (\n' +
            '    <Nav>\n' +
            '            {/* child comment, put {} around */}\n' +
            '            <Person\n' +
            '                /* multi\n' +
            '         line\n' +
            '         comment */\n' +
            '         //attr="test"\n' +
            '                name={window.isLoggedIn ? window.name : \'\'} // end of line comment\n' +
            '            />\n' +
            '        </Nav>\n' +
            ');\n' +
            'var qwer = <DropDown> A dropdown list <Menu> <MenuItem>Do Something</MenuItem> <MenuItem>Do Something Fun!</MenuItem> <MenuItem>Do Something Else</MenuItem> </Menu> </DropDown>;\n' +
            'render(dropdown);');

        // Handles messed up tags, as long as it isn't the same name
        // as the root tag. Also handles tags of same name as root tag
        // as long as nesting matches.
        bt(
            'xml=<a x="jn"><c></b></f><a><d jnj="jnn"><f></a ></nj></a>;',
            //  -- output --
            'xml = <a x="jn"><c></b></f><a><d jnj="jnn"><f></a ></nj></a>;');

        // If xml is not terminated, the remainder of the file is treated
        // as part of the xml-literal (passed through unaltered)
        test_fragment(
            'xml=<a></b>\n' +
            'c<b;',
            //  -- output --
            'xml = <a></b>\n' +
            'c<b;');

        // Issue #646 = whitespace is allowed in attribute declarations
        bt(
            'let a = React.createClass({\n' +
            '    render() {\n' +
            '        return (\n' +
            '            <p className=\'a\'>\n' +
            '                <span>c</span>\n' +
            '            </p>\n' +
            '        );\n' +
            '    }\n' +
            '});');
        bt(
            'let a = React.createClass({\n' +
            '    render() {\n' +
            '        return (\n' +
            '            <p className = \'b\'>\n' +
            '                <span>c</span>\n' +
            '            </p>\n' +
            '        );\n' +
            '    }\n' +
            '});');
        bt(
            'let a = React.createClass({\n' +
            '    render() {\n' +
            '        return (\n' +
            '            <p className = "c">\n' +
            '                <span>c</span>\n' +
            '            </p>\n' +
            '        );\n' +
            '    }\n' +
            '});');
        bt(
            'let a = React.createClass({\n' +
            '    render() {\n' +
            '        return (\n' +
            '            <{e}  className = {d}>\n' +
            '                <span>c</span>\n' +
            '            </{e}>\n' +
            '        );\n' +
            '    }\n' +
            '});');

        // Issue #914 - Multiline attribute in root tag
        bt(
            'return (\n' +
            '    <a href="#"\n' +
            '        onClick={e => {\n' +
            '            e.preventDefault()\n' +
            '            onClick()\n' +
            '       }}>\n' +
            '       {children}\n' +
            '    </a>\n' +
            ');');
        bt(
            'return (\n' +
            '    <{\n' +
            '        a + b\n' +
            '    } href="#"\n' +
            '        onClick={e => {\n' +
            '            e.preventDefault()\n' +
            '            onClick()\n' +
            '       }}>\n' +
            '       {children}\n' +
            '    </{\n' +
            '        a + b\n' +
            '    }>\n' +
            ');');
        bt(
            'return (\n' +
            '    <{\n' +
            '        a + b\n' +
            '    } href="#"\n' +
            '        onClick={e => {\n' +
            '            e.preventDefault()\n' +
            '            onClick()\n' +
            '       }}>\n' +
            '       {children}\n' +
            '    </{a + b}>\n' +
            '    );',
            //  -- output --
            'return (\n' +
            '    <{\n' +
            '        a + b\n' +
            '    } href="#"\n' +
            '        onClick={e => {\n' +
            '            e.preventDefault()\n' +
            '            onClick()\n' +
            '       }}>\n' +
            '       {children}\n' +
            '    </{a + b}>\n' +
            ');');
        bt(
            'class Columns extends React.Component {\n' +
            '    render() {\n' +
            '        return (\n' +
            '            <>\n' +
            '              <td>Hello</td>\n' +
            '              <td>World</td>\n' +
            '            </>\n' +
            '        );\n' +
            '    }\n' +
            '}');


        //============================================================
        //
        reset_options();
        set_name('');


        //============================================================
        // e4x disabled
        reset_options();
        set_name('e4x disabled');
        opts.e4x = false;
        bt(
            'xml=<a b="c"><d/><e>\n' +
            ' foo</e>x</a>;',
            //  -- output --
            'xml = < a b = "c" > < d / > < e >\n' +
            '    foo < /e>x</a > ;');


        //============================================================
        // Multiple braces
        reset_options();
        set_name('Multiple braces');
        bt(
            '{{}/z/}',
            //  -- output --
            '{\n' +
            '    {}\n' +
            '    /z/\n' +
            '}');


        //============================================================
        // Space before conditional - (space_before_conditional = "false")
        reset_options();
        set_name('Space before conditional - (space_before_conditional = "false")');
        opts.space_before_conditional = false;
        bt('if(a) b()');
        bt('while(a) b()');
        bt(
            'do\n' +
            '    c();\n' +
            'while(a) b()');
        bt('switch(a) b()');
        bt(
            'if(a)\n' +
            'b();',
            //  -- output --
            'if(a)\n' +
            '    b();');
        bt(
            'while(a)\n' +
            'b();',
            //  -- output --
            'while(a)\n' +
            '    b();');
        bt(
            'do\n' +
            'c();\n' +
            'while(a);',
            //  -- output --
            'do\n' +
            '    c();\n' +
            'while(a);');
        bt(
            'switch(a)\n' +
            'b()',
            //  -- output --
            'switch(a)\n' +
            '    b()');
        bt('return [];');
        bt('return ();');

        // Space before conditional - (space_before_conditional = "true")
        reset_options();
        set_name('Space before conditional - (space_before_conditional = "true")');
        opts.space_before_conditional = true;
        bt('if (a) b()');
        bt('while (a) b()');
        bt(
            'do\n' +
            '    c();\n' +
            'while (a) b()');
        bt('switch (a) b()');
        bt(
            'if(a)\n' +
            'b();',
            //  -- output --
            'if (a)\n' +
            '    b();');
        bt(
            'while(a)\n' +
            'b();',
            //  -- output --
            'while (a)\n' +
            '    b();');
        bt(
            'do\n' +
            'c();\n' +
            'while(a);',
            //  -- output --
            'do\n' +
            '    c();\n' +
            'while (a);');
        bt(
            'switch(a)\n' +
            'b()',
            //  -- output --
            'switch (a)\n' +
            '    b()');
        bt('return [];');
        bt('return ();');


        //============================================================
        // Beautify preserve formatting
        reset_options();
        set_name('Beautify preserve formatting');
        bt(
            '/* beautify preserve:start */\n' +
            '/* beautify preserve:end */');
        bt(
            '/* beautify preserve:start */\n' +
            '   var a = 1;\n' +
            '/* beautify preserve:end */');
        bt(
            'var a = 1;\n' +
            '/* beautify preserve:start */\n' +
            '   var a = 1;\n' +
            '/* beautify preserve:end */');
        bt('/* beautify preserve:start */     {asdklgh;y;;{}dd2d}/* beautify preserve:end */');
        bt(
            'var a =  1;\n' +
            '/* beautify preserve:start */\n' +
            '   var a = 1;\n' +
            '/* beautify preserve:end */',
            //  -- output --
            'var a = 1;\n' +
            '/* beautify preserve:start */\n' +
            '   var a = 1;\n' +
            '/* beautify preserve:end */');
        bt(
            'var a = 1;\n' +
            ' /* beautify preserve:start */\n' +
            '   var a = 1;\n' +
            '/* beautify preserve:end */',
            //  -- output --
            'var a = 1;\n' +
            '/* beautify preserve:start */\n' +
            '   var a = 1;\n' +
            '/* beautify preserve:end */');
        bt(
            'var a = {\n' +
            '    /* beautify preserve:start */\n' +
            '    one   :  1\n' +
            '    two   :  2,\n' +
            '    three :  3,\n' +
            '    ten   : 10\n' +
            '    /* beautify preserve:end */\n' +
            '};');
        bt(
            'var a = {\n' +
            '/* beautify preserve:start */\n' +
            '    one   :  1,\n' +
            '    two   :  2,\n' +
            '    three :  3,\n' +
            '    ten   : 10\n' +
            '/* beautify preserve:end */\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /* beautify preserve:start */\n' +
            '    one   :  1,\n' +
            '    two   :  2,\n' +
            '    three :  3,\n' +
            '    ten   : 10\n' +
            '/* beautify preserve:end */\n' +
            '};');

        // one space before and after required, only single spaces inside.
        bt(
            'var a = {\n' +
            '/*  beautify preserve:start  */\n' +
            '    one   :  1,\n' +
            '    two   :  2,\n' +
            '    three :  3,\n' +
            '    ten   : 10\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /*  beautify preserve:start  */\n' +
            '    one: 1,\n' +
            '    two: 2,\n' +
            '    three: 3,\n' +
            '    ten: 10\n' +
            '};');
        bt(
            'var a = {\n' +
            '/*beautify preserve:start*/\n' +
            '    one   :  1,\n' +
            '    two   :  2,\n' +
            '    three :  3,\n' +
            '    ten   : 10\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /*beautify preserve:start*/\n' +
            '    one: 1,\n' +
            '    two: 2,\n' +
            '    three: 3,\n' +
            '    ten: 10\n' +
            '};');
        bt(
            'var a = {\n' +
            '/*beautify  preserve:start*/\n' +
            '    one   :  1,\n' +
            '    two   :  2,\n' +
            '    three :  3,\n' +
            '    ten   : 10\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /*beautify  preserve:start*/\n' +
            '    one: 1,\n' +
            '    two: 2,\n' +
            '    three: 3,\n' +
            '    ten: 10\n' +
            '};');

        // Directive: ignore
        bt(
            '/* beautify ignore:start */\n' +
            '/* beautify ignore:end */');
        bt(
            '/* beautify ignore:start */\n' +
            '   var a,,,{ 1;\n' +
            '  /* beautify ignore:end */');
        bt(
            'var a = 1;\n' +
            '/* beautify ignore:start */\n' +
            '   var a = 1;\n' +
            '/* beautify ignore:end */');

        // ignore starts _after_ the start comment, ends after the end comment
        bt('/* beautify ignore:start */     {asdklgh;y;+++;dd2d}/* beautify ignore:end */');
        bt('/* beautify ignore:start */  {asdklgh;y;+++;dd2d}    /* beautify ignore:end */');
        bt(
            'var a =  1;\n' +
            '/* beautify ignore:start */\n' +
            '   var a,,,{ 1;\n' +
            '/*beautify ignore:end*/',
            //  -- output --
            'var a = 1;\n' +
            '/* beautify ignore:start */\n' +
            '   var a,,,{ 1;\n' +
            '/*beautify ignore:end*/');
        bt(
            'var a = 1;\n' +
            ' /* beautify ignore:start */\n' +
            '   var a,,,{ 1;\n' +
            '/* beautify ignore:end */',
            //  -- output --
            'var a = 1;\n' +
            '/* beautify ignore:start */\n' +
            '   var a,,,{ 1;\n' +
            '/* beautify ignore:end */');
        bt(
            'var a = {\n' +
            '    /* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    two   :  2,\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '    /* beautify ignore:end */\n' +
            '};');
        bt(
            'var a = {\n' +
            '/* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    two   :  2,\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    two   :  2,\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '};');

        // Directives - multiple and interacting
        bt(
            'var a = {\n' +
            '/* beautify preserve:start */\n' +
            '/* beautify preserve:start */\n' +
            '    one   :  1,\n' +
            '  /* beautify preserve:end */\n' +
            '    two   :  2,\n' +
            '    three :  3,\n' +
            '/* beautify preserve:start */\n' +
            '    ten   : 10\n' +
            '/* beautify preserve:end */\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /* beautify preserve:start */\n' +
            '/* beautify preserve:start */\n' +
            '    one   :  1,\n' +
            '  /* beautify preserve:end */\n' +
            '    two: 2,\n' +
            '    three: 3,\n' +
            '    /* beautify preserve:start */\n' +
            '    ten   : 10\n' +
            '/* beautify preserve:end */\n' +
            '};');
        bt(
            'var a = {\n' +
            '/* beautify ignore:start */\n' +
            '    one   :  1\n' +
            ' /* beautify ignore:end */\n' +
            '    two   :  2,\n' +
            '/* beautify ignore:start */\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /* beautify ignore:start */\n' +
            '    one   :  1\n' +
            ' /* beautify ignore:end */\n' +
            '    two: 2,\n' +
            '    /* beautify ignore:start */\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '};');

        // Starts can occur together, ignore:end must occur alone.
        bt(
            'var a = {\n' +
            '/* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    NOTE: ignore end block does not support starting other directives\n' +
            '    This does not match the ending the ignore...\n' +
            ' /* beautify ignore:end preserve:start */\n' +
            '    two   :  2,\n' +
            '/* beautify ignore:start */\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '    ==The next comment ends the starting ignore==\n' +
            '/* beautify ignore:end */\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    NOTE: ignore end block does not support starting other directives\n' +
            '    This does not match the ending the ignore...\n' +
            ' /* beautify ignore:end preserve:start */\n' +
            '    two   :  2,\n' +
            '/* beautify ignore:start */\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '    ==The next comment ends the starting ignore==\n' +
            '/* beautify ignore:end */\n' +
            '};');
        bt(
            'var a = {\n' +
            '/* beautify ignore:start preserve:start */\n' +
            '    one   :  {\n' +
            ' /* beautify ignore:end */\n' +
            '    two   :  2,\n' +
            '  /* beautify ignore:start */\n' +
            '    three :  {\n' +
            '/* beautify ignore:end */\n' +
            '    ten   : 10\n' +
            '   // This is all preserved\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /* beautify ignore:start preserve:start */\n' +
            '    one   :  {\n' +
            ' /* beautify ignore:end */\n' +
            '    two   :  2,\n' +
            '  /* beautify ignore:start */\n' +
            '    three :  {\n' +
            '/* beautify ignore:end */\n' +
            '    ten   : 10\n' +
            '   // This is all preserved\n' +
            '};');
        bt(
            'var a = {\n' +
            '/* beautify ignore:start preserve:start */\n' +
            '    one   :  {\n' +
            ' /* beautify ignore:end */\n' +
            '    two   :  2,\n' +
            '  /* beautify ignore:start */\n' +
            '    three :  {\n' +
            '/* beautify ignore:end */\n' +
            '    ten   : 10,\n' +
            '/* beautify preserve:end */\n' +
            '     eleven: 11\n' +
            '};',
            //  -- output --
            'var a = {\n' +
            '    /* beautify ignore:start preserve:start */\n' +
            '    one   :  {\n' +
            ' /* beautify ignore:end */\n' +
            '    two   :  2,\n' +
            '  /* beautify ignore:start */\n' +
            '    three :  {\n' +
            '/* beautify ignore:end */\n' +
            '    ten   : 10,\n' +
            '/* beautify preserve:end */\n' +
            '    eleven: 11\n' +
            '};');


        //============================================================
        // Comments and  tests
        reset_options();
        set_name('Comments and  tests');

        // #913
        bt(
            'class test {\n' +
            '    method1() {\n' +
            '        let resp = null;\n' +
            '    }\n' +
            '    /**\n' +
            '     * @param {String} id\n' +
            '     */\n' +
            '    method2(id) {\n' +
            '        let resp2 = null;\n' +
            '    }\n' +
            '}');

        // #1090
        bt(
            'for (var i = 0; i < 20; ++i) // loop\n' +
            '    if (i % 3) {\n' +
            '        console.log(i);\n' +
            '    }\n' +
            'console.log("done");');

        // #1043
        bt(
            'var o = {\n' +
            '    k: 0\n' +
            '}\n' +
            '// ...\n' +
            'foo(o)');

        // #713 and #964
        bt(
            'Meteor.call("foo", bar, function(err, result) {\n' +
            '    Session.set("baz", result.lorem)\n' +
            '})\n' +
            '//blah blah');

        // #815
        bt(
            'foo()\n' +
            '// this is a comment\n' +
            'bar()\n' +
            '\n' +
            'const foo = 5\n' +
            '// comment\n' +
            'bar()');

        // This shows current behavior.  Note #1069 is not addressed yet.
        bt(
            'if (modulus === 2) {\n' +
            '    // i might be odd here\n' +
            '    i += (i & 1);\n' +
            '    // now i is guaranteed to be even\n' +
            '    // this block is obviously about the statement above\n' +
            '\n' +
            '    // #1069 This should attach to the block below\n' +
            '    // this comment is about the block after it.\n' +
            '} else {\n' +
            '    // rounding up using integer arithmetic only\n' +
            '    if (i % modulus)\n' +
            '        i += modulus - (i % modulus);\n' +
            '    // now i is divisible by modulus\n' +
            '    // behavior of comments should be different for single statements vs block statements/expressions\n' +
            '}\n' +
            '\n' +
            'if (modulus === 2)\n' +
            '    // i might be odd here\n' +
            '    i += (i & 1);\n' +
            '// now i is guaranteed to be even\n' +
            '// non-braced comments unindent immediately\n' +
            '\n' +
            '// this comment is about the block after it.\n' +
            'else\n' +
            '    // rounding up using integer arithmetic only\n' +
            '    if (i % modulus)\n' +
            '        i += modulus - (i % modulus);\n' +
            '// behavior of comments should be different for single statements vs block statements/expressions');


        //============================================================
        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = <?php$view["name"]; ?>;', 'var a = <?php$view["name"]; ?>;');
        bt(
            'a = abc<?php\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '?>;');
        test_fragment(
            '<?php ?>\n' +
            'test.met<?php someValue ?>hod();');
        bt(
            '<?php "A" ?>abc<?php "D" ?>;\n' +
            '<?php "B" ?>.test();\n' +
            '" <?php   "C" \'D\'  ?>  "');
        bt(
            '<?php\n' +
            'echo "A";\n' +
            '?>;\n' +
            'test.method();');
        bt('"<?php";if(0){}"?>";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = <?=$view["name"]; ?>;', 'var a = <?=$view["name"]; ?>;');
        bt(
            'a = abc<?=\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '?>;');
        test_fragment(
            '<?= ?>\n' +
            'test.met<?= someValue ?>hod();');
        bt(
            '<?= "A" ?>abc<?= "D" ?>;\n' +
            '<?= "B" ?>.test();\n' +
            '" <?=   "C" \'D\'  ?>  "');
        bt(
            '<?=\n' +
            'echo "A";\n' +
            '?>;\n' +
            'test.method();');
        bt('"<?=";if(0){}"?>";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = <%$view["name"]; %>;', 'var a = <%$view["name"]; %>;');
        bt(
            'a = abc<%\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '%>;');
        test_fragment(
            '<% %>\n' +
            'test.met<% someValue %>hod();');
        bt(
            '<% "A" %>abc<% "D" %>;\n' +
            '<% "B" %>.test();\n' +
            '" <%   "C" \'D\'  %>  "');
        bt(
            '<%\n' +
            'echo "A";\n' +
            '%>;\n' +
            'test.method();');
        bt('"<%";if(0){}"%>";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = <%=$view["name"]; %>;', 'var a = <%=$view["name"]; %>;');
        bt(
            'a = abc<%=\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '%>;');
        test_fragment(
            '<%= %>\n' +
            'test.met<%= someValue %>hod();');
        bt(
            '<%= "A" %>abc<%= "D" %>;\n' +
            '<%= "B" %>.test();\n' +
            '" <%=   "C" \'D\'  %>  "');
        bt(
            '<%=\n' +
            'echo "A";\n' +
            '%>;\n' +
            'test.method();');
        bt('"<%=";if(0){}"%>";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {{$view["name"]; }};', 'var a = {{$view["name"]; }};');
        bt(
            'a = abc{{\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}};');
        test_fragment(
            '{{ }}\n' +
            'test.met{{ someValue }}hod();');
        bt(
            '{{ "A" }}abc{{ "D" }};\n' +
            '{{ "B" }}.test();\n' +
            '" {{   "C" \'D\'  }}  "');
        bt(
            '{{\n' +
            'echo "A";\n' +
            '}};\n' +
            'test.method();');
        bt('"{{";if(0){}"}}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {#$view["name"]; #};', 'var a = {#$view["name"]; #};');
        bt(
            'a = abc{#\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '#};');
        test_fragment(
            '{# #}\n' +
            'test.met{# someValue #}hod();');
        bt(
            '{# "A" #}abc{# "D" #};\n' +
            '{# "B" #}.test();\n' +
            '" {#   "C" \'D\'  #}  "');
        bt(
            '{#\n' +
            'echo "A";\n' +
            '#};\n' +
            'test.method();');
        bt('"{#";if(0){}"#}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {%$view["name"]; %};', 'var a = {%$view["name"]; %};');
        bt(
            'a = abc{%\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '%};');
        test_fragment(
            '{% %}\n' +
            'test.met{% someValue %}hod();');
        bt(
            '{% "A" %}abc{% "D" %};\n' +
            '{% "B" %}.test();\n' +
            '" {%   "C" \'D\'  %}  "');
        bt(
            '{%\n' +
            'echo "A";\n' +
            '%};\n' +
            'test.method();');
        bt('"{%";if(0){}"%}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {{$view["name"]; }};', 'var a = {{$view["name"]; }};');
        bt(
            'a = abc{{\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}};');
        test_fragment(
            '{{ }}\n' +
            'test.met{{ someValue }}hod();');
        bt(
            '{{ "A" }}abc{{ "D" }};\n' +
            '{{ "B" }}.test();\n' +
            '" {{   "C" \'D\'  }}  "');
        bt(
            '{{\n' +
            'echo "A";\n' +
            '}};\n' +
            'test.method();');
        bt('"{{";if(0){}"}}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {{{$view["name"]; }}};', 'var a = {{{$view["name"]; }}};');
        bt(
            'a = abc{{{\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}}};');
        test_fragment(
            '{{{ }}}\n' +
            'test.met{{{ someValue }}}hod();');
        bt(
            '{{{ "A" }}}abc{{{ "D" }}};\n' +
            '{{{ "B" }}}.test();\n' +
            '" {{{   "C" \'D\'  }}}  "');
        bt(
            '{{{\n' +
            'echo "A";\n' +
            '}}};\n' +
            'test.method();');
        bt('"{{{";if(0){}"}}}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {{^$view["name"]; }};', 'var a = {{^$view["name"]; }};');
        bt(
            'a = abc{{^\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}};');
        test_fragment(
            '{{^ }}\n' +
            'test.met{{^ someValue }}hod();');
        bt(
            '{{^ "A" }}abc{{^ "D" }};\n' +
            '{{^ "B" }}.test();\n' +
            '" {{^   "C" \'D\'  }}  "');
        bt(
            '{{^\n' +
            'echo "A";\n' +
            '}};\n' +
            'test.method();');
        bt('"{{^";if(0){}"}}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {{#$view["name"]; }};', 'var a = {{#$view["name"]; }};');
        bt(
            'a = abc{{#\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}};');
        test_fragment(
            '{{# }}\n' +
            'test.met{{# someValue }}hod();');
        bt(
            '{{# "A" }}abc{{# "D" }};\n' +
            '{{# "B" }}.test();\n' +
            '" {{#   "C" \'D\'  }}  "');
        bt(
            '{{#\n' +
            'echo "A";\n' +
            '}};\n' +
            'test.method();');
        bt('"{{#";if(0){}"}}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {{!$view["name"]; }};', 'var a = {{!$view["name"]; }};');
        bt(
            'a = abc{{!\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}};');
        test_fragment(
            '{{! }}\n' +
            'test.met{{! someValue }}hod();');
        bt(
            '{{! "A" }}abc{{! "D" }};\n' +
            '{{! "B" }}.test();\n' +
            '" {{!   "C" \'D\'  }}  "');
        bt(
            '{{!\n' +
            'echo "A";\n' +
            '}};\n' +
            'test.method();');
        bt('"{{!";if(0){}"}}";');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        opts.templating = ['django', 'erb', 'handlebars', 'php'];
        bt('var  a = {{!--$view["name"]; --}};', 'var a = {{!--$view["name"]; --}};');
        bt(
            'a = abc{{!--\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '--}};');
        test_fragment(
            '{{!-- --}}\n' +
            'test.met{{!-- someValue --}}hod();');
        bt(
            '{{!-- "A" --}}abc{{!-- "D" --}};\n' +
            '{{!-- "B" --}}.test();\n' +
            '" {{!--   "C" \'D\'  --}}  "');
        bt(
            '{{!--\n' +
            'echo "A";\n' +
            '--}};\n' +
            'test.method();');
        bt('"{{!--";if(0){}"--}}";');


        //============================================================
        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"<?php";if(0){}"?>";',
            //  -- output --
            '"<?php";\n' +
            'if (0) {}\n' +
            '"?>";');
        bt(
            '"<?php";if(0){}',
            //  -- output --
            '"<?php";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"<?=";if(0){}"?>";',
            //  -- output --
            '"<?=";\n' +
            'if (0) {}\n' +
            '"?>";');
        bt(
            '"<?=";if(0){}',
            //  -- output --
            '"<?=";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"<%";if(0){}"%>";',
            //  -- output --
            '"<%";\n' +
            'if (0) {}\n' +
            '"%>";');
        bt(
            '"<%";if(0){}',
            //  -- output --
            '"<%";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"<%=";if(0){}"%>";',
            //  -- output --
            '"<%=";\n' +
            'if (0) {}\n' +
            '"%>";');
        bt(
            '"<%=";if(0){}',
            //  -- output --
            '"<%=";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{{";if(0){}"}}";',
            //  -- output --
            '"{{";\n' +
            'if (0) {}\n' +
            '"}}";');
        bt(
            '"{{";if(0){}',
            //  -- output --
            '"{{";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{#";if(0){}"#}";',
            //  -- output --
            '"{#";\n' +
            'if (0) {}\n' +
            '"#}";');
        bt(
            '"{#";if(0){}',
            //  -- output --
            '"{#";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{%";if(0){}"%}";',
            //  -- output --
            '"{%";\n' +
            'if (0) {}\n' +
            '"%}";');
        bt(
            '"{%";if(0){}',
            //  -- output --
            '"{%";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{{";if(0){}"}}";',
            //  -- output --
            '"{{";\n' +
            'if (0) {}\n' +
            '"}}";');
        bt(
            '"{{";if(0){}',
            //  -- output --
            '"{{";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{{{";if(0){}"}}}";',
            //  -- output --
            '"{{{";\n' +
            'if (0) {}\n' +
            '"}}}";');
        bt(
            '"{{{";if(0){}',
            //  -- output --
            '"{{{";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{{^";if(0){}"}}";',
            //  -- output --
            '"{{^";\n' +
            'if (0) {}\n' +
            '"}}";');
        bt(
            '"{{^";if(0){}',
            //  -- output --
            '"{{^";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{{#";if(0){}"}}";',
            //  -- output --
            '"{{#";\n' +
            'if (0) {}\n' +
            '"}}";');
        bt(
            '"{{#";if(0){}',
            //  -- output --
            '"{{#";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{{!";if(0){}"}}";',
            //  -- output --
            '"{{!";\n' +
            'if (0) {}\n' +
            '"}}";');
        bt(
            '"{{!";if(0){}',
            //  -- output --
            '"{{!";\n' +
            'if (0) {}');

        // Templating disabled - ensure formatting - ()
        reset_options();
        set_name('Templating disabled - ensure formatting - ()');
        opts.templating = ['auto'];
        bt(
            '"{{!--";if(0){}"--}}";',
            //  -- output --
            '"{{!--";\n' +
            'if (0) {}\n' +
            '"--}}";');
        bt(
            '"{{!--";if(0){}',
            //  -- output --
            '"{{!--";\n' +
            'if (0) {}');


        //============================================================
        // jslint and space after anon function - (jslint_happy = "true", space_after_anon_function = "true")
        reset_options();
        set_name('jslint and space after anon function - (jslint_happy = "true", space_after_anon_function = "true")');
        opts.jslint_happy = true;
        opts.space_after_anon_function = true;
        bt(
            'a=typeof(x)',
            //  -- output --
            'a = typeof (x)');
        bt(
            'x();\n' +
            '\n' +
            'function(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function () {}');
        bt(
            'x();\n' +
            '\n' +
            'function y(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function y() {}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function () {}\n' +
            '}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function y(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function y() {}\n' +
            '}');
        bt(
            'function () {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}');
        bt(
            'switch(x) {case 0: case 1: a(); break; default: break}',
            //  -- output --
            'switch (x) {\n' +
            'case 0:\n' +
            'case 1:\n' +
            '    a();\n' +
            '    break;\n' +
            'default:\n' +
            '    break\n' +
            '}');
        bt(
            'switch(x){case -1:break;case !y:break;}',
            //  -- output --
            'switch (x) {\n' +
            'case -1:\n' +
            '    break;\n' +
            'case !y:\n' +
            '    break;\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x) {case 0: case 1:{a(); break;} default: break}',
            //  -- output --
            'switch (x) {\n' +
            'case 0:\n' +
            'case 1: {\n' +
            '    a();\n' +
            '    break;\n' +
            '}\n' +
            'default:\n' +
            '    break\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x){case -1:break;case !y:{break;}}',
            //  -- output --
            'switch (x) {\n' +
            'case -1:\n' +
            '    break;\n' +
            'case !y: {\n' +
            '    break;\n' +
            '}\n' +
            '}');

        // Issue #1622 - basic class with function definitions
        bt(
            'class blah {\n' +
            '    constructor() {\n' +
            '        this.doStuff()\n' +
            '    }\n' +
            '    doStuff() {\n' +
            '        console.log("stuff")\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class with extends and function definitions
        bt(
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            'this.y = 1;\n' +
            '    }\n' +
            '}',
            //  -- output --
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            '        this.y = 1;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class/extends as a property
        bt(
            'var a.class = {\n' +
            ' ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            ' bb.s(),\n' +
            '})',
            //  -- output --
            'var a.class = {\n' +
            '    ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            '    bb.s(),\n' +
            '})');

        // typical greasemonkey start
        test_fragment(
            '// comment 2\n' +
            '(function ()');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function () {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function yoohoo() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function yoohoo() {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {},\n' +
            'd = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function () {},\n' +
            '    d = \'\';');
        bt(
            'var o2=$.extend(a);function(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function () {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'var o2=$.extend(a);function yoohoo(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function yoohoo() {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'function*() {\n' +
            '    yield 1;\n' +
            '}',
            //  -- output --
            'function* () {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* yoohoo() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'async x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'var a={data(){},\n' +
            'data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    data() {},\n' +
            '    data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            'data(){},\n' +
            'data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {data(){},\n' +
            'data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '}');
        bt(
            'var a={*data(){},*data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    * data() {},\n' +
            '    * data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            '*data(){},*data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {*data(){},*data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '}');

        // jslint and space after anon function - (jslint_happy = "true", space_after_anon_function = "false")
        reset_options();
        set_name('jslint and space after anon function - (jslint_happy = "true", space_after_anon_function = "false")');
        opts.jslint_happy = true;
        opts.space_after_anon_function = false;
        bt(
            'a=typeof(x)',
            //  -- output --
            'a = typeof (x)');
        bt(
            'x();\n' +
            '\n' +
            'function(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function () {}');
        bt(
            'x();\n' +
            '\n' +
            'function y(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function y() {}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function () {}\n' +
            '}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function y(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function y() {}\n' +
            '}');
        bt(
            'function () {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}');
        bt(
            'switch(x) {case 0: case 1: a(); break; default: break}',
            //  -- output --
            'switch (x) {\n' +
            'case 0:\n' +
            'case 1:\n' +
            '    a();\n' +
            '    break;\n' +
            'default:\n' +
            '    break\n' +
            '}');
        bt(
            'switch(x){case -1:break;case !y:break;}',
            //  -- output --
            'switch (x) {\n' +
            'case -1:\n' +
            '    break;\n' +
            'case !y:\n' +
            '    break;\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x) {case 0: case 1:{a(); break;} default: break}',
            //  -- output --
            'switch (x) {\n' +
            'case 0:\n' +
            'case 1: {\n' +
            '    a();\n' +
            '    break;\n' +
            '}\n' +
            'default:\n' +
            '    break\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x){case -1:break;case !y:{break;}}',
            //  -- output --
            'switch (x) {\n' +
            'case -1:\n' +
            '    break;\n' +
            'case !y: {\n' +
            '    break;\n' +
            '}\n' +
            '}');

        // Issue #1622 - basic class with function definitions
        bt(
            'class blah {\n' +
            '    constructor() {\n' +
            '        this.doStuff()\n' +
            '    }\n' +
            '    doStuff() {\n' +
            '        console.log("stuff")\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class with extends and function definitions
        bt(
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            'this.y = 1;\n' +
            '    }\n' +
            '}',
            //  -- output --
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            '        this.y = 1;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class/extends as a property
        bt(
            'var a.class = {\n' +
            ' ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            ' bb.s(),\n' +
            '})',
            //  -- output --
            'var a.class = {\n' +
            '    ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            '    bb.s(),\n' +
            '})');

        // typical greasemonkey start
        test_fragment(
            '// comment 2\n' +
            '(function ()');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function () {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function yoohoo() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function yoohoo() {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {},\n' +
            'd = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function () {},\n' +
            '    d = \'\';');
        bt(
            'var o2=$.extend(a);function(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function () {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'var o2=$.extend(a);function yoohoo(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function yoohoo() {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'function*() {\n' +
            '    yield 1;\n' +
            '}',
            //  -- output --
            'function* () {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* yoohoo() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'async x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'var a={data(){},\n' +
            'data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    data() {},\n' +
            '    data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            'data(){},\n' +
            'data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {data(){},\n' +
            'data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '}');
        bt(
            'var a={*data(){},*data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    * data() {},\n' +
            '    * data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            '*data(){},*data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {*data(){},*data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '}');

        // jslint and space after anon function - (jslint_happy = "false", space_after_anon_function = "true")
        reset_options();
        set_name('jslint and space after anon function - (jslint_happy = "false", space_after_anon_function = "true")');
        opts.jslint_happy = false;
        opts.space_after_anon_function = true;
        bt(
            'a=typeof(x)',
            //  -- output --
            'a = typeof (x)');
        bt(
            'x();\n' +
            '\n' +
            'function(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function () {}');
        bt(
            'x();\n' +
            '\n' +
            'function y(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function y() {}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function () {}\n' +
            '}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function y(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function y() {}\n' +
            '}');
        bt(
            'function () {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}');
        bt(
            'switch(x) {case 0: case 1: a(); break; default: break}',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '    case 1:\n' +
            '        a();\n' +
            '        break;\n' +
            '    default:\n' +
            '        break\n' +
            '}');
        bt(
            'switch(x){case -1:break;case !y:break;}',
            //  -- output --
            'switch (x) {\n' +
            '    case -1:\n' +
            '        break;\n' +
            '    case !y:\n' +
            '        break;\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x) {case 0: case 1:{a(); break;} default: break}',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '    case 1: {\n' +
            '        a();\n' +
            '        break;\n' +
            '    }\n' +
            '    default:\n' +
            '        break\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x){case -1:break;case !y:{break;}}',
            //  -- output --
            'switch (x) {\n' +
            '    case -1:\n' +
            '        break;\n' +
            '    case !y: {\n' +
            '        break;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - basic class with function definitions
        bt(
            'class blah {\n' +
            '    constructor() {\n' +
            '        this.doStuff()\n' +
            '    }\n' +
            '    doStuff() {\n' +
            '        console.log("stuff")\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class with extends and function definitions
        bt(
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            'this.y = 1;\n' +
            '    }\n' +
            '}',
            //  -- output --
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            '        this.y = 1;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class/extends as a property
        bt(
            'var a.class = {\n' +
            ' ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            ' bb.s(),\n' +
            '})',
            //  -- output --
            'var a.class = {\n' +
            '    ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            '    bb.s(),\n' +
            '})');

        // typical greasemonkey start
        test_fragment(
            '// comment 2\n' +
            '(function ()');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function () {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function yoohoo() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function yoohoo() {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {},\n' +
            'd = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function () {},\n' +
            '    d = \'\';');
        bt(
            'var o2=$.extend(a);function(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function () {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'var o2=$.extend(a);function yoohoo(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function yoohoo() {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'function*() {\n' +
            '    yield 1;\n' +
            '}',
            //  -- output --
            'function* () {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* yoohoo() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'async x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'var a={data(){},\n' +
            'data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    data() {},\n' +
            '    data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            'data(){},\n' +
            'data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {data(){},\n' +
            'data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '}');
        bt(
            'var a={*data(){},*data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    * data() {},\n' +
            '    * data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            '*data(){},*data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {*data(){},*data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '}');

        // jslint and space after anon function - (jslint_happy = "false", space_after_anon_function = "false")
        reset_options();
        set_name('jslint and space after anon function - (jslint_happy = "false", space_after_anon_function = "false")');
        opts.jslint_happy = false;
        opts.space_after_anon_function = false;
        bt(
            'a=typeof(x)',
            //  -- output --
            'a = typeof(x)');
        bt(
            'x();\n' +
            '\n' +
            'function(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function() {}');
        bt(
            'x();\n' +
            '\n' +
            'function y(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function y() {}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function() {}\n' +
            '}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function y(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function y() {}\n' +
            '}');
        bt(
            'function () {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}',
            //  -- output --
            'function() {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}');
        bt(
            'switch(x) {case 0: case 1: a(); break; default: break}',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '    case 1:\n' +
            '        a();\n' +
            '        break;\n' +
            '    default:\n' +
            '        break\n' +
            '}');
        bt(
            'switch(x){case -1:break;case !y:break;}',
            //  -- output --
            'switch (x) {\n' +
            '    case -1:\n' +
            '        break;\n' +
            '    case !y:\n' +
            '        break;\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x) {case 0: case 1:{a(); break;} default: break}',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '    case 1: {\n' +
            '        a();\n' +
            '        break;\n' +
            '    }\n' +
            '    default:\n' +
            '        break\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x){case -1:break;case !y:{break;}}',
            //  -- output --
            'switch (x) {\n' +
            '    case -1:\n' +
            '        break;\n' +
            '    case !y: {\n' +
            '        break;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - basic class with function definitions
        bt(
            'class blah {\n' +
            '    constructor() {\n' +
            '        this.doStuff()\n' +
            '    }\n' +
            '    doStuff() {\n' +
            '        console.log("stuff")\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class with extends and function definitions
        bt(
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            'this.y = 1;\n' +
            '    }\n' +
            '}',
            //  -- output --
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            '        this.y = 1;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class/extends as a property
        bt(
            'var a.class = {\n' +
            ' ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            ' bb.s(),\n' +
            '})',
            //  -- output --
            'var a.class = {\n' +
            '    ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            '    bb.s(),\n' +
            '})');

        // typical greasemonkey start
        test_fragment(
            '// comment 2\n' +
            '(function()');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function() {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function yoohoo() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function yoohoo() {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {},\n' +
            'd = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function() {},\n' +
            '    d = \'\';');
        bt(
            'var o2=$.extend(a);function(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function() {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'var o2=$.extend(a);function yoohoo(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function yoohoo() {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'function*() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* yoohoo() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'async x() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'var a={data(){},\n' +
            'data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    data() {},\n' +
            '    data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            'data(){},\n' +
            'data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {data(){},\n' +
            'data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    data() {},\n' +
            '    data2() {},\n' +
            '    a: 1\n' +
            '}');
        bt(
            'var a={*data(){},*data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    * data() {},\n' +
            '    * data2() {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            '*data(){},*data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {*data(){},*data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    * data() {},\n' +
            '    * data2() {},\n' +
            '    a: 1\n' +
            '}');

        // jslint and space after anon function - (space_after_named_function = "true")
        reset_options();
        set_name('jslint and space after anon function - (space_after_named_function = "true")');
        opts.space_after_named_function = true;
        bt(
            'a=typeof(x)',
            //  -- output --
            'a = typeof(x)');
        bt(
            'x();\n' +
            '\n' +
            'function(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function() {}');
        bt(
            'x();\n' +
            '\n' +
            'function y(){}',
            //  -- output --
            'x();\n' +
            '\n' +
            'function y () {}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function() {}\n' +
            '}');
        bt(
            'x();\n' +
            '\n' +
            'var x = {\n' +
            'x: function y(){}\n' +
            '}',
            //  -- output --
            'x();\n' +
            '\n' +
            'var x = {\n' +
            '    x: function y () {}\n' +
            '}');
        bt(
            'function () {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}',
            //  -- output --
            'function() {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}');
        bt(
            'switch(x) {case 0: case 1: a(); break; default: break}',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '    case 1:\n' +
            '        a();\n' +
            '        break;\n' +
            '    default:\n' +
            '        break\n' +
            '}');
        bt(
            'switch(x){case -1:break;case !y:break;}',
            //  -- output --
            'switch (x) {\n' +
            '    case -1:\n' +
            '        break;\n' +
            '    case !y:\n' +
            '        break;\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x) {case 0: case 1:{a(); break;} default: break}',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '    case 1: {\n' +
            '        a();\n' +
            '        break;\n' +
            '    }\n' +
            '    default:\n' +
            '        break\n' +
            '}');

        // Issue #1357
        bt(
            'switch(x){case -1:break;case !y:{break;}}',
            //  -- output --
            'switch (x) {\n' +
            '    case -1:\n' +
            '        break;\n' +
            '    case !y: {\n' +
            '        break;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - basic class with function definitions
        bt(
            'class blah {\n' +
            '    constructor() {\n' +
            '        this.doStuff()\n' +
            '    }\n' +
            '    doStuff() {\n' +
            '        console.log("stuff")\n' +
            '    }\n' +
            '}',
            //  -- output --
            'class blah {\n' +
            '    constructor () {\n' +
            '        this.doStuff()\n' +
            '    }\n' +
            '    doStuff () {\n' +
            '        console.log("stuff")\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class with extends and function definitions
        bt(
            'class blah extends something {\n' +
            '    constructor() {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction() {\n' +
            'this.y = 1;\n' +
            '    }\n' +
            '}',
            //  -- output --
            'class blah extends something {\n' +
            '    constructor () {\n' +
            '        this.zz = 2 + 2;\n' +
            '    }\n' +
            '    someOtherFunction () {\n' +
            '        this.y = 1;\n' +
            '    }\n' +
            '}');

        // Issue #1622 - class/extends as a property
        bt(
            'var a.class = {\n' +
            ' ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            ' bb.s(),\n' +
            '})',
            //  -- output --
            'var a.class = {\n' +
            '    ...abc(),\n' +
            '}\n' +
            'b.extends({\n' +
            '    bb.s(),\n' +
            '})');

        // typical greasemonkey start
        test_fragment(
            '// comment 2\n' +
            '(function()');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function() {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function yoohoo() {}, d = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function yoohoo () {},\n' +
            '    d = \'\';');
        bt(
            'var a2, b2, c2, d2 = 0, c = function() {},\n' +
            'd = \'\';',
            //  -- output --
            'var a2, b2, c2, d2 = 0,\n' +
            '    c = function() {},\n' +
            '    d = \'\';');
        bt(
            'var o2=$.extend(a);function(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function() {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'var o2=$.extend(a);function yoohoo(){alert(x);}',
            //  -- output --
            'var o2 = $.extend(a);\n' +
            '\n' +
            'function yoohoo () {\n' +
            '    alert(x);\n' +
            '}');
        bt(
            'function*() {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* yoohoo() {\n' +
            '    yield 1;\n' +
            '}',
            //  -- output --
            'function* yoohoo () {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'function* x() {\n' +
            '    yield 1;\n' +
            '}',
            //  -- output --
            'function* x () {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'async x() {\n' +
            '    yield 1;\n' +
            '}',
            //  -- output --
            'async x () {\n' +
            '    yield 1;\n' +
            '}');
        bt(
            'var a={data(){},\n' +
            'data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    data () {},\n' +
            '    data2 () {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            'data(){},\n' +
            'data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    data () {},\n' +
            '    data2 () {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {data(){},\n' +
            'data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    data () {},\n' +
            '    data2 () {},\n' +
            '    a: 1\n' +
            '}');
        bt(
            'var a={*data(){},*data2(){}}',
            //  -- output --
            'var a = {\n' +
            '    * data () {},\n' +
            '    * data2 () {}\n' +
            '}');
        bt(
            'new Vue({\n' +
            '*data(){},*data2(){}, a:1})',
            //  -- output --
            'new Vue({\n' +
            '    * data () {},\n' +
            '    * data2 () {},\n' +
            '    a: 1\n' +
            '})');
        bt(
            'export default {*data(){},*data2(){},\n' +
            'a:1}',
            //  -- output --
            'export default {\n' +
            '    * data () {},\n' +
            '    * data2 () {},\n' +
            '    a: 1\n' +
            '}');


        //============================================================
        // Regression tests
        reset_options();
        set_name('Regression tests');

        // Issue 241
        bt(
            'obj\n' +
            '    .last({\n' +
            '        foo: 1,\n' +
            '        bar: 2\n' +
            '    });\n' +
            'var test = 1;');

        // Issue #772
        bt(
            'this.initAttributes([\n' +
            '"name",\n' +
            '["parent", null, "parentName"],\n' +
            '"length",\n' +
            '["id", this.name],\n' +
            ']);',
            //  -- output --
            'this.initAttributes([\n' +
            '    "name",\n' +
            '    ["parent", null, "parentName"],\n' +
            '    "length",\n' +
            '    ["id", this.name],\n' +
            ']);');

        // Issue #1663
        bt(
            '{\n' +
            '    /* howdy\n' +
            '    \n' +
            '    */\n' +
            '}');

        // #1095 - Return without semicolon followed by prefix on a new line
        bt(
            'function x(){\n' +
            'return\n' +
            '++a\n' +
            '}\n' +
            '\n' +
            'while(true) {\n' +
            'return\n' +
            '--b\n' +
            '}',
            //  -- output --
            'function x() {\n' +
            '    return\n' +
            '    ++a\n' +
            '}\n' +
            '\n' +
            'while (true) {\n' +
            '    return\n' +
            '    --b\n' +
            '}');

        // #1095
        bt(
            'function test(){\n' +
            'if(x) return\n' +
            '++x\n' +
            'var y= 1;\n' +
            '}\n' +
            'function t1(){\n' +
            'if(cc) return;\n' +
            'else return\n' +
            '--cc\n' +
            '}',
            //  -- output --
            'function test() {\n' +
            '    if (x) return\n' +
            '    ++x\n' +
            '    var y = 1;\n' +
            '}\n' +
            '\n' +
            'function t1() {\n' +
            '    if (cc) return;\n' +
            '    else return\n' +
            '    --cc\n' +
            '}');

        // #1095 - Return with semicolon followed by a prefix on a new line
        bt(
            'function x(){\n' +
            'return; ++a\n' +
            '}\n' +
            '\n' +
            'while(true){return; --b\n' +
            '}',
            //  -- output --
            'function x() {\n' +
            '    return;\n' +
            '    ++a\n' +
            '}\n' +
            '\n' +
            'while (true) {\n' +
            '    return;\n' +
            '    --b\n' +
            '}');

        // #1838 - handle class and interface word as an object property
        bt(
            '{\n' +
            '    class: {\n' +
            '        a: 1,\n' +
            '        b: 2,\n' +
            '        c: 3,\n' +
            '    }\n' +
            '    interface: {\n' +
            '        a: 1,\n' +
            '        b: 2,\n' +
            '        c: 3,\n' +
            '    }\n' +
            '}');

        // #1838 - handle class word as an object property but with space after colon
        bt(
            '{\n' +
            '    class : { a: 1,\n' +
            'b: 2,c : 3\n' +
            '    }\n' +
            '}',
            //  -- output --
            '{\n' +
            '    class: {\n' +
            '        a: 1,\n' +
            '        b: 2,\n' +
            '        c: 3\n' +
            '    }\n' +
            '}');

        // #1838 - handle class word as an object property but without spaces
        bt(
            '{class:{a:1,b:2,c:3,}}',
            //  -- output --
            '{\n' +
            '    class: {\n' +
            '        a: 1,\n' +
            '        b: 2,\n' +
            '        c: 3,\n' +
            '    }\n' +
            '}');

        // #1838 - handle class word as a nested object property
        bt(
            '{x:{a:1,class:2,c:3,}}',
            //  -- output --
            '{\n' +
            '    x: {\n' +
            '        a: 1,\n' +
            '        class: 2,\n' +
            '        c: 3,\n' +
            '    }\n' +
            '}');
        bt(
            'obj\n' +
            '    .last(a, function() {\n' +
            '        var test;\n' +
            '    });\n' +
            'var test = 1;');
        bt(
            'obj.first()\n' +
            '    .second()\n' +
            '    .last(function(err, response) {\n' +
            '        console.log(err);\n' +
            '    });');

        // Issue 268 and 275
        bt(
            'obj.last(a, function() {\n' +
            '    var test;\n' +
            '});\n' +
            'var test = 1;');
        bt(
            'obj.last(a,\n' +
            '    function() {\n' +
            '        var test;\n' +
            '    });\n' +
            'var test = 1;');
        bt(
            '(function() {if (!window.FOO) window.FOO || (window.FOO = function() {var b = {bar: "zort"};});})();',
            //  -- output --
            '(function() {\n' +
            '    if (!window.FOO) window.FOO || (window.FOO = function() {\n' +
            '        var b = {\n' +
            '            bar: "zort"\n' +
            '        };\n' +
            '    });\n' +
            '})();');

        // Issue 281
        bt(
            'define(["dojo/_base/declare", "my/Employee", "dijit/form/Button",\n' +
            '    "dojo/_base/lang", "dojo/Deferred"\n' +
            '], function(declare, Employee, Button, lang, Deferred) {\n' +
            '    return declare(Employee, {\n' +
            '        constructor: function() {\n' +
            '            new Button({\n' +
            '                onClick: lang.hitch(this, function() {\n' +
            '                    new Deferred().then(lang.hitch(this, function() {\n' +
            '                        this.salary * 0.25;\n' +
            '                    }));\n' +
            '                })\n' +
            '            });\n' +
            '        }\n' +
            '    });\n' +
            '});');
        bt(
            'define(["dojo/_base/declare", "my/Employee", "dijit/form/Button",\n' +
            '        "dojo/_base/lang", "dojo/Deferred"\n' +
            '    ],\n' +
            '    function(declare, Employee, Button, lang, Deferred) {\n' +
            '        return declare(Employee, {\n' +
            '            constructor: function() {\n' +
            '                new Button({\n' +
            '                    onClick: lang.hitch(this, function() {\n' +
            '                        new Deferred().then(lang.hitch(this, function() {\n' +
            '                            this.salary * 0.25;\n' +
            '                        }));\n' +
            '                    })\n' +
            '                });\n' +
            '            }\n' +
            '        });\n' +
            '    });');

        // Issue 459
        bt(
            '(function() {\n' +
            '    return {\n' +
            '        foo: function() {\n' +
            '            return "bar";\n' +
            '        },\n' +
            '        bar: ["bar"]\n' +
            '    };\n' +
            '}());');

        // Issue 505 - strings should end at newline unless continued by backslash
        bt(
            'var name = "a;\n' +
            'name = "b";');
        bt(
            'var name = "a;\\\n' +
            '    name = b";');

        // Issue 514 - some operators require spaces to distinguish them
        bt('var c = "_ACTION_TO_NATIVEAPI_" + ++g++ + +new Date;');
        bt('var c = "_ACTION_TO_NATIVEAPI_" - --g-- - -new Date;');

        // Issue 440 - reserved words can be used as object property names
        bt(
            'a = {\n' +
            '    function: {},\n' +
            '    "function": {},\n' +
            '    throw: {},\n' +
            '    "throw": {},\n' +
            '    var: {},\n' +
            '    "var": {},\n' +
            '    set: {},\n' +
            '    "set": {},\n' +
            '    get: {},\n' +
            '    "get": {},\n' +
            '    if: {},\n' +
            '    "if": {},\n' +
            '    then: {},\n' +
            '    "then": {},\n' +
            '    else: {},\n' +
            '    "else": {},\n' +
            '    yay: {}\n' +
            '};');

        // Issue 331 - if-else with braces edge case
        bt(
            'if(x){a();}else{b();}if(y){c();}',
            //  -- output --
            'if (x) {\n' +
            '    a();\n' +
            '} else {\n' +
            '    b();\n' +
            '}\n' +
            'if (y) {\n' +
            '    c();\n' +
            '}');

        // Issue #1683 - switch-case wrong indentation
        bt(
            'switch (x) { case 0: if (y == z) { a(); } else { b(); } case 1: c(); }',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '        if (y == z) {\n' +
            '            a();\n' +
            '        } else {\n' +
            '            b();\n' +
            '        }\n' +
            '    case 1:\n' +
            '        c();\n' +
            '}');

        // Issue 485 - ensure function declarations behave the same in arrays as elsewhere
        bt(
            'var v = ["a",\n' +
            '    function() {\n' +
            '        return;\n' +
            '    }, {\n' +
            '        id: 1\n' +
            '    }\n' +
            '];');
        bt(
            'var v = ["a", function() {\n' +
            '    return;\n' +
            '}, {\n' +
            '    id: 1\n' +
            '}];');

        // Issue 382 - initial totally cursory support for es6 module export
        bt(
            'module "Even" {\n' +
            '    import odd from "Odd";\n' +
            '    export function sum(x, y) {\n' +
            '        return x + y;\n' +
            '    }\n' +
            '    export var pi = 3.141593;\n' +
            '    export default moduleName;\n' +
            '}');
        bt(
            'module "Even" {\n' +
            '    export default function div(x, y) {}\n' +
            '}');

        // Issue 889 - export default { ... }
        bt(
            'export default {\n' +
            '    func1() {},\n' +
            '    func2() {}\n' +
            '    func3() {}\n' +
            '}');
        bt(
            'export default {\n' +
            '    a() {\n' +
            '        return 1;\n' +
            '    },\n' +
            '    b() {\n' +
            '        return 2;\n' +
            '    },\n' +
            '    c() {\n' +
            '        return 3;\n' +
            '    }\n' +
            '}');

        // Issue 508
        bt('set["name"]');
        bt('get["name"]');
        test_fragment(
            'a = {\n' +
            '    set b(x) {},\n' +
            '    c: 1,\n' +
            '    d: function() {}\n' +
            '};');
        test_fragment(
            'a = {\n' +
            '    get b() {\n' +
            '        retun 0;\n' +
            '    },\n' +
            '    c: 1,\n' +
            '    d: function() {}\n' +
            '};');

        // Issue 298 - do not under indent if/while/for condtionals experesions
        bt(
            '\'use strict\';\n' +
            'if ([].some(function() {\n' +
            '        return false;\n' +
            '    })) {\n' +
            '    console.log("hello");\n' +
            '}');

        // Issue 298 - do not under indent if/while/for condtionals experesions
        bt(
            '\'use strict\';\n' +
            'if ([].some(function() {\n' +
            '        return false;\n' +
            '    })) {\n' +
            '    console.log("hello");\n' +
            '}');

        // Issue 552 - Typescript?  Okay... we didn't break it before, so try not to break it now.
        bt(
            'class Test {\n' +
            '    blah: string[];\n' +
            '    foo(): number {\n' +
            '        return 0;\n' +
            '    }\n' +
            '    bar(): number {\n' +
            '        return 0;\n' +
            '    }\n' +
            '}');

        // Issue 1544 - Typescript declare formatting (no newline).
        bt(
            'declare const require: any;\n' +
            'declare function greet(greeting: string): void;\n' +
            'declare var foo: number;\n' +
            'declare namespace myLib {\n' +
            '    function makeGreeting(s: string): string;\n' +
            '    let numberOfGreetings: number;\n' +
            '}\n' +
            'declare let test: any;');
        bt(
            'interface Test {\n' +
            '    blah: string[];\n' +
            '    foo(): number {\n' +
            '        return 0;\n' +
            '    }\n' +
            '    bar(): number {\n' +
            '        return 0;\n' +
            '    }\n' +
            '}');

        // Issue 583 - Functions with comments after them should still indent correctly.
        bt(
            'function exit(code) {\n' +
            '    setTimeout(function() {\n' +
            '        phantom.exit(code);\n' +
            '    }, 0);\n' +
            '    phantom.onError = function() {};\n' +
            '}\n' +
            '// Comment');

        // Issue 806 - newline arrow functions
        bt(
            'a.b("c",\n' +
            '    () => d.e\n' +
            ')');

        // Issue 810 - es6 object literal detection
        bt(
            'function badFormatting() {\n' +
            '    return {\n' +
            '        a,\n' +
            '        b: c,\n' +
            '        d: e,\n' +
            '        f: g,\n' +
            '        h,\n' +
            '        i,\n' +
            '        j: k\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            'function goodFormatting() {\n' +
            '    return {\n' +
            '        a: b,\n' +
            '        c,\n' +
            '        d: e,\n' +
            '        f: g,\n' +
            '        h,\n' +
            '        i,\n' +
            '        j: k\n' +
            '    }\n' +
            '}');

        // Issue 602 - ES6 object literal shorthand functions
        bt(
            'return {\n' +
            '    fn1() {},\n' +
            '    fn2() {}\n' +
            '}');
        bt(
            'throw {\n' +
            '    fn1() {},\n' +
            '    fn2() {}\n' +
            '}');
        bt(
            'foo({\n' +
            '    fn1(a) {}\n' +
            '    fn2(a) {}\n' +
            '})');
        bt(
            'foo("text", {\n' +
            '    fn1(a) {}\n' +
            '    fn2(a) {}\n' +
            '})');
        bt(
            'oneArg = {\n' +
            '    fn1(a) {\n' +
            '        do();\n' +
            '    },\n' +
            '    fn2() {}\n' +
            '}');
        bt(
            'multiArg = {\n' +
            '    fn1(a, b, c) {\n' +
            '        do();\n' +
            '    },\n' +
            '    fn2() {}\n' +
            '}');
        bt(
            'noArgs = {\n' +
            '    fn1() {\n' +
            '        do();\n' +
            '    },\n' +
            '    fn2() {}\n' +
            '}');
        bt(
            'emptyFn = {\n' +
            '    fn1() {},\n' +
            '    fn2() {}\n' +
            '}');
        bt(
            'nested = {\n' +
            '    fns: {\n' +
            '        fn1() {},\n' +
            '        fn2() {}\n' +
            '    }\n' +
            '}');
        bt(
            'array = [{\n' +
            '    fn1() {},\n' +
            '    prop: val,\n' +
            '    fn2() {}\n' +
            '}]');
        bt(
            'expr = expr ? expr : {\n' +
            '    fn1() {},\n' +
            '    fn2() {}\n' +
            '}');
        bt(
            'strange = valid + {\n' +
            '    fn1() {},\n' +
            '    fn2() {\n' +
            '        return 1;\n' +
            '    }\n' +
            '}.fn2()');

        // Issue 854 - Arrow function with statement block
        bt(
            'test(() => {\n' +
            '    var a = {}\n' +
            '\n' +
            '    a.what = () => true ? 1 : 2\n' +
            '\n' +
            '    a.thing = () => {\n' +
            '        b();\n' +
            '    }\n' +
            '})');

        // Issue 1727 - Optional chaining
        bt('true?.1:.2', 'true ? .1 : .2');

        // Issue 406 - Multiline array
        bt(
            'var tempName = [\n' +
            '    "temp",\n' +
            '    process.pid,\n' +
            '    (Math.random() * 0x1000000000).toString(36),\n' +
            '    new Date().getTime()\n' +
            '].join("-");');

        // Issue 1801 - Optional chaining w/ obj?.[expr] syntax
        bt(
            'let nestedProp = obj?.["prop" + "Name"];\n' +
            'let arrayItem = arr?.[42];');

        // Issue 1374 - Parameters starting with ! or [ merged into single line
        bt(
            'fn(\n' +
            '    1,\n' +
            '    !1,\n' +
            '    1,\n' +
            '    [1]\n' +
            ')');

        // Issue 1288 - Negative numbers remove newlines in array
        bt(
            'var array = [\n' +
            '    -1,\n' +
            '    0,\n' +
            '    "a",\n' +
            '    -2,\n' +
            '    1,\n' +
            '    -3,\n' +
            '];');

        // Issue 1229 - Negated expressions in array
        bt(
            'a = [\n' +
            '    true && 1,\n' +
            '    true && 1,\n' +
            '    true && 1\n' +
            ']\n' +
            'a = [\n' +
            '    !true && 1,\n' +
            '    !true && 1,\n' +
            '    !true && 1\n' +
            ']');

        // Issue #996 - Input ends with backslash throws exception
        test_fragment(
            'sd = 1;\n' +
            '/');

        // Issue #1079 - unbraced if with comments should still look right
        bt(
            'if (console.log)\n' +
            '    for (var i = 0; i < 20; ++i)\n' +
            '        if (i % 3)\n' +
            '            console.log(i);\n' +
            '// all done\n' +
            'console.log("done");');

        // Issue #1085 - function should not have blank line in a number of cases
        bt(
            'var transformer =\n' +
            '    options.transformer ||\n' +
            '    globalSettings.transformer ||\n' +
            '    function(x) {\n' +
            '        return x;\n' +
            '    };');

        // Issue #1794 - support nullish-coalescing
        bt('a = b ?? c');

        // Issue #569 - function should not have blank line in a number of cases
        bt(
            '(function(global) {\n' +
            '    "use strict";\n' +
            '\n' +
            '    /* jshint ignore:start */\n' +
            '    include "somefile.js"\n' +
            '    /* jshint ignore:end */\n' +
            '}(this));');
        bt(
            'function bindAuthEvent(eventName) {\n' +
            '    self.auth.on(eventName, function(event, meta) {\n' +
            '        self.emit(eventName, event, meta);\n' +
            '    });\n' +
            '}\n' +
            '["logged_in", "logged_out", "signed_up", "updated_user"].forEach(bindAuthEvent);\n' +
            '\n' +
            'function bindBrowserEvent(eventName) {\n' +
            '    browser.on(eventName, function(event, meta) {\n' +
            '        self.emit(eventName, event, meta);\n' +
            '    });\n' +
            '}\n' +
            '["navigating"].forEach(bindBrowserEvent);');

        // Issue #892 - new line between chained methods
        bt(
            'foo\n' +
            '    .who()\n' +
            '\n' +
            '    .knows()\n' +
            '    // comment\n' +
            '    .nothing() // comment\n' +
            '\n' +
            '    .more()');

        // Issue #1107 - Missing space between words for label
        bt(
            'function f(a) {c: do if (x) {} else if (y) {} while(0); return 0;}',
            //  -- output --
            'function f(a) {\n' +
            '    c: do\n' +
            '        if (x) {} else if (y) {}\n' +
            '    while (0);\n' +
            '    return 0;\n' +
            '}');
        bt(
            'function f(a) {c: if (x) {} else if (y) {} return 0;}',
            //  -- output --
            'function f(a) {\n' +
            '    c: if (x) {} else if (y) {}\n' +
            '    return 0;\n' +
            '}');


        //============================================================
        // Test non-positionable-ops
        reset_options();
        set_name('Test non-positionable-ops');
        bt('a += 2;');
        bt('a -= 2;');
        bt('a *= 2;');
        bt('a /= 2;');
        bt('a %= 2;');
        bt('a &= 2;');
        bt('a ^= 2;');
        bt('a |= 2;');
        bt('a **= 2;');
        bt('a <<= 2;');
        bt('a >>= 2;');


        //============================================================
        //
        reset_options();
        set_name('');

        // exponent literals
        bt('a = 1e10');
        bt('a = 1.3e10');
        bt('a = 1.3e-10');
        bt('a = -12345.3e-10');
        bt('a = .12345e-10');
        bt('a = 06789e-10');
        bt('a = e - 10');
        bt('a = 1.3e+10');
        bt('a = 1.e-7');
        bt('a = -12345.3e+10');
        bt('a = .12345e+10');
        bt('a = 06789e+10');
        bt('a = e + 10');
        bt('a=0e-12345.3e-10', 'a = 0e-12345 .3e-10');
        bt('a=0.e-12345.3e-10', 'a = 0.e-12345 .3e-10');
        bt('a=0x.e-12345.3e-10', 'a = 0x.e - 12345.3e-10');
        bt('a=0x0.e-12345.3e-10', 'a = 0x0.e - 12345.3e-10');
        bt('a=0x0.0e-12345.3e-10', 'a = 0x0 .0e-12345 .3e-10');
        bt('a=0g-12345.3e-10', 'a = 0 g - 12345.3e-10');
        bt('a=0.g-12345.3e-10', 'a = 0. g - 12345.3e-10');
        bt('a=0x.g-12345.3e-10', 'a = 0x.g - 12345.3e-10');
        bt('a=0x0.g-12345.3e-10', 'a = 0x0.g - 12345.3e-10');
        bt('a=0x0.0g-12345.3e-10', 'a = 0x0 .0 g - 12345.3e-10');

        // exponent literals with underscore
        bt('a = 1_1e10');
        bt('a = 1_.3e10');
        bt('a = 1_1.3e10');
        bt('a = 1__1.3e10');
        bt('a = 1._3e10');
        bt('a = 1.3_e10');
        bt('a = 1.3e_10');
        bt('a = 1.3e1_0');
        bt('a = 1.3e10_');

        // Decimal literals
        bt('a = 0123456789;');
        bt('a = 9876543210;');
        bt('a = 5647308291;');
        bt('a=030e-5', 'a = 030e-5');
        bt('a=00+4', 'a = 00 + 4');
        bt('a=32+4', 'a = 32 + 4');
        bt('a=0.6g+4', 'a = 0.6 g + 4');
        bt('a=01.10', 'a = 01.10');
        bt('a=a.10', 'a = a .10');
        bt('a=00B0x0', 'a = 00 B0x0');
        bt('a=00B0xb0', 'a = 00 B0xb0');
        bt('a=00B0x0b0', 'a = 00 B0x0b0');
        bt('a=0090x0', 'a = 0090 x0');
        bt('a=0g0b0o0', 'a = 0 g0b0o0');

        // Decimal literals with underscore
        bt('a = 0_123456789');
        bt('a = 0__123456789');
        bt('a = 0__');
        bt('a = 0_1_2_3');
        bt('a = 0_1_2_3_');

        // Hexadecimal literals
        bt('a = 0x0123456789abcdef;');
        bt('a = 0X0123456789ABCDEF;');
        bt('a = 0xFeDcBa9876543210;');
        bt('a=0x30e-5', 'a = 0x30e - 5');
        bt('a=0xF0+4', 'a = 0xF0 + 4');
        bt('a=0Xff+4', 'a = 0Xff + 4');
        bt('a=0Xffg+4', 'a = 0Xff g + 4');
        bt('a=0x01.10', 'a = 0x01 .10');
        bt('a = 0xb0ce;');
        bt('a = 0x0b0;');
        bt('a=0x0B0x0', 'a = 0x0B0 x0');
        bt('a=0x0B0xb0', 'a = 0x0B0 xb0');
        bt('a=0x0B0x0b0', 'a = 0x0B0 x0b0');
        bt('a=0X090x0', 'a = 0X090 x0');
        bt('a=0Xg0b0o0', 'a = 0X g0b0o0');

        // Hexadecimal literals with underscore
        bt('a = 0x0_123456789abcdef');
        bt('a = 0x0__0123456789abcdef');
        bt('a = 0x_0123456789abcdef');
        bt('a = 0x__');
        bt('a = 0x0_1_a_3');
        bt('a = 0x_1_2_F_');

        // Octal literals
        bt('a = 0o01234567;');
        bt('a = 0O01234567;');
        bt('a = 0o34120675;');
        bt('a=0o30e-5', 'a = 0o30 e - 5');
        bt('a=0o70+4', 'a = 0o70 + 4');
        bt('a=0O77+4', 'a = 0O77 + 4');
        bt('a=0O778+4', 'a = 0O77 8 + 4');
        bt('a=0O77a+4', 'a = 0O77 a + 4');
        bt('a=0o01.10', 'a = 0o01 .10');
        bt('a=0o0B0x0', 'a = 0o0 B0x0');
        bt('a=0o0B0xb0', 'a = 0o0 B0xb0');
        bt('a=0o0B0x0b0', 'a = 0o0 B0x0b0');
        bt('a=0O090x0', 'a = 0O0 90 x0');
        bt('a=0Og0b0o0', 'a = 0O g0b0o0');

        // Octal literals with underscore
        bt('a = 0o0_1234567');
        bt('a = 0o0__1234567');
        bt('a = 0o_01234567');
        bt('a = 0o__');
        bt('a = 0o0_1_2_3');
        bt('a = 0o_1_2_3_');

        // Binary literals
        bt('a = 0b010011;');
        bt('a = 0B010011;');
        bt('a = 0b01001100001111;');
        bt('a=0b10e-5', 'a = 0b10 e - 5');
        bt('a=0b10+4', 'a = 0b10 + 4');
        bt('a=0B11+4', 'a = 0B11 + 4');
        bt('a=0B112+4', 'a = 0B11 2 + 4');
        bt('a=0B11a+4', 'a = 0B11 a + 4');
        bt('a=0b01.10', 'a = 0b01 .10');
        bt('a=0b0B0x0', 'a = 0b0 B0x0');
        bt('a=0b0B0xb0', 'a = 0b0 B0xb0');
        bt('a=0b0B0x0b0', 'a = 0b0 B0x0b0');
        bt('a=0B090x0', 'a = 0B0 90 x0');
        bt('a=0Bg0b0o0', 'a = 0B g0b0o0');

        // Binary literals with underscore
        bt('a = 0b0_10011');
        bt('a = 0b0__10011');
        bt('a = 0b_010011');
        bt('a = 0b__');
        bt('a = 0b0_1_1_1');
        bt('a = 0b_1_0_1_');
        bt('a = 0B010_0_11;');
        bt('a = 0b01_0011_0000_1111;');

        // BigInt literals
        bt('a = 1n;');
        bt('a = 1234567890123456789n;');
        bt('a = -1234567890123456789n;');
        bt('a = 1234567890123456789 N;');
        bt('a=0b10e-5n', 'a = 0b10 e - 5n');
        bt('a=.0n', 'a = .0 n');
        bt('a=1.0n', 'a = 1.0 n');
        bt('a=1e0n', 'a = 1e0 n');
        bt('a=0n11a+4', 'a = 0n 11 a + 4');

        // BigInt literals with underscore
        bt('a = 0_123456789n');
        bt('a = 0__123456789n');
        bt('a = 0__n');
        bt('a = 0_1_2_3n');
        bt('a = 0_1_2_3_n');

        // BigInt hexadecimal literals
        bt('a = 0x0123456789abcdefn;');
        bt('a = 0X0123456789ABCDEFn;');
        bt('a = 0xFeDcBa9876543210n;');
        bt('a=0x30en-5', 'a = 0x30en - 5');
        bt('a=0xF0n+4', 'a = 0xF0n + 4');
        bt('a=0Xffn+4', 'a = 0Xffn + 4');
        bt('a=0Xffng+4', 'a = 0Xffn g + 4');
        bt('a=0x01n.10', 'a = 0x01n .10');
        bt('a = 0xb0cen;');
        bt('a = 0x0b0n;');
        bt('a=0x0B0nx0', 'a = 0x0B0n x0');
        bt('a=0x0B0nxb0', 'a = 0x0B0n xb0');
        bt('a=0x0B0nx0b0', 'a = 0x0B0n x0b0');
        bt('a=0X090nx0', 'a = 0X090n x0');

        // BigInt hexadecimal literals with underscore
        bt('a = 0x0_123456789abcdefn');
        bt('a = 0x0__0123456789abcdefn');
        bt('a = 0x_0123456789abcdefn');
        bt('a = 0x__n');
        bt('a = 0x0_1_a_3n');
        bt('a = 0x_1_2_F_n');

        // BigInt octal literals
        bt('a = 0o01234567n;');
        bt('a = 0O01234567n;');
        bt('a = 0o34120675n;');
        bt('a=0o30ne-5', 'a = 0o30n e - 5');
        bt('a=0o70n+4', 'a = 0o70n + 4');
        bt('a=0O77n+4', 'a = 0O77n + 4');
        bt('a=0O77n8+4', 'a = 0O77n 8 + 4');
        bt('a=0O77na+4', 'a = 0O77n a + 4');
        bt('a=0o01n.10', 'a = 0o01n .10');
        bt('a=0o0nB0x0', 'a = 0o0n B0x0');
        bt('a=0o0nB0xb0', 'a = 0o0n B0xb0');
        bt('a=0o0nB0x0b0', 'a = 0o0n B0x0b0');
        bt('a=0O0n90x0', 'a = 0O0n 90 x0');

        // BigInt octal literals with underscore
        bt('a = 0o0_1234567n');
        bt('a = 0o0__1234567n');
        bt('a = 0o_01234567n');
        bt('a = 0o__n');
        bt('a = 0o0_1_2_3n');
        bt('a = 0o_1_2_3_n');

        // BigInt binary literals
        bt('a = 0b010011n;');
        bt('a = 0B010011n;');
        bt('a = 0b01001100001111n;');
        bt('a=0b10ne-5', 'a = 0b10n e - 5');
        bt('a=0b10n+4', 'a = 0b10n + 4');
        bt('a=0B11n+4', 'a = 0B11n + 4');
        bt('a=0B11n2+4', 'a = 0B11n 2 + 4');
        bt('a=0B11na+4', 'a = 0B11n a + 4');
        bt('a=0b01n.10', 'a = 0b01n .10');
        bt('a=0b0nB0x0', 'a = 0b0n B0x0');
        bt('a=0b0nB0xb0', 'a = 0b0n B0xb0');
        bt('a=0b0nB0x0b0', 'a = 0b0n B0x0b0');
        bt('a=0B0n90x0', 'a = 0B0n 90 x0');

        // BigInt binary literals with underscore
        bt('a = 0b0_10011n');
        bt('a = 0b0__10011n');
        bt('a = 0b_010011');
        bt('a = 0b__n');
        bt('a = 0b0_1_1_1n');
        bt('a = 0b_1_0_1_n');
        bt('a = 0B010_0_11n;');
        bt('a = 0b01_0011_0000_1111n;');


        //============================================================
        // brace_style ,preserve-inline tests - (brace_style = ""collapse,preserve-inline"")
        reset_options();
        set_name('brace_style ,preserve-inline tests - (brace_style = ""collapse,preserve-inline"")');
        opts.brace_style = 'collapse,preserve-inline';
        bt('import { asdf } from "asdf";');
        bt('import { get } from "asdf";');
        bt('function inLine() { console.log("oh em gee"); }');
        bt('if (cancer) { console.log("Im sorry but you only have so long to live..."); }');
        bt('if (ding) { console.log("dong"); } else { console.log("dang"); }');
        bt(
            'function kindaComplex() {\n' +
            '    var a = 2;\n' +
            '    var obj = {};\n' +
            '    var obj2 = { a: "a", b: "b" };\n' +
            '    var obj3 = {\n' +
            '        c: "c",\n' +
            '        d: "d",\n' +
            '        e: "e"\n' +
            '    };\n' +
            '}');
        bt(
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '             console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}',
            //  -- output --
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '            console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}');

        // brace_style ,preserve-inline tests - (brace_style = ""expand,preserve-inline"")
        reset_options();
        set_name('brace_style ,preserve-inline tests - (brace_style = ""expand,preserve-inline"")');
        opts.brace_style = 'expand,preserve-inline';
        bt('import { asdf } from "asdf";');
        bt('import { get } from "asdf";');
        bt('function inLine() { console.log("oh em gee"); }');
        bt('if (cancer) { console.log("Im sorry but you only have so long to live..."); }');
        bt(
            'if (ding) { console.log("dong"); } else { console.log("dang"); }',
            //  -- output --
            'if (ding) { console.log("dong"); }\n' +
            'else { console.log("dang"); }');
        bt(
            'function kindaComplex() {\n' +
            '    var a = 2;\n' +
            '    var obj = {};\n' +
            '    var obj2 = { a: "a", b: "b" };\n' +
            '    var obj3 = {\n' +
            '        c: "c",\n' +
            '        d: "d",\n' +
            '        e: "e"\n' +
            '    };\n' +
            '}',
            //  -- output --
            'function kindaComplex()\n' +
            '{\n' +
            '    var a = 2;\n' +
            '    var obj = {};\n' +
            '    var obj2 = { a: "a", b: "b" };\n' +
            '    var obj3 = {\n' +
            '        c: "c",\n' +
            '        d: "d",\n' +
            '        e: "e"\n' +
            '    };\n' +
            '}');
        bt(
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '             console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}',
            //  -- output --
            'function complex()\n' +
            '{\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b()\n' +
            '        {\n' +
            '            console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}');

        // brace_style ,preserve-inline tests - (brace_style = ""end-expand,preserve-inline"")
        reset_options();
        set_name('brace_style ,preserve-inline tests - (brace_style = ""end-expand,preserve-inline"")');
        opts.brace_style = 'end-expand,preserve-inline';
        bt('import { asdf } from "asdf";');
        bt('import { get } from "asdf";');
        bt('function inLine() { console.log("oh em gee"); }');
        bt('if (cancer) { console.log("Im sorry but you only have so long to live..."); }');
        bt(
            'if (ding) { console.log("dong"); } else { console.log("dang"); }',
            //  -- output --
            'if (ding) { console.log("dong"); }\n' +
            'else { console.log("dang"); }');
        bt(
            'function kindaComplex() {\n' +
            '    var a = 2;\n' +
            '    var obj = {};\n' +
            '    var obj2 = { a: "a", b: "b" };\n' +
            '    var obj3 = {\n' +
            '        c: "c",\n' +
            '        d: "d",\n' +
            '        e: "e"\n' +
            '    };\n' +
            '}');
        bt(
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '             console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}',
            //  -- output --
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '            console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}');

        // brace_style ,preserve-inline tests - (brace_style = ""none,preserve-inline"")
        reset_options();
        set_name('brace_style ,preserve-inline tests - (brace_style = ""none,preserve-inline"")');
        opts.brace_style = 'none,preserve-inline';
        bt('import { asdf } from "asdf";');
        bt('import { get } from "asdf";');
        bt('function inLine() { console.log("oh em gee"); }');
        bt('if (cancer) { console.log("Im sorry but you only have so long to live..."); }');
        bt('if (ding) { console.log("dong"); } else { console.log("dang"); }');
        bt(
            'function kindaComplex() {\n' +
            '    var a = 2;\n' +
            '    var obj = {};\n' +
            '    var obj2 = { a: "a", b: "b" };\n' +
            '    var obj3 = {\n' +
            '        c: "c",\n' +
            '        d: "d",\n' +
            '        e: "e"\n' +
            '    };\n' +
            '}');
        bt(
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '             console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}',
            //  -- output --
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '            console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}');

        // brace_style ,preserve-inline tests - (brace_style = ""collapse-preserve-inline"")
        reset_options();
        set_name('brace_style ,preserve-inline tests - (brace_style = ""collapse-preserve-inline"")');
        opts.brace_style = 'collapse-preserve-inline';
        bt('import { asdf } from "asdf";');
        bt('import { get } from "asdf";');
        bt('function inLine() { console.log("oh em gee"); }');
        bt('if (cancer) { console.log("Im sorry but you only have so long to live..."); }');
        bt('if (ding) { console.log("dong"); } else { console.log("dang"); }');
        bt(
            'function kindaComplex() {\n' +
            '    var a = 2;\n' +
            '    var obj = {};\n' +
            '    var obj2 = { a: "a", b: "b" };\n' +
            '    var obj3 = {\n' +
            '        c: "c",\n' +
            '        d: "d",\n' +
            '        e: "e"\n' +
            '    };\n' +
            '}');
        bt(
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '             console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}',
            //  -- output --
            'function complex() {\n' +
            '    console.log("wowe");\n' +
            '    (function() { var a = 2; var b = 3; })();\n' +
            '    $.each(arr, function(el, idx) { return el; });\n' +
            '    var obj = {\n' +
            '        a: function() { console.log("test"); },\n' +
            '        b() {\n' +
            '            console.log("test2");\n' +
            '        }\n' +
            '    };\n' +
            '}');


        //============================================================
        // Destructured and related
        reset_options();
        set_name('Destructured and related');
        opts.brace_style = 'collapse,preserve-inline';

        // Issue 382 - import destructured
        bt(
            'module "Even" {\n' +
            '    import { odd, oddly } from "Odd";\n' +
            '}');
        bt(
            'import defaultMember from "module-name";\n' +
            'import * as name from "module-name";\n' +
            'import { member } from "module-name";\n' +
            'import { member as alias } from "module-name";\n' +
            'import { member1, member2 } from "module-name";\n' +
            'import { member1, member2 as alias2 } from "module-name";\n' +
            'import defaultMember, { member, member2 } from "module-name";\n' +
            'import defaultMember, * as name from "module-name";\n' +
            'import "module-name";\n' +
            'import("module-name")');

        // Issue #1393 - dynamic import()
        bt(
            'if (from < to) {\n' +
            '    import(`dynamic${library}`);\n' +
            '} else {\n' +
            '    import("otherdynamic");\n' +
            '}');

        // Issue #1197 - dynamic import() arrow syntax
        bt('frontend = Async(() => import("../frontend").then(m => m.default      ))', 'frontend = Async(() => import("../frontend").then(m => m.default))');

        // Issue #1978 - import.meta syntax support
        bt('let       x =      import.meta', 'let x = import.meta');

        // Issue 858 - from is a keyword only after import
        bt(
            'if (from < to) {\n' +
            '    from++;\n' +
            '} else {\n' +
            '    from--;\n' +
            '}');

        // Issue 511 - destrutured
        bt(
            'var { b, c } = require("../stores");\n' +
            'var { ProjectStore } = require("../stores");\n' +
            '\n' +
            'function takeThing({ prop }) {\n' +
            '    console.log("inner prop", prop)\n' +
            '}');

        // Issue 315 - Short objects
        bt('var a = { b: { c: { d: e } } };');
        bt(
            'var a = {\n' +
            '    b: {\n' +
            '        c: { d: e }\n' +
            '        c3: { d: e }\n' +
            '    },\n' +
            '    b2: { c: { d: e } }\n' +
            '};');

        // Issue 370 - Short objects in array
        bt(
            'var methods = [\n' +
            '    { name: "to" },\n' +
            '    { name: "step" },\n' +
            '    { name: "move" },\n' +
            '    { name: "min" },\n' +
            '    { name: "max" }\n' +
            '];');

        // Issue 838 - Short objects in array
        bt(
            'function(url, callback) {\n' +
            '    var script = document.createElement("script")\n' +
            '    if (true) script.onreadystatechange = function() {\n' +
            '        foo();\n' +
            '    }\n' +
            '    else script.onload = callback;\n' +
            '}');

        // Issue 578 - Odd indenting after function
        bt(
            'function bindAuthEvent(eventName) {\n' +
            '    self.auth.on(eventName, function(event, meta) {\n' +
            '        self.emit(eventName, event, meta);\n' +
            '    });\n' +
            '}\n' +
            '["logged_in", "logged_out", "signed_up", "updated_user"].forEach(bindAuthEvent);');

        // Issue #487 - some short expressions examples
        bt(
            'if (a == 1) { a++; }\n' +
            'a = { a: a };\n' +
            'UserDB.findOne({ username: "xyz" }, function(err, user) {});\n' +
            'import { fs } from "fs";');

        // Issue #982 - Fixed return expression collapse-preserve-inline
        bt(
            'function foo(arg) {\n' +
            '    if (!arg) { a(); }\n' +
            '    if (!arg) { return false; }\n' +
            '    if (!arg) { throw "inline"; }\n' +
            '    return true;\n' +
            '}');

        // Issue #338 - Short expressions
        bt(
            'if (someCondition) { return something; }\n' +
            'if (someCondition) {\n' +
            '    return something;\n' +
            '}\n' +
            'if (someCondition) { break; }\n' +
            'if (someCondition) {\n' +
            '    return something;\n' +
            '}');

        // Issue #1283 - Javascript ++ Operator get wrong indent
        bt(
            '{this.foo++\n' +
            'bar}',
            //  -- output --
            '{\n' +
            '    this.foo++\n' +
            '    bar\n' +
            '}');

        // Issue #1283 - Javascript ++ Operator get wrong indent (2)
        bt(
            'axios.interceptors.request.use(\n' +
            '    config => {\n' +
            '        // loading\n' +
            '        window.store.loading++\n' +
            '        let extraParams = {}\n' +
            '    }\n' +
            ')');

        // Issue ##1846 - in keyword in class method causes indentation problem
        bt(
            'class {\n' +
            '  get a() {\n' +
            '\n' +
            '\n' +
            '  }\n' +
            '\n' +
            '\n' +
            '  in() {\n' +
            '\n' +
            '\n' +
            '  }\n' +
            '\n' +
            '\n' +
            '  b() {\n' +
            '\n' +
            '\n' +
            '  }\n' +
            '}',
            //  -- output --
            'class {\n' +
            '    get a() {\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    in() {\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    b() {\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '}');

        // Related to Issue ##1846 - Do not indent 'in' keyword if not a class method
        bt(
            'function test() {\n' +
            'for x in nums {}\n' +
            '"make" in car\n' +
            '3 in number;\n' +
            '}',
            //  -- output --
            'function test() {\n' +
            '    for x in nums {}\n' +
            '    "make" in car\n' +
            '    3 in number;\n' +
            '}');

        // Related to Issue ##1846 - of keyword in class method causes indentation problem
        bt(
            'class {\n' +
            '  get a() {\n' +
            '\n' +
            '\n' +
            '  }\n' +
            '\n' +
            '\n' +
            '  of() {\n' +
            '\n' +
            '\n' +
            '  }\n' +
            '\n' +
            '\n' +
            '  b() {\n' +
            '\n' +
            '\n' +
            '  }\n' +
            '}',
            //  -- output --
            'class {\n' +
            '    get a() {\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    of() {\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    b() {\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '}');

        // Issue #1950: Do not remove whitespace after number - test scenario: number before a dot
        bt('1000000000000001000 .toFixed(0)!==1000000000000001024', '1000000000000001000 .toFixed(0) !== 1000000000000001024');

        // Issue #1950: Do not remove whitespace after number - test scenario: variable ends with a number before a dot
        bt('a.b21 . performAction()', 'a.b21.performAction()');


        //============================================================
        // keep_array_indentation false
        reset_options();
        set_name('keep_array_indentation false');
        opts.keep_array_indentation = false;
        bt(
            'a  = ["a", "b", "c",\n' +
            '   "d", "e", "f"]',
            //  -- output --
            'a = ["a", "b", "c",\n' +
            '    "d", "e", "f"\n' +
            ']');
        bt(
            'a  = ["a", "b", "c",\n' +
            '   "d", "e", "f",\n' +
            '        "g", "h", "i"]',
            //  -- output --
            'a = ["a", "b", "c",\n' +
            '    "d", "e", "f",\n' +
            '    "g", "h", "i"\n' +
            ']');
        bt(
            'a  = ["a", "b", "c",\n' +
            '       "d", "e", "f",\n' +
            '            "g", "h", "i"]',
            //  -- output --
            'a = ["a", "b", "c",\n' +
            '    "d", "e", "f",\n' +
            '    "g", "h", "i"\n' +
            ']');
        bt(
            'var  x = [{}\n' +
            ']',
            //  -- output --
            'var x = [{}]');
        bt(
            'var x = [{foo:bar}\n' +
            ']',
            //  -- output --
            'var x = [{\n' +
            '    foo: bar\n' +
            '}]');
        bt(
            'a  = ["something",\n' +
            '    "completely",\n' +
            '    "different"];\n' +
            'if (x);',
            //  -- output --
            'a = ["something",\n' +
            '    "completely",\n' +
            '    "different"\n' +
            '];\n' +
            'if (x);');
        bt('a = ["a","b","c"]', 'a = ["a", "b", "c"]');
        bt('a = ["a",   "b","c"]', 'a = ["a", "b", "c"]');
        bt(
            'x = [{"a":0}]',
            //  -- output --
            'x = [{\n' +
            '    "a": 0\n' +
            '}]');
        bt(
            '{a([[a1]], {b;});}',
            //  -- output --
            '{\n' +
            '    a([\n' +
            '        [a1]\n' +
            '    ], {\n' +
            '        b;\n' +
            '    });\n' +
            '}');
        bt(
            'a ();\n' +
            '   [\n' +
            '   ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '   ].toString();',
            //  -- output --
            'a();\n' +
            '[\n' +
            '    ["sdfsdfsd"],\n' +
            '    ["sdfsdfsdf"]\n' +
            '].toString();');
        bt(
            'a ();\n' +
            'a = [\n' +
            '   ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '   ].toString();',
            //  -- output --
            'a();\n' +
            'a = [\n' +
            '    ["sdfsdfsd"],\n' +
            '    ["sdfsdfsdf"]\n' +
            '].toString();');
        bt(
            'function()  {\n' +
            '    Foo([\n' +
            '        ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '    ]);\n' +
            '}',
            //  -- output --
            'function() {\n' +
            '    Foo([\n' +
            '        ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '    ]);\n' +
            '}');
        bt(
            'function  foo() {\n' +
            '    return [\n' +
            '        "one",\n' +
            '        "two"\n' +
            '    ];\n' +
            '}',
            //  -- output --
            'function foo() {\n' +
            '    return [\n' +
            '        "one",\n' +
            '        "two"\n' +
            '    ];\n' +
            '}');
        bt(
            'function foo() {\n' +
            '    return [\n' +
            '        {\n' +
            '            one: "x",\n' +
            '            two: [\n' +
            '                {\n' +
            '                    id: "a",\n' +
            '                    name: "apple"\n' +
            '                }, {\n' +
            '                    id: "b",\n' +
            '                    name: "banana"\n' +
            '                }\n' +
            '            ]\n' +
            '        }\n' +
            '    ];\n' +
            '}',
            //  -- output --
            'function foo() {\n' +
            '    return [{\n' +
            '        one: "x",\n' +
            '        two: [{\n' +
            '            id: "a",\n' +
            '            name: "apple"\n' +
            '        }, {\n' +
            '            id: "b",\n' +
            '            name: "banana"\n' +
            '        }]\n' +
            '    }];\n' +
            '}');
        bt(
            'function foo() {\n' +
            '   return [\n' +
            '      {\n' +
            '         one: "x",\n' +
            '         two: [\n' +
            '            {\n' +
            '               id: "a",\n' +
            '               name: "apple"\n' +
            '            }, {\n' +
            '               id: "b",\n' +
            '               name: "banana"\n' +
            '            }\n' +
            '         ]\n' +
            '      }\n' +
            '   ];\n' +
            '}',
            //  -- output --
            'function foo() {\n' +
            '    return [{\n' +
            '        one: "x",\n' +
            '        two: [{\n' +
            '            id: "a",\n' +
            '            name: "apple"\n' +
            '        }, {\n' +
            '            id: "b",\n' +
            '            name: "banana"\n' +
            '        }]\n' +
            '    }];\n' +
            '}');


        //============================================================
        // keep_array_indentation true
        reset_options();
        set_name('keep_array_indentation true');
        opts.keep_array_indentation = true;
        bt(
            'a  = ["a", "b", "c",\n' +
            '   "d", "e", "f"]',
            //  -- output --
            'a = ["a", "b", "c",\n' +
            '   "d", "e", "f"]');
        bt(
            'a  = ["a", "b", "c",\n' +
            '   "d", "e", "f",\n' +
            '        "g", "h", "i"]',
            //  -- output --
            'a = ["a", "b", "c",\n' +
            '   "d", "e", "f",\n' +
            '        "g", "h", "i"]');
        bt(
            'a  = ["a", "b", "c",\n' +
            '       "d", "e", "f",\n' +
            '            "g", "h", "i"]',
            //  -- output --
            'a = ["a", "b", "c",\n' +
            '       "d", "e", "f",\n' +
            '            "g", "h", "i"]');
        bt(
            'var  x = [{}\n' +
            ']',
            //  -- output --
            'var x = [{}\n' +
            ']');
        bt(
            'var x = [{foo:bar}\n' +
            ']',
            //  -- output --
            'var x = [{\n' +
            '        foo: bar\n' +
            '    }\n' +
            ']');
        bt(
            'a  = ["something",\n' +
            '    "completely",\n' +
            '    "different"];\n' +
            'if (x);',
            //  -- output --
            'a = ["something",\n' +
            '    "completely",\n' +
            '    "different"];\n' +
            'if (x);');
        bt('a = ["a","b","c"]', 'a = ["a", "b", "c"]');
        bt('a = ["a",   "b","c"]', 'a = ["a", "b", "c"]');
        bt(
            'x = [{"a":0}]',
            //  -- output --
            'x = [{\n' +
            '    "a": 0\n' +
            '}]');
        bt(
            '{a([[a1]], {b;});}',
            //  -- output --
            '{\n' +
            '    a([[a1]], {\n' +
            '        b;\n' +
            '    });\n' +
            '}');
        bt(
            'a ();\n' +
            '   [\n' +
            '   ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '   ].toString();',
            //  -- output --
            'a();\n' +
            '   [\n' +
            '   ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '   ].toString();');
        bt(
            'a ();\n' +
            'a = [\n' +
            '   ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '   ].toString();',
            //  -- output --
            'a();\n' +
            'a = [\n' +
            '   ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '   ].toString();');
        bt(
            'function()  {\n' +
            '    Foo([\n' +
            '        ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '    ]);\n' +
            '}',
            //  -- output --
            'function() {\n' +
            '    Foo([\n' +
            '        ["sdfsdfsd"],\n' +
            '        ["sdfsdfsdf"]\n' +
            '    ]);\n' +
            '}');
        bt(
            'function  foo() {\n' +
            '    return [\n' +
            '        "one",\n' +
            '        "two"\n' +
            '    ];\n' +
            '}',
            //  -- output --
            'function foo() {\n' +
            '    return [\n' +
            '        "one",\n' +
            '        "two"\n' +
            '    ];\n' +
            '}');
        bt(
            'function  foo() {\n' +
            '    return [\n' +
            '        {\n' +
            '            one: "x",\n' +
            '            two: [\n' +
            '                {\n' +
            '                    id: "a",\n' +
            '                    name: "apple"\n' +
            '                }, {\n' +
            '                    id: "b",\n' +
            '                    name: "banana"\n' +
            '                }\n' +
            '            ]\n' +
            '        }\n' +
            '    ];\n' +
            '}',
            //  -- output --
            'function foo() {\n' +
            '    return [\n' +
            '        {\n' +
            '            one: "x",\n' +
            '            two: [\n' +
            '                {\n' +
            '                    id: "a",\n' +
            '                    name: "apple"\n' +
            '                }, {\n' +
            '                    id: "b",\n' +
            '                    name: "banana"\n' +
            '                }\n' +
            '            ]\n' +
            '        }\n' +
            '    ];\n' +
            '}');


        //============================================================
        // indent_empty_lines true
        reset_options();
        set_name('indent_empty_lines true');
        opts.indent_empty_lines = true;
        test_fragment(
            'var a = 1;\n' +
            '\n' +
            'var b = 1;');
        test_fragment(
            'var a = 1;\n' +
            '        \n' +
            'var b = 1;',
            //  -- output --
            'var a = 1;\n' +
            '\n' +
            'var b = 1;');
        test_fragment(
            '{\n' +
            '    var a = 1;\n' +
            '        \n' +
            '    var b = 1;\n' +
            '\n' +
            '}',
            //  -- output --
            '{\n' +
            '    var a = 1;\n' +
            '    \n' +
            '    var b = 1;\n' +
            '    \n' +
            '}');
        test_fragment(
            '{\n' +
            '\n' +
            '    var a = 1;\n' +
            '\n' +
            '\n' +
            '\n' +
            '    var b = 1;\n' +
            '\n' +
            '}',
            //  -- output --
            '{\n' +
            '    \n' +
            '    var a = 1;\n' +
            '    \n' +
            '    \n' +
            '    \n' +
            '    var b = 1;\n' +
            '    \n' +
            '}');
        test_fragment(
            '{\n' +
            '\n' +
            '    var a = 1;\n' +
            '\n' +
            'function A() {\n' +
            '\n' +
            '}\n' +
            '\n' +
            '    var b = 1;\n' +
            '\n' +
            '}',
            //  -- output --
            '{\n' +
            '    \n' +
            '    var a = 1;\n' +
            '    \n' +
            '    function A() {\n' +
            '        \n' +
            '    }\n' +
            '    \n' +
            '    var b = 1;\n' +
            '    \n' +
            '}');


        //============================================================
        // indent_empty_lines false
        reset_options();
        set_name('indent_empty_lines false');
        opts.indent_empty_lines = false;
        test_fragment(
            'var a = 1;\n' +
            '\n' +
            'var b = 1;');
        test_fragment(
            'var a = 1;\n' +
            '        \n' +
            'var b = 1;',
            //  -- output --
            'var a = 1;\n' +
            '\n' +
            'var b = 1;');
        test_fragment(
            '{\n' +
            '    var a = 1;\n' +
            '        \n' +
            '    var b = 1;\n' +
            '\n' +
            '}',
            //  -- output --
            '{\n' +
            '    var a = 1;\n' +
            '\n' +
            '    var b = 1;\n' +
            '\n' +
            '}');
        test_fragment(
            '{\n' +
            '\n' +
            '    var a = 1;\n' +
            '\n' +
            '\n' +
            '\n' +
            '    var b = 1;\n' +
            '\n' +
            '}');
        test_fragment(
            '{\n' +
            '\n' +
            '    var a = 1;\n' +
            '\n' +
            'function A() {\n' +
            '\n' +
            '}\n' +
            '\n' +
            '    var b = 1;\n' +
            '\n' +
            '}',
            //  -- output --
            '{\n' +
            '\n' +
            '    var a = 1;\n' +
            '\n' +
            '    function A() {\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '    var b = 1;\n' +
            '\n' +
            '}');


        //============================================================
        // Old tests
        reset_options();
        set_name('Old tests');
        bt('');
        test_fragment('   return .5');
        test_fragment(
            '   return .5;\n' +
            '   a();');
        test_fragment(
            '    return .5;\n' +
            '    a();');
        test_fragment(
            '     return .5;\n' +
            '     a();');
        test_fragment('   < div');
        bt('a        =          1', 'a = 1');
        bt('a=1', 'a = 1');
        bt('(3) / 2');
        bt('["a", "b"].join("")');
        bt(
            'a();\n' +
            '\n' +
            'b();');
        bt(
            'var a = 1 var b = 2',
            //  -- output --
            'var a = 1\n' +
            'var b = 2');
        bt(
            'var a=1, b=c[d], e=6;',
            //  -- output --
            'var a = 1,\n' +
            '    b = c[d],\n' +
            '    e = 6;');
        bt(
            'var a,\n' +
            '    b,\n' +
            '    c;');
        bt(
            'let a = 1 let b = 2',
            //  -- output --
            'let a = 1\n' +
            'let b = 2');
        bt(
            'let a=1, b=c[d], e=6;',
            //  -- output --
            'let a = 1,\n' +
            '    b = c[d],\n' +
            '    e = 6;');
        bt(
            'let a,\n' +
            '    b,\n' +
            '    c;');
        bt(
            'const a = 1 const b = 2',
            //  -- output --
            'const a = 1\n' +
            'const b = 2');
        bt(
            'const a=1, b=c[d], e=6;',
            //  -- output --
            'const a = 1,\n' +
            '    b = c[d],\n' +
            '    e = 6;');
        bt(
            'const a,\n' +
            '    b,\n' +
            '    c;');
        bt('a = " 12345 "');
        bt('a = \' 12345 \'');
        bt('if (a == 1) b = 2;');
        bt(
            'if(1){2}else{3}',
            //  -- output --
            'if (1) {\n' +
            '    2\n' +
            '} else {\n' +
            '    3\n' +
            '}');
        bt('if(1||2);', 'if (1 || 2);');
        bt('(a==1)||(b==2)', '(a == 1) || (b == 2)');
        bt(
            'var a = 1 if (2) 3;',
            //  -- output --
            'var a = 1\n' +
            'if (2) 3;');
        bt('a = a + 1');
        bt('a = a == 1');
        bt('/12345[^678]*9+/.match(a)');
        bt('a /= 5');
        bt('a = 0.5 * 3');
        bt('a *= 10.55');
        bt('a < .5');
        bt('a <= .5');
        bt('a<.5', 'a < .5');
        bt('a<=.5', 'a <= .5');
        bt('a = [1, 2, 3, 4]');
        bt('F*(g/=f)*g+b', 'F * (g /= f) * g + b');
        bt(
            'a.b({c:d})',
            //  -- output --
            'a.b({\n' +
            '    c: d\n' +
            '})');
        bt(
            'a.b\n' +
            '(\n' +
            '{\n' +
            'c:\n' +
            'd\n' +
            '}\n' +
            ')',
            //  -- output --
            'a.b({\n' +
            '    c: d\n' +
            '})');
        bt(
            'a.b({c:"d"})',
            //  -- output --
            'a.b({\n' +
            '    c: "d"\n' +
            '})');
        bt(
            'a.b\n' +
            '(\n' +
            '{\n' +
            'c:\n' +
            '"d"\n' +
            '}\n' +
            ')',
            //  -- output --
            'a.b({\n' +
            '    c: "d"\n' +
            '})');
        bt('a=!b', 'a = !b');
        bt('a=!!b', 'a = !!b');
        bt('a?b:c', 'a ? b : c');
        bt('a?1:2', 'a ? 1 : 2');
        bt('a?(b):c', 'a ? (b) : c');
        bt(
            'x={a:1,b:w=="foo"?x:y,c:z}',
            //  -- output --
            'x = {\n' +
            '    a: 1,\n' +
            '    b: w == "foo" ? x : y,\n' +
            '    c: z\n' +
            '}');
        bt('x=a?b?c?d:e:f:g;', 'x = a ? b ? c ? d : e : f : g;');
        bt(
            'x=a?b?c?d:{e1:1,e2:2}:f:g;',
            //  -- output --
            'x = a ? b ? c ? d : {\n' +
            '    e1: 1,\n' +
            '    e2: 2\n' +
            '} : f : g;');
        bt('function void(void) {}');
        bt('if(!a)foo();', 'if (!a) foo();');
        bt('a=~a', 'a = ~a');
        bt(
            'a;/*comment*/b;',
            //  -- output --
            'a; /*comment*/\n' +
            'b;');
        bt(
            'a;/* comment */b;',
            //  -- output --
            'a; /* comment */\n' +
            'b;');

        // simple comments don't get touched at all
        test_fragment(
            'a;/*\n' +
            'comment\n' +
            '*/b;',
            //  -- output --
            'a;\n' +
            '/*\n' +
            'comment\n' +
            '*/\n' +
            'b;');
        bt(
            'a;/**\n' +
            '* javadoc\n' +
            '*/b;',
            //  -- output --
            'a;\n' +
            '/**\n' +
            ' * javadoc\n' +
            ' */\n' +
            'b;');
        test_fragment(
            'a;/**\n' +
            '\n' +
            'no javadoc\n' +
            '*/b;',
            //  -- output --
            'a;\n' +
            '/**\n' +
            '\n' +
            'no javadoc\n' +
            '*/\n' +
            'b;');

        // comment blocks detected and reindented even w/o javadoc starter
        bt(
            'a;/*\n' +
            '* javadoc\n' +
            '*/b;',
            //  -- output --
            'a;\n' +
            '/*\n' +
            ' * javadoc\n' +
            ' */\n' +
            'b;');
        bt('if(a)break;', 'if (a) break;');
        bt(
            'if(a){break}',
            //  -- output --
            'if (a) {\n' +
            '    break\n' +
            '}');
        bt('if((a))foo();', 'if ((a)) foo();');
        bt('for(var i=0;;) a', 'for (var i = 0;;) a');
        bt(
            'for(var i=0;;)\n' +
            'a',
            //  -- output --
            'for (var i = 0;;)\n' +
            '    a');
        bt('a++;');
        bt('for(;;i++)a()', 'for (;; i++) a()');
        bt(
            'for(;;i++)\n' +
            'a()',
            //  -- output --
            'for (;; i++)\n' +
            '    a()');
        bt('for(;;++i)a', 'for (;; ++i) a');
        bt('return(1)', 'return (1)');
        bt(
            'try{a();}catch(b){c();}finally{d();}',
            //  -- output --
            'try {\n' +
            '    a();\n' +
            '} catch (b) {\n' +
            '    c();\n' +
            '} finally {\n' +
            '    d();\n' +
            '}');

        //  magic function call
        bt('(xx)()');

        // another magic function call
        bt('a[1]()');
        bt(
            'if(a){b();}else if(c) foo();',
            //  -- output --
            'if (a) {\n' +
            '    b();\n' +
            '} else if (c) foo();');
        bt(
            'switch(x) {case 0: case 1: a(); break; default: break}',
            //  -- output --
            'switch (x) {\n' +
            '    case 0:\n' +
            '    case 1:\n' +
            '        a();\n' +
            '        break;\n' +
            '    default:\n' +
            '        break\n' +
            '}');
        bt(
            'switch(x) {default: case 1: a(); break; case 0: break}',
            //  -- output --
            'switch (x) {\n' +
            '    default:\n' +
            '    case 1:\n' +
            '        a();\n' +
            '        break;\n' +
            '    case 0:\n' +
            '        break\n' +
            '}');
        bt(
            'switch(x){case -1:break;case !y:break;}',
            //  -- output --
            'switch (x) {\n' +
            '    case -1:\n' +
            '        break;\n' +
            '    case !y:\n' +
            '        break;\n' +
            '}');
        bt('a !== b');
        bt(
            'if (a) b(); else c();',
            //  -- output --
            'if (a) b();\n' +
            'else c();');

        // typical greasemonkey start
        bt(
            '// comment\n' +
            '(function something() {})');

        // duplicating newlines
        bt(
            '{\n' +
            '\n' +
            '    x();\n' +
            '\n' +
            '}');
        bt('if (a in b) foo();');
        bt('if (a of b) foo();');
        bt('if (a of [1, 2, 3]) foo();');
        bt(
            'if(X)if(Y)a();else b();else c();',
            //  -- output --
            'if (X)\n' +
            '    if (Y) a();\n' +
            '    else b();\n' +
            'else c();');
        bt(
            'if (foo) bar();\n' +
            'else break');
        bt('var a, b;');
        bt('var a = new function();');
        test_fragment('new function');
        bt('var a, b');
        bt(
            '{a:1, b:2}',
            //  -- output --
            '{\n' +
            '    a: 1,\n' +
            '    b: 2\n' +
            '}');
        bt(
            'a={1:[-1],2:[+1]}',
            //  -- output --
            'a = {\n' +
            '    1: [-1],\n' +
            '    2: [+1]\n' +
            '}');
        bt(
            'var l = {\'a\':\'1\', \'b\':\'2\'}',
            //  -- output --
            'var l = {\n' +
            '    \'a\': \'1\',\n' +
            '    \'b\': \'2\'\n' +
            '}');
        bt('if (template.user[n] in bk) foo();');
        bt('return 45');
        bt(
            'return this.prevObject ||\n' +
            '\n' +
            '    this.constructor(null);');
        bt('If[1]');
        bt('Then[1]');
        bt('a = 1;// comment', 'a = 1; // comment');
        bt('a = 1; // comment');
        bt(
            'a = 1;\n' +
            ' // comment',
            //  -- output --
            'a = 1;\n' +
            '// comment');
        bt('a = [-1, -1, -1]');
        bt(
            '// a\n' +
            '// b\n' +
            '\n' +
            '\n' +
            '\n' +
            '// c\n' +
            '// d');
        bt(
            '// func-comment\n' +
            '\n' +
            'function foo() {}\n' +
            '\n' +
            '// end-func-comment');

        // The exact formatting these should have is open for discussion, but they are at least reasonable
        bt(
            'a = [ // comment\n' +
            '    -1, -1, -1\n' +
            ']');
        bt(
            'var a = [ // comment\n' +
            '    -1, -1, -1\n' +
            ']');
        bt(
            'a = [ // comment\n' +
            '    -1, // comment\n' +
            '    -1, -1\n' +
            ']');
        bt(
            'var a = [ // comment\n' +
            '    -1, // comment\n' +
            '    -1, -1\n' +
            ']');
        bt(
            'o = [{a:b},{c:d}]',
            //  -- output --
            'o = [{\n' +
            '    a: b\n' +
            '}, {\n' +
            '    c: d\n' +
            '}]');

        // was: extra space appended
        bt(
            'if (a) {\n' +
            '    do();\n' +
            '}');

        // if/else statement with empty body
        bt(
            'if (a) {\n' +
            '// comment\n' +
            '}else{\n' +
            '// comment\n' +
            '}',
            //  -- output --
            'if (a) {\n' +
            '    // comment\n' +
            '} else {\n' +
            '    // comment\n' +
            '}');

        // multiple comments indentation
        bt(
            'if (a) {\n' +
            '// comment\n' +
            '// comment\n' +
            '}',
            //  -- output --
            'if (a) {\n' +
            '    // comment\n' +
            '    // comment\n' +
            '}');
        bt(
            'if (a) b() else c();',
            //  -- output --
            'if (a) b()\n' +
            'else c();');
        bt(
            'if (a) b() else if c() d();',
            //  -- output --
            'if (a) b()\n' +
            'else if c() d();');
        bt('{}');
        bt(
            '{\n' +
            '\n' +
            '}');
        bt(
            'do { a(); } while ( 1 );',
            //  -- output --
            'do {\n' +
            '    a();\n' +
            '} while (1);');
        bt('do {} while (1);');
        bt(
            'do {\n' +
            '} while (1);',
            //  -- output --
            'do {} while (1);');
        bt(
            'do {\n' +
            '\n' +
            '} while (1);');
        bt('var a = x(a, b, c)');
        bt(
            'delete x if (a) b();',
            //  -- output --
            'delete x\n' +
            'if (a) b();');
        bt(
            'delete x[x] if (a) b();',
            //  -- output --
            'delete x[x]\n' +
            'if (a) b();');
        bt('for(var a=1,b=2)d', 'for (var a = 1, b = 2) d');
        bt('for(var a=1,b=2,c=3) d', 'for (var a = 1, b = 2, c = 3) d');
        bt(
            'for(var a=1,b=2,c=3;d<3;d++)\n' +
            'e',
            //  -- output --
            'for (var a = 1, b = 2, c = 3; d < 3; d++)\n' +
            '    e');
        bt(
            'function x(){(a||b).c()}',
            //  -- output --
            'function x() {\n' +
            '    (a || b).c()\n' +
            '}');
        bt(
            'function x(){return - 1}',
            //  -- output --
            'function x() {\n' +
            '    return -1\n' +
            '}');
        bt(
            'function x(){return ! a}',
            //  -- output --
            'function x() {\n' +
            '    return !a\n' +
            '}');
        bt('x => x');
        bt('(x) => x');
        bt(
            'x => { x }',
            //  -- output --
            'x => {\n' +
            '    x\n' +
            '}');
        bt(
            '(x) => { x }',
            //  -- output --
            '(x) => {\n' +
            '    x\n' +
            '}');

        // a common snippet in jQuery plugins
        bt(
            'settings = $.extend({},defaults,settings);',
            //  -- output --
            'settings = $.extend({}, defaults, settings);');
        bt('$http().then().finally().default()');
        bt(
            '$http()\n' +
            '.then()\n' +
            '.finally()\n' +
            '.default()',
            //  -- output --
            '$http()\n' +
            '    .then()\n' +
            '    .finally()\n' +
            '    .default()');
        bt('$http().when.in.new.catch().throw()');
        bt(
            '$http()\n' +
            '.when\n' +
            '.in\n' +
            '.new\n' +
            '.catch()\n' +
            '.throw()',
            //  -- output --
            '$http()\n' +
            '    .when\n' +
            '    .in\n' +
            '    .new\n' +
            '    .catch()\n' +
            '    .throw()');
        bt(
            '{xxx;}()',
            //  -- output --
            '{\n' +
            '    xxx;\n' +
            '}()');
        bt(
            'a = \'a\'\n' +
            'b = \'b\'');
        bt('a = /reg/exp');
        bt('a = /reg/');
        bt('/abc/.test()');
        bt('/abc/i.test()');
        bt(
            '{/abc/i.test()}',
            //  -- output --
            '{\n' +
            '    /abc/i.test()\n' +
            '}');
        bt('var x=(a)/a;', 'var x = (a) / a;');
        bt('x != -1');
        bt('for (; s-->0;)t', 'for (; s-- > 0;) t');
        bt('for (; s++>0;)u', 'for (; s++ > 0;) u');
        bt('a = s++>s--;', 'a = s++ > s--;');
        bt('a = s++>--s;', 'a = s++ > --s;');
        bt(
            '{x=#1=[]}',
            //  -- output --
            '{\n' +
            '    x = #1=[]\n' +
            '}');
        bt(
            '{a:#1={}}',
            //  -- output --
            '{\n' +
            '    a: #1={}\n' +
            '}');
        bt(
            '{a:#1#}',
            //  -- output --
            '{\n' +
            '    a: #1#\n' +
            '}');
        test_fragment('"incomplete-string');
        test_fragment('\'incomplete-string');
        test_fragment('/incomplete-regex');
        test_fragment('`incomplete-template-string');
        test_fragment(
            '{a:1},{a:2}',
            //  -- output --
            '{\n' +
            '    a: 1\n' +
            '}, {\n' +
            '    a: 2\n' +
            '}');
        test_fragment(
            'var ary=[{a:1}, {a:2}];',
            //  -- output --
            'var ary = [{\n' +
            '    a: 1\n' +
            '}, {\n' +
            '    a: 2\n' +
            '}];');

        // incomplete
        test_fragment(
            '{a:#1',
            //  -- output --
            '{\n' +
            '    a: #1');

        // incomplete
        test_fragment(
            '{a:#',
            //  -- output --
            '{\n' +
            '    a: #');

        // incomplete
        test_fragment(
            '}}}',
            //  -- output --
            '}\n' +
            '}\n' +
            '}');
        test_fragment(
            '<!--\n' +
            'void();\n' +
            '// -->');

        // incomplete regexp
        test_fragment('a=/regexp', 'a = /regexp');
        bt(
            '{a:#1=[],b:#1#,c:#999999#}',
            //  -- output --
            '{\n' +
            '    a: #1=[],\n' +
            '    b: #1#,\n' +
            '    c: #999999#\n' +
            '}');
        bt(
            'do{x()}while(a>1)',
            //  -- output --
            'do {\n' +
            '    x()\n' +
            '} while (a > 1)');
        bt(
            'x(); /reg/exp.match(something)',
            //  -- output --
            'x();\n' +
            '/reg/exp.match(something)');
        test_fragment(
            'something();(',
            //  -- output --
            'something();\n' +
            '(');
        test_fragment(
            '#!she/bangs, she bangs\n' +
            'f=1',
            //  -- output --
            '#!she/bangs, she bangs\n' +
            '\n' +
            'f = 1');
        test_fragment(
            '#!she/bangs, she bangs\n' +
            '\n' +
            'f=1',
            //  -- output --
            '#!she/bangs, she bangs\n' +
            '\n' +
            'f = 1');
        test_fragment(
            '#!she/bangs, she bangs\n' +
            '\n' +
            '/* comment */');
        test_fragment(
            '#!she/bangs, she bangs\n' +
            '\n' +
            '\n' +
            '/* comment */');
        test_fragment('#');
        test_fragment('#!');
        test_fragment('#include');
        test_fragment('#include "settings.jsxinc"');
        test_fragment(
            '#include "settings.jsxinc"\n' +
            '\n' +
            '\n' +
            '/* comment */');
        test_fragment(
            '#include "settings.jsxinc"\n' +
            '\n' +
            '\n' +
            '#include "settings.jsxinc"\n' +
            '\n' +
            '\n' +
            '/* comment */');
        bt('function namespace::something()');
        test_fragment(
            '<!--\n' +
            'something();\n' +
            '-->');
        test_fragment(
            '<!--\n' +
            'if(i<0){bla();}\n' +
            '-->',
            //  -- output --
            '<!--\n' +
            'if (i < 0) {\n' +
            '    bla();\n' +
            '}\n' +
            '-->');
        bt(
            '{foo();--bar;}',
            //  -- output --
            '{\n' +
            '    foo();\n' +
            '    --bar;\n' +
            '}');
        bt(
            '{foo();++bar;}',
            //  -- output --
            '{\n' +
            '    foo();\n' +
            '    ++bar;\n' +
            '}');
        bt(
            '{--bar;}',
            //  -- output --
            '{\n' +
            '    --bar;\n' +
            '}');
        bt(
            '{++bar;}',
            //  -- output --
            '{\n' +
            '    ++bar;\n' +
            '}');
        bt('if(true)++a;', 'if (true) ++a;');
        bt(
            'if(true)\n' +
            '++a;',
            //  -- output --
            'if (true)\n' +
            '    ++a;');
        bt('if(true)--a;', 'if (true) --a;');
        bt(
            'if(true)\n' +
            '--a;',
            //  -- output --
            'if (true)\n' +
            '    --a;');
        bt('elem[array]++;');
        bt('elem++ * elem[array]++;');
        bt('elem-- * -elem[array]++;');
        bt('elem-- + elem[array]++;');
        bt('elem-- - elem[array]++;');
        bt('elem-- - -elem[array]++;');
        bt('elem-- - +elem[array]++;');

        // Handling of newlines around unary ++ and -- operators
        bt(
            '{foo\n' +
            '++bar;}',
            //  -- output --
            '{\n' +
            '    foo\n' +
            '    ++bar;\n' +
            '}');
        bt(
            '{foo++\n' +
            'bar;}',
            //  -- output --
            '{\n' +
            '    foo++\n' +
            '    bar;\n' +
            '}');

        // This is invalid, but harder to guard against. Issue #203.
        bt(
            '{foo\n' +
            '++\n' +
            'bar;}',
            //  -- output --
            '{\n' +
            '    foo\n' +
            '    ++\n' +
            '    bar;\n' +
            '}');

        // regexps
        bt(
            'a(/abc\\/\\/def/);b()',
            //  -- output --
            'a(/abc\\/\\/def/);\n' +
            'b()');
        bt(
            'a(/a[b\\[\\]c]d/);b()',
            //  -- output --
            'a(/a[b\\[\\]c]d/);\n' +
            'b()');

        // incomplete char class
        test_fragment('a(/a[b\\[');

        // allow unescaped / in char classes
        bt(
            'a(/[a/b]/);b()',
            //  -- output --
            'a(/[a/b]/);\n' +
            'b()');
        bt('typeof /foo\\//;');
        bt('throw /foo\\//;');
        bt('do /foo\\//;');
        bt('return /foo\\//;');
        bt(
            'switch (a) {\n' +
            '    case /foo\\//:\n' +
            '        b\n' +
            '}');
        bt(
            'if (a) /foo\\//\n' +
            'else /foo\\//;');
        bt('if (foo) /regex/.test();');
        bt('for (index in [1, 2, 3]) /^test$/i.test(s)');
        bt(
            'function foo() {\n' +
            '    return [\n' +
            '        "one",\n' +
            '        "two"\n' +
            '    ];\n' +
            '}');
        bt(
            'a=[[1,2],[4,5],[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    function() {},\n' +
            '    [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    function() {},\n' +
            '    function() {},\n' +
            '    [7, 8]\n' +
            ']');
        bt(
            'a=[[1,2],[4,5],function(){},[7,8]]',
            //  -- output --
            'a = [\n' +
            '    [1, 2],\n' +
            '    [4, 5],\n' +
            '    function() {},\n' +
            '    [7, 8]\n' +
            ']');
        bt('a=[b,c,function(){},function(){},d]', 'a = [b, c, function() {}, function() {}, d]');
        bt(
            'a=[b,c,\n' +
            'function(){},function(){},d]',
            //  -- output --
            'a = [b, c,\n' +
            '    function() {},\n' +
            '    function() {},\n' +
            '    d\n' +
            ']');
        bt('a=[a[1],b[4],c[d[7]]]', 'a = [a[1], b[4], c[d[7]]]');
        bt('[1,2,[3,4,[5,6],7],8]', '[1, 2, [3, 4, [5, 6], 7], 8]');
        bt(
            '[[["1","2"],["3","4"]],[["5","6","7"],["8","9","0"]],[["1","2","3"],["4","5","6","7"],["8","9","0"]]]',
            //  -- output --
            '[\n' +
            '    [\n' +
            '        ["1", "2"],\n' +
            '        ["3", "4"]\n' +
            '    ],\n' +
            '    [\n' +
            '        ["5", "6", "7"],\n' +
            '        ["8", "9", "0"]\n' +
            '    ],\n' +
            '    [\n' +
            '        ["1", "2", "3"],\n' +
            '        ["4", "5", "6", "7"],\n' +
            '        ["8", "9", "0"]\n' +
            '    ]\n' +
            ']');
        bt(
            '{[x()[0]];indent;}',
            //  -- output --
            '{\n' +
            '    [x()[0]];\n' +
            '    indent;\n' +
            '}');
        bt(
            '/*\n' +
            ' foo trailing space    \n' +
            ' * bar trailing space   \n' +
            '**/');
        bt(
            '{\n' +
            '    /*\n' +
            '    foo    \n' +
            '    * bar    \n' +
            '    */\n' +
            '}');
        bt('return ++i');
        bt(
            'obj.num++\n' +
            'foo()\n' +
            'bar()\n' +
            '\n' +
            'obj.num--\n' +
            'foo()\n' +
            'bar()');
        bt('return !!x');
        bt('return !x');
        bt('return [1,2]', 'return [1, 2]');
        bt('return;');
        bt(
            'return\n' +
            'func');
        bt('catch(e)', 'catch (e)');
        bt(
            'var a=1,b={foo:2,bar:3},{baz:4,wham:5},c=4;',
            //  -- output --
            'var a = 1,\n' +
            '    b = {\n' +
            '        foo: 2,\n' +
            '        bar: 3\n' +
            '    },\n' +
            '    {\n' +
            '        baz: 4,\n' +
            '        wham: 5\n' +
            '    }, c = 4;');
        bt(
            'var a=1,b={foo:2,bar:3},{baz:4,wham:5},\n' +
            'c=4;',
            //  -- output --
            'var a = 1,\n' +
            '    b = {\n' +
            '        foo: 2,\n' +
            '        bar: 3\n' +
            '    },\n' +
            '    {\n' +
            '        baz: 4,\n' +
            '        wham: 5\n' +
            '    },\n' +
            '    c = 4;');

        // inline comment
        bt(
            'function x(/*int*/ start, /*string*/ foo)',
            //  -- output --
            'function x( /*int*/ start, /*string*/ foo)');

        // javadoc comment
        bt(
            '/**\n' +
            '* foo\n' +
            '*/',
            //  -- output --
            '/**\n' +
            ' * foo\n' +
            ' */');
        bt(
            '{\n' +
            '/**\n' +
            '* foo\n' +
            '*/\n' +
            '}',
            //  -- output --
            '{\n' +
            '    /**\n' +
            '     * foo\n' +
            '     */\n' +
            '}');

        // starless block comment
        bt(
            '/**\n' +
            'foo\n' +
            '*/');
        bt(
            '/**\n' +
            'foo\n' +
            '**/');
        bt(
            '/**\n' +
            'foo\n' +
            'bar\n' +
            '**/');
        bt(
            '/**\n' +
            'foo\n' +
            '\n' +
            'bar\n' +
            '**/');
        bt(
            '/**\n' +
            'foo\n' +
            '    bar\n' +
            '**/');
        bt(
            '{\n' +
            '/**\n' +
            'foo\n' +
            '*/\n' +
            '}',
            //  -- output --
            '{\n' +
            '    /**\n' +
            '    foo\n' +
            '    */\n' +
            '}');
        bt(
            '{\n' +
            '/**\n' +
            'foo\n' +
            '**/\n' +
            '}',
            //  -- output --
            '{\n' +
            '    /**\n' +
            '    foo\n' +
            '    **/\n' +
            '}');
        bt(
            '{\n' +
            '/**\n' +
            'foo\n' +
            'bar\n' +
            '**/\n' +
            '}',
            //  -- output --
            '{\n' +
            '    /**\n' +
            '    foo\n' +
            '    bar\n' +
            '    **/\n' +
            '}');
        bt(
            '{\n' +
            '/**\n' +
            'foo\n' +
            '\n' +
            'bar\n' +
            '**/\n' +
            '}',
            //  -- output --
            '{\n' +
            '    /**\n' +
            '    foo\n' +
            '\n' +
            '    bar\n' +
            '    **/\n' +
            '}');
        bt(
            '{\n' +
            '/**\n' +
            'foo\n' +
            '    bar\n' +
            '**/\n' +
            '}',
            //  -- output --
            '{\n' +
            '    /**\n' +
            '    foo\n' +
            '        bar\n' +
            '    **/\n' +
            '}');
        bt(
            '{\n' +
            '    /**\n' +
            '    foo\n' +
            'bar\n' +
            '    **/\n' +
            '}');
        bt(
            'var a,b,c=1,d,e,f=2;',
            //  -- output --
            'var a, b, c = 1,\n' +
            '    d, e, f = 2;');
        bt(
            'var a,b,c=[],d,e,f=2;',
            //  -- output --
            'var a, b, c = [],\n' +
            '    d, e, f = 2;');
        bt(
            'function() {\n' +
            '    var a, b, c, d, e = [],\n' +
            '        f;\n' +
            '}');
        bt(
            'do/regexp/;\n' +
            'while(1);',
            //  -- output --
            'do /regexp/;\n' +
            'while (1);');
        bt(
            'var a = a,\n' +
            'a;\n' +
            'b = {\n' +
            'b\n' +
            '}',
            //  -- output --
            'var a = a,\n' +
            '    a;\n' +
            'b = {\n' +
            '    b\n' +
            '}');
        bt(
            'var a = a,\n' +
            '    /* c */\n' +
            '    b;');
        bt(
            'var a = a,\n' +
            '    // c\n' +
            '    b;');

        // weird element referencing
        bt('foo.("bar");');
        bt(
            'if (a) a()\n' +
            'else b()\n' +
            'newline()');
        bt(
            'if (a) a()\n' +
            'newline()');
        bt('a=typeof(x)', 'a = typeof(x)');
        bt(
            'var a = function() {\n' +
            '        return null;\n' +
            '    },\n' +
            '    b = false;');
        bt(
            'var a = function() {\n' +
            '    func1()\n' +
            '}');
        bt(
            'var a = function() {\n' +
            '    func1()\n' +
            '}\n' +
            'var b = function() {\n' +
            '    func2()\n' +
            '}');

        // code with and without semicolons
        bt(
            'var whatever = require("whatever");\n' +
            'function() {\n' +
            '    a = 6;\n' +
            '}',
            //  -- output --
            'var whatever = require("whatever");\n' +
            '\n' +
            'function() {\n' +
            '    a = 6;\n' +
            '}');
        bt(
            'var whatever = require("whatever")\n' +
            'function() {\n' +
            '    a = 6\n' +
            '}',
            //  -- output --
            'var whatever = require("whatever")\n' +
            '\n' +
            'function() {\n' +
            '    a = 6\n' +
            '}');
        bt(
            '{"x":[{"a":1,"b":3},\n' +
            '7,8,8,8,8,{"b":99},{"a":11}]}',
            //  -- output --
            '{\n' +
            '    "x": [{\n' +
            '            "a": 1,\n' +
            '            "b": 3\n' +
            '        },\n' +
            '        7, 8, 8, 8, 8, {\n' +
            '            "b": 99\n' +
            '        }, {\n' +
            '            "a": 11\n' +
            '        }\n' +
            '    ]\n' +
            '}');
        bt(
            '{"x":[{"a":1,"b":3},7,8,8,8,8,{"b":99},{"a":11}]}',
            //  -- output --
            '{\n' +
            '    "x": [{\n' +
            '        "a": 1,\n' +
            '        "b": 3\n' +
            '    }, 7, 8, 8, 8, 8, {\n' +
            '        "b": 99\n' +
            '    }, {\n' +
            '        "a": 11\n' +
            '    }]\n' +
            '}');
        bt(
            '{"1":{"1a":"1b"},"2"}',
            //  -- output --
            '{\n' +
            '    "1": {\n' +
            '        "1a": "1b"\n' +
            '    },\n' +
            '    "2"\n' +
            '}');
        bt(
            '{a:{a:b},c}',
            //  -- output --
            '{\n' +
            '    a: {\n' +
            '        a: b\n' +
            '    },\n' +
            '    c\n' +
            '}');
        bt(
            '{[y[a]];keep_indent;}',
            //  -- output --
            '{\n' +
            '    [y[a]];\n' +
            '    keep_indent;\n' +
            '}');
        bt(
            'if (x) {y} else { if (x) {y}}',
            //  -- output --
            'if (x) {\n' +
            '    y\n' +
            '} else {\n' +
            '    if (x) {\n' +
            '        y\n' +
            '    }\n' +
            '}');
        bt(
            'if (foo) one()\n' +
            'two()\n' +
            'three()');
        bt(
            'if (1 + foo() && bar(baz()) / 2) one()\n' +
            'two()\n' +
            'three()');
        bt(
            'if (1 + foo() && bar(baz()) / 2) one();\n' +
            'two();\n' +
            'three();');
        bt(
            'var a=1,b={bang:2},c=3;',
            //  -- output --
            'var a = 1,\n' +
            '    b = {\n' +
            '        bang: 2\n' +
            '    },\n' +
            '    c = 3;');
        bt(
            'var a={bing:1},b=2,c=3;',
            //  -- output --
            'var a = {\n' +
            '        bing: 1\n' +
            '    },\n' +
            '    b = 2,\n' +
            '    c = 3;');

        // Issue #1896: Handle newlines with bitwise ~ operator
        bt(
            'if (foo) {\n' +
            'var bar = 1;\n' +
            '~bar ? 0 : 1\n' +
            ' }',
            //  -- output --
            'if (foo) {\n' +
            '    var bar = 1;\n' +
            '    ~bar ? 0 : 1\n' +
            '}');


    }

    function beautifier_unconverted_tests()
    {
        sanitytest = test_obj;

        reset_options();
        //============================================================
        test_fragment(null, '');

        reset_options();
        //============================================================
        // Test user pebkac protection, converts dash names to underscored names
        opts["end-with-newline"] = true;
        test_fragment(null, '\n');

        reset_options();
        //============================================================
        opts.indent_size = 1;
        opts.indent_char = ' ';
        bt('{ one_char() }', "{\n one_char()\n}");

        bt('var a,b=1,c=2', 'var a, b = 1,\n c = 2');

        opts.indent_size = 4;
        opts.indent_char = ' ';
        bt('{ one_char() }', "{\n    one_char()\n}");

        opts.indent_size = 1;
        opts.indent_char = "\t";
        bt('{ one_char() }', "{\n\tone_char()\n}");
        bt('x = a ? b : c; x;', 'x = a ? b : c;\nx;');

        //set to something else than it should change to, but with tabs on, should override
        opts.indent_size = 5;
        opts.indent_char = ' ';
        opts.indent_with_tabs = true;

        bt('{ one_char() }', "{\n\tone_char()\n}");
        bt('x = a ? b : c; x;', 'x = a ? b : c;\nx;');

        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.indent_with_tabs = false;

        reset_options();
        //============================================================
        // tests for brace positioning
        beautify_brace_tests('expand');
        beautify_brace_tests('collapse');
        beautify_brace_tests('end-expand');
        beautify_brace_tests('none');


        bt('"foo""bar""baz"', '"foo"\n"bar"\n"baz"');
        bt("'foo''bar''baz'", "'foo'\n'bar'\n'baz'");

        bt("{\n    get foo() {}\n}");
        bt("{\n    var a = get\n    foo();\n}");
        bt("{\n    set foo() {}\n}");
        bt("{\n    var a = set\n    foo();\n}");
        bt("var x = {\n    get function()\n}");
        bt("var x = {\n    set function()\n}");

        // According to my current research get/set have no special meaning outside of an object literal
        bt("var x = set\n\na() {}", "var x = set\n\na() {}");
        bt("var x = set\n\nfunction() {}", "var x = set\n\nfunction() {}");

        bt('for () /abc/.test()');
        bt('if (k) /aaa/m.test(v) && l();');
        bt('switch (true) {\n    case /swf/i.test(foo):\n        bar();\n}');
        bt('createdAt = {\n    type: Date,\n    default: Date.now\n}');
        bt('switch (createdAt) {\n    case a:\n        Date,\n    default:\n        Date.now\n}');

        reset_options();
        //============================================================
        opts.preserve_newlines = true;
        bt("var a = 'foo' +\n    'bar';");


        reset_options();
        //============================================================
        opts.unescape_strings = false;
        bt('"\\\\s"'); // == "\\s" in the js source
        bt("'\\\\s'"); // == '\\s' in the js source
        bt("'\\\\\\s'"); // == '\\\s' in the js source
        bt("'\\s'"); // == '\s' in the js source
        bt('"•"');
        bt('"—"');
        bt('"\\x41\\x42\\x43\\x01"', '"\\x41\\x42\\x43\\x01"');
        bt('"\\u2022"', '"\\u2022"');
        bt('a = /\s+/');
        // bt('a = /\\x41/','a = /A/');
        bt('"\\u2022";a = /\s+/;"\\x41\\x42\\x43\\x01".match(/\\x41/);','"\\u2022";\na = /\s+/;\n"\\x41\\x42\\x43\\x01".match(/\\x41/);');
        test_fragment('"\\x22\\x27",\'\\x22\\x27\',"\\x5c",\'\\x5c\',"\\xff and \\xzz","unicode \\u0000 \\u0022 \\u0027 \\u005c \\uffff \\uzzzz"', '"\\x22\\x27", \'\\x22\\x27\', "\\x5c", \'\\x5c\', "\\xff and \\xzz", "unicode \\u0000 \\u0022 \\u0027 \\u005c \\uffff \\uzzzz"');

        opts.unescape_strings = true;
        test_fragment('"\\x20\\x40\\x4a"', '" @J"');
        test_fragment('"\\xff\\x40\\x4a"');
        test_fragment('"\\u0072\\u016B\\u0137\\u012B\\u0074\\u0069\\u0073"', '"\u0072\u016B\u0137\u012B\u0074\u0069\u0073"');
        test_fragment('"Google Chrome est\\u00E1 actualizado."', '"Google Chrome está actualizado."');
        test_fragment('"\\x22\\x27",\'\\x22\\x27\',"\\x5c",\'\\x5c\',"\\xff and \\xzz","unicode \\u0000 \\u0022 \\u0027 \\u005c \\uffff"',
           '"\\"\\\'", \'\\"\\\'\', "\\\\", \'\\\\\', "\\xff and \\xzz", "unicode \\u0000 \\" \\\' \\\\ ' + unicode_char(0xffff) + '"');

        // For error case, return the string unchanged
        test_fragment('"\\x22\\x27",\'\\x22\\x27\',"\\x5c",\'\\x5c\',"\\xff and \\xzz","unicode \\u0000 \\u0022 \\u0027 \\u005c \\uffff \\uzzzz"',
            '"\\"\\\'", \'\\"\\\'\', "\\\\", \'\\\\\', "\\xff and \\xzz", "unicode \\u0000 \\u0022 \\u0027 \\u005c \\uffff \\uzzzz"');

        reset_options();
        //============================================================
        Urlencoded.run_tests(sanitytest);
    }

    beautifier_tests();
    beautifier_unconverted_tests();
}

if (typeof exports !== "undefined") {
    exports.run_javascript_tests = run_javascript_tests;
}
//== js/test/generated/beautify-javascript-tests.js end


//== js/test/generated/beautify-css-tests.js
/*
    AUTO-GENERATED. DO NOT MODIFY.
    Script: test/generate-tests.js
    Template: test/data/css/node.mustache
    Data: test/data/css/tests.js

  The MIT License (MIT)

  Copyright (c) 2007-2018 Einar Lielmanis, Liam Newman, and contributors.

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
/*jshint unused:false */
/*jshint strict:false */

function run_css_tests(test_obj, Urlencoded, js_beautify, html_beautify, css_beautify)
{

    var default_opts = {
        indent_size: 4,
        indent_char: ' ',
        preserve_newlines: true,
        jslint_happy: false,
        keep_array_indentation: false,
        brace_style: 'collapse',
        space_before_conditional: true,
        break_chained_methods: false,
        selector_separator: '\n',
        end_with_newline: false
    };
    var opts;

    default_opts.indent_size = 4;
    default_opts.indent_char = ' ';
    default_opts.selector_separator_newline = true;
    default_opts.brace_style = 'collapse';
    default_opts.end_with_newline = false;
    default_opts.newline_between_rules = false;
    default_opts.space_around_combinator = false;
    default_opts.preserve_newlines = false;
    default_opts.space_around_selector_separator = false;

    function reset_options()
    {
        opts = {}; for(var p in default_opts) opts[p] = default_opts[p];
        test_name = 'css-beautify';
    }

    function test_beautifier(input)
    {
        return css_beautify(input, opts);
    }

    var sanitytest;
    var test_name = '';


    function set_name(name)
    {
        name = (name || '').trim();
        if (name) {
            test_name = name.replace(/\r/g, '\\r').replace(/\n/g, '\\n');
        }
    }

    // test the input on beautifier with the current flag settings
    // does not check the indentation / surroundings as bt() does
    function test_fragment(input, expected)
    {
        var success = true;
        sanitytest.test_function(test_beautifier, test_name);
        expected = expected || expected === '' ? expected : input;
        success = success && sanitytest.expect(input, expected);
        // if the expected is different from input, run it again
        // expected output should be unchanged when run twice.
        if (success && expected !== input) {
            success = success && sanitytest.expect(expected, expected);
        }

        // Everywhere we do newlines, they should be replaced with opts.eol
        sanitytest.test_function(test_beautifier, 'eol ' + test_name);
        opts.eol = '\r\\n';
        expected = expected.replace(/[\n]/g, '\r\n');
        opts.disabled = true;
        success = success && sanitytest.expect(input, input || '');
        success = success && sanitytest.expect('\n\n' + expected, '\n\n' + expected);
        opts.disabled = false;
        success = success && sanitytest.expect(input, expected);
        if (success && input && input.indexOf('\n') !== -1) {
            input = input.replace(/[\n]/g, '\r\n');
            sanitytest.expect(input, expected);
            // Ensure support for auto eol detection
            opts.eol = 'auto';
            success = success && sanitytest.expect(input, expected);
        }
        opts.eol = '\n';
        return success;
    }

    // test css
    function t(input, expectation)
    {
        var success = true;
        var wrapped_input, wrapped_expectation;

        expectation = expectation || expectation === '' ? expectation : input;
        success = success && test_fragment(input, expectation);

        return success;
    }

    function unicode_char(value) {
        return String.fromCharCode(value);
    }

    function beautifier_tests()
    {
        sanitytest = test_obj;

        reset_options();
        //============================================================
        t(".tabs {}");


        //============================================================
        // End With Newline - (end_with_newline = "true")
        reset_options();
        set_name('End With Newline - (end_with_newline = "true")');
        opts.end_with_newline = true;
        test_fragment('', '\n');
        test_fragment('   .tabs{}', '   .tabs {}\n');
        test_fragment(
            '   \n' +
            '\n' +
            '.tabs{}\n' +
            '\n' +
            '\n' +
            '\n',
            //  -- output --
            '   .tabs {}\n');
        test_fragment('\n');

        // End With Newline - (end_with_newline = "false")
        reset_options();
        set_name('End With Newline - (end_with_newline = "false")');
        opts.end_with_newline = false;
        test_fragment('');
        test_fragment('   .tabs{}', '   .tabs {}');
        test_fragment(
            '   \n' +
            '\n' +
            '.tabs{}\n' +
            '\n' +
            '\n' +
            '\n',
            //  -- output --
            '   .tabs {}');
        test_fragment('\n', '');


        //============================================================
        // Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false")');
        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.indent_with_tabs = false;
        test_fragment('   a');
        test_fragment(
            '   .a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '   .a {\n' +
            '       text-align: right;\n' +
            '   }');
        test_fragment(
            '   // This is a random comment\n' +
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '   // This is a random comment\n' +
            '   .a {\n' +
            '       text-align: right;\n' +
            '   }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "0")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "0")');
        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.indent_with_tabs = false;
        opts.indent_level = 0;
        test_fragment('   a');
        test_fragment(
            '   .a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '   .a {\n' +
            '       text-align: right;\n' +
            '   }');
        test_fragment(
            '   // This is a random comment\n' +
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '   // This is a random comment\n' +
            '   .a {\n' +
            '       text-align: right;\n' +
            '   }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "1")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "1")');
        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.indent_with_tabs = false;
        opts.indent_level = 1;
        test_fragment('   a', '    a');
        test_fragment(
            '   .a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '    .a {\n' +
            '        text-align: right;\n' +
            '    }');
        test_fragment(
            '   // This is a random comment\n' +
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '    // This is a random comment\n' +
            '    .a {\n' +
            '        text-align: right;\n' +
            '    }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "2")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "2")');
        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.indent_with_tabs = false;
        opts.indent_level = 2;
        test_fragment('a', '        a');
        test_fragment(
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '        .a {\n' +
            '            text-align: right;\n' +
            '        }');
        test_fragment(
            '// This is a random comment\n' +
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '        // This is a random comment\n' +
            '        .a {\n' +
            '            text-align: right;\n' +
            '        }');

        // Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "true", indent_level = "2")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "true", indent_level = "2")');
        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.indent_with_tabs = true;
        opts.indent_level = 2;
        test_fragment('a', '\t\ta');
        test_fragment(
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '\t\t.a {\n' +
            '\t\t\ttext-align: right;\n' +
            '\t\t}');
        test_fragment(
            '// This is a random comment\n' +
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '\t\t// This is a random comment\n' +
            '\t\t.a {\n' +
            '\t\t\ttext-align: right;\n' +
            '\t\t}');

        // Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "0")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_size = "4", indent_char = "" "", indent_with_tabs = "false", indent_level = "0")');
        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.indent_with_tabs = false;
        opts.indent_level = 0;
        test_fragment('\t   a');
        test_fragment(
            '\t   .a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '\t   .a {\n' +
            '\t       text-align: right;\n' +
            '\t   }');
        test_fragment(
            '\t   // This is a random comment\n' +
            '.a {\n' +
            '  text-align: right;\n' +
            '}',
            //  -- output --
            '\t   // This is a random comment\n' +
            '\t   .a {\n' +
            '\t       text-align: right;\n' +
            '\t   }');


        //============================================================
        // Empty braces
        reset_options();
        set_name('Empty braces');
        t('.tabs{}', '.tabs {}');
        t('.tabs { }', '.tabs {}');
        t('.tabs    {    }', '.tabs {}');
        t(
            '.tabs    \n' +
            '{\n' +
            '    \n' +
            '  }',
            //  -- output --
            '.tabs {}');


        //============================================================
        //
        reset_options();
        set_name('');
        t(
            '#cboxOverlay {\n' +
            '    background: url(images/overlay.png) repeat 0 0;\n' +
            '    opacity: 0.9;\n' +
            '    filter: alpha(opacity = 90);\n' +
            '}',
            //  -- output --
            '#cboxOverlay {\n' +
            '    background: url(images/overlay.png) repeat 0 0;\n' +
            '    opacity: 0.9;\n' +
            '    filter: alpha(opacity=90);\n' +
            '}');

        // simple data uri base64 test
        t(
            'a { background: url(data:image/gif;base64,R0lGODlhCwALAJEAAAAAAP///xUVFf///yH5BAEAAAMALAAAAAALAAsAAAIPnI+py+0/hJzz0IruwjsVADs=); }',
            //  -- output --
            'a {\n' +
            '    background: url(data:image/gif;base64,R0lGODlhCwALAJEAAAAAAP///xUVFf///yH5BAEAAAMALAAAAAALAAsAAAIPnI+py+0/hJzz0IruwjsVADs=);\n' +
            '}');

        // non-base64 data
        t(
            'a { background: url(data:text/html,%3Ch1%3EHello%2C%20World!%3C%2Fh1%3E); }',
            //  -- output --
            'a {\n' +
            '    background: url(data:text/html,%3Ch1%3EHello%2C%20World!%3C%2Fh1%3E);\n' +
            '}');

        // Beautifier does not fix or mitigate bad data uri
        t(
            'a { background: url(data:  image/gif   base64,R0lGODlhCwALAJEAAAAAAP///xUVFf///yH5BAEAAAMALAAAAAALAAsAAAIPnI+py+0/hJzz0IruwjsVADs=); }',
            //  -- output --
            'a {\n' +
            '    background: url(data:  image/gif   base64,R0lGODlhCwALAJEAAAAAAP///xUVFf///yH5BAEAAAMALAAAAAALAAsAAAIPnI+py+0/hJzz0IruwjsVADs=);\n' +
            '}');


        //============================================================
        // Support simple language specific option inheritance/overriding - (indent_char = "" "", indent_size = "4", js = "{ "indent_size": 3 }", css = "{ "indent_size": 5 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_char = "" "", indent_size = "4", js = "{ "indent_size": 3 }", css = "{ "indent_size": 5 }")');
        opts.indent_char = ' ';
        opts.indent_size = 4;
        opts.js = { 'indent_size': 3 };
        opts.css = { 'indent_size': 5 };
        t(
            '.selector {\n' +
            '     font-size: 12px;\n' +
            '}');

        // Support simple language specific option inheritance/overriding - (indent_char = "" "", indent_size = "4", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 } }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_char = "" "", indent_size = "4", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 } }")');
        opts.indent_char = ' ';
        opts.indent_size = 4;
        opts.html = { 'js': { 'indent_size': 3 }, 'css': { 'indent_size': 5 } };
        t(
            '.selector {\n' +
            '    font-size: 12px;\n' +
            '}');

        // Support simple language specific option inheritance/overriding - (indent_char = "" "", indent_size = "9", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 8 }, "indent_size": 2}", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_char = "" "", indent_size = "9", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 8 }, "indent_size": 2}", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")');
        opts.indent_char = ' ';
        opts.indent_size = 9;
        opts.html = { 'js': { 'indent_size': 3 }, 'css': { 'indent_size': 8 }, 'indent_size': 2};
        opts.js = { 'indent_size': 5 };
        opts.css = { 'indent_size': 3 };
        t(
            '.selector {\n' +
            '   font-size: 12px;\n' +
            '}');


        //============================================================
        // Space Around Combinator - (space_around_combinator = "true")
        reset_options();
        set_name('Space Around Combinator - (space_around_combinator = "true")');
        opts.space_around_combinator = true;
        t('a>b{}', 'a > b {}');
        t('a~b{}', 'a ~ b {}');
        t('a+b{}', 'a + b {}');
        t('a+b>c{}', 'a + b > c {}');
        t('a > b{}', 'a > b {}');
        t('a ~ b{}', 'a ~ b {}');
        t('a + b{}', 'a + b {}');
        t('a + b > c{}', 'a + b > c {}');
        t(
            'a > b{width: calc(100% + 45px);}',
            //  -- output --
            'a > b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a ~ b{width: calc(100% + 45px);}',
            //  -- output --
            'a ~ b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a + b{width: calc(100% + 45px);}',
            //  -- output --
            'a + b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a + b > c{width: calc(100% + 45px);}',
            //  -- output --
            'a + b > c {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');

        // Space Around Combinator - (space_around_combinator = "false")
        reset_options();
        set_name('Space Around Combinator - (space_around_combinator = "false")');
        opts.space_around_combinator = false;
        t('a>b{}', 'a>b {}');
        t('a~b{}', 'a~b {}');
        t('a+b{}', 'a+b {}');
        t('a+b>c{}', 'a+b>c {}');
        t('a > b{}', 'a>b {}');
        t('a ~ b{}', 'a~b {}');
        t('a + b{}', 'a+b {}');
        t('a + b > c{}', 'a+b>c {}');
        t(
            'a > b{width: calc(100% + 45px);}',
            //  -- output --
            'a>b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a ~ b{width: calc(100% + 45px);}',
            //  -- output --
            'a~b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a + b{width: calc(100% + 45px);}',
            //  -- output --
            'a+b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a + b > c{width: calc(100% + 45px);}',
            //  -- output --
            'a+b>c {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');

        // Space Around Combinator - (space_around_selector_separator = "true")
        reset_options();
        set_name('Space Around Combinator - (space_around_selector_separator = "true")');
        opts.space_around_selector_separator = true;
        t('a>b{}', 'a > b {}');
        t('a~b{}', 'a ~ b {}');
        t('a+b{}', 'a + b {}');
        t('a+b>c{}', 'a + b > c {}');
        t('a > b{}', 'a > b {}');
        t('a ~ b{}', 'a ~ b {}');
        t('a + b{}', 'a + b {}');
        t('a + b > c{}', 'a + b > c {}');
        t(
            'a > b{width: calc(100% + 45px);}',
            //  -- output --
            'a > b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a ~ b{width: calc(100% + 45px);}',
            //  -- output --
            'a ~ b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a + b{width: calc(100% + 45px);}',
            //  -- output --
            'a + b {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');
        t(
            'a + b > c{width: calc(100% + 45px);}',
            //  -- output --
            'a + b > c {\n' +
            '    width: calc(100% + 45px);\n' +
            '}');


        //============================================================
        // Issue 1373 -- Correct spacing around [attribute~=value]
        reset_options();
        set_name('Issue 1373 -- Correct spacing around [attribute~=value]');
        t('header>div[class~="div-all"]');


        //============================================================
        // Selector Separator - (selector_separator_newline = "false", selector_separator = "" "", newline_between_rules = "true")
        reset_options();
        set_name('Selector Separator - (selector_separator_newline = "false", selector_separator = "" "", newline_between_rules = "true")');
        opts.selector_separator_newline = false;
        opts.selector_separator = " ";
        opts.newline_between_rules = true;
        t(
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '#bla, #foo{color:green}\n' +
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}\n' +
            '\n' +
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '@media print {.tab{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {.tab,.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab, .bat {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {// comment\n' +
            '//comment 2\n' +
            '.bat{}}',
            //  -- output --
            '@media print {\n' +
            '\n' +
            '    // comment\n' +
            '    //comment 2\n' +
            '    .bat {}\n' +
            '}');
        t(
            '#bla, #foo{color:black}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: black\n' +
            '}');
        t(
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}\n' +
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}',
            //  -- output --
            'a:first-child, a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child, div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            'a:first-child, a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child, div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}');

        // Selector Separator - (selector_separator_newline = "false", selector_separator = "" "", newline_between_rules = "false")
        reset_options();
        set_name('Selector Separator - (selector_separator_newline = "false", selector_separator = "" "", newline_between_rules = "false")');
        opts.selector_separator_newline = false;
        opts.selector_separator = " ";
        opts.newline_between_rules = false;
        t(
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '#bla, #foo{color:green}\n' +
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}\n' +
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '@media print {.tab{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {.tab,.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab, .bat {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {// comment\n' +
            '//comment 2\n' +
            '.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    // comment\n' +
            '    //comment 2\n' +
            '    .bat {}\n' +
            '}');
        t(
            '#bla, #foo{color:black}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: black\n' +
            '}');
        t(
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}\n' +
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}',
            //  -- output --
            'a:first-child, a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child, div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            'a:first-child, a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child, div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}');

        // Selector Separator - (selector_separator_newline = "false", selector_separator = ""  "", newline_between_rules = "false")
        reset_options();
        set_name('Selector Separator - (selector_separator_newline = "false", selector_separator = ""  "", newline_between_rules = "false")');
        opts.selector_separator_newline = false;
        opts.selector_separator = "  ";
        opts.newline_between_rules = false;
        t(
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '#bla, #foo{color:green}\n' +
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}\n' +
            '#bla, #foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '@media print {.tab{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {.tab,.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab, .bat {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {// comment\n' +
            '//comment 2\n' +
            '.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    // comment\n' +
            '    //comment 2\n' +
            '    .bat {}\n' +
            '}');
        t(
            '#bla, #foo{color:black}',
            //  -- output --
            '#bla, #foo {\n' +
            '    color: black\n' +
            '}');
        t(
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}\n' +
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}',
            //  -- output --
            'a:first-child, a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child, div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            'a:first-child, a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child, div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}');

        // Selector Separator - (selector_separator_newline = "true", selector_separator = "" "", newline_between_rules = "true")
        reset_options();
        set_name('Selector Separator - (selector_separator_newline = "true", selector_separator = "" "", newline_between_rules = "true")');
        opts.selector_separator_newline = true;
        opts.selector_separator = " ";
        opts.newline_between_rules = true;
        t(
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '#bla, #foo{color:green}\n' +
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}\n' +
            '\n' +
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '@media print {.tab{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {.tab,.bat{}}',
            //  -- output --
            '@media print {\n' +
            '\n' +
            '    .tab,\n    .bat {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {// comment\n' +
            '//comment 2\n' +
            '.bat{}}',
            //  -- output --
            '@media print {\n' +
            '\n' +
            '    // comment\n' +
            '    //comment 2\n' +
            '    .bat {}\n' +
            '}');
        t(
            '#bla, #foo{color:black}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: black\n' +
            '}');
        t(
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}\n' +
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}',
            //  -- output --
            'a:first-child,\na:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child,\n    div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            'a:first-child,\na:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child,\n    div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}');

        // Selector Separator - (selector_separator_newline = "true", selector_separator = "" "", newline_between_rules = "false")
        reset_options();
        set_name('Selector Separator - (selector_separator_newline = "true", selector_separator = "" "", newline_between_rules = "false")');
        opts.selector_separator_newline = true;
        opts.selector_separator = " ";
        opts.newline_between_rules = false;
        t(
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '#bla, #foo{color:green}\n' +
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}\n' +
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '@media print {.tab{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {.tab,.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab,\n    .bat {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {// comment\n' +
            '//comment 2\n' +
            '.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    // comment\n' +
            '    //comment 2\n' +
            '    .bat {}\n' +
            '}');
        t(
            '#bla, #foo{color:black}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: black\n' +
            '}');
        t(
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}\n' +
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}',
            //  -- output --
            'a:first-child,\na:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child,\n    div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            'a:first-child,\na:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child,\n    div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}');

        // Selector Separator - (selector_separator_newline = "true", selector_separator = ""  "", newline_between_rules = "false")
        reset_options();
        set_name('Selector Separator - (selector_separator_newline = "true", selector_separator = ""  "", newline_between_rules = "false")');
        opts.selector_separator_newline = true;
        opts.selector_separator = "  ";
        opts.newline_between_rules = false;
        t(
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '#bla, #foo{color:green}\n' +
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}\n' +
            '#bla,\n#foo {\n' +
            '    color: green\n' +
            '}');
        t(
            '@media print {.tab{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {.tab,.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    .tab,\n    .bat {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {// comment\n' +
            '//comment 2\n' +
            '.bat{}}',
            //  -- output --
            '@media print {\n' +
            '    // comment\n' +
            '    //comment 2\n' +
            '    .bat {}\n' +
            '}');
        t(
            '#bla, #foo{color:black}',
            //  -- output --
            '#bla,\n#foo {\n' +
            '    color: black\n' +
            '}');
        t(
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}\n' +
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}',
            //  -- output --
            'a:first-child,\na:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child,\n    div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            'a:first-child,\na:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child,\n    div:hover {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}');

        // Selector Separator - (selector_separator_newline = "true", selector_separator = ""  "", brace_style = ""expand"", newline_between_rules = "false")
        reset_options();
        set_name('Selector Separator - (selector_separator_newline = "true", selector_separator = ""  "", brace_style = ""expand"", newline_between_rules = "false")');
        opts.selector_separator_newline = true;
        opts.selector_separator = "  ";
        opts.brace_style = "expand";
        opts.newline_between_rules = false;
        t(
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo\n{\n' +
            '    color: green\n' +
            '}');
        t(
            '#bla, #foo{color:green}\n' +
            '#bla, #foo{color:green}',
            //  -- output --
            '#bla,\n#foo\n{\n' +
            '    color: green\n' +
            '}\n' +
            '#bla,\n#foo\n{\n' +
            '    color: green\n' +
            '}');
        t(
            '@media print {.tab{}}',
            //  -- output --
            '@media print\n{\n' +
            '    .tab\n    {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {.tab,.bat{}}',
            //  -- output --
            '@media print\n{\n' +
            '    .tab,\n    .bat\n    {}\n' +
            '}');

        // This is bug #1489
        t(
            '@media print {// comment\n' +
            '//comment 2\n' +
            '.bat{}}',
            //  -- output --
            '@media print\n{\n' +
            '    // comment\n' +
            '    //comment 2\n' +
            '    .bat\n    {}\n' +
            '}');
        t(
            '#bla, #foo{color:black}',
            //  -- output --
            '#bla,\n#foo\n{\n' +
            '    color: black\n' +
            '}');
        t(
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}\n' +
            'a:first-child,a:first-child{color:red;div:first-child,div:hover{color:black;}}',
            //  -- output --
            'a:first-child,\na:first-child\n{\n' +
            '    color: red;\n' +
            '    div:first-child,\n    div:hover\n    {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            'a:first-child,\na:first-child\n{\n' +
            '    color: red;\n' +
            '    div:first-child,\n    div:hover\n    {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}');


        //============================================================
        // Preserve Newlines - (preserve_newlines = "true")
        reset_options();
        set_name('Preserve Newlines - (preserve_newlines = "true")');
        opts.preserve_newlines = true;
        t('.div {}\n\n.span {}');
        t(
            '#bla, #foo{\n' +
            '    color:black;\n\n    font-size: 12px;\n' +
            '}',
            //  -- output --
            '#bla,\n' +
            '#foo {\n' +
            '    color: black;\n\n    font-size: 12px;\n' +
            '}');

        // Preserve Newlines - (preserve_newlines = "false")
        reset_options();
        set_name('Preserve Newlines - (preserve_newlines = "false")');
        opts.preserve_newlines = false;
        t('.div {}\n\n.span {}', '.div {}\n.span {}');
        t(
            '#bla, #foo{\n' +
            '    color:black;\n\n    font-size: 12px;\n' +
            '}',
            //  -- output --
            '#bla,\n' +
            '#foo {\n' +
            '    color: black;\n    font-size: 12px;\n' +
            '}');


        //============================================================
        // Preserve Newlines and newline_between_rules
        reset_options();
        set_name('Preserve Newlines and newline_between_rules');
        opts.preserve_newlines = true;
        opts.newline_between_rules = true;
        t(
            '.div {}.span {}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '#bla, #foo{\n' +
            '    color:black;\n' +
            '    font-size: 12px;\n' +
            '}',
            //  -- output --
            '#bla,\n' +
            '#foo {\n' +
            '    color: black;\n' +
            '    font-size: 12px;\n' +
            '}');
        t(
            '#bla, #foo{\n' +
            '    color:black;\n' +
            '\n' +
            '\n' +
            '    font-size: 12px;\n' +
            '}',
            //  -- output --
            '#bla,\n' +
            '#foo {\n' +
            '    color: black;\n' +
            '\n' +
            '\n' +
            '    font-size: 12px;\n' +
            '}');
        t(
            '#bla,\n' +
            '\n' +
            '#foo {\n' +
            '    color: black;\n' +
            '    font-size: 12px;\n' +
            '}');
        t(
            'a {\n' +
            '    b: c;\n' +
            '\n' +
            '\n' +
            '    d: {\n' +
            '        e: f;\n' +
            '    }\n' +
            '}');
        t(
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            'html {}\n' +
            '\n' +
            '/*this is a comment*/');
        t(
            '.div {\n' +
            '    a: 1;\n' +
            '\n' +
            '\n' +
            '    b: 2;\n' +
            '}\n' +
            '\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '    a: 1;\n' +
            '}');
        t(
            '.div {\n' +
            '\n' +
            '\n' +
            '    a: 1;\n' +
            '\n' +
            '\n' +
            '    b: 2;\n' +
            '}\n' +
            '\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '    a: 1;\n' +
            '}');
        t(
            '@media screen {\n' +
            '    .div {\n' +
            '        a: 1;\n' +
            '\n' +
            '\n' +
            '        b: 2;\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '\n' +
            '    .span {\n' +
            '        a: 1;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '.span {}');


        //============================================================
        // Preserve Newlines and add tabs
        reset_options();
        set_name('Preserve Newlines and add tabs');
        opts.preserve_newlines = true;
        t(
            '.tool-tip {\n' +
            '    position: relative;\n' +
            '\n' +
            '        \n' +
            '    .tool-tip-content {\n' +
            '        &>* {\n' +
            '            margin-top: 0;\n' +
            '        }\n' +
            '        \n' +
            '\n' +
            '        .mixin-box-shadow(.2rem .2rem .5rem rgba(0, 0, 0, .15));\n' +
            '        padding: 1rem;\n' +
            '        position: absolute;\n' +
            '        z-index: 10;\n' +
            '    }\n' +
            '}',
            //  -- output --
            '.tool-tip {\n' +
            '    position: relative;\n' +
            '\n' +
            '\n' +
            '    .tool-tip-content {\n' +
            '        &>* {\n' +
            '            margin-top: 0;\n' +
            '        }\n' +
            '\n\n        .mixin-box-shadow(.2rem .2rem .5rem rgba(0, 0, 0, .15));\n' +
            '        padding: 1rem;\n' +
            '        position: absolute;\n' +
            '        z-index: 10;\n' +
            '    }\n' +
            '}');


        //============================================================
        // Issue #1338 -- Preserve Newlines within CSS rules
        reset_options();
        set_name('Issue #1338 -- Preserve Newlines within CSS rules');
        opts.preserve_newlines = true;
        t(
            'body {\n' +
            '    grid-template-areas:\n' +
            '        "header header"\n' +
            '        "main   sidebar"\n' +
            '        "footer footer";\n' +
            '}');


        //============================================================
        // Newline Between Rules - (newline_between_rules = "true")
        reset_options();
        set_name('Newline Between Rules - (newline_between_rules = "true")');
        opts.newline_between_rules = true;
        t(
            '.div {}\n' +
            '.span {}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '.div{}\n' +
            '   \n' +
            '.span{}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '.div {}    \n' +
            '  \n' +
            '.span { } \n',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '.div {\n' +
            '    \n' +
            '} \n' +
            '  .span {\n' +
            ' }  ',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '    margin: 0; /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue"\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue"\n' +
            '        }\n' +
            '\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{color:red;div:first-child{color:black;}}\n' +
            '.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{color:red;div:not(.peq){color:black;}}\n' +
            '.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '    }\n' +
            '\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    .list-group-item {}\n' +
            '\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            '\n' +
            '.list-group-condensed {}');
        t(
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '\n' +
            '    .list-group-icon {}\n' +
            '\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            '\n' +
            '.list-group-condensed {}');
        t(
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    //this is my pre-comment\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    //this is a comment\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '    //this is also a comment\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '\n' +
            '    //this is my pre-comment\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '\n' +
            '    //this is a comment\n' +
            '    .list-group-icon {}\n' +
            '\n' +
            '    //this is also a comment\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            '\n' +
            '.list-group-condensed {}');
        t(
            '.list-group {\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            'color: #38a0e5;\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            'color: #38a0e5;\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    color: #38a0e5;\n' +
            '\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '\n' +
            '    color: #38a0e5;\n' +
            '\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '\n' +
            '    color: #38a0e5;\n' +
            '\n' +
            '    .list-group-icon {}\n' +
            '\n' +
            '    color: #38a0e5;\n' +
            '\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            '\n' +
            'color: #38a0e5;\n' +
            '\n' +
            '.list-group-condensed {}');
        t(
            '@media only screen and (max-width: 40em) {\n' +
            'header {\n' +
            '    margin: 0 auto;\n' +
            '    padding: 10px;\n' +
            '    background: red;\n' +
            '    }\n' +
            'main {\n' +
            '    margin: 20px auto;\n' +
            '    padding: 4px;\n' +
            '    background: blue;\n' +
            '    }\n' +
            '}',
            //  -- output --
            '@media only screen and (max-width: 40em) {\n' +
            '    header {\n' +
            '        margin: 0 auto;\n' +
            '        padding: 10px;\n' +
            '        background: red;\n' +
            '    }\n' +
            '\n' +
            '    main {\n' +
            '        margin: 20px auto;\n' +
            '        padding: 4px;\n' +
            '        background: blue;\n' +
            '    }\n' +
            '}');
        t(
            '.preloader {\n' +
            '    height: 20px;\n' +
            '    .line {\n' +
            '        width: 1px;\n' +
            '        height: 12px;\n' +
            '        background: #38a0e5;\n' +
            '        margin: 0 1px;\n' +
            '        display: inline-block;\n' +
            '        &.line-1 {\n' +
            '            animation-delay: 800ms;\n' +
            '        }\n' +
            '        &.line-2 {\n' +
            '            animation-delay: 600ms;\n' +
            '        }\n' +
            '    }\n' +
            '    div {\n' +
            '        color: #38a0e5;\n' +
            '        font-family: "Arial", sans-serif;\n' +
            '        font-size: 10px;\n' +
            '        margin: 5px 0;\n' +
            '    }\n' +
            '}',
            //  -- output --
            '.preloader {\n' +
            '    height: 20px;\n' +
            '\n' +
            '    .line {\n' +
            '        width: 1px;\n' +
            '        height: 12px;\n' +
            '        background: #38a0e5;\n' +
            '        margin: 0 1px;\n' +
            '        display: inline-block;\n' +
            '\n' +
            '        &.line-1 {\n' +
            '            animation-delay: 800ms;\n' +
            '        }\n' +
            '\n' +
            '        &.line-2 {\n' +
            '            animation-delay: 600ms;\n' +
            '        }\n' +
            '    }\n' +
            '\n' +
            '    div {\n' +
            '        color: #38a0e5;\n' +
            '        font-family: "Arial", sans-serif;\n' +
            '        font-size: 10px;\n' +
            '        margin: 5px 0;\n' +
            '    }\n' +
            '}');

        // Newline Between Rules - (newline_between_rules = "false")
        reset_options();
        set_name('Newline Between Rules - (newline_between_rules = "false")');
        opts.newline_between_rules = false;
        t(
            '.div {}\n' +
            '.span {}');
        t(
            '.div{}\n' +
            '   \n' +
            '.span{}',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '.div {}    \n' +
            '  \n' +
            '.span { } \n',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '.div {\n' +
            '    \n' +
            '} \n' +
            '  .span {\n' +
            ' }  ',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '    margin: 0; /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{height:15px;}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue"\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{color:red;div:first-child{color:black;}}\n' +
            '.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{color:red;div:not(.peq){color:black;}}\n' +
            '.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '    }\n' +
            '\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    .list-group-item {}\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            '.list-group-condensed {}');
        t(
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '    .list-group-icon {}\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            '.list-group-condensed {}');
        t(
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    //this is my pre-comment\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    //this is a comment\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '    //this is also a comment\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '    //this is my pre-comment\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '    //this is a comment\n' +
            '    .list-group-icon {}\n' +
            '    //this is also a comment\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            '.list-group-condensed {}');
        t(
            '.list-group {\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-item {\n' +
            '        a:1\n' +
            '    }\n' +
            'color: #38a0e5;\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-icon {\n' +
            '    }\n' +
            '}\n' +
            'color: #38a0e5;\n' +
            '.list-group-condensed {\n' +
            '}',
            //  -- output --
            '.list-group {\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-item {\n' +
            '        a: 1\n' +
            '    }\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-icon {}\n' +
            '    color: #38a0e5;\n' +
            '    .list-group-icon {}\n' +
            '}\n' +
            'color: #38a0e5;\n' +
            '.list-group-condensed {}');
        t(
            '@media only screen and (max-width: 40em) {\n' +
            'header {\n' +
            '    margin: 0 auto;\n' +
            '    padding: 10px;\n' +
            '    background: red;\n' +
            '    }\n' +
            'main {\n' +
            '    margin: 20px auto;\n' +
            '    padding: 4px;\n' +
            '    background: blue;\n' +
            '    }\n' +
            '}',
            //  -- output --
            '@media only screen and (max-width: 40em) {\n' +
            '    header {\n' +
            '        margin: 0 auto;\n' +
            '        padding: 10px;\n' +
            '        background: red;\n' +
            '    }\n' +
            '    main {\n' +
            '        margin: 20px auto;\n' +
            '        padding: 4px;\n' +
            '        background: blue;\n' +
            '    }\n' +
            '}');
        t(
            '.preloader {\n' +
            '    height: 20px;\n' +
            '    .line {\n' +
            '        width: 1px;\n' +
            '        height: 12px;\n' +
            '        background: #38a0e5;\n' +
            '        margin: 0 1px;\n' +
            '        display: inline-block;\n' +
            '        &.line-1 {\n' +
            '            animation-delay: 800ms;\n' +
            '        }\n' +
            '        &.line-2 {\n' +
            '            animation-delay: 600ms;\n' +
            '        }\n' +
            '    }\n' +
            '    div {\n' +
            '        color: #38a0e5;\n' +
            '        font-family: "Arial", sans-serif;\n' +
            '        font-size: 10px;\n' +
            '        margin: 5px 0;\n' +
            '    }\n' +
            '}');


        //============================================================
        // Functions braces
        reset_options();
        set_name('Functions braces');
        t('.tabs(){}', '.tabs() {}');
        t('.tabs (){}', '.tabs () {}');
        t(
            '.tabs (pa, pa(1,2)), .cols { }',
            //  -- output --
            '.tabs (pa, pa(1, 2)),\n' +
            '.cols {}');
        t(
            '.tabs(pa, pa(1,2)), .cols { }',
            //  -- output --
            '.tabs(pa, pa(1, 2)),\n' +
            '.cols {}');
        t('.tabs (   )   {    }', '.tabs () {}');
        t('.tabs(   )   {    }', '.tabs() {}');
        t(
            '.tabs  (t, t2)  \n' +
            '{\n' +
            '  key: val(p1  ,p2);  \n' +
            '  }',
            //  -- output --
            '.tabs (t, t2) {\n' +
            '    key: val(p1, p2);\n' +
            '}');
        t(
            '.box-shadow(@shadow: 0 1px 3px rgba(0, 0, 0, .25)) {\n' +
            '    -webkit-box-shadow: @shadow;\n' +
            '    -moz-box-shadow: @shadow;\n' +
            '    box-shadow: @shadow;\n' +
            '}');


        //============================================================
        // Beautify preserve formatting
        reset_options();
        set_name('Beautify preserve formatting');
        opts.indent_size = 4;
        opts.indent_char = ' ';
        opts.preserve_newlines = true;

        // Directive: ignore
        t(
            '/* beautify ignore:start */\n' +
            '/* beautify ignore:end */');
        t(
            '/* beautify ignore:start */\n' +
            '   var a,,,{ 1;\n' +
            ' .div {}/* beautify ignore:end */');
        t(
            '.div {}\n' +
            '\n' +
            '/* beautify ignore:start */\n' +
            '   .div {}var a = 1;\n' +
            '/* beautify ignore:end */');

        // ignore starts _after_ the start comment, ends after the end comment
        t('/* beautify ignore:start */     {asdklgh;y;+++;dd2d}/* beautify ignore:end */');
        t('/* beautify ignore:start */  {asdklgh;y;+++;dd2d}    /* beautify ignore:end */');
        t(
            '.div {}/* beautify ignore:start */\n' +
            '   .div {}var a,,,{ 1;\n' +
            '/*beautify ignore:end*/',
            //  -- output --
            '.div {}\n' +
            '/* beautify ignore:start */\n' +
            '   .div {}var a,,,{ 1;\n' +
            '/*beautify ignore:end*/');
        t(
            '.div {}\n' +
            '  /* beautify ignore:start */\n' +
            '   .div {}var a,,,{ 1;\n' +
            '/* beautify ignore:end */',
            //  -- output --
            '.div {}\n' +
            '/* beautify ignore:start */\n' +
            '   .div {}var a,,,{ 1;\n' +
            '/* beautify ignore:end */');
        t(
            '.div {\n' +
            '    /* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    two   :  2,\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '    /* beautify ignore:end */\n' +
            '}');
        t(
            '.div {\n' +
            '/* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    two   :  2,\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '}',
            //  -- output --
            '.div {\n' +
            '    /* beautify ignore:start */\n' +
            '    one   :  1\n' +
            '    two   :  2,\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '}');
        t(
            '.div {\n' +
            '/* beautify ignore:start */\n' +
            '    one   :  1\n' +
            ' /* beautify ignore:end */\n' +
            '    two   :  2,\n' +
            '/* beautify ignore:start */\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '}',
            //  -- output --
            '.div {\n' +
            '    /* beautify ignore:start */\n' +
            '    one   :  1\n' +
            ' /* beautify ignore:end */\n' +
            '    two : 2,\n' +
            '    /* beautify ignore:start */\n' +
            '    three :  {\n' +
            '    ten   : 10\n' +
            '/* beautify ignore:end */\n' +
            '}');


        //============================================================
        // Comments - (preserve_newlines = "false", newline_between_rules = "false")
        reset_options();
        set_name('Comments - (preserve_newlines = "false", newline_between_rules = "false")');
        opts.preserve_newlines = false;
        opts.newline_between_rules = false;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '.rule {}');
        t(
            '.tabs{/* test */}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {/* non-header */width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {margin: 0;/* This is a comment including an url http://domain.com/path/to/file.ext */}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '.tabs{width:10px;}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{//comment\n' +
            '//2nd single line comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another nl\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{width: 10px;   // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{width: 10px;\n' +
            '        // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' *//* another comment */body{}',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '}.demob {text-align: right;}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {text-align:left;}//demob instructions for LESS note visibility only\n' +
            '.demob {text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '.span {}',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '/**//**///\n' +
            '/**/.div{}/**//**///\n' +
            '/**/.span {}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '.div{}//\n' +
            '.span {}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {margin: 0; /* This is a comment including an url http://domain.com/path/to/file.ext */}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}.div{height:15px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {background-image: url(foo@2x.png);    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {    #foo:hover {        background-image: url(foo@2x.png);    }    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {    font-family: "Bitstream Vera Serif Bold";    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");}\n' +
            '@media screen {    #foo:hover {        background-image: url(foo.png);    }    @media screen and (min-device-pixel-ratio: 2) {        @font-face {            font-family: "Helvetica Neue";        }        #foo:hover {            background-image: url(foo@2x.png);        }    }}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{color:red;div:first-child{color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{color:red;div:not(.peq){color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "false", newline_between_rules = "false")
        reset_options();
        set_name('Comments - (preserve_newlines = "false", newline_between_rules = "false")');
        opts.preserve_newlines = false;
        opts.newline_between_rules = false;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '/* test */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '\n' +
            '\n' +
            '/* non-header */\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0;\n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '//comment\n' +
            '\n' +
            '\n' +
            '//2nd single line comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another nl\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;   // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;\n' +
            '\n' +
            '\n' +
            '        // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body{}\n' +
            '\n' +
            '\n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            'text-align:left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0; \n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            '\n' +
            '\n' +
            'background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '\n' +
            '\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '\n' +
            '\n' +
            '        @font-face {\n' +
            '\n' +
            '\n' +
            '            font-family: "Helvetica Neue";\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '        #foo:hover {\n' +
            '\n' +
            '\n' +
            '            background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:first-child{\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:not(.peq){\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "false", newline_between_rules = "false")
        reset_options();
        set_name('Comments - (preserve_newlines = "false", newline_between_rules = "false")');
        opts.preserve_newlines = false;
        opts.newline_between_rules = false;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '        \n' +
            '    \n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '/* test */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '        \n' +
            '    \n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '        \n' +
            '    \n' +
            '/* non-header */\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0;\n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '            \n' +
            '   \n' +
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '//comment\n' +
            '            \n' +
            '   \n' +
            '//2nd single line comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another nl\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;   // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;\n' +
            '            \n' +
            '   \n' +
            '        // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '        \n' +
            '    \n' +
            '/* another comment */\n' +
            '        \n' +
            '    \n' +
            'body{}\n' +
            '        \n' +
            '    \n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '//demob instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '            \n' +
            '   \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0; \n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            '        \n' +
            '    \n' +
            'background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '        \n' +
            '    \n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        \n' +
            '    \n' +
            '        @font-face {\n' +
            '        \n' +
            '    \n' +
            '            font-family: "Helvetica Neue";\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '        #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '            background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:not(.peq){\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "false")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "false")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = false;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '.rule {}');
        t(
            '.tabs{/* test */}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {/* non-header */width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {margin: 0;/* This is a comment including an url http://domain.com/path/to/file.ext */}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '.tabs{width:10px;}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{//comment\n' +
            '//2nd single line comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another nl\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{width: 10px;   // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{width: 10px;\n' +
            '        // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' *//* another comment */body{}',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '}.demob {text-align: right;}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {text-align:left;}//demob instructions for LESS note visibility only\n' +
            '.demob {text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '.span {}',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '/**//**///\n' +
            '/**/.div{}/**//**///\n' +
            '/**/.span {}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '.div{}//\n' +
            '.span {}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {margin: 0; /* This is a comment including an url http://domain.com/path/to/file.ext */}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}.div{height:15px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {background-image: url(foo@2x.png);    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {    #foo:hover {        background-image: url(foo@2x.png);    }    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {    font-family: "Bitstream Vera Serif Bold";    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");}\n' +
            '@media screen {    #foo:hover {        background-image: url(foo.png);    }    @media screen and (min-device-pixel-ratio: 2) {        @font-face {            font-family: "Helvetica Neue";        }        #foo:hover {            background-image: url(foo@2x.png);        }    }}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{color:red;div:first-child{color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{color:red;div:not(.peq){color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "false")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "false")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = false;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '/* test */\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '/* non-header */\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            'margin: 0;\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '// comment\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '// comment\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '.tabs{\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '//comment\n' +
            '//2nd single line comment\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            'height:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            'height:10px;//another nl\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{\n' +
            'width: 10px;   // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            'width: 10px;\n' +
            '        // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body{}\n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '.demob {\n' +
            'text-align: right;\n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            'text-align:left;\n' +
            '}\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '.span {\n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '.span {}');
        t(
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div{}\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {\n' +
            '}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '.div{}\n' +
            '//\n' +
            '.span {\n' +
            '}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            'margin: 0; \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            'background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{\n' +
            'color:red;\n' +
            'div:first-child{\n' +
            'color:black;\n' +
            '}\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{\n' +
            'color:red;\n' +
            'div:not(.peq){\n' +
            'color:black;\n' +
            '}\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "false")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "false")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = false;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '        \n' +
            '    \n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '/* test */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* test */\n' +
            '\n' +
            '\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '        \n' +
            '    \n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '        \n' +
            '    \n' +
            '/* non-header */\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* non-header */\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0;\n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '//comment\n' +
            '            \n' +
            '   \n' +
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '//comment\n' +
            '            \n' +
            '   \n' +
            '//2nd single line comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    //comment\n' +
            '\n' +
            '\n' +
            '    //2nd single line comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another nl\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another nl\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;   // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;\n' +
            '            \n' +
            '   \n' +
            '        // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '    // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '        \n' +
            '    \n' +
            '/* another comment */\n' +
            '        \n' +
            '    \n' +
            'body{}\n' +
            '        \n' +
            '    \n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right;\n' +
            '\n' +
            '\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '//demob instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            '    text-align: left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '//\n' +
            '            \n' +
            '   \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0; \n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '#foo {\n' +
            '        \n' +
            '    \n' +
            'background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '\n' +
            '\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@font-face {\n' +
            '        \n' +
            '    \n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        \n' +
            '    \n' +
            '        @font-face {\n' +
            '        \n' +
            '    \n' +
            '            font-family: "Helvetica Neue";\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '        #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '            background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '\n' +
            '\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '\n' +
            '\n' +
            '        @font-face {\n' +
            '\n' +
            '\n' +
            '            font-family: "Helvetica Neue";\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '        #foo:hover {\n' +
            '\n' +
            '\n' +
            '            background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:first-child {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:not(.peq){\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "false")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "false")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = false;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '/* test */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* test */\n' +
            '\n' +
            '\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '\n' +
            '\n' +
            '/* non-header */\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* non-header */\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0;\n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '//comment\n' +
            '\n' +
            '\n' +
            '//2nd single line comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    //comment\n' +
            '\n' +
            '\n' +
            '    //2nd single line comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another nl\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another nl\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;   // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;\n' +
            '\n' +
            '\n' +
            '        // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '    // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body{}\n' +
            '\n' +
            '\n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right;\n' +
            '\n' +
            '\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            'text-align:left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            '    text-align: left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '//\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0; \n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '#foo {\n' +
            '\n' +
            '\n' +
            'background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '\n' +
            '\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@font-face {\n' +
            '\n' +
            '\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '\n' +
            '\n' +
            '        @font-face {\n' +
            '\n' +
            '\n' +
            '            font-family: "Helvetica Neue";\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '        #foo:hover {\n' +
            '\n' +
            '\n' +
            '            background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:first-child{\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:first-child {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:not(.peq){\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');

        // Comments - (preserve_newlines = "false", newline_between_rules = "true")
        reset_options();
        set_name('Comments - (preserve_newlines = "false", newline_between_rules = "true")');
        opts.preserve_newlines = false;
        opts.newline_between_rules = true;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{/* test */}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {/* non-header */width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {margin: 0;/* This is a comment including an url http://domain.com/path/to/file.ext */}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '.tabs{width:10px;}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{//comment\n' +
            '//2nd single line comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another nl\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{width: 10px;   // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{width: 10px;\n' +
            '        // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' *//* another comment */body{}',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '}.demob {text-align: right;}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {text-align:left;}//demob instructions for LESS note visibility only\n' +
            '.demob {text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '.span {}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '/**//**///\n' +
            '/**/.div{}/**//**///\n' +
            '/**/.span {}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '.div{}//\n' +
            '.span {}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {margin: 0; /* This is a comment including an url http://domain.com/path/to/file.ext */}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}.div{height:15px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {background-image: url(foo@2x.png);    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {    #foo:hover {        background-image: url(foo@2x.png);    }    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {    font-family: "Bitstream Vera Serif Bold";    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");}\n' +
            '@media screen {    #foo:hover {        background-image: url(foo.png);    }    @media screen and (min-device-pixel-ratio: 2) {        @font-face {            font-family: "Helvetica Neue";        }        #foo:hover {            background-image: url(foo@2x.png);        }    }}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{color:red;div:first-child{color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{color:red;div:not(.peq){color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "false", newline_between_rules = "true")
        reset_options();
        set_name('Comments - (preserve_newlines = "false", newline_between_rules = "true")');
        opts.preserve_newlines = false;
        opts.newline_between_rules = true;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '/* test */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '\n' +
            '\n' +
            '/* non-header */\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0;\n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '//comment\n' +
            '\n' +
            '\n' +
            '//2nd single line comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another nl\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;   // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;\n' +
            '\n' +
            '\n' +
            '        // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body{}\n' +
            '\n' +
            '\n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            'text-align:left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0; \n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            '\n' +
            '\n' +
            'background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '\n' +
            '\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '\n' +
            '\n' +
            '        @font-face {\n' +
            '\n' +
            '\n' +
            '            font-family: "Helvetica Neue";\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '        #foo:hover {\n' +
            '\n' +
            '\n' +
            '            background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:first-child{\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:not(.peq){\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "false", newline_between_rules = "true")
        reset_options();
        set_name('Comments - (preserve_newlines = "false", newline_between_rules = "true")');
        opts.preserve_newlines = false;
        opts.newline_between_rules = true;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '        \n' +
            '    \n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '/* test */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '        \n' +
            '    \n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '        \n' +
            '    \n' +
            '/* non-header */\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0;\n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '            \n' +
            '   \n' +
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '//comment\n' +
            '            \n' +
            '   \n' +
            '//2nd single line comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another nl\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;   // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;\n' +
            '            \n' +
            '   \n' +
            '        // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '        \n' +
            '    \n' +
            '/* another comment */\n' +
            '        \n' +
            '    \n' +
            'body{}\n' +
            '        \n' +
            '    \n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '//demob instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '            \n' +
            '   \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0; \n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            '        \n' +
            '    \n' +
            'background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '        \n' +
            '    \n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        \n' +
            '    \n' +
            '        @font-face {\n' +
            '        \n' +
            '    \n' +
            '            font-family: "Helvetica Neue";\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '        #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '            background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:not(.peq){\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "true")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "true")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = true;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{/* test */}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {/* non-header */width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {margin: 0;/* This is a comment including an url http://domain.com/path/to/file.ext */}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{// comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '.tabs{width:10px;}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{//comment\n' +
            '//2nd single line comment\n' +
            'width:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another nl\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{width: 10px;   // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{width: 10px;\n' +
            '        // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' *//* another comment */body{}',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '}.demob {text-align: right;}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {text-align:left;}//demob instructions for LESS note visibility only\n' +
            '.demob {text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '.span {}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '/**//**///\n' +
            '/**/.div{}/**//**///\n' +
            '/**/.span {}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '.div{}//\n' +
            '.span {}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {margin: 0; /* This is a comment including an url http://domain.com/path/to/file.ext */}\n' +
            '.div{height:15px;}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}.div{height:15px;}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {background-image: url(foo@2x.png);    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {    #foo:hover {        background-image: url(foo@2x.png);    }    @font-face {        font-family: "Bitstream Vera Serif Bold";        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");    }}.div{height:15px;}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {    font-family: "Bitstream Vera Serif Bold";    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");}\n' +
            '@media screen {    #foo:hover {        background-image: url(foo.png);    }    @media screen and (min-device-pixel-ratio: 2) {        @font-face {            font-family: "Helvetica Neue";        }        #foo:hover {            background-image: url(foo@2x.png);        }    }}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{color:red;div:first-child{color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{color:red;div:not(.peq){color:black;}}.div{height:15px;}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "true")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "true")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = true;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '/* test */\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* test */\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '/* non-header */\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    /* non-header */\n' +
            '    width: 10px;\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            'margin: 0;\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '// comment\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '// comment\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    // comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '//comment\n' +
            '.tabs{\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '//comment\n' +
            '.tabs {\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            '//comment\n' +
            '//2nd single line comment\n' +
            'width:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    //comment\n' +
            '    //2nd single line comment\n' +
            '    width: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            'height:10px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px;\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            'height:10px;//another nl\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another nl\n' +
            '}');
        t(
            '.tabs{\n' +
            'width: 10px;   // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            'width: 10px;\n' +
            '        // comment follows rule\n' +
            '// another comment new line\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px;\n' +
            '    // comment follows rule\n' +
            '    // another comment new line\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body{}\n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '/* another comment */\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '.demob {\n' +
            'text-align: right;\n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '}\n' +
            '\n' +
            '.demob {\n' +
            '    text-align: right;\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            'text-align:left;\n' +
            '}\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '    text-align: left;\n' +
            '}\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '.demob {\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '.span {\n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '.span {}');
        t(
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div{}\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {\n' +
            '}',
            //  -- output --
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.div {}\n' +
            '\n' +
            '/**/\n' +
            '/**/\n' +
            '//\n' +
            '/**/\n' +
            '.span {}');
        t(
            '//\n' +
            '.div{}\n' +
            '//\n' +
            '.span {\n' +
            '}',
            //  -- output --
            '//\n' +
            '.div {}\n' +
            '\n' +
            '//\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            'margin: 0; \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '    margin: 0;\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '.tabs{\n' +
            'width:10px;//end of line comment\n' +
            'height:10px;//another\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '    width: 10px; //end of line comment\n' +
            '    height: 10px; //another\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '#foo {\n' +
            'background-image: url(foo@2x.png);\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo@2x.png);\n' +
            '    }\n' +
            '\n' +
            '    @font-face {\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '}\n' +
            '\n' +
            '@media screen {\n' +
            '    #foo:hover {\n' +
            '        background-image: url(foo.png);\n' +
            '    }\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        @font-face {\n' +
            '            font-family: "Helvetica Neue";\n' +
            '        }\n' +
            '\n' +
            '        #foo:hover {\n' +
            '            background-image: url(foo@2x.png);\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            'a:first-child{\n' +
            'color:red;\n' +
            'div:first-child{\n' +
            'color:black;\n' +
            '}\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:first-child {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');
        t(
            'a:first-child{\n' +
            'color:red;\n' +
            'div:not(.peq){\n' +
            'color:black;\n' +
            '}\n' +
            '}\n' +
            '.div{\n' +
            'height:15px;\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '    color: red;\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '        color: black;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.div {\n' +
            '    height: 15px;\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "true")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "true")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = true;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '/* test */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* test */\n' +
            '\n' +
            '\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '\n' +
            '\n' +
            '/* non-header */\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* non-header */\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0;\n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '// comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            '//comment\n' +
            '\n' +
            '\n' +
            '//2nd single line comment\n' +
            '\n' +
            '\n' +
            'width:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    //comment\n' +
            '\n' +
            '\n' +
            '    //2nd single line comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another nl\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another nl\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;   // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width: 10px;\n' +
            '\n' +
            '\n' +
            '        // comment follows rule\n' +
            '\n' +
            '\n' +
            '// another comment new line\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '    // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body{}\n' +
            '\n' +
            '\n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right;\n' +
            '\n' +
            '\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            'text-align:left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            '    text-align: left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '//\n' +
            '\n' +
            '\n' +
            '.div{}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '//\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '\n' +
            '\n' +
            'margin: 0; \n' +
            '\n' +
            '\n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '\n' +
            '\n' +
            'width:10px;//end of line comment\n' +
            '\n' +
            '\n' +
            'height:10px;//another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '#foo {\n' +
            '\n' +
            '\n' +
            'background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '\n' +
            '\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@font-face {\n' +
            '\n' +
            '\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '\n' +
            '\n' +
            '        @font-face {\n' +
            '\n' +
            '\n' +
            '            font-family: "Helvetica Neue";\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '        #foo:hover {\n' +
            '\n' +
            '\n' +
            '            background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:first-child{\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:first-child {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '\n' +
            '\n' +
            'color:red;\n' +
            '\n' +
            '\n' +
            'div:not(.peq){\n' +
            '\n' +
            '\n' +
            'color:black;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div{\n' +
            '\n' +
            '\n' +
            'height:15px;\n' +
            '\n' +
            '\n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');

        // Comments - (preserve_newlines = "true", newline_between_rules = "true")
        reset_options();
        set_name('Comments - (preserve_newlines = "true", newline_between_rules = "true")');
        opts.preserve_newlines = true;
        opts.newline_between_rules = true;
        t('/* header comment newlines on */');
        t(
            '@import "custom.css";\n' +
            '        \n' +
            '    \n' +
            '.rule{}',
            //  -- output --
            '@import "custom.css";\n' +
            '\n' +
            '\n' +
            '.rule {}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '/* test */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* test */\n' +
            '\n' +
            '\n' +
            '}');

        // #1185
        t(
            '/* header */\n' +
            '        \n' +
            '    \n' +
            '.tabs{}',
            //  -- output --
            '/* header */\n' +
            '\n' +
            '\n' +
            '.tabs {}');
        t(
            '.tabs {\n' +
            '        \n' +
            '    \n' +
            '/* non-header */\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    /* non-header */\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t('/* header');
        t('// comment');
        t('/*');
        t('//');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0;\n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}');

        // single line comment support (less/sass)
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '// comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    // comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '//comment\n' +
            '            \n' +
            '   \n' +
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//comment\n' +
            '\n' +
            '\n' +
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            '//comment\n' +
            '            \n' +
            '   \n' +
            '//2nd single line comment\n' +
            '            \n' +
            '   \n' +
            'width:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    //comment\n' +
            '\n' +
            '\n' +
            '    //2nd single line comment\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another nl\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another nl\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;   // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #1165
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width: 10px;\n' +
            '            \n' +
            '   \n' +
            '        // comment follows rule\n' +
            '            \n' +
            '   \n' +
            '// another comment new line\n' +
            '            \n' +
            '   \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px;\n' +
            '\n' +
            '\n' +
            '    // comment follows rule\n' +
            '\n' +
            '\n' +
            '    // another comment new line\n' +
            '\n' +
            '\n' +
            '}');

        // #736
        t(
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '        \n' +
            '    \n' +
            '/* another comment */\n' +
            '        \n' +
            '    \n' +
            'body{}\n' +
            '        \n' +
            '    \n',
            //  -- output --
            '/*\n' +
            ' * comment\n' +
            ' */\n' +
            '\n' +
            '\n' +
            '/* another comment */\n' +
            '\n' +
            '\n' +
            'body {}');

        // #1348
        t(
            '.demoa1 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left; //demoa1 instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.demoa1 {\n' +
            '\n' +
            '\n' +
            '    text-align: left; //demoa1 instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right;\n' +
            '\n' +
            '\n' +
            '}');

        // #1440
        t(
            '#search-text {\n' +
            '  width: 43%;\n' +
            '  // height: 100%;\n' +
            '  border: none;\n' +
            '}',
            //  -- output --
            '#search-text {\n' +
            '    width: 43%;\n' +
            '    // height: 100%;\n' +
            '    border: none;\n' +
            '}');
        t(
            '.demoa2 {\n' +
            '        \n' +
            '    \n' +
            'text-align:left;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '//demob instructions for LESS note visibility only\n' +
            '            \n' +
            '   \n' +
            '.demob {\n' +
            '        \n' +
            '    \n' +
            'text-align: right}',
            //  -- output --
            '.demoa2 {\n' +
            '\n' +
            '\n' +
            '    text-align: left;\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '//demob instructions for LESS note visibility only\n' +
            '\n' +
            '\n' +
            '.demob {\n' +
            '\n' +
            '\n' +
            '    text-align: right\n' +
            '}');

        // new lines between rules - #531 and #857
        t(
            '.div{}\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.div {}\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '/**/\n' +
            '        \n' +
            '    \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '/**/\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '//\n' +
            '            \n' +
            '   \n' +
            '.div{}\n' +
            '        \n' +
            '    \n' +
            '//\n' +
            '            \n' +
            '   \n' +
            '.span {\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '//\n' +
            '\n' +
            '\n' +
            '.div {}\n' +
            '\n' +
            '\n' +
            '//\n' +
            '\n' +
            '\n' +
            '.span {}');
        t(
            '.selector1 {\n' +
            '        \n' +
            '    \n' +
            'margin: 0; \n' +
            '        \n' +
            '    \n' +
            '/* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.selector1 {\n' +
            '\n' +
            '\n' +
            '    margin: 0;\n' +
            '\n' +
            '\n' +
            '    /* This is a comment including an url http://domain.com/path/to/file.ext */\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '.tabs{\n' +
            '        \n' +
            '    \n' +
            'width:10px;//end of line comment\n' +
            '            \n' +
            '   \n' +
            'height:10px;//another\n' +
            '            \n' +
            '   \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '.tabs {\n' +
            '\n' +
            '\n' +
            '    width: 10px; //end of line comment\n' +
            '\n' +
            '\n' +
            '    height: 10px; //another\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '#foo {\n' +
            '        \n' +
            '    \n' +
            'background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '#foo {\n' +
            '\n' +
            '\n' +
            '    background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @font-face {\n' +
            '        \n' +
            '    \n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @font-face {\n' +
            '\n' +
            '\n' +
            '        font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '        src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            '@font-face {\n' +
            '        \n' +
            '    \n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '        \n' +
            '    \n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '            \n' +
            '   \n' +
            '@media screen {\n' +
            '        \n' +
            '    \n' +
            '    #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '        background-image: url(foo.png);\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '        \n' +
            '    \n' +
            '        @font-face {\n' +
            '        \n' +
            '    \n' +
            '            font-family: "Helvetica Neue";\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '        #foo:hover {\n' +
            '        \n' +
            '    \n' +
            '            background-image: url(foo@2x.png);\n' +
            '        \n' +
            '    \n' +
            '        }\n' +
            '        \n' +
            '    \n' +
            '    }\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            '@font-face {\n' +
            '\n' +
            '\n' +
            '    font-family: "Bitstream Vera Serif Bold";\n' +
            '\n' +
            '\n' +
            '    src: url("http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf");\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '@media screen {\n' +
            '\n' +
            '\n' +
            '    #foo:hover {\n' +
            '\n' +
            '\n' +
            '        background-image: url(foo.png);\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '    @media screen and (min-device-pixel-ratio: 2) {\n' +
            '\n' +
            '\n' +
            '        @font-face {\n' +
            '\n' +
            '\n' +
            '            font-family: "Helvetica Neue";\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '        #foo:hover {\n' +
            '\n' +
            '\n' +
            '            background-image: url(foo@2x.png);\n' +
            '\n' +
            '\n' +
            '        }\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:first-child {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');
        t(
            'a:first-child{\n' +
            '        \n' +
            '    \n' +
            'color:red;\n' +
            '        \n' +
            '    \n' +
            'div:not(.peq){\n' +
            '        \n' +
            '    \n' +
            'color:black;\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '}\n' +
            '        \n' +
            '    \n' +
            '.div{\n' +
            '        \n' +
            '    \n' +
            'height:15px;\n' +
            '        \n' +
            '    \n' +
            '}',
            //  -- output --
            'a:first-child {\n' +
            '\n' +
            '\n' +
            '    color: red;\n' +
            '\n' +
            '\n' +
            '    div:not(.peq) {\n' +
            '\n' +
            '\n' +
            '        color: black;\n' +
            '\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '\n' +
            '}\n' +
            '\n' +
            '\n' +
            '.div {\n' +
            '\n' +
            '\n' +
            '    height: 15px;\n' +
            '\n' +
            '\n' +
            '}');


        //============================================================
        // Handle LESS property name interpolation
        reset_options();
        set_name('Handle LESS property name interpolation');
        t(
            'tag {\n' +
            '    @{prop}: none;\n' +
            '}');
        t(
            'tag{@{prop}:none;}',
            //  -- output --
            'tag {\n' +
            '    @{prop}: none;\n' +
            '}');
        t(
            'tag{ @{prop}: none;}',
            //  -- output --
            'tag {\n' +
            '    @{prop}: none;\n' +
            '}');

        // can also be part of property name
        t(
            'tag {\n' +
            '    dynamic-@{prop}: none;\n' +
            '}');
        t(
            'tag{dynamic-@{prop}:none;}',
            //  -- output --
            'tag {\n' +
            '    dynamic-@{prop}: none;\n' +
            '}');
        t(
            'tag{ dynamic-@{prop}: none;}',
            //  -- output --
            'tag {\n' +
            '    dynamic-@{prop}: none;\n' +
            '}');


        //============================================================
        // Handle LESS property name interpolation, test #631
        reset_options();
        set_name('Handle LESS property name interpolation, test #631');
        t(
            '.generate-columns(@n, @i: 1) when (@i =< @n) {\n' +
            '    .column-@{i} {\n' +
            '        width: (@i * 100% / @n);\n' +
            '    }\n' +
            '    .generate-columns(@n, (@i + 1));\n' +
            '}');
        t(
            '.generate-columns(@n,@i:1) when (@i =< @n){.column-@{i}{width:(@i * 100% / @n);}.generate-columns(@n,(@i + 1));}',
            //  -- output --
            '.generate-columns(@n, @i: 1) when (@i =< @n) {\n' +
            '    .column-@{i} {\n' +
            '        width: (@i * 100% / @n);\n' +
            '    }\n' +
            '    .generate-columns(@n, (@i + 1));\n' +
            '}');


        //============================================================
        // Handle LESS function parameters
        reset_options();
        set_name('Handle LESS function parameters');
        t(
            'div{.px2rem(width,12);}',
            //  -- output --
            'div {\n' +
            '    .px2rem(width, 12);\n' +
            '}');
        t(
            'div {\n' +
            '    background: url("//test.com/dummy.png");\n' +
            '    .px2rem(width, 12);\n' +
            '}');


        //============================================================
        // Psuedo-classes vs Variables
        reset_options();
        set_name('Psuedo-classes vs Variables');
        t('@page :first {}');

        // Assume the colon goes with the @name. If we're in LESS, this is required regardless of the at-string.
        t('@page:first {}', '@page: first {}');
        t('@page: first {}');


        //============================================================
        // Issue 1411 -- LESS Variable Assignment Spacing
        reset_options();
        set_name('Issue 1411 -- LESS Variable Assignment Spacing');
        t(
            '@set: {\n' +
            '    one: blue;\n' +
            '    two: green;\n' +
            '    three: red;\n' +
            '}\n' +
            '.set {\n' +
            '    each(@set, {\n' +
            '        @{key}-@{index}: @value;\n' +
            '    });\n' +
            '}');
        t('@light-blue: @nice-blue + #111;');


        //============================================================
        // SASS/SCSS
        reset_options();
        set_name('SASS/SCSS');

        // Basic Interpolation
        t(
            'p {\n' +
            '    $font-size: 12px;\n' +
            '    $line-height: 30px;\n' +
            '    font: #{$font-size}/#{$line-height};\n' +
            '}');
        t('p.#{$name} {}');
        t(
            '@mixin itemPropertiesCoverItem($items, $margin) {\n' +
            '    width: calc((100% - ((#{$items} - 1) * #{$margin}rem)) / #{$items});\n' +
            '    margin: 1.6rem #{$margin}rem 1.6rem 0;\n' +
            '}');

        // Multiple filed issues in LESS due to not(:blah)
        t('&:first-of-type:not(:last-child) {}');

        // #1236 - maps standard
        t(
            '$theme-colors: (\n' +
            '    primary: $blue,\n' +
            '    secondary: "gray-600"\n' +
            ');');

        // #1236 - maps single line
        t(
            '$theme-colors:(primary: $blue,     secondary: "$gray-600");',
            //  -- output --
            '$theme-colors: (\n' +
            '    primary: $blue,\n' +
            '    secondary: "$gray-600"\n' +
            ');');

        // #1236 - maps with functions
        t(
            '$maps:(x: 80px,     y: "something",    \n' +
            'z: calc(10 + 10)\n' +
            ');',
            //  -- output --
            '$maps: (\n' +
            '    x: 80px,\n' +
            '    y: "something",\n' +
            '    z: calc(10 + 10)\n' +
            ');');
        t(
            'div {\n' +
            '    &:not(:first-of-type) {\n' +
            '        background: red;\n' +
            '    }\n' +
            '}');


        //============================================================
        // Proper handling of colon in selectors
        reset_options();
        set_name('Proper handling of colon in selectors');
        opts.selector_separator_newline = false;
        t('a :b {}');
        t('a ::b {}');
        t('a:b {}');
        t('a::b {}');
        t(
            'a {}, a::b {}, a   ::b {}, a:b {}, a   :b {}',
            //  -- output --
            'a {}\n' +
            ', a::b {}\n' +
            ', a ::b {}\n' +
            ', a:b {}\n' +
            ', a :b {}');
        t(
            '.card-blue ::-webkit-input-placeholder {\n' +
            '    color: #87D1FF;\n' +
            '}');
        t(
            'div [attr] :not(.class) {\n' +
            '    color: red;\n' +
            '}');

        // Issue #1233
        t(
            '.one {\n' +
            '    color: #FFF;\n' +
            '    // pseudo-element\n' +
            '    span:not(*::selection) {\n' +
            '        margin-top: 0;\n' +
            '    }\n' +
            '}\n' +
            '.two {\n' +
            '    color: #000;\n' +
            '    // pseudo-class\n' +
            '    span:not(*:active) {\n' +
            '        margin-top: 0;\n' +
            '    }\n' +
            '}');


        //============================================================
        // Regresssion Tests
        reset_options();
        set_name('Regresssion Tests');
        opts.selector_separator_newline = false;
        t(
            '@media(min-width:768px) {\n' +
            '    .selector::after {\n' +
            '        /* property: value */\n' +
            '    }\n' +
            '    .other-selector {\n' +
            '        /* property: value */\n' +
            '    }\n' +
            '}');

        // #1236 - SCSS/SASS Maps with selector_separator_newline = false
        t('$font-weights: ("regular": 400, "medium": 500, "bold": 700);');
        t(
            '.fa-rotate-270 {\n' +
            '    filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=3);\n' +
            '}');

        // #2056 - Extra space before !important added
        t(
            '.x {\n' +
            '    $d: a !default;\n' +
            '}');
        t(
            '.x {\n' +
            '    $d: a !default;\n' +
            '    @if $x !=0 {\n' +
            '        color: $var !important;\n' +
            '    }\n' +
            '}');

        // #2051 - css format removes space after quoted value
        t(
            'q {\n' +
            '    quotes: \'"\' \'"\' "\'" "\'";\n' +
            '    quotes: "some" \'thing\' "different";\n' +
            '    quotes: \'some\' "thing" \'different\';\n' +
            '}');


        //============================================================
        // Regression tests - with default options
        reset_options();
        set_name('Regression tests - with default options');

        // Issue #1798 - space after strings in preserved
        t('@use "variables" as *;');

        // Issue #1976 - support the new @forwards syntax
        t(
            '@forwards "a" with (\n' +
            '   $a: 2\n' +
            ');',
            //  -- output --
            '@forwards "a" with ($a: 2);');


        //============================================================
        // Issue #1817
        reset_options();
        set_name('Issue #1817');

        // ensure that properties that are expected to have multiline values persist new lines
        t(
            '.grid {\n' +
            '    grid-template:\n' +
            '        "top-bar top-bar" 100px\n' +
            '        "left-bar center" 100px;\n' +
            '}');

        // property values that have string followed by other identifiers should persist spacing
        t(
            '.grid {grid-template: "top-bar" 100px;}',
            //  -- output --
            '.grid {\n' +
            '    grid-template: "top-bar" 100px;\n' +
            '}');
        t(
            'div {\n' +
            'grid-template-areas: "a"\n' +
            ' "b" \n' +
            '                    "c";\n' +
            '}',
            //  -- output --
            'div {\n' +
            '    grid-template-areas: "a"\n' +
            '        "b"\n' +
            '        "c";\n' +
            '}');
        t(
            'div {\n' +
            'grid-template: "a a a" 20%\n' +
            ' [main-top] "b b b" 1fr\n' +
            '                    "b b b" auto;\n' +
            '}',
            //  -- output --
            'div {\n' +
            '    grid-template: "a a a" 20%\n' +
            '        [main-top] "b b b" 1fr\n' +
            '        "b b b" auto;\n' +
            '}');


        //============================================================
        // Issue #645, #1233
        reset_options();
        set_name('Issue #645, #1233');
        opts.selector_separator_newline = true;
        opts.preserve_newlines = true;
        opts.newline_between_rules = true;
        t(
            '/* Comment above first rule */\n' +
            '\n' +
            'body {\n' +
            '    display: none;\n' +
            '}\n' +
            '\n' +
            '/* Comment between rules */\n' +
            '\n' +
            'ul,\n' +
            '\n' +
            '/* Comment between selectors */\n' +
            '\n' +
            'li {\n' +
            '    display: none;\n' +
            '}\n' +
            '\n' +
            '/* Comment after last rule */');
        t(
            '.one  {\n' +
            '    color: #FFF;\n' +
            '    // pseudo-element\n' +
            '    span:not(*::selection) {\n' +
            '        margin-top: 0;\n' +
            '    }\n' +
            '}\n' +
            '.two {\n' +
            '    color: #000;\n' +
            '    // pseudo-class\n' +
            '    span:not(*:active) {\n' +
            '        margin-top: 0;\n' +
            '    }\n' +
            '}',
            //  -- output --
            '.one {\n' +
            '    color: #FFF;\n' +
            '\n' +
            '    // pseudo-element\n' +
            '    span:not(*::selection) {\n' +
            '        margin-top: 0;\n' +
            '    }\n' +
            '}\n' +
            '\n' +
            '.two {\n' +
            '    color: #000;\n' +
            '\n' +
            '    // pseudo-class\n' +
            '    span:not(*:active) {\n' +
            '        margin-top: 0;\n' +
            '    }\n' +
            '}');


        //============================================================
        // Extend Tests
        reset_options();
        set_name('Extend Tests');
        t(
            '.btn-group-radios {\n' +
            '    .btn:hover {\n' +
            '        &:hover,\n' +
            '        &:focus {\n' +
            '            @extend .btn-blue:hover;\n' +
            '        }\n' +
            '    }\n' +
            '}');
        t(
            '.item-warning {\n' +
            '    @extend btn-warning:hover;\n' +
            '}\n' +
            '.item-warning-wrong {\n' +
            '    @extend btn-warning: hover;\n' +
            '}');
        t(
            '.item-warning {\n' +
            '    @extend .color1, .hover2;\n' +
            '}');


        //============================================================
        // Import Tests
        reset_options();
        set_name('Import Tests');
        t(
            '@import "custom.css";.rule{}\n' +
            'a, p {}',
            //  -- output --
            '@import "custom.css";\n' +
            '.rule {}\n' +
            'a,\n' +
            'p {}');
        t(
            '@import url("bluish.css") projection,tv;.rule{}\n' +
            'a, p {}',
            //  -- output --
            '@import url("bluish.css") projection, tv;\n' +
            '.rule {}\n' +
            'a,\n' +
            'p {}');


        //============================================================
        // Important
        reset_options();
        set_name('Important');
        t(
            'a {\n' +
            '    color: blue  !important;\n' +
            '}',
            //  -- output --
            'a {\n' +
            '    color: blue !important;\n' +
            '}');
        t(
            'a {\n' +
            '    color: blue!important;\n' +
            '}',
            //  -- output --
            'a {\n' +
            '    color: blue !important;\n' +
            '}');
        t(
            'a {\n' +
            '    color: blue !important;\n' +
            '}');
        t(
            '.blue\\! {\n' +
            '    color: blue !important;\n' +
            '}');


        //============================================================
        // Escape
        reset_options();
        set_name('Escape');
        t(
            'a:not(.color\\:blue) {\n' +
            '    color: blue !important;\n' +
            '}');
        t(
            '.blue\\:very {\n' +
            '    color: blue !important;\n' +
            '}');
        test_fragment('a:not(.color\\');
        test_fragment('a:not\\');


        //============================================================
        // indent_empty_lines true
        reset_options();
        set_name('indent_empty_lines true');
        opts.indent_empty_lines = true;
        opts.preserve_newlines = true;
        test_fragment(
            'a {\n' +
            '\n' +
            'width: auto;\n' +
            '\n' +
            'height: auto;\n' +
            '\n' +
            '}',
            //  -- output --
            'a {\n' +
            '    \n' +
            '    width: auto;\n' +
            '    \n' +
            '    height: auto;\n' +
            '    \n' +
            '}');


        //============================================================
        // indent_empty_lines false
        reset_options();
        set_name('indent_empty_lines false');
        opts.indent_empty_lines = false;
        opts.preserve_newlines = true;
        test_fragment(
            'a {\n' +
            '\n' +
            '    width: auto;\n' +
            '\n' +
            '    height: auto;\n' +
            '\n' +
            '}');


        //============================================================
        // brace_style = expand - (brace_style = ""expand"", selector_separator_newline = "false", newline_between_rules = "true")
        reset_options();
        set_name('brace_style = expand - (brace_style = ""expand"", selector_separator_newline = "false", newline_between_rules = "true")');
        opts.brace_style = 'expand';
        opts.selector_separator_newline = false;
        opts.newline_between_rules = true;
        t(
            'a, b, .c {\n' +
            '    width: auto;\n' +
            '  \n' +
            '    height: auto;\n' +
            '}',
            //  -- output --
            'a, b, .c\n' +
            '{\n' +
            '    width: auto;\n' +
            '    height: auto;\n' +
            '}');

        // edge case - empty line after { should not be indented without indent_empty_lines
        t(
            'a, b, .c {\n' +
            '\n' +
            '    width: auto;\n' +
            '}',
            //  -- output --
            'a, b, .c\n' +
            '{\n' +
            '    width: auto;\n' +
            '}');

        // mixins call with object notation, and brace_style="expand"
        t(
            '.example({\n' +
            '    color:red;\n' +
            '});',
            //  -- output --
            '.example(\n' +
            '    {\n' +
            '        color:red;\n' +
            '    }\n' +
            ');');

        // integration test of newline_between_rules, imports, and brace_style="expand"
        t(
            '.a{} @import "custom.css";.rule{}',
            //  -- output --
            '.a\n' +
            '{}\n' +
            '\n' +
            '@import "custom.css";\n' +
            '\n' +
            '.rule\n' +
            '{}');

        // brace_style = expand - (brace_style = ""expand"", indent_empty_lines = "true", selector_separator_newline = "false", preserve_newlines = "true")
        reset_options();
        set_name('brace_style = expand - (brace_style = ""expand"", indent_empty_lines = "true", selector_separator_newline = "false", preserve_newlines = "true")');
        opts.brace_style = 'expand';
        opts.indent_empty_lines = true;
        opts.selector_separator_newline = false;
        opts.preserve_newlines = true;
        t(
            'a, b, .c {\n' +
            '    width: auto;\n' +
            '  \n' +
            '    height: auto;\n' +
            '}',
            //  -- output --
            'a, b, .c\n' +
            '{\n' +
            '    width: auto;\n' +
            '    \n    height: auto;\n' +
            '}');

        // edge case - empty line after { should not be indented without indent_empty_lines
        t(
            'a, b, .c {\n' +
            '\n' +
            '    width: auto;\n' +
            '}',
            //  -- output --
            'a, b, .c\n' +
            '{\n' +
            '    \n    width: auto;\n' +
            '}');

        // mixins call with object notation, and brace_style="expand"
        t(
            '.example({\n' +
            '    color:red;\n' +
            '});',
            //  -- output --
            '.example(\n' +
            '    {\n' +
            '        color:red;\n' +
            '    }\n' +
            ');');

        // integration test of newline_between_rules, imports, and brace_style="expand"
        t(
            '.a{} @import "custom.css";.rule{}',
            //  -- output --
            '.a\n' +
            '{}\n' +
            '@import "custom.css";\n' +
            '.rule\n' +
            '{}');

        // brace_style = expand - (brace_style = ""expand"", indent_empty_lines = "false", selector_separator_newline = "false", preserve_newlines = "true")
        reset_options();
        set_name('brace_style = expand - (brace_style = ""expand"", indent_empty_lines = "false", selector_separator_newline = "false", preserve_newlines = "true")');
        opts.brace_style = 'expand';
        opts.indent_empty_lines = false;
        opts.selector_separator_newline = false;
        opts.preserve_newlines = true;
        t(
            'a, b, .c {\n' +
            '    width: auto;\n' +
            '  \n' +
            '    height: auto;\n' +
            '}',
            //  -- output --
            'a, b, .c\n' +
            '{\n' +
            '    width: auto;\n' +
            '\n    height: auto;\n' +
            '}');

        // edge case - empty line after { should not be indented without indent_empty_lines
        t(
            'a, b, .c {\n' +
            '\n' +
            '    width: auto;\n' +
            '}',
            //  -- output --
            'a, b, .c\n' +
            '{\n' +
            '\n    width: auto;\n' +
            '}');

        // mixins call with object notation, and brace_style="expand"
        t(
            '.example({\n' +
            '    color:red;\n' +
            '});',
            //  -- output --
            '.example(\n' +
            '    {\n' +
            '        color:red;\n' +
            '    }\n' +
            ');');

        // integration test of newline_between_rules, imports, and brace_style="expand"
        t(
            '.a{} @import "custom.css";.rule{}',
            //  -- output --
            '.a\n' +
            '{}\n' +
            '@import "custom.css";\n' +
            '.rule\n' +
            '{}');


        //============================================================
        // LESS mixins
        reset_options();
        set_name('LESS mixins');
        t(
            '.btn {\n' +
            '    .generate-animation(@mykeyframes, 1.4s, .5s, 1, ease-out);\n' +
            '}\n' +
            '.mymixin(@color: #ccc; @border-width: 1px) {\n' +
            '    border: @border-width solid @color;\n' +
            '}\n' +
            'strong {\n' +
            '    &:extend(a:hover);\n' +
            '}');
        t(
            '.test {\n' +
            '    .example({\n' +
            '        color:red;\n' +
            '    });\n' +
            '}');
        t(
            '.example2({\n' +
            '    display:none;\n' +
            '});');
        t(
            '.aa {\n' +
            '    .mq-medium(a, {\n' +
            '        background: red;\n' +
            '    });\n' +
            '}');
        t(
            '@selectors: blue, green, red;\n' +
            'each(@selectors, {\n' +
            '    .sel-@{value} {\n' +
            '        a: b;\n' +
            '    }\n' +
            '});');

        // Ensure simple closing parens do not break behavior
        t(
            'strong {\n' +
            '    &:extend(a:hover));\n' +
            '}\n' +
            '.btn {\n' +
            '    .generate-animation(@mykeyframes, 1.4s, .5s, 1, ease-out);\n' +
            '}\n' +
            '.mymixin(@color: #ccc; @border-width: 1px) {\n' +
            '    border: @border-width solid @color;\n' +
            '}\n' +
            'strong {\n' +
            '    &:extend(a:hover);\n' +
            '}');

        // indent multi-line parens
        t(
            '.btn {\n' +
            '    .generate-animation(@mykeyframes, 1.4s,\n' +
            '        .5s, 1, ease-out);\n' +
            '}\n' +
            '.mymixin(@color: #ccc;\n' +
            '    @border-width: 1px) {\n' +
            '    border: @border-width solid @color;\n' +
            '}');

        // format inside mixin parens
        t(
            '.btn {\n' +
            '    .generate-animation(@mykeyframes,1.4s,.5s,1,ease-out);\n' +
            '}\n' +
            '.mymixin(@color:#ccc;@border-width:1px) {\n' +
            '    border:@border-width solid @color;\n' +
            '}',
            //  -- output --
            '.btn {\n' +
            '    .generate-animation(@mykeyframes, 1.4s, .5s, 1, ease-out);\n' +
            '}\n' +
            '.mymixin(@color: #ccc; @border-width: 1px) {\n' +
            '    border: @border-width solid @color;\n' +
            '}');


        //============================================================
        // Preserve Newlines and max number of new lines
        reset_options();
        set_name('Preserve Newlines and max number of new lines');
        opts.preserve_newlines = true;
        opts.max_preserve_newlines = 2;
        t(
            'p {\n' +
            '\n' +
            '\n' +
            '\n' +
            '    color: blue;\n' +
            '}',
            //  -- output --
            'p {\n' +
            '\n' +
            '    color: blue;\n' +
            '}');
        t(
            'p {\n' +
            '\n' +
            '    color: blue;\n' +
            '}');
        t(
            'p {\n' +
            '    color: blue;\n' +
            '}');


        //============================================================
        //
        reset_options();
        set_name('');


    }

    function beautifier_unconverted_tests()
    {
        sanitytest = test_obj;

        reset_options();
        //============================================================
        test_fragment(null, '');

        reset_options();
        //============================================================
        // Test user pebkac protection, converts dash names to underscored names
        opts["end-with-newline"] = true;
        test_fragment(null, '\n');

        reset_options();
        //============================================================
        // test basic css beautifier
        t(".tabs {}");
        t(".tabs{color:red;}", ".tabs {\n    color: red;\n}");
        t(".tabs{color:rgb(255, 255, 0)}", ".tabs {\n    color: rgb(255, 255, 0)\n}");
        t(".tabs{background:url('back.jpg')}", ".tabs {\n    background: url('back.jpg')\n}");
        t("#bla, #foo{color:red}", "#bla,\n#foo {\n    color: red\n}");
        t("@media print {.tab{}}", "@media print {\n    .tab {}\n}");
        t("@media print {.tab{background-image:url(foo@2x.png)}}", "@media print {\n    .tab {\n        background-image: url(foo@2x.png)\n    }\n}");

        t("a:before {\n" +
            "    content: 'a{color:black;}\"\"\\'\\'\"\\n\\n\\na{color:black}\';\n" +
            "}");

        //lead-in whitespace determines base-indent.
        // lead-in newlines are stripped.
        t("\n\na, img {padding: 0.2px}", "a,\nimg {\n    padding: 0.2px\n}");
        t("   a, img {padding: 0.2px}", "   a,\n   img {\n       padding: 0.2px\n   }");
        t("      \na, img {padding: 0.2px}", "      a,\n      img {\n          padding: 0.2px\n      }");
        t("\n\n     a, img {padding: 0.2px}", "a,\nimg {\n    padding: 0.2px\n}");

        // separate selectors
        t("#bla, #foo{color:red}", "#bla,\n#foo {\n    color: red\n}");
        t("a, img {padding: 0.2px}", "a,\nimg {\n    padding: 0.2px\n}");

        // block nesting
        t("#foo {\n    background-image: url(foo@2x.png);\n    @font-face {\n        font-family: 'Bitstream Vera Serif Bold';\n        src: url('http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf');\n    }\n}");
        t("@media screen {\n    #foo:hover {\n        background-image: url(foo@2x.png);\n    }\n    @font-face {\n        font-family: 'Bitstream Vera Serif Bold';\n        src: url('http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf');\n    }\n}");
/*
@font-face {
    font-family: 'Bitstream Vera Serif Bold';
    src: url('http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf');
}
@media screen {
    #foo:hover {
        background-image: url(foo.png);
    }
    @media screen and (min-device-pixel-ratio: 2) {
        @font-face {
            font-family: 'Helvetica Neue'
        }
        #foo:hover {
            background-image: url(foo@2x.png);
        }
    }
}
*/
        t("@font-face {\n    font-family: 'Bitstream Vera Serif Bold';\n    src: url('http://developer.mozilla.org/@api/deki/files/2934/=VeraSeBd.ttf');\n}\n@media screen {\n    #foo:hover {\n        background-image: url(foo.png);\n    }\n    @media screen and (min-device-pixel-ratio: 2) {\n        @font-face {\n            font-family: 'Helvetica Neue'\n        }\n        #foo:hover {\n            background-image: url(foo@2x.png);\n        }\n    }\n}");

        // less-css cases
        t('.well{@well-bg:@bg-color;@well-fg:@fg-color;}','.well {\n    @well-bg: @bg-color;\n    @well-fg: @fg-color;\n}');
        t('.well {&.active {\nbox-shadow: 0 1px 1px @border-color, 1px 0 1px @border-color;}}',
            '.well {\n' +
            '    &.active {\n' +
            '        box-shadow: 0 1px 1px @border-color, 1px 0 1px @border-color;\n' +
            '    }\n' +
            '}');
        t('a {\n' +
            '    color: blue;\n' +
            '    &:hover {\n' +
            '        color: green;\n' +
            '    }\n' +
            '    & & &&&.active {\n' +
            '        color: green;\n' +
            '    }\n' +
            '}');

        // Not sure if this is sensible
        // but I believe it is correct to not remove the space in "&: hover".
        t('a {\n' +
            '    &: hover {\n' +
            '        color: green;\n' +
            '    }\n' +
            '}');

        // import
        t('@import "test";');

        // don't break nested pseudo-classes
        t("a:first-child{color:red;div:first-child{color:black;}}",
            "a:first-child {\n    color: red;\n    div:first-child {\n        color: black;\n    }\n}");

        // handle SASS/LESS parent reference
        t("div{&:first-letter {text-transform: uppercase;}}",
            "div {\n    &:first-letter {\n        text-transform: uppercase;\n    }\n}");

        //nested modifiers (&:hover etc)
        t(".tabs{&:hover{width:10px;}}", ".tabs {\n    &:hover {\n        width: 10px;\n    }\n}");
        t(".tabs{&.big{width:10px;}}", ".tabs {\n    &.big {\n        width: 10px;\n    }\n}");
        t(".tabs{&>big{width:10px;}}", ".tabs {\n    &>big {\n        width: 10px;\n    }\n}");
        t(".tabs{&+.big{width:10px;}}", ".tabs {\n    &+.big {\n        width: 10px;\n    }\n}");

        //nested rules
        t(".tabs{.child{width:10px;}}", ".tabs {\n    .child {\n        width: 10px;\n    }\n}");

        //variables
        t("@myvar:10px;.tabs{width:10px;}", "@myvar: 10px;\n.tabs {\n    width: 10px;\n}");
        t("@myvar:10px; .tabs{width:10px;}", "@myvar: 10px;\n.tabs {\n    width: 10px;\n}");

        //mixins
        t("div{.px2rem(width,12);}", "div {\n    .px2rem(width, 12);\n}");
        // mixin next to 'background: url("...")' should not add a line break after the comma
        t("div {\n    background: url(\"//test.com/dummy.png\");\n    .px2rem(width, 12);\n}");

        // test options
        opts.indent_size = 2;
        opts.indent_char = ' ';
        opts.selector_separator_newline = false;

        // pseudo-classes and pseudo-elements
        t("#foo:hover {\n  background-image: url(foo@2x.png)\n}");
        t("#foo *:hover {\n  color: purple\n}");
        t("::selection {\n  color: #ff0000;\n}");

        // TODO: don't break nested pseduo-classes
        t("@media screen {.tab,.bat:hover {color:red}}", "@media screen {\n  .tab, .bat:hover {\n    color: red\n  }\n}");

        // particular edge case with braces and semicolons inside tags that allows custom text
        t("a:not(\"foobar\\\";{}omg\"){\ncontent: 'example\\';{} text';\ncontent: \"example\\\";{} text\";}",
            "a:not(\"foobar\\\";{}omg\") {\n  content: 'example\\';{} text';\n  content: \"example\\\";{} text\";\n}");

        // may not eat the space before "["
        t('html.js [data-custom="123"] {\n  opacity: 1.00;\n}');
        t('html.js *[data-custom="123"] {\n  opacity: 1.00;\n}');
    }

    beautifier_tests();
    beautifier_unconverted_tests();
}

if (typeof exports !== "undefined") {
    exports.run_css_tests = run_css_tests;
}
//== js/test/generated/beautify-css-tests.js end


//== js/test/generated/beautify-html-tests.js
/*
    AUTO-GENERATED. DO NOT MODIFY.
    Script: test/generate-tests.js
    Template: test/data/html/node.mustache
    Data: test/data/html/tests.js

  The MIT License (MIT)

  Copyright (c) 2007-2018 Einar Lielmanis, Liam Newman, and contributors.

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation files
  (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge,
  publish, distribute, sublicense, and/or sell copies of the Software,
  and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
/*jshint unused:false */
/*jshint strict:false */

function run_html_tests(test_obj, Urlencoded, js_beautify, html_beautify, css_beautify)
{

    var default_opts = {
        indent_size: 4,
        indent_char: ' ',
        preserve_newlines: true,
        jslint_happy: false,
        keep_array_indentation: false,
        brace_style: 'collapse',
        space_before_conditional: true,
        break_chained_methods: false,
        selector_separator: '\n',
        end_with_newline: false
    };
    var opts;

    default_opts.indent_size = 4;
    default_opts.indent_char = ' ';
    default_opts.indent_with_tabs = false;
    default_opts.preserve_newlines = true;
    default_opts.jslint_happy = false;
    default_opts.keep_array_indentation = false;
    default_opts.brace_style = 'collapse';
    default_opts.extra_liners = ['html', 'head', '/html'];

    function reset_options()
    {
        opts = {}; for(var p in default_opts) opts[p] = default_opts[p];
        test_name = 'html-beautify';
    }

    function test_beautifier(input)
    {
        return html_beautify(input, opts);
    }

    var sanitytest;
    var test_name = '';

    function set_name(name)
    {
        name = (name || '').trim();
        if (name) {
            test_name = name.replace(/\r/g, '\\r').replace(/\n/g, '\\n');
        }
    }

    // test the input on beautifier with the current flag settings
    // does not check the indentation / surroundings as bt() does
    function test_fragment(input, expected)
    {
        var success = true;
        sanitytest.test_function(test_beautifier, test_name);
        expected = expected || expected === '' ? expected : input;
        success = success && sanitytest.expect(input, expected);
        // if the expected is different from input, run it again
        // expected output should be unchanged when run twice.
        if (success && expected !== input) {
            success = success && sanitytest.expect(expected, expected);
        }

        // Everywhere we do newlines, they should be replaced with opts.eol
        sanitytest.test_function(test_beautifier, 'eol ' + test_name);
        opts.eol = '\r\\n';
        expected = expected.replace(/[\n]/g, '\r\n');
        opts.disabled = true;
        success = success && sanitytest.expect(input, input || '');
        success = success && sanitytest.expect('\n\n' + expected, '\n\n' + expected);
        opts.disabled = false;
        success = success && sanitytest.expect(input, expected);
        if (success && input && input.indexOf('\n') !== -1) {
            input = input.replace(/[\n]/g, '\r\n');
            sanitytest.expect(input, expected);
            // Ensure support for auto eol detection
            opts.eol = 'auto';
            success = success && sanitytest.expect(input, expected);
        }
        opts.eol = '\n';
        return success;
    }

    // test html
    function bth(input, expectation)
    {
        var success = true;

        var wrapped_input, wrapped_expectation, field_input, field_expectation;

        expectation = expectation || expectation === '' ? expectation : input;
        success = success && test_fragment(input, expectation);

        if (opts.indent_size === 4 && input) {
          var indent_string = opts.indent_with_tabs ? '\t' : '    ';
            wrapped_input = '<div>\n' + input.replace(/^(.+)$/mg, indent_string + '$1') + '\n' + indent_string + '<span>inline</span>\n</div>';
            wrapped_expectation = '<div>\n' + expectation.replace(/^(.+)$/mg, indent_string + '$1') + '\n' + indent_string + '<span>inline</span>\n</div>';
            success = success && test_fragment(wrapped_input, wrapped_expectation);
        }
        return success;
    }

    function unicode_char(value) {
        return String.fromCharCode(value);
    }

    function beautifier_tests()
    {
        sanitytest = test_obj;

        reset_options();
        //============================================================
        bth('');


        //============================================================
        // Unicode Support
        reset_options();
        set_name('Unicode Support');
        bth('<p>Hello' + unicode_char(160) + unicode_char(3232) + '_' + unicode_char(3232) + 'world!</p>');


        //============================================================
        // Handle inline and block elements differently - ()
        reset_options();
        set_name('Handle inline and block elements differently - ()');
        test_fragment(
            '<body><h1>Block</h1></body>',
            //  -- output --
            '<body>\n' +
            '    <h1>Block</h1>\n' +
            '</body>');
        test_fragment('<body><i>Inline</i></body>');
        bth(
            '<svg xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" preserveAspectRatio="none" x="0" y="0" viewBox="0 0 900 710" width="100%" height="100%">\n' +
            '<circle id="mycircle" \n' +
            'cx="182.901" cy="91.4841" \n' +
            'style="fill:rosybrown;stroke:black;stroke-width:1px;" r="48" /></svg>',
            //  -- output --
            '<svg xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" preserveAspectRatio="none" x="0" y="0" viewBox="0 0 900 710" width="100%" height="100%">\n' +
            '    <circle id="mycircle" cx="182.901" cy="91.4841" style="fill:rosybrown;stroke:black;stroke-width:1px;" r="48" />\n' +
            '</svg>');
        bth(
            '<div class="col-xs-2">\n' +
            '<input type="radio" class="control-label" ng-disabled="!col" ng-model="col" value="2" class="form-control" id="coli" name="coli" />\n' +
            '<label for="coli" class="control-label">Collision</label></div>',
            //  -- output --
            '<div class="col-xs-2">\n' +
            '    <input type="radio" class="control-label" ng-disabled="!col" ng-model="col" value="2" class="form-control" id="coli" name="coli" />\n' +
            '    <label for="coli" class="control-label">Collision</label>\n' +
            '</div>');
        bth(
            '<label class="col-xs-2">Collision\n' +
            '<input type="radio" class="control-label" ng-disabled="!col" ng-model="col" value="2" class="form-control" id="coli" name="coli" /></label>',
            //  -- output --
            '<label class="col-xs-2">Collision\n' +
            '    <input type="radio" class="control-label" ng-disabled="!col" ng-model="col" value="2" class="form-control" id="coli" name="coli" /></label>');
        bth(
            '<div class="col-xs-2">Collision\n' +
            '<input type="radio" class="control-label" ng-disabled="!col" ng-model="col" value="2" class="form-control" id="coli" name="coli" /></div>',
            //  -- output --
            '<div class="col-xs-2">Collision\n' +
            '    <input type="radio" class="control-label" ng-disabled="!col" ng-model="col" value="2" class="form-control" id="coli" name="coli" />\n' +
            '</div>');


        //============================================================
        // End With Newline - (end_with_newline = "true")
        reset_options();
        set_name('End With Newline - (end_with_newline = "true")');
        opts.end_with_newline = true;
        test_fragment('', '\n');
        test_fragment('<div></div>', '<div></div>\n');
        test_fragment('\n');

        // End With Newline - (end_with_newline = "false")
        reset_options();
        set_name('End With Newline - (end_with_newline = "false")');
        opts.end_with_newline = false;
        test_fragment('');
        test_fragment('<div></div>');
        test_fragment('\n', '');


        //============================================================
        // Support Indent Level Options and Base Indent Autodetection - ()
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - ()');
        test_fragment('   a');
        test_fragment(
            '   <div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '   <div>\n' +
            '       <p>This is my sentence.</p>\n' +
            '   </div>');
        test_fragment(
            '   // This is a random comment\n' +
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '   // This is a random comment\n' +
            '   <div>\n' +
            '       <p>This is my sentence.</p>\n' +
            '   </div>');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")');
        opts.indent_level = 0;
        test_fragment('   a');
        test_fragment(
            '   <div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '   <div>\n' +
            '       <p>This is my sentence.</p>\n' +
            '   </div>');
        test_fragment(
            '   // This is a random comment\n' +
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '   // This is a random comment\n' +
            '   <div>\n' +
            '       <p>This is my sentence.</p>\n' +
            '   </div>');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "1")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "1")');
        opts.indent_level = 1;
        test_fragment('   a', '    a');
        test_fragment(
            '   <div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '    <div>\n' +
            '        <p>This is my sentence.</p>\n' +
            '    </div>');
        test_fragment(
            '   // This is a random comment\n' +
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '    // This is a random comment\n' +
            '    <div>\n' +
            '        <p>This is my sentence.</p>\n' +
            '    </div>');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "2")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "2")');
        opts.indent_level = 2;
        test_fragment('a', '        a');
        test_fragment(
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '        <div>\n' +
            '            <p>This is my sentence.</p>\n' +
            '        </div>');
        test_fragment(
            '// This is a random comment\n' +
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '        // This is a random comment\n' +
            '        <div>\n' +
            '            <p>This is my sentence.</p>\n' +
            '        </div>');

        // Support Indent Level Options and Base Indent Autodetection - (indent_with_tabs = "true", indent_level = "2")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_with_tabs = "true", indent_level = "2")');
        opts.indent_with_tabs = true;
        opts.indent_level = 2;
        test_fragment('a', '\t\ta');
        test_fragment(
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '\t\t<div>\n' +
            '\t\t\t<p>This is my sentence.</p>\n' +
            '\t\t</div>');
        test_fragment(
            '// This is a random comment\n' +
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '\t\t// This is a random comment\n' +
            '\t\t<div>\n' +
            '\t\t\t<p>This is my sentence.</p>\n' +
            '\t\t</div>');

        // Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")
        reset_options();
        set_name('Support Indent Level Options and Base Indent Autodetection - (indent_level = "0")');
        opts.indent_level = 0;
        test_fragment('\t   a');
        test_fragment(
            '\t   <div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '\t   <div>\n' +
            '\t       <p>This is my sentence.</p>\n' +
            '\t   </div>');
        test_fragment(
            '\t   // This is a random comment\n' +
            '<div>\n' +
            '  <p>This is my sentence.</p>\n' +
            '</div>',
            //  -- output --
            '\t   // This is a random comment\n' +
            '\t   <div>\n' +
            '\t       <p>This is my sentence.</p>\n' +
            '\t   </div>');


        //============================================================
        // Custom Extra Liners (empty) - (extra_liners = "[]")
        reset_options();
        set_name('Custom Extra Liners (empty) - (extra_liners = "[]")');
        opts.extra_liners = [];
        test_fragment(
            '<html><head><meta></head><body><div><p>x</p></div></body></html>',
            //  -- output --
            '<html>\n' +
            '<head>\n' +
            '    <meta>\n' +
            '</head>\n' +
            '<body>\n' +
            '    <div>\n' +
            '        <p>x</p>\n' +
            '    </div>\n' +
            '</body>\n' +
            '</html>');


        //============================================================
        // Custom Extra Liners (default) - (extra_liners = "null")
        reset_options();
        set_name('Custom Extra Liners (default) - (extra_liners = "null")');
        opts.extra_liners = null;
        test_fragment(
            '<html><head></head><body></body></html>',
            //  -- output --
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');


        //============================================================
        // Custom Extra Liners (p, string) - (extra_liners = ""p,/p"")
        reset_options();
        set_name('Custom Extra Liners (p, string) - (extra_liners = ""p,/p"")');
        opts.extra_liners = 'p,/p';
        test_fragment(
            '<html><head><meta></head><body><div><p>x</p></div></body></html>',
            //  -- output --
            '<html>\n' +
            '<head>\n' +
            '    <meta>\n' +
            '</head>\n' +
            '<body>\n' +
            '    <div>\n' +
            '\n' +
            '        <p>x\n' +
            '\n' +
            '        </p>\n' +
            '    </div>\n' +
            '</body>\n' +
            '</html>');


        //============================================================
        // Custom Extra Liners (p) - (extra_liners = "["p", "/p"]")
        reset_options();
        set_name('Custom Extra Liners (p) - (extra_liners = "["p", "/p"]")');
        opts.extra_liners = ['p', '/p'];
        test_fragment(
            '<html><head><meta></head><body><div><p>x</p></div></body></html>',
            //  -- output --
            '<html>\n' +
            '<head>\n' +
            '    <meta>\n' +
            '</head>\n' +
            '<body>\n' +
            '    <div>\n' +
            '\n' +
            '        <p>x\n' +
            '\n' +
            '        </p>\n' +
            '    </div>\n' +
            '</body>\n' +
            '</html>');


        //============================================================
        // Tests for script and style Commented and cdata wapping (#1641)
        reset_options();
        set_name('Tests for script and style Commented and cdata wapping (#1641)');
        opts.templating = 'php';
        bth(
            '<style><!----></style>',
            //  -- output --
            '<style>\n' +
            '    <!--\n' +
            '    -->\n' +
            '</style>');
        bth(
            '<style><!--\n' +
            '--></style>',
            //  -- output --
            '<style>\n' +
            '    <!--\n' +
            '    -->\n' +
            '</style>');
        bth(
            '<style><!-- the rest of this   line is   ignored\n' +
            '\n' +
            '\n' +
            '\n' +
            '--></style>',
            //  -- output --
            '<style>\n' +
            '    <!-- the rest of this   line is   ignored\n' +
            '    -->\n' +
            '</style>');
        bth(
            '<style type="test/null"><!--\n' +
            '\n' +
            '\t  \n' +
            '\n' +
            '--></style>',
            //  -- output --
            '<style type="test/null">\n' +
            '    <!--\n' +
            '    -->\n' +
            '</style>');
        bth(
            '<script><!--\n' +
            'console.log("</script>" + "</style>");\n' +
            '--></script>',
            //  -- output --
            '<script>\n' +
            '    <!--\n' +
            '    console.log("</script>" + "</style>");\n' +
            '    -->\n' +
            '</script>');

        // If wrapping is incomplete, print remaining unchanged.
        test_fragment(
            '<div>\n' +
            '<script><!--\n' +
            'console.log("</script>" + "</style>");\n' +
            ' </script>\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <script><!--\n' +
            'console.log("</script>" + "</style>");\n' +
            ' </script>\n' +
            '</div>');
        bth(
            '<style><!--\n' +
            '.selector {\n' +
            '    font-family: "</script></style>";\n' +
            '    }\n' +
            '--></style>',
            //  -- output --
            '<style>\n' +
            '    <!--\n' +
            '    .selector {\n' +
            '        font-family: "</script></style>";\n' +
            '    }\n' +
            '    -->\n' +
            '</style>');
        bth(
            '<script type="test/null">\n' +
            '    <!--\n' +
            '   console.log("</script>" + "</style>");\n' +
            '    console.log("</script>" + "</style>");\n' +
            '--></script>',
            //  -- output --
            '<script type="test/null">\n' +
            '    <!--\n' +
            '    console.log("</script>" + "</style>");\n' +
            '     console.log("</script>" + "</style>");\n' +
            '    -->\n' +
            '</script>');
        bth(
            '<script type="test/null"><!--\n' +
            ' console.log("</script>" + "</style>");\n' +
            '      console.log("</script>" + "</style>");\n' +
            '--></script>',
            //  -- output --
            '<script type="test/null">\n' +
            '    <!--\n' +
            '    console.log("</script>" + "</style>");\n' +
            '         console.log("</script>" + "</style>");\n' +
            '    -->\n' +
            '</script>');
        bth(
            '<script><![CDATA[\n' +
            'console.log("</script>" + "</style>");\n' +
            ']]></script>',
            //  -- output --
            '<script>\n' +
            '    <![CDATA[\n' +
            '    console.log("</script>" + "</style>");\n' +
            '    ]]>\n' +
            '</script>');
        bth(
            '<style><![CDATA[\n' +
            '.selector {\n' +
            '    font-family: "</script></style>";\n' +
            '    }\n' +
            ']]></style>',
            //  -- output --
            '<style>\n' +
            '    <![CDATA[\n' +
            '    .selector {\n' +
            '        font-family: "</script></style>";\n' +
            '    }\n' +
            '    ]]>\n' +
            '</style>');
        bth(
            '<script type="test/null">\n' +
            '    <![CDATA[\n' +
            '   console.log("</script>" + "</style>");\n' +
            '    console.log("</script>" + "</style>");\n' +
            ']]></script>',
            //  -- output --
            '<script type="test/null">\n' +
            '    <![CDATA[\n' +
            '    console.log("</script>" + "</style>");\n' +
            '     console.log("</script>" + "</style>");\n' +
            '    ]]>\n' +
            '</script>');
        bth(
            '<script type="test/null"><![CDATA[\n' +
            ' console.log("</script>" + "</style>");\n' +
            '      console.log("</script>" + "</style>");\n' +
            ']]></script>',
            //  -- output --
            '<script type="test/null">\n' +
            '    <![CDATA[\n' +
            '    console.log("</script>" + "</style>");\n' +
            '         console.log("</script>" + "</style>");\n' +
            '    ]]>\n' +
            '</script>');

        // Issue #1687 - start with <?php ?>
        bth(
            '<script>\n' +
            '<?php ?>\n' +
            'var b={\n' +
            'a:function _test( ){\n' +
            'return 1;\n' +
            '}\n' +
            '}\n' +
            '</script>',
            //  -- output --
            '<script>\n' +
            '    <?php ?>\n' +
            '    var b = {\n' +
            '        a: function _test() {\n' +
            '            return 1;\n' +
            '        }\n' +
            '    }\n' +
            '</script>');


        //============================================================
        // Tests for script and style types (issue 453, 821)
        reset_options();
        set_name('Tests for script and style types (issue 453, 821)');
        bth('<script type="text/unknown"><div></div></script>');
        bth('<script type="text/unknown">Blah Blah Blah</script>');
        bth('<script type="text/unknown">    Blah Blah Blah   </script>', '<script type="text/unknown"> Blah Blah Blah   </script>');
        bth(
            '<script type="text/javascript"><div></div></script>',
            //  -- output --
            '<script type="text/javascript">\n' +
            '    < div > < /div>\n' +
            '</script>');
        bth(
            '<script><div></div></script>',
            //  -- output --
            '<script>\n' +
            '    < div > < /div>\n' +
            '</script>');

        // text/html should beautify as html
        bth(
            '<script type="text/html">\n' +
            '<div>\n' +
            '<div></div><div></div></div></script>',
            //  -- output --
            '<script type="text/html">\n' +
            '    <div>\n' +
            '        <div></div>\n' +
            '        <div></div>\n' +
            '    </div>\n' +
            '</script>');

        // null beatifier behavior - should still indent
        test_fragment(
            '<script type="test/null">\n' +
            '    <div>\n' +
            '  <div></div><div></div></div></script>',
            //  -- output --
            '<script type="test/null">\n' +
            '    <div>\n' +
            '      <div></div><div></div></div>\n' +
            '</script>');
        bth(
            '<script type="test/null">\n' +
            '   <div>\n' +
            '     <div></div><div></div></div></script>',
            //  -- output --
            '<script type="test/null">\n' +
            '    <div>\n' +
            '      <div></div><div></div></div>\n' +
            '</script>');
        bth(
            '<script type="test/null">\n' +
            '<div>\n' +
            '<div></div><div></div></div></script>',
            //  -- output --
            '<script type="test/null">\n' +
            '    <div>\n' +
            '    <div></div><div></div></div>\n' +
            '</script>');
        bth(
            '<script>var foo = "bar";</script>',
            //  -- output --
            '<script>\n' +
            '    var foo = "bar";\n' +
            '</script>');

        // Issue #1606 - type attribute on other element
        bth(
            '<script>\n' +
            'console.log(1  +  1);\n' +
            '</script>\n' +
            '\n' +
            '<input type="submit"></input>',
            //  -- output --
            '<script>\n' +
            '    console.log(1 + 1);\n' +
            '</script>\n' +
            '\n' +
            '<input type="submit"></input>');
        bth(
            '<script type="text/javascript">console.log(1  +  1);</script>',
            //  -- output --
            '<script type="text/javascript">\n' +
            '    console.log(1 + 1);\n' +
            '</script>');

        // Issue #1706 - es script module
        bth(
            '<script type="module">console.log(1  +  1);</script>',
            //  -- output --
            '<script type="module">\n' +
            '    console.log(1 + 1);\n' +
            '</script>');
        bth(
            '<script type="application/javascript">var foo = "bar";</script>',
            //  -- output --
            '<script type="application/javascript">\n' +
            '    var foo = "bar";\n' +
            '</script>');
        bth(
            '<script type="application/javascript;version=1.8">var foo = "bar";</script>',
            //  -- output --
            '<script type="application/javascript;version=1.8">\n' +
            '    var foo = "bar";\n' +
            '</script>');
        bth(
            '<script type="application/x-javascript">var foo = "bar";</script>',
            //  -- output --
            '<script type="application/x-javascript">\n' +
            '    var foo = "bar";\n' +
            '</script>');
        bth(
            '<script type="application/ecmascript">var foo = "bar";</script>',
            //  -- output --
            '<script type="application/ecmascript">\n' +
            '    var foo = "bar";\n' +
            '</script>');
        bth(
            '<script type="dojo/aspect">this.domNode.style.display="none";</script>',
            //  -- output --
            '<script type="dojo/aspect">\n' +
            '    this.domNode.style.display = "none";\n' +
            '</script>');
        bth(
            '<script type="dojo/method">this.domNode.style.display="none";</script>',
            //  -- output --
            '<script type="dojo/method">\n' +
            '    this.domNode.style.display = "none";\n' +
            '</script>');
        bth(
            '<script type="text/javascript1.5">var foo = "bar";</script>',
            //  -- output --
            '<script type="text/javascript1.5">\n' +
            '    var foo = "bar";\n' +
            '</script>');
        bth(
            '<script type="application/json">{"foo":"bar"}</script>',
            //  -- output --
            '<script type="application/json">\n' +
            '    {\n' +
            '        "foo": "bar"\n' +
            '    }\n' +
            '</script>');
        bth(
            '<script type="application/ld+json">{"foo":"bar"}</script>',
            //  -- output --
            '<script type="application/ld+json">\n' +
            '    {\n' +
            '        "foo": "bar"\n' +
            '    }\n' +
            '</script>');
        bth('<style type="text/unknown"><tag></tag></style>');
        bth(
            '<style type="text/css"><tag></tag></style>',
            //  -- output --
            '<style type="text/css">\n' +
            '    <tag></tag>\n' +
            '</style>');
        bth(
            '<style><tag></tag></style>',
            //  -- output --
            '<style>\n' +
            '    <tag></tag>\n' +
            '</style>');
        bth(
            '<style>.selector {font-size:12px;}</style>',
            //  -- output --
            '<style>\n' +
            '    .selector {\n' +
            '        font-size: 12px;\n' +
            '    }\n' +
            '</style>');
        bth(
            '<style type="text/css">.selector {font-size:12px;}</style>',
            //  -- output --
            '<style type="text/css">\n' +
            '    .selector {\n' +
            '        font-size: 12px;\n' +
            '    }\n' +
            '</style>');


        //============================================================
        // Attribute Wrap alignment with spaces and tabs - (wrap_attributes = ""force-aligned"", indent_with_tabs = "true")
        reset_options();
        set_name('Attribute Wrap alignment with spaces and tabs - (wrap_attributes = ""force-aligned"", indent_with_tabs = "true")');
        opts.wrap_attributes = 'force-aligned';
        opts.indent_with_tabs = true;
        test_fragment(
            '<div><div a="1" b="2"><div>test</div></div></div>',
            //  -- output --
            '<div>\n' +
            '\t<div a="1"\n' +
            '\t\t b="2">\n' +
            '\t\t<div>test</div>\n' +
            '\t</div>\n' +
            '</div>');
        test_fragment(
            '<input type="radio"\n' +
            '\t   name="garage"\n' +
            '\t   id="garage-02"\n' +
            '\t   class="ns-e-togg__radio ns-js-form-binding"\n' +
            '\t   value="02"\n' +
            '\t   {{#ifCond data.antragsart "05"}}\n' +
            '\t   checked="checked"\n' +
            '\t   {{/ifCond}}>');
        test_fragment(
            '<div>\n' +
            '\t<input type="radio"\n' +
            '\t\t   name="garage"\n' +
            '\t\t   id="garage-02"\n' +
            '\t\t   class="ns-e-togg__radio ns-js-form-binding"\n' +
            '\t\t   value="02"\n' +
            '\t\t   {{#ifCond data.antragsart "05"}}\n' +
            '\t\t   checked="checked"\n' +
            '\t\t   {{/ifCond}}>\n' +
            '</div>');
        test_fragment(
            '---\n' +
            'layout: mainLayout.html\n' +
            'page: default.html\n' +
            '---\n' +
            '\n' +
            '<div>\n' +
            '\t{{> componentXYZ my.data.key}}\n' +
            '\t{{> componentABC my.other.data.key}}\n' +
            '\t<span>Hello World</span>\n' +
            '\t<p>Your paragraph</p>\n' +
            '</div>');

        // Attribute Wrap alignment with spaces and tabs - (wrap_attributes = ""force"", indent_with_tabs = "true")
        reset_options();
        set_name('Attribute Wrap alignment with spaces and tabs - (wrap_attributes = ""force"", indent_with_tabs = "true")');
        opts.wrap_attributes = 'force';
        opts.indent_with_tabs = true;
        test_fragment(
            '<div><div a="1" b="2"><div>test</div></div></div>',
            //  -- output --
            '<div>\n' +
            '\t<div a="1"\n' +
            '\t\tb="2">\n' +
            '\t\t<div>test</div>\n' +
            '\t</div>\n' +
            '</div>');
        test_fragment(
            '<input type="radio"\n' +
            '\tname="garage"\n' +
            '\tid="garage-02"\n' +
            '\tclass="ns-e-togg__radio ns-js-form-binding"\n' +
            '\tvalue="02"\n' +
            '\t{{#ifCond data.antragsart "05"}}\n' +
            '\tchecked="checked"\n' +
            '\t{{/ifCond}}>');
        test_fragment(
            '<div>\n' +
            '\t<input type="radio"\n' +
            '\t\tname="garage"\n' +
            '\t\tid="garage-02"\n' +
            '\t\tclass="ns-e-togg__radio ns-js-form-binding"\n' +
            '\t\tvalue="02"\n' +
            '\t\t{{#ifCond data.antragsart "05"}}\n' +
            '\t\tchecked="checked"\n' +
            '\t\t{{/ifCond}}>\n' +
            '</div>');
        test_fragment(
            '---\n' +
            'layout: mainLayout.html\n' +
            'page: default.html\n' +
            '---\n' +
            '\n' +
            '<div>\n' +
            '\t{{> componentXYZ my.data.key}}\n' +
            '\t{{> componentABC my.other.data.key}}\n' +
            '\t<span>Hello World</span>\n' +
            '\t<p>Your paragraph</p>\n' +
            '</div>');


        //============================================================
        // Attribute Wrap de-indent - (wrap_attributes = ""force-aligned"", indent_with_tabs = "false")
        reset_options();
        set_name('Attribute Wrap de-indent - (wrap_attributes = ""force-aligned"", indent_with_tabs = "false")');
        opts.wrap_attributes = 'force-aligned';
        opts.indent_with_tabs = false;
        bth(
            '<div a="1" b="2"><div>test</div></div>',
            //  -- output --
            '<div a="1"\n' +
            '     b="2">\n' +
            '    <div>test</div>\n' +
            '</div>');
        bth(
            '<p>\n' +
            '    <a href="/test/" target="_blank"><img src="test.jpg" /></a><a href="/test/" target="_blank"><img src="test.jpg" /></a>\n' +
            '</p>',
            //  -- output --
            '<p>\n' +
            '    <a href="/test/"\n' +
            '       target="_blank"><img src="test.jpg" /></a><a href="/test/"\n' +
            '       target="_blank"><img src="test.jpg" /></a>\n' +
            '</p>');
        bth(
            '<p>\n' +
            '    <span data-not-a-href="/test/" data-totally-not-a-target="_blank"><img src="test.jpg" /></span><span data-not-a-href="/test/" data-totally-not-a-target="_blank"><img src="test.jpg" /></span>\n' +
            '</p>',
            //  -- output --
            '<p>\n' +
            '    <span data-not-a-href="/test/"\n' +
            '          data-totally-not-a-target="_blank"><img src="test.jpg" /></span><span data-not-a-href="/test/"\n' +
            '          data-totally-not-a-target="_blank"><img src="test.jpg" /></span>\n' +
            '</p>');


        //============================================================
        // Issue #1403 -- no extra newlines in force-aligned wrap_attributes - (wrap_attributes = ""force-aligned"")
        reset_options();
        set_name('Issue #1403 -- no extra newlines in force-aligned wrap_attributes - (wrap_attributes = ""force-aligned"")');
        opts.wrap_attributes = 'force-aligned';
        test_fragment(
            '<button class="btn btn-primary" ng-click="shipment.editSendDate = false;sampleTracking.updateShipmentDates({shipment_id: shipment.shipment_id, sent_timestamp: shipment.sending_date})" type="button">Save</button>',
            //  -- output --
            '<button class="btn btn-primary"\n' +
            '        ng-click="shipment.editSendDate = false;sampleTracking.updateShipmentDates({shipment_id: shipment.shipment_id, sent_timestamp: shipment.sending_date})"\n' +
            '        type="button">Save</button>');


        //============================================================
        // unformatted_content_delimiter ^^
        reset_options();
        set_name('unformatted_content_delimiter ^^');
        opts.wrap_line_length = 80;
        opts.unformatted_content_delimiter = '^^';
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 ^^09 0010 0011 0012 0013 0014 0015 ^^16 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008\n' +
            '    ^^09 0010 0011 0012 0013 0014 0015 ^^16 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   ^^10   0011   0012   0013   0014   0015   0016   0^^7   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009\n' +
            '    ^^10   0011   0012   0013   0014   0015   0016   0^^7 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0^^0   0011   0012   0013   0014   0015   0016   0^^7   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0^^0 0011 0012 0013 0014\n' +
            '    0015 0016 0^^7 0018 0019 0020</span>');


        //============================================================
        // Attribute Wrap - (wrap_attributes = ""force"")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force"")');
        opts.wrap_attributes = 'force';
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo"\n' +
            '    attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1=12345\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12\n' +
            '    attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '    rel="stylesheet"\n' +
            '    type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""force"", wrap_line_length = "80")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force"", wrap_line_length = "80")');
        opts.wrap_attributes = 'force';
        opts.wrap_line_length = 80;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013\n' +
            '    0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010\n' +
            '    <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013\n' +
            '    0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic\n' +
            '    <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment(
            '<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>',
            //  -- output --
            '<p>Our forms for collecting address-related information follow a standard\n' +
            '    design. Specific input elements willl vary according to the form’s audience\n' +
            '    and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo"\n' +
            '    attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1=12345\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12\n' +
            '    attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '    href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '    rel="stylesheet"\n' +
            '    type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""force"", wrap_attributes_indent_size = "8")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force"", wrap_attributes_indent_size = "8")');
        opts.wrap_attributes = 'force';
        opts.wrap_attributes_indent_size = 8;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div attr0\n' +
            '        attr1="123"\n' +
            '        data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '        attr0\n' +
            '        attr1="123"\n' +
            '        data-attr2="hello    t here"\n' +
            '        heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img attr0\n' +
            '        attr1="123"\n' +
            '        data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo"\n' +
            '        attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '        attr0\n' +
            '        attr1=12345\n' +
            '        data-attr2="hello    t here"\n' +
            '        heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12\n' +
            '        attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '        rel="stylesheet"\n' +
            '        type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""auto"", wrap_line_length = "80", wrap_attributes_indent_size = "0")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""auto"", wrap_line_length = "80", wrap_attributes_indent_size = "0")');
        opts.wrap_attributes = 'auto';
        opts.wrap_line_length = 80;
        opts.wrap_attributes_indent_size = 0;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013\n' +
            '    0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010\n' +
            '    <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013\n' +
            '    0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic\n' +
            '    <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment(
            '<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>',
            //  -- output --
            '<p>Our forms for collecting address-related information follow a standard\n' +
            '    design. Specific input elements willl vary according to the form’s audience\n' +
            '    and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment('<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123"\n' +
            'data-attr2="hello    t here"\n' +
            'heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment('<img attr0 attr1="123" data-attr2="hello    t here"/>', '<img attr0 attr1="123" data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo" attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1=12345\n' +
            'data-attr2="hello    t here"\n' +
            'heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12 attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            'href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            'rel="stylesheet" type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""auto"", wrap_line_length = "80", wrap_attributes_indent_size = "4")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""auto"", wrap_line_length = "80", wrap_attributes_indent_size = "4")');
        opts.wrap_attributes = 'auto';
        opts.wrap_line_length = 80;
        opts.wrap_attributes_indent_size = 4;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013\n' +
            '    0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010\n' +
            '    <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013\n' +
            '    0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic\n' +
            '    <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment(
            '<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>',
            //  -- output --
            '<p>Our forms for collecting address-related information follow a standard\n' +
            '    design. Specific input elements willl vary according to the form’s audience\n' +
            '    and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment('<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment('<img attr0 attr1="123" data-attr2="hello    t here"/>', '<img attr0 attr1="123" data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo" attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1=12345\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12 attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '    href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '    rel="stylesheet" type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""auto"", wrap_line_length = "0")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""auto"", wrap_line_length = "0")');
        opts.wrap_attributes = 'auto';
        opts.wrap_line_length = 0;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment('<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>');
        test_fragment('<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment('<img attr0 attr1="123" data-attr2="hello    t here"/>', '<img attr0 attr1="123" data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo" attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment('<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>', '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1=12345 data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12 attr2="bar" />');
        bth('<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""force-aligned"")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-aligned"")');
        opts.wrap_attributes = 'force-aligned';
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '     attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo"\n' +
            '      attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '     attr0\n' +
            '     attr1=12345\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12\n' +
            '      attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '      rel="stylesheet"\n' +
            '      type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""force-aligned"", wrap_line_length = "80")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-aligned"", wrap_line_length = "80")');
        opts.wrap_attributes = 'force-aligned';
        opts.wrap_line_length = 80;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013\n' +
            '    0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010\n' +
            '    <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013\n' +
            '    0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic\n' +
            '    <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment(
            '<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>',
            //  -- output --
            '<p>Our forms for collecting address-related information follow a standard\n' +
            '    design. Specific input elements willl vary according to the form’s audience\n' +
            '    and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '     attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo"\n' +
            '      attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '     attr0\n' +
            '     attr1=12345\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12\n' +
            '      attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '      rel="stylesheet"\n' +
            '      type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""aligned-multiple"", wrap_line_length = "80")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""aligned-multiple"", wrap_line_length = "80")');
        opts.wrap_attributes = 'aligned-multiple';
        opts.wrap_line_length = 80;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013\n' +
            '    0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010\n' +
            '    <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013\n' +
            '    0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic\n' +
            '    <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment(
            '<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>',
            //  -- output --
            '<p>Our forms for collecting address-related information follow a standard\n' +
            '    design. Specific input elements willl vary according to the form’s audience\n' +
            '    and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment('<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123"\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment('<img attr0 attr1="123" data-attr2="hello    t here"/>', '<img attr0 attr1="123" data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo" attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1=12345\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12 attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '      rel="stylesheet" type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""aligned-multiple"")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""aligned-multiple"")');
        opts.wrap_attributes = 'aligned-multiple';
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment('<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>');
        test_fragment('<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment('<img attr0 attr1="123" data-attr2="hello    t here"/>', '<img attr0 attr1="123" data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo" attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment('<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>', '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1=12345 data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12 attr2="bar" />');
        bth('<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""force-aligned"", wrap_attributes_indent_size = "8")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-aligned"", wrap_attributes_indent_size = "8")');
        opts.wrap_attributes = 'force-aligned';
        opts.wrap_attributes_indent_size = 8;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here">This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '     attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img attr0\n' +
            '     attr1="123"\n' +
            '     data-attr2="hello    t here" />');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1="foo"\n' +
            '      attr2="bar" />');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '     attr0\n' +
            '     attr1=12345\n' +
            '     data-attr2="hello    t here"\n' +
            '     heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root attr1=foo12\n' +
            '      attr2="bar" />');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '      rel="stylesheet"\n' +
            '      type="text/css">');

        // Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "4")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "4")');
        opts.wrap_attributes = 'force-expand-multiline';
        opts.wrap_attributes_indent_size = 4;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '>This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '    lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '/>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '    attr1="foo"\n' +
            '    attr2="bar"\n' +
            '/>');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '    lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1=12345\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '    attr1=foo12\n' +
            '    attr2="bar"\n' +
            '/>');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '    href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '    rel="stylesheet"\n' +
            '    type="text/css"\n' +
            '>');

        // Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "4", wrap_line_length = "80")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "4", wrap_line_length = "80")');
        opts.wrap_attributes = 'force-expand-multiline';
        opts.wrap_attributes_indent_size = 4;
        opts.wrap_line_length = 80;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013\n' +
            '    0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010\n' +
            '    <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013\n' +
            '    0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic\n' +
            '    <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment(
            '<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>',
            //  -- output --
            '<p>Our forms for collecting address-related information follow a standard\n' +
            '    design. Specific input elements willl vary according to the form’s audience\n' +
            '    and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '>This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '    lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img\n' +
            '    attr0\n' +
            '    attr1="123"\n' +
            '    data-attr2="hello    t here"\n' +
            '/>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '    attr1="foo"\n' +
            '    attr2="bar"\n' +
            '/>');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '    lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '    attr0\n' +
            '    attr1=12345\n' +
            '    data-attr2="hello    t here"\n' +
            '    heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '    attr1=foo12\n' +
            '    attr2="bar"\n' +
            '/>');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '    href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '    rel="stylesheet"\n' +
            '    type="text/css"\n' +
            '>');

        // Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "8")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "8")');
        opts.wrap_attributes = 'force-expand-multiline';
        opts.wrap_attributes_indent_size = 8;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div\n' +
            '        attr0\n' +
            '        attr1="123"\n' +
            '        data-attr2="hello    t here"\n' +
            '>This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '        lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '        attr0\n' +
            '        attr1="123"\n' +
            '        data-attr2="hello    t here"\n' +
            '        heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img\n' +
            '        attr0\n' +
            '        attr1="123"\n' +
            '        data-attr2="hello    t here"\n' +
            '/>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '        attr1="foo"\n' +
            '        attr2="bar"\n' +
            '/>');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '        lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '        attr0\n' +
            '        attr1=12345\n' +
            '        data-attr2="hello    t here"\n' +
            '        heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '        attr1=foo12\n' +
            '        attr2="bar"\n' +
            '/>');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '        href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '        rel="stylesheet"\n' +
            '        type="text/css"\n' +
            '>');

        // Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "4", indent_with_tabs = "true")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "4", indent_with_tabs = "true")');
        opts.wrap_attributes = 'force-expand-multiline';
        opts.wrap_attributes_indent_size = 4;
        opts.indent_with_tabs = true;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div\n' +
            '\tattr0\n' +
            '\tattr1="123"\n' +
            '\tdata-attr2="hello    t here"\n' +
            '>This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '\tlookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '\tattr0\n' +
            '\tattr1="123"\n' +
            '\tdata-attr2="hello    t here"\n' +
            '\theymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img\n' +
            '\tattr0\n' +
            '\tattr1="123"\n' +
            '\tdata-attr2="hello    t here"\n' +
            '/>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '\tattr1="foo"\n' +
            '\tattr2="bar"\n' +
            '/>');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '\tlookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '\tattr0\n' +
            '\tattr1=12345\n' +
            '\tdata-attr2="hello    t here"\n' +
            '\theymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '\tattr1=foo12\n' +
            '\tattr2="bar"\n' +
            '/>');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '\thref="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '\trel="stylesheet"\n' +
            '\ttype="text/css"\n' +
            '>');

        // Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "7", indent_with_tabs = "true")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_attributes_indent_size = "7", indent_with_tabs = "true")');
        opts.wrap_attributes = 'force-expand-multiline';
        opts.wrap_attributes_indent_size = 7;
        opts.indent_with_tabs = true;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment('<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>', '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment('<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div\n' +
            '\t   attr0\n' +
            '\t   attr1="123"\n' +
            '\t   data-attr2="hello    t here"\n' +
            '>This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '\t   lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '\t   attr0\n' +
            '\t   attr1="123"\n' +
            '\t   data-attr2="hello    t here"\n' +
            '\t   heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img\n' +
            '\t   attr0\n' +
            '\t   attr1="123"\n' +
            '\t   data-attr2="hello    t here"\n' +
            '/>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '\t   attr1="foo"\n' +
            '\t   attr2="bar"\n' +
            '/>');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '\t   lookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '\t   attr0\n' +
            '\t   attr1=12345\n' +
            '\t   data-attr2="hello    t here"\n' +
            '\t   heymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '\t   attr1=foo12\n' +
            '\t   attr2="bar"\n' +
            '/>');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '\t   href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '\t   rel="stylesheet"\n' +
            '\t   type="text/css"\n' +
            '>');

        // Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_line_length = "80", indent_with_tabs = "true")
        reset_options();
        set_name('Attribute Wrap - (wrap_attributes = ""force-expand-multiline"", wrap_line_length = "80", indent_with_tabs = "true")');
        opts.wrap_attributes = 'force-expand-multiline';
        opts.wrap_line_length = 80;
        opts.indent_with_tabs = true;
        bth('<div  >This is some text</div>', '<div>This is some text</div>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '\t0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0   0001   0002   0003   0004   0005   0006   0007   0008   0009   0010   0011   0012   0013   0014   0015   0016   0017   0018   0019   0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '\t0015 0016 0017 0018 0019 0020</span>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\t0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '\t0015 0016 0017 0018 0019 0020</span>');

        // issue #869
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013\n' +
            '\t0014&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1324
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009  0010 <span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010\n' +
            '\t<span>&nbsp;</span>&nbsp;0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic 0013 0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic 0013\n' +
            '\t0014' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // issue #1496 and #1324 - respect unicode non-breaking space
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011  unic <span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 unic\n' +
            '\t<span>' + unicode_char(160) + '</span>' + unicode_char(160) + '0015 0016 0017 0018 0019 0020</span>');

        // Issue 1222 -- P tags are formatting correctly
        test_fragment(
            '<p>Our forms for collecting address-related information follow a standard design. Specific input elements willl vary according to the form’s audience and purpose.</p>',
            //  -- output --
            '<p>Our forms for collecting address-related information follow a standard\n' +
            '\tdesign. Specific input elements willl vary according to the form’s audience\n' +
            '\tand purpose.</p>');
        bth('<div attr="123"  >This is some text</div>', '<div attr="123">This is some text</div>');
        test_fragment(
            '<div attr0 attr1="123" data-attr2="hello    t here">This is some text</div>',
            //  -- output --
            '<div\n' +
            '\tattr0\n' +
            '\tattr1="123"\n' +
            '\tdata-attr2="hello    t here"\n' +
            '>This is some text</div>');
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0="true" attr0 attr1="123" data-attr2="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '\tlookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '\tattr0\n' +
            '\tattr1="123"\n' +
            '\tdata-attr2="hello    t here"\n' +
            '\theymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<img attr0 attr1="123" data-attr2="hello    t here"/>',
            //  -- output --
            '<img\n' +
            '\tattr0\n' +
            '\tattr1="123"\n' +
            '\tdata-attr2="hello    t here"\n' +
            '/>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1="foo" attr2="bar"/>',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '\tattr1="foo"\n' +
            '\tattr2="bar"\n' +
            '/>');

        // Issue #1094 - Beautify correctly without quotes and with extra spaces
        test_fragment(
            '<div lookatthissuperduperlongattributenamewhoahcrazy0 =    "true" attr0 attr1=  12345 data-attr2   ="hello    t here" heymanimreallylongtoowhocomesupwiththesenames="false">This is text</div>',
            //  -- output --
            '<div\n' +
            '\tlookatthissuperduperlongattributenamewhoahcrazy0="true"\n' +
            '\tattr0\n' +
            '\tattr1=12345\n' +
            '\tdata-attr2="hello    t here"\n' +
            '\theymanimreallylongtoowhocomesupwiththesenames="false"\n' +
            '>This is text</div>');
        test_fragment(
            '<?xml version="1.0" encoding="UTF-8" ?><root attr1   =   foo12   attr2  ="bar"    />',
            //  -- output --
            '<?xml version="1.0" encoding="UTF-8" ?>\n' +
            '<root\n' +
            '\tattr1=foo12\n' +
            '\tattr2="bar"\n' +
            '/>');
        bth(
            '<link href="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin" rel="stylesheet" type="text/css">',
            //  -- output --
            '<link\n' +
            '\thref="//fonts.googleapis.com/css?family=Open+Sans:300italic,400italic,600italic,700italic,400,600,700,300&amp;subset=latin"\n' +
            '\trel="stylesheet"\n' +
            '\ttype="text/css"\n' +
            '>');


        //============================================================
        // Issue #1335 -- <button> Bug with force-expand-multiline formatting
        reset_options();
        set_name('Issue #1335 -- <button> Bug with force-expand-multiline formatting');
        opts.wrap_attributes = 'force-expand-multiline';
        test_fragment(
            '<button\n' +
            '    class="my-class"\n' +
            '    id="id1"\n' +
            '>\n' +
            '    Button 1\n' +
            '</button>\n' +
            '\n' +
            '<button\n' +
            '    class="my-class"\n' +
            '    id="id2"\n' +
            '>\n' +
            '    Button 2\n' +
            '</button>');
        bth(
            '<button>\n' +
            '    <span>foo</span>\n' +
            '<p>bar</p>\n' +
            '</button>',
            //  -- output --
            '<button>\n' +
            '    <span>foo</span>\n' +
            '    <p>bar</p>\n' +
            '</button>');


        //============================================================
        // Issue #1125 -- Add preserve and preserve_aligned attribute options - (wrap_attributes = ""preserve-aligned"")
        reset_options();
        set_name('Issue #1125 -- Add preserve and preserve_aligned attribute options - (wrap_attributes = ""preserve-aligned"")');
        opts.wrap_attributes = 'preserve-aligned';
        bth(
            '<input type="text"     class="form-control"  autocomplete="off"\n' +
            '[(ngModel)]="myValue"          [disabled]="isDisabled" [placeholder]="placeholder"\n' +
            '[typeahead]="suggestionsSource" [typeaheadOptionField]="suggestionValueField" [typeaheadItemTemplate]="suggestionTemplate"   [typeaheadWaitMs]="300"\n' +
            '(typeaheadOnSelect)="onSuggestionSelected($event)" />',
            //  -- output --
            '<input type="text" class="form-control" autocomplete="off"\n' +
            '       [(ngModel)]="myValue" [disabled]="isDisabled" [placeholder]="placeholder"\n' +
            '       [typeahead]="suggestionsSource" [typeaheadOptionField]="suggestionValueField" [typeaheadItemTemplate]="suggestionTemplate" [typeaheadWaitMs]="300"\n' +
            '       (typeaheadOnSelect)="onSuggestionSelected($event)" />');

        // Issue #1125 -- Add preserve and preserve_aligned attribute options - (wrap_attributes = ""preserve"")
        reset_options();
        set_name('Issue #1125 -- Add preserve and preserve_aligned attribute options - (wrap_attributes = ""preserve"")');
        opts.wrap_attributes = 'preserve';
        bth(
            '<input type="text"     class="form-control"  autocomplete="off"\n' +
            '[(ngModel)]="myValue"          [disabled]="isDisabled" [placeholder]="placeholder"\n' +
            '[typeahead]="suggestionsSource" [typeaheadOptionField]="suggestionValueField" [typeaheadItemTemplate]="suggestionTemplate"   [typeaheadWaitMs]="300"\n' +
            '(typeaheadOnSelect)="onSuggestionSelected($event)" />',
            //  -- output --
            '<input type="text" class="form-control" autocomplete="off"\n' +
            '    [(ngModel)]="myValue" [disabled]="isDisabled" [placeholder]="placeholder"\n' +
            '    [typeahead]="suggestionsSource" [typeaheadOptionField]="suggestionValueField" [typeaheadItemTemplate]="suggestionTemplate" [typeaheadWaitMs]="300"\n' +
            '    (typeaheadOnSelect)="onSuggestionSelected($event)" />');


        //============================================================
        // Handlebars Indenting Off
        reset_options();
        set_name('Handlebars Indenting Off');
        opts.indent_handlebars = false;
        test_fragment(
            '{{#if 0}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 0}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}');
        test_fragment(
            '<div>\n' +
            '{{#each thing}}\n' +
            '    {{name}}\n' +
            '{{/each}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#each thing}}\n' +
            '    {{name}}\n' +
            '    {{/each}}\n' +
            '</div>');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '   {{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. Just treated as text here.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '<div>\n' +
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '    {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '    {{em-input label="Place*" property="place" type="text" placeholder=""}}\n' +
            '</div>');
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{translate "onText"}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{translate "offText"}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '{{translate "onText"}}\n' +
            '{{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '{{translate "offText"}}\n' +
            '{{/if}}');


        //============================================================
        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{field}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{field}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{field}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{field}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{field}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{field}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{field}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{field}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{field}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{field}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{field}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{field}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}{{field}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{field}}{{/if}}',
            //  -- output --
            '{{#if words}}{{field}}{{/if}}');
        bth(
            '{{#if     words}}{{field}}{{/if}}',
            //  -- output --
            '{{#if words}}{{field}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{field}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{field}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{field}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{field}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{field}}\n' +
            '    {{else}}\n' +
            '    {{field}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{field}}\n' +
            '{{else}}\n' +
            '    {{field}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{field}}\n' +
            '    {{else}}\n' +
            '{{field}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{field}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{field}}\n' +
            '    {{else}}\n' +
            '        {{field}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{field}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{field}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{field}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{field}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{field}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{field}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{field}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{field}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{field}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{field}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{field}}</span>');
        bth('<{{ele}} unformatted="{{#if}}{{field}}{{/if}}">{{field}}</{{ele}}>');
        bth('<div unformatted="{{#if}}{{field}}{{/if}}">{{field}}</div>');
        bth('<div unformatted="{{#if  }}    {{field}}{{/if}}">{{field}}</div>');
        bth('<div class="{{#if thingIs "value"}}{{field}}{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}{{field}}{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}{{field}}{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}{{field}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}',
            //  -- output --
            '{{#if words}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}');
        bth(
            '{{#if     words}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}',
            //  -- output --
            '{{#if words}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '    {{else}}\n' +
            '    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{else}}\n' +
            '    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '    {{else}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '    {{else}}\n' +
            '        {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</span>');
        bth('<{{ele}} unformatted="{{#if}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}">{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</{{ele}}>');
        bth('<div unformatted="{{#if}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}">{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</div>');
        bth('<div unformatted="{{#if  }}    {{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}">{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}</div>');
        bth('<div class="{{#if thingIs "value"}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! comment}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! comment}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! comment}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! comment}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! comment}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! comment}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! comment}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! comment}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{! comment}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{! comment}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{! comment}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{! comment}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}{{! comment}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{! comment}}{{/if}}',
            //  -- output --
            '{{#if words}}{{! comment}}{{/if}}');
        bth(
            '{{#if     words}}{{! comment}}{{/if}}',
            //  -- output --
            '{{#if words}}{{! comment}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{! comment}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{! comment}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{! comment}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{! comment}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{! comment}}\n' +
            '    {{else}}\n' +
            '    {{! comment}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{! comment}}\n' +
            '{{else}}\n' +
            '    {{! comment}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{! comment}}\n' +
            '    {{else}}\n' +
            '{{! comment}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{! comment}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{! comment}}\n' +
            '    {{else}}\n' +
            '        {{! comment}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{! comment}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{! comment}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{! comment}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{! comment}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{! comment}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{! comment}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{! comment}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{! comment}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{! comment}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{! comment}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{! comment}}</span>');
        bth('<{{ele}} unformatted="{{#if}}{{! comment}}{{/if}}">{{! comment}}</{{ele}}>');
        bth('<div unformatted="{{#if}}{{! comment}}{{/if}}">{{! comment}}</div>');
        bth('<div unformatted="{{#if  }}    {{! comment}}{{/if}}">{{! comment}}</div>');
        bth('<div class="{{#if thingIs "value"}}{{! comment}}{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}{{! comment}}{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}{{! comment}}{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}{{! comment}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- comment--}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- comment--}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- comment--}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- comment--}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- comment--}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- comment--}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- comment--}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- comment--}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{!-- comment--}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{!-- comment--}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{!-- comment--}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{!-- comment--}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}{{!-- comment--}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{!-- comment--}}{{/if}}',
            //  -- output --
            '{{#if words}}{{!-- comment--}}{{/if}}');
        bth(
            '{{#if     words}}{{!-- comment--}}{{/if}}',
            //  -- output --
            '{{#if words}}{{!-- comment--}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{!-- comment--}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{!-- comment--}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{!-- comment--}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{!-- comment--}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{!-- comment--}}\n' +
            '    {{else}}\n' +
            '    {{!-- comment--}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{!-- comment--}}\n' +
            '{{else}}\n' +
            '    {{!-- comment--}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{!-- comment--}}\n' +
            '    {{else}}\n' +
            '{{!-- comment--}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{!-- comment--}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{!-- comment--}}\n' +
            '    {{else}}\n' +
            '        {{!-- comment--}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{!-- comment--}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{!-- comment--}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{!-- comment--}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{!-- comment--}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{!-- comment--}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{!-- comment--}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{!-- comment--}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{!-- comment--}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{!-- comment--}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{!-- comment--}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{!-- comment--}}</span>');
        bth('<{{ele}} unformatted="{{#if}}{{!-- comment--}}{{/if}}">{{!-- comment--}}</{{ele}}>');
        bth('<div unformatted="{{#if}}{{!-- comment--}}{{/if}}">{{!-- comment--}}</div>');
        bth('<div unformatted="{{#if  }}    {{!-- comment--}}{{/if}}">{{!-- comment--}}</div>');
        bth('<div class="{{#if thingIs "value"}}{{!-- comment--}}{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}{{!-- comment--}}{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}{{!-- comment--}}{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}{{!-- comment--}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{{unescaped_variable}}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{{unescaped_variable}}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{{unescaped_variable}}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{{unescaped_variable}}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{{unescaped_variable}}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{{unescaped_variable}}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{{unescaped_variable}}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{{unescaped_variable}}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{{unescaped_variable}}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{{unescaped_variable}}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{{unescaped_variable}}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{{unescaped_variable}}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}{{{unescaped_variable}}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{{unescaped_variable}}}{{/if}}',
            //  -- output --
            '{{#if words}}{{{unescaped_variable}}}{{/if}}');
        bth(
            '{{#if     words}}{{{unescaped_variable}}}{{/if}}',
            //  -- output --
            '{{#if words}}{{{unescaped_variable}}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{{unescaped_variable}}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{{unescaped_variable}}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{{unescaped_variable}}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{{unescaped_variable}}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{{unescaped_variable}}}\n' +
            '    {{else}}\n' +
            '    {{{unescaped_variable}}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{{unescaped_variable}}}\n' +
            '{{else}}\n' +
            '    {{{unescaped_variable}}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{{unescaped_variable}}}\n' +
            '    {{else}}\n' +
            '{{{unescaped_variable}}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{{unescaped_variable}}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{{unescaped_variable}}}\n' +
            '    {{else}}\n' +
            '        {{{unescaped_variable}}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{{unescaped_variable}}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{{unescaped_variable}}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{{unescaped_variable}}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{{unescaped_variable}}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{{unescaped_variable}}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{{unescaped_variable}}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{{unescaped_variable}}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{{unescaped_variable}}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{{unescaped_variable}}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{{unescaped_variable}}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{{unescaped_variable}}}</span>');
        bth('<{{ele}} unformatted="{{#if}}{{{unescaped_variable}}}{{/if}}">{{{unescaped_variable}}}</{{ele}}>');
        bth('<div unformatted="{{#if}}{{{unescaped_variable}}}{{/if}}">{{{unescaped_variable}}}</div>');
        bth('<div unformatted="{{#if  }}    {{{unescaped_variable}}}{{/if}}">{{{unescaped_variable}}}</div>');
        bth('<div class="{{#if thingIs "value"}}{{{unescaped_variable}}}{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}{{{unescaped_variable}}}{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}{{{unescaped_variable}}}{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}{{{unescaped_variable}}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}',
            //  -- output --
            '{{#if words}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}');
        bth(
            '{{#if     words}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}',
            //  -- output --
            '{{#if words}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '    {{else}}\n' +
            '    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{else}}\n' +
            '    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '    {{else}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '    {{else}}\n' +
            '        {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</span>');
        bth('<{{ele}} unformatted="{{#if}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}">{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</{{ele}}>');
        bth('<div unformatted="{{#if}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}">{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</div>');
        bth('<div unformatted="{{#if  }}    {{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}">{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}</div>');
        bth('<div class="{{#if thingIs "value"}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}{{Hello "woRld"}} {{!-- comment--}} {{heLloWorlD}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}',
            //  -- output --
            '{{#if words}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}');
        bth(
            '{{#if     words}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}',
            //  -- output --
            '{{#if words}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {pre{{field1}} {{field2}} {{field3}}post\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {pre{{field1}} {{field2}} {{field3}}post\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {pre{{field1}} {{field2}} {{field3}}post\n' +
            '    {{else}}\n' +
            '    {pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{else}}\n' +
            '    {pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {pre{{field1}} {{field2}} {{field3}}post\n' +
            '    {{else}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {pre{{field1}} {{field2}} {{field3}}post\n' +
            '    {{else}}\n' +
            '        {pre{{field1}} {{field2}} {{field3}}post\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {pre{{field1}} {{field2}} {{field3}}post\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{pre{{field1}} {{field2}} {{field3}}post\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {pre{{field1}} {{field2}} {{field3}}post\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {pre{{field1}} {{field2}} {{field3}}post\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{pre{{field1}} {{field2}} {{field3}}post</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{pre{{field1}} {{field2}} {{field3}}post</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{pre{{field1}} {{field2}} {{field3}}post</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{pre{{field1}} {{field2}} {{field3}}post</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{pre{{field1}} {{field2}} {{field3}}post</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{pre{{field1}} {{field2}} {{field3}}post</span>');
        bth('<{{ele}} unformatted="{{#if}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}">{pre{{field1}} {{field2}} {{field3}}post</{{ele}}>');
        bth('<div unformatted="{{#if}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}">{pre{{field1}} {{field2}} {{field3}}post</div>');
        bth('<div unformatted="{{#if  }}    {pre{{field1}} {{field2}} {{field3}}post{{/if}}">{pre{{field1}} {{field2}} {{field3}}post</div>');
        bth('<div class="{{#if thingIs "value"}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}{pre{{field1}} {{field2}} {{field3}}post{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth(
            '{{#if 0}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}',
            //  -- output --
            '{{#if words}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}');
        bth(
            '{{#if     words}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}',
            //  -- output --
            '{{#if words}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '    {{else}}\n' +
            '    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{else}}\n' +
            '    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '    {{else}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '    {{else}}\n' +
            '        {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</span>');
        bth(
            '<{{ele}} unformatted="{{#if}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}">{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</{{ele}}>');
        bth(
            '<div unformatted="{{#if}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}">{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</div>');
        bth(
            '<div unformatted="{{#if  }}    {{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}">{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}</div>');
        bth(
            '<div class="{{#if thingIs "value"}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}"></div>');
        bth(
            '<div class="{{#if thingIs \'value\'}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}"></div>');
        bth(
            '<div class=\'{{#if thingIs "value"}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}\'></div>');
        bth(
            '<div class=\'{{#if thingIs \'value\'}}{{! \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth(
            '{{#if 0}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}',
            //  -- output --
            '{{#if words}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}');
        bth(
            '{{#if     words}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}',
            //  -- output --
            '{{#if words}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '    {{else}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{else}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '    {{else}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '    {{else}}\n' +
            '        {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</span>');
        bth(
            '<{{ele}} unformatted="{{#if}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}">{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</{{ele}}>');
        bth(
            '<div unformatted="{{#if}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}">{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</div>');
        bth(
            '<div unformatted="{{#if  }}    {{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}">{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}</div>');
        bth(
            '<div class="{{#if thingIs "value"}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}"></div>');
        bth(
            '<div class="{{#if thingIs \'value\'}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}"></div>');
        bth(
            '<div class=\'{{#if thingIs "value"}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}\'></div>');
        bth(
            '<div class=\'{{#if thingIs \'value\'}}{{!-- \n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            '--}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true")');
        opts.indent_handlebars = true;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth(
            '{{#if 0}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}',
            //  -- output --
            '{{#if words}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}');
        bth(
            '{{#if     words}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}',
            //  -- output --
            '{{#if words}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '    {{else}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{else}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '    {{else}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '    {{else}}\n' +
            '        {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            '{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}" {{else}}class="{{class2}}" {{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</span>');
        bth(
            '<{{ele}} unformatted="{{#if}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}">{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</{{ele}}>');
        bth(
            '<div unformatted="{{#if}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}">{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</div>');
        bth(
            '<div unformatted="{{#if  }}    {{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}">{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}</div>');
        bth(
            '<div class="{{#if thingIs "value"}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}"></div>');
        bth(
            '<div class="{{#if thingIs \'value\'}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}"></div>');
        bth(
            '<div class=\'{{#if thingIs "value"}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}\'></div>');
        bth(
            '<div class=\'{{#if thingIs \'value\'}}{{!-- \n' +
            ' mult-line\n' +
            'comment \n' +
            '{{#> component}}\n' +
            ' mult-line\n' +
            'comment  \n' +
            '     with spacing\n' +
            ' {{/ component}}--}}{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');

        // Handlebars Indenting On - (indent_handlebars = "true", wrap_line_length = "80")
        reset_options();
        set_name('Handlebars Indenting On - (indent_handlebars = "true", wrap_line_length = "80")');
        opts.indent_handlebars = true;
        opts.wrap_line_length = 80;
        bth('{{page-title}}');
        bth(
            '{{page-title}}\n' +
            '{{a}}\n' +
            '{{value-title}}');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            'content\n' +
            '{{#if condition}}\n' +
            '    <div  class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            'content',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            'content\n' +
            '{{#if condition}}\n' +
            '    <div class="some">{{helper "hello"}}<strong>{{helper "world"}}</strong>\n' +
            '    </div>\n' +
            '{{/if}}\n' +
            'content');
        bth(
            '{{textarea value=someContent}}\n' +
            '\n' +
            'content\n' +
            '{{#if condition}}\n' +
            '    <div  class="some-class-detail">{{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong></div>\n' +
            '{{/if}}\n' +
            'content',
            //  -- output --
            '{{textarea value=someContent}}\n' +
            '\n' +
            'content\n' +
            '{{#if condition}}\n' +
            '    <div class="some-class-detail">\n' +
            '        {{helper "hello"}}<strong>{{helper "world"}}</strong>{{helper "hello"}}<strong>{{helper "world"}}</strong>\n' +
            '    </div>\n' +
            '{{/if}}\n' +
            'content');

        // error case
        bth(
            '{{page-title}}\n' +
            '{{ myHelper someValue}}\n' +
            'content\n' +
            '{{value-title}}');

        // Issue #1469 - preserve newlines inside handlebars, including first one. BUG: does not fix indenting inside handlebars.
        bth(
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            'content\n' +
            '   {{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '       {{em-input label="Place*" property="place" type="text" placeholder=""}}',
            //  -- output --
            '{{em-input\n' +
            '  label="Some Labe" property="amt"\n' +
            '  type="text" placeholder=""}}\n' +
            'content\n' +
            '{{em-input label="Type*"\n' +
            'property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth(
            '{{em-input label="Some Labe" property="amt" type="text" placeholder=""}}\n' +
            'content\n' +
            '{{em-input label="Type*" property="type" type="text" placeholder="(LTD)"}}\n' +
            '{{em-input label="Place*" property="place" type="text" placeholder=""}}');
        bth('{{#if 0}}{{/if}}');
        bth('{{#if 0}}content{{/if}}');
        bth(
            '{{#if 0}}\n' +
            '{{/if}}');
        bth(
            '{{#if     words}}{{/if}}',
            //  -- output --
            '{{#if words}}{{/if}}');
        bth(
            '{{#if     words}}content{{/if}}',
            //  -- output --
            '{{#if words}}content{{/if}}');
        bth(
            '{{#if     words}}content{{/if}}',
            //  -- output --
            '{{#if words}}content{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '<div>\n' +
            '</div>\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '{{#if 1}}\n' +
            '{{/if}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    {{#if 1}}\n' +
            '    {{/if}}\n' +
            '</div>');
        bth(
            '{{#if}}\n' +
            '{{#each}}\n' +
            '{{#if}}\n' +
            'content\n' +
            '{{/if}}\n' +
            '{{#if}}\n' +
            'content\n' +
            '{{/if}}\n' +
            '{{/each}}\n' +
            '{{/if}}',
            //  -- output --
            '{{#if}}\n' +
            '    {{#each}}\n' +
            '        {{#if}}\n' +
            '            content\n' +
            '        {{/if}}\n' +
            '        {{#if}}\n' +
            '            content\n' +
            '        {{/if}}\n' +
            '    {{/each}}\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    <div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '    <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '<div>\n' +
            '    <small>SMALL TEXT</small>\n' +
            '    <span>\n' +
            '        {{#if isOwner}}\n' +
            '            <span><i class="fa fa-close"></i></span>\n' +
            '        {{else}}\n' +
            '            <span><i class="fa fa-bolt"></i></span>\n' +
            '        {{/if}}\n' +
            '    </span>\n' +
            '    <strong>{{userName}}:&nbsp;</strong>{{text}}\n' +
            '</div>');
        bth(
            '{{#if `this.customerSegment == "Active"`}}\n' +
            '    ...\n' +
            '{{/if}}');
        bth(
            '{{#isDealLink}}\n' +
            '&nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}',
            //  -- output --
            '{{#isDealLink}}\n' +
            '    &nbsp;&nbsp;<a target="_blank" href="{{dealLink}}" class="weak">See</a>\n' +
            '{{/isDealLink}}');
        bth(
            '{{#if 1}}\n' +
            '    content\n' +
            '    {{else}}\n' +
            '    content\n' +
            '{{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '    content\n' +
            '{{else}}\n' +
            '    content\n' +
            '{{/if}}');
        bth(
            '{{#if 1}}\n' +
            '    {{else}}\n' +
            '    {{/if}}',
            //  -- output --
            '{{#if 1}}\n' +
            '{{else}}\n' +
            '{{/if}}');
        bth(
            '{{#if thing}}\n' +
            '{{#if otherthing}}\n' +
            '    content\n' +
            '    {{else}}\n' +
            'content\n' +
            '    {{/if}}\n' +
            '       {{else}}\n' +
            'content\n' +
            '{{/if}}',
            //  -- output --
            '{{#if thing}}\n' +
            '    {{#if otherthing}}\n' +
            '        content\n' +
            '    {{else}}\n' +
            '        content\n' +
            '    {{/if}}\n' +
            '{{else}}\n' +
            '    content\n' +
            '{{/if}}');

        // ISSUE #800 and #1123: else if and #unless
        bth(
            '{{#if callOn}}\n' +
            '{{#unless callOn}}\n' +
            '      content\n' +
            '   {{else}}\n' +
            '{{translate "offText"}}\n' +
            '{{/unless callOn}}\n' +
            '   {{else if (eq callOn false)}}\n' +
            'content\n' +
            '        {{/if}}',
            //  -- output --
            '{{#if callOn}}\n' +
            '    {{#unless callOn}}\n' +
            '        content\n' +
            '    {{else}}\n' +
            '        {{translate "offText"}}\n' +
            '    {{/unless callOn}}\n' +
            '{{else if (eq callOn false)}}\n' +
            '    content\n' +
            '{{/if}}');
        bth(
            '<div {{someStyle}}>  </div>',
            //  -- output --
            '<div {{someStyle}}> </div>');

        // only partial support for complex templating in attributes
        bth(
            '<dIv {{#if test}}class="foo"{{/if}}>content</dIv>',
            //  -- output --
            '<dIv {{#if test}}class="foo" {{/if}}>content</dIv>');
        test_fragment(
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"{{else}}class="{{class2}}"{{/if}}>content</diV>',
            //  -- output --
            '<diV {{#if thing}}{{somestyle}}class_spacing_for="{{class}}"\n' +
            '    {{else}}class="{{class2}}" {{/if}}>content</diV>');

        // partiial support for templating in attributes
        bth(
            '<span {{#if condition}}class="foo"{{/if}}>content</span>',
            //  -- output --
            '<span {{#if condition}}class="foo" {{/if}}>content</span>');
        bth('<{{ele}} unformatted="{{#if}}content{{/if}}">content</{{ele}}>');
        bth('<div unformatted="{{#if}}content{{/if}}">content</div>');
        bth('<div unformatted="{{#if  }}    content{{/if}}">content</div>');
        bth('<div class="{{#if thingIs "value"}}content{{/if}}"></div>');
        bth('<div class="{{#if thingIs \'value\'}}content{{/if}}"></div>');
        bth('<div class=\'{{#if thingIs "value"}}content{{/if}}\'></div>');
        bth('<div class=\'{{#if thingIs \'value\'}}content{{/if}}\'></div>');
        bth('<span>{{condition < 0 ? "result1" : "result2"}}</span>');
        bth('<span>{{condition1 && condition2 && condition3 && condition4 < 0 ? "resForTrue" : "resForFalse"}}</span>');


        //============================================================
        // Handlebars Else If, Each, and Inverted Section tag indenting
        reset_options();
        set_name('Handlebars Else If, Each, and Inverted Section tag indenting');
        opts.indent_handlebars = true;
        bth(
            '{{#if test}}<div></div>{{else}}<div></div>{{/if}}',
            //  -- output --
            '{{#if test}}\n' +
            '    <div></div>\n' +
            '{{else}}\n' +
            '    <div></div>\n' +
            '{{/if}}');
        bth('{{#if test}}<span></span>{{else}}<span></span>{{/if}}');
        bth(
            '<a class="navbar-brand">\n' +
            '    {{#if connected}}\n' +
            '        <i class="fa fa-link" style="color:green"></i> {{else if sleep}}\n' +
            '        <i class="fa fa-sleep" style="color:yellow"></i>\n' +
            '    {{else}}\n' +
            '        <i class="fa fa-unlink" style="color:red"></i>\n' +
            '    {{/if}}\n' +
            '</a>',
            //  -- output --
            '<a class="navbar-brand">\n' +
            '    {{#if connected}}\n' +
            '        <i class="fa fa-link" style="color:green"></i>\n' +
            '    {{else if sleep}}\n' +
            '        <i class="fa fa-sleep" style="color:yellow"></i>\n' +
            '    {{else}}\n' +
            '        <i class="fa fa-unlink" style="color:red"></i>\n' +
            '    {{/if}}\n' +
            '</a>');
        bth(
            '{{#each clinics as |clinic|}}\n' +
            '    <p>{{clinic.name}}</p>\n' +
            '{{else}}\n' +
            '    <p>Unfortunately no clinics found.</p>\n' +
            '{{/each}}');

        // Issue #1623 - Fix indentation of `^` inverted section tags in Handlebars/Mustache code
        bth(
            '{{^inverted-condition}}\n' +
            '    <p>Unfortunately this condition is false.</p>\n' +
            '{{/inverted-condition}}');

        // Issue #1756 - Fix indentation of partials
        bth(
            '{{#*inline "myPartial"}}\n' +
            '    <p>Unfortunately this condition is false.</p>\n' +
            '{{/inline}}');
        bth(
            '{{#> myPartial}}\n' +
            '    <p>Unfortunately this condition is false.</p>\n' +
            '{{/myPartial}}');

        // Issue #1946 - Indentation of partial blocks with whitespace following partial name
        bth(
            '{{#> myPartial }}\n' +
            '    <p>Unfortunately this condition is false.</p>\n' +
            '{{/myPartial}}');

        // Issue #1946 - Indentation of partial blocks with parameters
        bth(
            '{{#> myPartial param="test"}}\n' +
            '    <p>Unfortunately this condition is false.</p>\n' +
            '{{/myPartial}}');

        // Issue #1946 - Indentation of inline partials with parameters
        bth(
            '{{#*inline "myPartial" param="test"}}\n' +
            '    <p>Unfortunately this condition is false.</p>\n' +
            '{{/inline}}');


        //============================================================
        // Unclosed html elements
        reset_options();
        set_name('Unclosed html elements');
        bth(
            '<source>\n' +
            '<source>');
        bth(
            '<br>\n' +
            '<br>');
        bth(
            '<input>\n' +
            '<input>');
        bth(
            '<meta>\n' +
            '<meta>');
        bth(
            '<link>\n' +
            '<link>');
        bth(
            '<colgroup>\n' +
            '    <col>\n' +
            '    <col>\n' +
            '</colgroup>');
        bth(
            '<source>\n' +
            '    <source>',
            //  -- output --
            '<source>\n' +
            '<source>');
        bth(
            '<br>\n' +
            '    <br>',
            //  -- output --
            '<br>\n' +
            '<br>');
        bth(
            '<input>\n' +
            '    <input>',
            //  -- output --
            '<input>\n' +
            '<input>');
        bth(
            '<meta>\n' +
            '    <meta>',
            //  -- output --
            '<meta>\n' +
            '<meta>');
        bth(
            '<link>\n' +
            '    <link>',
            //  -- output --
            '<link>\n' +
            '<link>');
        bth(
            '<colgroup>\n' +
            '        <col>\n' +
            '        <col>\n' +
            '</colgroup>',
            //  -- output --
            '<colgroup>\n' +
            '    <col>\n' +
            '    <col>\n' +
            '</colgroup>');


        //============================================================
        // Optional html elements
        reset_options();
        set_name('Optional html elements');
        test_fragment(
            '<li>test content\n' +
            '<li>test content\n' +
            '<li>test content');
        bth(
            '<ol>\n' +
            '    <li>test content\n' +
            '    <li>test content\n' +
            '    <li>test content\n' +
            '</ol>');
        bth(
            '<ol>\n' +
            '    <li>\n' +
            '        test content\n' +
            '    <li>\n' +
            '        <ul>\n' +
            '            <li> extra text\n' +
            '            <li> depth check\n' +
            '        </ul>\n' +
            '    <li> test content\n' +
            '    <li>\n' +
            '        test content\n' +
            '</ol>');
        bth(
            '<dl>\n' +
            '    <dt>\n' +
            '        test content\n' +
            '    <dt>\n' +
            '        test content\n' +
            '    <dd>\n' +
            '        test content\n' +
            '    <dd>\n' +
            '        test content\n' +
            '    <dt>\n' +
            '        test content\n' +
            '    <dd>\n' +
            '        <dl>\n' +
            '            <dt>\n' +
            '                test content\n' +
            '            <dt>\n' +
            '                test content\n' +
            '            <dd>\n' +
            '                test content\n' +
            '        </dl>\n' +
            '</dl>');
        bth(
            '<select>\n' +
            '    <optgroup>\n' +
            '        test content\n' +
            '    <optgroup>\n' +
            '        test content\n' +
            '        <option>\n' +
            '            test content\n' +
            '        <option>\n' +
            '            test content\n' +
            '    <optgroup>\n' +
            '        test content\n' +
            '        <option>\n' +
            '            <p>test content\n' +
            '        <option>\n' +
            '            test content\n' +
            '</select>');

        // Regression test for #1649
        bth(
            '<table>\n' +
            '    <tbody>\n' +
            '        <tr>\n' +
            '            <td>\n' +
            '                <table>\n' +
            '                    <thead>\n' +
            '                        <th>\n' +
            '                        </th>\n' +
            '                    </thead>\n' +
            '                    <tbody>\n' +
            '                        <tr>\n' +
            '                            <td>\n' +
            '                            </td>\n' +
            '                        </tr>\n' +
            '                    </tbody>\n' +
            '                </table>\n' +
            '            </td>\n' +
            '        </tr>\n' +
            '    </tbody>\n' +
            '</table>');
        bth(
            '<table>\n' +
            '    <caption>37547 TEE Electric Powered Rail Car Train Functions (Abbreviated)\n' +
            '    <colgroup>\n' +
            '        <col>\n' +
            '        <col>\n' +
            '        <col>\n' +
            '    <thead>\n' +
            '        <tr>\n' +
            '            <th>Function\n' +
            '            <th>Control Unit\n' +
            '            <th>Central Station\n' +
            '    <tbody>\n' +
            '        <tr>\n' +
            '            <td>Headlights\n' +
            '            <td>✔\n' +
            '            <td>✔\n' +
            '        <tr>\n' +
            '            <td>Interior Lights\n' +
            '            <td>✔\n' +
            '            <td>✔\n' +
            '        <tr>\n' +
            '            <td>Electric locomotive operating sounds\n' +
            '            <td>✔\n' +
            '                <table>\n' +
            '                    <caption>37547 TEE Electric Powered Rail Car Train Functions (Abbreviated)\n' +
            '                    <colgroup>\n' +
            '                        <col>\n' +
            '                        <col>\n' +
            '                        <col>\n' +
            '                    <thead>\n' +
            '                        <tr>\n' +
            '                            <th>Function\n' +
            '                            <th>\n' +
            '                                <p>Control Unit\n' +
            '                            <th>Central Station\n' +
            '                    <tbody>\n' +
            '                        <tr>\n' +
            '                            <td>Headlights\n' +
            '                            <td>✔\n' +
            '                            <td>✔\n' +
            '                        <tr>\n' +
            '                            <td>Interior Lights\n' +
            '                            <td>✔\n' +
            '                            <td>✔\n' +
            '                        <tr>\n' +
            '                            <td>Electric locomotive operating sounds\n' +
            '                            <td>✔\n' +
            '                            <td>✔\n' +
            '                        <tr>\n' +
            '                            <td>Engineer’s cab lighting\n' +
            '                            <td>\n' +
            '                            <td>✔\n' +
            '                        <tr>\n' +
            '                            <td>Station Announcements - Swiss\n' +
            '                            <td>\n' +
            '                            <td>✔\n' +
            '                    <tfoot>\n' +
            '                        <tr>\n' +
            '                            <td>Station Announcements - Swiss\n' +
            '                            <td>\n' +
            '                            <td>✔\n' +
            '                </table>\n' +
            '            <td>✔\n' +
            '        <tr>\n' +
            '            <td>Engineer’s cab lighting\n' +
            '            <td>\n' +
            '            <td>✔\n' +
            '        <tr>\n' +
            '            <td>Station Announcements - Swiss\n' +
            '            <td>\n' +
            '            <td>✔\n' +
            '    <tfoot>\n' +
            '        <tr>\n' +
            '            <td>Station Announcements - Swiss\n' +
            '            <td>\n' +
            '            <td>✔\n' +
            '</table>');

        // Regression test for #1213
        bth(
            '<ul><li>ab<li>cd</li><li>cd</li></ul><dl><dt>ef<dt>gh</dt><dt>gh</dt></dl>\n' +
            '<ul><li>ab</li><li>cd<li>cd</li></ul><dl><dt>ef</dt><dt>gh<dt>gh</dt></dl>',
            //  -- output --
            '<ul>\n' +
            '    <li>ab\n' +
            '    <li>cd</li>\n' +
            '    <li>cd</li>\n' +
            '</ul>\n' +
            '<dl>\n' +
            '    <dt>ef\n' +
            '    <dt>gh</dt>\n' +
            '    <dt>gh</dt>\n' +
            '</dl>\n' +
            '<ul>\n' +
            '    <li>ab</li>\n' +
            '    <li>cd\n' +
            '    <li>cd</li>\n' +
            '</ul>\n' +
            '<dl>\n' +
            '    <dt>ef</dt>\n' +
            '    <dt>gh\n' +
            '    <dt>gh</dt>\n' +
            '</dl>');

        // P element optional closing tag - #1503
        bth(
            '<p><p><dl><dt>ef<dt><p>gh</dt><dt>gh</dt></dl><p><h3>headers are outside paragraphs</h3>\n' +
            '<p>.<textarea><p><p>.</textarea><textarea><p><p>.</textarea><p>.<p>.</p>',
            //  -- output --
            '<p>\n' +
            '<p>\n' +
            '<dl>\n' +
            '    <dt>ef\n' +
            '    <dt>\n' +
            '        <p>gh\n' +
            '    </dt>\n' +
            '    <dt>gh</dt>\n' +
            '</dl>\n' +
            '<p>\n' +
            '<h3>headers are outside paragraphs</h3>\n' +
            '<p>.<textarea><p><p>.</textarea><textarea><p><p>.</textarea>\n' +
            '<p>.\n' +
            '<p>.</p>');


        //============================================================
        // Unformatted tags
        reset_options();
        set_name('Unformatted tags');
        bth(
            '<ol>\n' +
            '    <li>b<pre>c</pre></li>\n' +
            '</ol>',
            //  -- output --
            '<ol>\n' +
            '    <li>b\n' +
            '        <pre>c</pre>\n' +
            '    </li>\n' +
            '</ol>');
        bth(
            '<ol>\n' +
            '    <li>b<code>c</code></li>\n' +
            '</ol>');
        bth(
            '<ul>\n' +
            '    <li>\n' +
            '        <span class="octicon octicon-person"></span>\n' +
            '        <a href="/contact/">Kontakt</a>\n' +
            '    </li>\n' +
            '</ul>');
        bth('<div class="searchform"><input type="text" value="" name="s" id="s" /><input type="submit" id="searchsubmit" value="Search" /></div>');
        bth('<div class="searchform"><input type="text" value="" name="s" id="s"><input type="submit" id="searchsubmit" value="Search"></div>');
        bth(
            '<p>\n' +
            '    <a href="/test/"><img src="test.jpg" /></a>\n' +
            '</p>');
        bth(
            '<p>\n' +
            '    <a href="/test/"><img src="test.jpg" /></a><a href="/test/"><img src="test.jpg" /></a>\n' +
            '</p>');
        bth(
            '<p>\n' +
            '    <a href="/test/"><img src="test.jpg" /></a><a href="/test/"><img src="test.jpg" /></a><a href="/test/"><img src="test.jpg" /></a><a href="/test/"><img src="test.jpg" /></a>\n' +
            '</p>');
        bth(
            '<p>\n' +
            '    <span>image: <img src="test.jpg" /></span><span>image: <img src="test.jpg" /></span>\n' +
            '</p>');
        bth(
            '<p>\n' +
            '    <strong>image: <img src="test.jpg" /></strong><strong>image: <img src="test.jpg" /></strong>\n' +
            '</p>');


        //============================================================
        // File starting with comment
        reset_options();
        set_name('File starting with comment');
        bth(
            '<!--sample comment -->\n' +
            '\n' +
            '<html>\n' +
            '<body>\n' +
            '    <span>a span</span>\n' +
            '</body>\n' +
            '\n' +
            '</html>');


        //============================================================
        // ISSUE #545 and #944 Ignore directive works in html
        reset_options();
        set_name('ISSUE #545 and #944 Ignore directive works in html');

        // ignore starts _after_ the start comment, ends after the end comment
        bth(
            '<div>\n' +
            '    <!-- beautify ignore:start -->\n' +
            '@{\n' +
            '\n' +
            '    ViewBag.Title = "Dashboard";\n' +
            '    string firstName = string.Empty;\n' +
            '    string userId = ViewBag.UserId;\n' +
            '\n' +
            '    if( !string.IsNullOrEmpty(ViewBag.FirstName ) ) {\n' +
            '\n' +
            '         firstName = "<h2>Hi " + ViewBag.FirstName + "</h2>";\n' +
            '\n' +
            '    }\n' +
            '\n' +
            '}\n' +
            ' <!-- beautify ignore:end -->\n' +
            '\n' +
            '    <header class="layout-header">\n' +
            '\n' +
            '        <h2 id="logo"><a href="/">Logo</a></h2>\n' +
            '\n' +
            '        <ul class="social">\n' +
            '\n' +
            '            <li class="facebook"><a href="#">Facebook</a></li>\n' +
            '            <li class="twitter"><a href="#">Twitter</a></li>\n' +
            '\n' +
            '        </ul>\n' +
            '\n' +
            '    </header>\n' +
            '</div>');


        //============================================================
        // Issue 1478 - Space handling inside self closing tag
        reset_options();
        set_name('Issue 1478 - Space handling inside self closing tag');
        bth(
            '<div>\n' +
            '    <br/>\n' +
            '    <br />\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <br />\n' +
            '    <br />\n' +
            '</div>');


        //============================================================
        // Single line comment after closing tag
        reset_options();
        set_name('Single line comment after closing tag');
        bth(
            '<div class="col">\n' +
            '    <div class="row">\n' +
            '        <div class="card">\n' +
            '\n' +
            '            <h1>Some heading</h1>\n' +
            '            <p>Some text for the card.</p>\n' +
            '            <img src="some/image.jpg" alt="">\n' +
            '\n' +
            '            </div>    <!-- /.card -->\n' +
            '    </div>\n' +
            '            <!-- /.row -->\n' +
            '</div> <!-- /.col -->',
            //  -- output --
            '<div class="col">\n' +
            '    <div class="row">\n' +
            '        <div class="card">\n' +
            '\n' +
            '            <h1>Some heading</h1>\n' +
            '            <p>Some text for the card.</p>\n' +
            '            <img src="some/image.jpg" alt="">\n' +
            '\n' +
            '        </div> <!-- /.card -->\n' +
            '    </div>\n' +
            '    <!-- /.row -->\n' +
            '</div> <!-- /.col -->');


        //============================================================
        // Regression Tests
        reset_options();
        set_name('Regression Tests');

        // #1202
        bth('<a class="js-open-move-from-header" href="#">5A - IN-SPRINT TESTING</a>');
        test_fragment('<a ">9</a">');
        bth('<a href="javascript:;" id="_h_url_paid_pro3" onmousedown="_h_url_click_paid_pro(this);" rel="nofollow" class="pro-title" itemprop="name">WA GlassKote</a>');
        bth('<a href="/b/yergey-brewing-a-beer-has-no-name/1745600">"A Beer Has No Name"</a>');

        // #1304
        bth('<label>Every</label><input class="scheduler__minutes-input" type="text">');

        // #1377
        bth(
            '<a href=\'\' onclick=\'doIt("<?php echo str_replace("\'", "\\ ", $var); ?>  "); \'>\n' +
            '    Test\n' +
            '</a>\n' +
            '\n' +
            '<?php include_once $_SERVER[\'DOCUMENT_ROOT\'] . "/shared/helpModal.php";  ?>');

        // #1736 - unquoted attribute with slashes
        bth(
            '<div>\n' +
            '    <a href=http://www.example.com></a>\n' +
            '</div>');


        //============================================================
        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        bth('<h1  class="content-page-header"><?php$view["name"]; ?></h1>', '<h1 class="content-page-header"><?php$view["name"]; ?></h1>');
        bth(
            '<?php\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '?>');
        test_fragment(
            '<?php ?>\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '<?php "A" ?>abc<?php "D" ?>\n' +
            '<?php "B" ?>\n' +
            '<?php "C" ?>');
        bth(
            '<?php\n' +
            'echo "A";\n' +
            '?>\n' +
            '<span>Test</span>');
        bth('<<?php html_element(); ?> <?phplanguage_attributes();?>>abc</<?php html_element(); ?>>');
        bth('<input type="text" value="<?php$x["test"] . $x[\'test\']?>">');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        bth('<h1  class="content-page-header"><?=$view["name"]; ?></h1>', '<h1 class="content-page-header"><?=$view["name"]; ?></h1>');
        bth(
            '<?=\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '?>');
        test_fragment(
            '<?= ?>\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '<?= "A" ?>abc<?= "D" ?>\n' +
            '<?= "B" ?>\n' +
            '<?= "C" ?>');
        bth(
            '<?=\n' +
            'echo "A";\n' +
            '?>\n' +
            '<span>Test</span>');
        bth('<<?= html_element(); ?> <?=language_attributes();?>>abc</<?= html_element(); ?>>');
        bth('<input type="text" value="<?=$x["test"] . $x[\'test\']?>">');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        bth('<h1  class="content-page-header"><? $view["name"]; ?></h1>', '<h1 class="content-page-header"><? $view["name"]; ?></h1>');
        bth(
            '<? \n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '?>');
        test_fragment(
            '<?  ?>\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '<?  "A" ?>abc<?  "D" ?>\n' +
            '<?  "B" ?>\n' +
            '<?  "C" ?>');
        bth(
            '<? \n' +
            'echo "A";\n' +
            '?>\n' +
            '<span>Test</span>');
        bth('<<?  html_element(); ?> <? language_attributes();?>>abc</<?  html_element(); ?>>');
        bth('<input type="text" value="<? $x["test"] . $x[\'test\']?>">');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        bth('<h1  class="content-page-header"><%$view["name"]; %></h1>', '<h1 class="content-page-header"><%$view["name"]; %></h1>');
        bth(
            '<%\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '%>');
        test_fragment(
            '<% %>\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '<% "A" %>abc<% "D" %>\n' +
            '<% "B" %>\n' +
            '<% "C" %>');
        bth(
            '<%\n' +
            'echo "A";\n' +
            '%>\n' +
            '<span>Test</span>');
        bth('<<% html_element(); %> <%language_attributes();%>>abc</<% html_element(); %>>');
        bth('<input type="text" value="<%$x["test"] . $x[\'test\']%>">');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        bth('<h1  class="content-page-header">{{$view["name"]; }}</h1>', '<h1 class="content-page-header">{{$view["name"]; }}</h1>');
        bth(
            '{{\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}}');
        test_fragment(
            '{{ }}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{{ "A" }}abc{{ "D" }}\n' +
            '{{ "B" }}\n' +
            '{{ "C" }}');
        bth(
            '{{\n' +
            'echo "A";\n' +
            '}}\n' +
            '<span>Test</span>');
        bth('<{{ html_element(); }} {{language_attributes();}}>abc</{{ html_element(); }}>');
        bth('<input type="text" value="{{$x["test"] . $x[\'test\']}}">');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        bth('<h1  class="content-page-header">{#$view["name"]; #}</h1>', '<h1 class="content-page-header">{#$view["name"]; #}</h1>');
        bth(
            '{#\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '#}');
        test_fragment(
            '{# #}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{# "A" #}abc{# "D" #}\n' +
            '{# "B" #}\n' +
            '{# "C" #}');
        bth(
            '{#\n' +
            'echo "A";\n' +
            '#}\n' +
            '<span>Test</span>');
        bth('<{# html_element(); #} {#language_attributes();#}>abc</{# html_element(); #}>');
        bth('<input type="text" value="{#$x["test"] . $x[\'test\']#}">');

        // minimal template handling - ()
        reset_options();
        set_name('minimal template handling - ()');
        bth('<h1  class="content-page-header">{%$view["name"]; %}</h1>', '<h1 class="content-page-header">{%$view["name"]; %}</h1>');
        bth(
            '{%\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '%}');
        test_fragment(
            '{% %}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{% "A" %}abc{% "D" %}\n' +
            '{% "B" %}\n' +
            '{% "C" %}');
        bth(
            '{%\n' +
            'echo "A";\n' +
            '%}\n' +
            '<span>Test</span>');
        bth('<{% html_element(); %} {%language_attributes();%}>abc</{% html_element(); %}>');
        bth('<input type="text" value="{%$x["test"] . $x[\'test\']%}">');

        // minimal template handling - (templating = ""smarty"")
        reset_options();
        set_name('minimal template handling - (templating = ""smarty"")');
        opts.templating = 'smarty';
        bth('<h1  class="content-page-header">{a$view["name"]; a}</h1>', '<h1 class="content-page-header">{a$view["name"]; a}</h1>');
        bth(
            '{a\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            'a}');
        test_fragment(
            '{a a}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{a "A" a}abc{a "D" a}\n' +
            '{a "B" a}\n' +
            '{a "C" a}');
        bth(
            '{a\n' +
            'echo "A";\n' +
            'a}\n' +
            '<span>Test</span>');
        bth('<{a html_element(); a} {alanguage_attributes();a}>abc</{a html_element(); a}>');
        bth('<input type="text" value="{a$x["test"] . $x[\'test\']a}">');

        // minimal template handling - (templating = ""smarty"")
        reset_options();
        set_name('minimal template handling - (templating = ""smarty"")');
        opts.templating = 'smarty';
        bth('<h1  class="content-page-header">{*$view["name"]; *}</h1>', '<h1 class="content-page-header">{*$view["name"]; *}</h1>');
        bth(
            '{*\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '*}');
        test_fragment(
            '{* *}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{* "A" *}abc{* "D" *}\n' +
            '{* "B" *}\n' +
            '{* "C" *}');
        bth(
            '{*\n' +
            'echo "A";\n' +
            '*}\n' +
            '<span>Test</span>');
        bth('<{* html_element(); *} {*language_attributes();*}>abc</{* html_element(); *}>');
        bth('<input type="text" value="{*$x["test"] . $x[\'test\']*}">');

        // minimal template handling - (templating = ""smarty"")
        reset_options();
        set_name('minimal template handling - (templating = ""smarty"")');
        opts.templating = 'smarty';
        bth('<h1  class="content-page-header">{literal}$view["name"]; {/literal}</h1>', '<h1 class="content-page-header">{literal}$view["name"]; {/literal}</h1>');
        bth(
            '{literal}\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '{/literal}');
        test_fragment(
            '{literal} {/literal}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{literal} "A" {/literal}abc{literal} "D" {/literal}\n' +
            '{literal} "B" {/literal}\n' +
            '{literal} "C" {/literal}');
        bth(
            '{literal}\n' +
            'echo "A";\n' +
            '{/literal}\n' +
            '<span>Test</span>');
        bth('<{literal} html_element(); {/literal} {literal}language_attributes();{/literal}>abc</{literal} html_element(); {/literal}>');
        bth('<input type="text" value="{literal}$x["test"] . $x[\'test\']{/literal}">');

        // minimal template handling - (indent_handlebars = "false")
        reset_options();
        set_name('minimal template handling - (indent_handlebars = "false")');
        opts.indent_handlebars = false;
        bth('<h1  class="content-page-header">{{$view["name"]; }}</h1>', '<h1 class="content-page-header">{{$view["name"]; }}</h1>');
        bth(
            '{{\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}}');
        test_fragment(
            '{{ }}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{{ "A" }}abc{{ "D" }}\n' +
            '{{ "B" }}\n' +
            '{{ "C" }}');
        bth(
            '{{\n' +
            'echo "A";\n' +
            '}}\n' +
            '<span>Test</span>');
        bth('<{{ html_element(); }} {{language_attributes();}}>abc</{{ html_element(); }}>');
        bth('<input type="text" value="{{$x["test"] . $x[\'test\']}}">');

        // minimal template handling - (indent_handlebars = "false")
        reset_options();
        set_name('minimal template handling - (indent_handlebars = "false")');
        opts.indent_handlebars = false;
        bth('<h1  class="content-page-header">{{#$view["name"]; }}</h1>', '<h1 class="content-page-header">{{#$view["name"]; }}</h1>');
        bth(
            '{{#\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}}');
        test_fragment(
            '{{# }}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{{# "A" }}abc{{# "D" }}\n' +
            '{{# "B" }}\n' +
            '{{# "C" }}');
        bth(
            '{{#\n' +
            'echo "A";\n' +
            '}}\n' +
            '<span>Test</span>');
        bth('<{{# html_element(); }} {{#language_attributes();}}>abc</{{# html_element(); }}>');
        bth('<input type="text" value="{{#$x["test"] . $x[\'test\']}}">');

        // minimal template handling - (indent_handlebars = "false")
        reset_options();
        set_name('minimal template handling - (indent_handlebars = "false")');
        opts.indent_handlebars = false;
        bth('<h1  class="content-page-header">{{!$view["name"]; }}</h1>', '<h1 class="content-page-header">{{!$view["name"]; }}</h1>');
        bth(
            '{{!\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '}}');
        test_fragment(
            '{{! }}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{{! "A" }}abc{{! "D" }}\n' +
            '{{! "B" }}\n' +
            '{{! "C" }}');
        bth(
            '{{!\n' +
            'echo "A";\n' +
            '}}\n' +
            '<span>Test</span>');
        bth('<{{! html_element(); }} {{!language_attributes();}}>abc</{{! html_element(); }}>');
        bth('<input type="text" value="{{!$x["test"] . $x[\'test\']}}">');

        // minimal template handling - (indent_handlebars = "false")
        reset_options();
        set_name('minimal template handling - (indent_handlebars = "false")');
        opts.indent_handlebars = false;
        bth('<h1  class="content-page-header">{{!--$view["name"]; --}}</h1>', '<h1 class="content-page-header">{{!--$view["name"]; --}}</h1>');
        bth(
            '{{!--\n' +
            'for($i = 1; $i <= 100; $i++;) {\n' +
            '    #count to 100!\n' +
            '    echo($i . "</br>");\n' +
            '}\n' +
            '--}}');
        test_fragment(
            '{{!-- --}}\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<head></head>\n' +
            '\n' +
            '<body></body>\n' +
            '\n' +
            '</html>');
        bth(
            '{{!-- "A" --}}abc{{!-- "D" --}}\n' +
            '{{!-- "B" --}}\n' +
            '{{!-- "C" --}}');
        bth(
            '{{!--\n' +
            'echo "A";\n' +
            '--}}\n' +
            '<span>Test</span>');
        bth('<{{!-- html_element(); --}} {{!--language_attributes();--}}>abc</{{!-- html_element(); --}}>');
        bth('<input type="text" value="{{!--$x["test"] . $x[\'test\']--}}">');


        //============================================================
        // Support simple language specific option inheritance/overriding - (js = "{ "indent_size": 3 }", css = "{ "indent_size": 5 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (js = "{ "indent_size": 3 }", css = "{ "indent_size": 5 }")');
        opts.js = { 'indent_size': 3 };
        opts.css = { 'indent_size': 5 };
        test_fragment(
            '<head>\n' +
            '    <script>\n' +
            '        if (a == b) {\n' +
            '           test();\n' +
            '        }\n' +
            '    </script>\n' +
            '    <style>\n' +
            '        .selector {\n' +
            '             font-size: 12px;\n' +
            '        }\n' +
            '    </style>\n' +
            '</head>');
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '    <script>\n' +
            '        if (a == b) {\n' +
            '           test();\n' +
            '        }\n' +
            '    </script>\n' +
            '    <style>\n' +
            '        .selector {\n' +
            '             font-size: 12px;\n' +
            '        }\n' +
            '    </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '    <script src="one.js"></script> <!-- one -->\n' +
            '    <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Support simple language specific option inheritance/overriding - (html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 } }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 } }")');
        opts.html = { 'js': { 'indent_size': 3 }, 'css': { 'indent_size': 5 } };
        test_fragment(
            '<head>\n' +
            '    <script>\n' +
            '        if (a == b) {\n' +
            '           test();\n' +
            '        }\n' +
            '    </script>\n' +
            '    <style>\n' +
            '        .selector {\n' +
            '             font-size: 12px;\n' +
            '        }\n' +
            '    </style>\n' +
            '</head>');
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '    <script>\n' +
            '        if (a == b) {\n' +
            '           test();\n' +
            '        }\n' +
            '    </script>\n' +
            '    <style>\n' +
            '        .selector {\n' +
            '             font-size: 12px;\n' +
            '        }\n' +
            '    </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '    <script src="one.js"></script> <!-- one -->\n' +
            '    <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Support simple language specific option inheritance/overriding - (indent_size = "9", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_size = "9", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")');
        opts.indent_size = 9;
        opts.js = { 'indent_size': 5 };
        opts.css = { 'indent_size': 3 };
        test_fragment(
            '<head>\n' +
            '         <script>\n' +
            '                  if (a == b) {\n' +
            '                       test();\n' +
            '                  }\n' +
            '         </script>\n' +
            '         <style>\n' +
            '                  .selector {\n' +
            '                     font-size: 12px;\n' +
            '                  }\n' +
            '         </style>\n' +
            '</head>');
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '         <script>\n' +
            '                  if (a == b) {\n' +
            '                       test();\n' +
            '                  }\n' +
            '         </script>\n' +
            '         <style>\n' +
            '                  .selector {\n' +
            '                     font-size: 12px;\n' +
            '                  }\n' +
            '         </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '         <script src="one.js"></script> <!-- one -->\n' +
            '         <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Support simple language specific option inheritance/overriding - (indent_size = "9", js = "{ "indent_size": 5, "disabled": true }", css = "{ "indent_size": 3 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_size = "9", js = "{ "indent_size": 5, "disabled": true }", css = "{ "indent_size": 3 }")');
        opts.indent_size = 9;
        opts.js = { 'indent_size': 5, 'disabled': true };
        opts.css = { 'indent_size': 3 };
        test_fragment(
            '<head>\n' +
            '         <script>\n' +
            '                  if (a == b) {\n' +
            '                       test();\n' +
            '                  }\n' +
            '         </script>\n' +
            '         <style>\n' +
            '                  .selector {\n' +
            '                     font-size: 12px;\n' +
            '                  }\n' +
            '         </style>\n' +
            '</head>');
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '         <script>\n' +
            '                  if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '         </script>\n' +
            '         <style>\n' +
            '                  .selector {\n' +
            '                     font-size: 12px;\n' +
            '                  }\n' +
            '         </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '         <script src="one.js"></script> <!-- one -->\n' +
            '         <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Support simple language specific option inheritance/overriding - (indent_size = "9", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3, "disabled": true }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_size = "9", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3, "disabled": true }")');
        opts.indent_size = 9;
        opts.js = { 'indent_size': 5 };
        opts.css = { 'indent_size': 3, 'disabled': true };
        test_fragment(
            '<head>\n' +
            '         <script>\n' +
            '                  if (a == b) {\n' +
            '                       test();\n' +
            '                  }\n' +
            '         </script>\n' +
            '         <style>\n' +
            '                  .selector {\n' +
            '                     font-size: 12px;\n' +
            '                  }\n' +
            '         </style>\n' +
            '</head>');
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '         <script>\n' +
            '                  if (a == b) {\n' +
            '                       test();\n' +
            '                  }\n' +
            '         </script>\n' +
            '         <style>\n' +
            '                  .selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '         </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '         <script src="one.js"></script> <!-- one -->\n' +
            '         <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Support simple language specific option inheritance/overriding - (indent_size = "9", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 }, "indent_size": 2}", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_size = "9", html = "{ "js": { "indent_size": 3 }, "css": { "indent_size": 5 }, "indent_size": 2}", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")');
        opts.indent_size = 9;
        opts.html = { 'js': { 'indent_size': 3 }, 'css': { 'indent_size': 5 }, 'indent_size': 2};
        opts.js = { 'indent_size': 5 };
        opts.css = { 'indent_size': 3 };
        test_fragment(
            '<head>\n' +
            '  <script>\n' +
            '    if (a == b) {\n' +
            '       test();\n' +
            '    }\n' +
            '  </script>\n' +
            '  <style>\n' +
            '    .selector {\n' +
            '         font-size: 12px;\n' +
            '    }\n' +
            '  </style>\n' +
            '</head>');
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '  <script>\n' +
            '    if (a == b) {\n' +
            '       test();\n' +
            '    }\n' +
            '  </script>\n' +
            '  <style>\n' +
            '    .selector {\n' +
            '         font-size: 12px;\n' +
            '    }\n' +
            '  </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '  <script src="one.js"></script> <!-- one -->\n' +
            '  <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Support simple language specific option inheritance/overriding - (indent_size = "9", html = "{ "js": { "indent_size": 3, "disabled": true }, "css": { "indent_size": 5 }, "indent_size": 2}", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")
        reset_options();
        set_name('Support simple language specific option inheritance/overriding - (indent_size = "9", html = "{ "js": { "indent_size": 3, "disabled": true }, "css": { "indent_size": 5 }, "indent_size": 2}", js = "{ "indent_size": 5 }", css = "{ "indent_size": 3 }")');
        opts.indent_size = 9;
        opts.html = { 'js': { 'indent_size': 3, 'disabled': true }, 'css': { 'indent_size': 5 }, 'indent_size': 2};
        opts.js = { 'indent_size': 5 };
        opts.css = { 'indent_size': 3 };
        test_fragment(
            '<head>\n' +
            '  <script>\n' +
            '    if (a == b) {\n' +
            '       test();\n' +
            '    }\n' +
            '  </script>\n' +
            '  <style>\n' +
            '    .selector {\n' +
            '         font-size: 12px;\n' +
            '    }\n' +
            '  </style>\n' +
            '</head>');
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '  <script>\n' +
            '    if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '  </script>\n' +
            '  <style>\n' +
            '    .selector {\n' +
            '         font-size: 12px;\n' +
            '    }\n' +
            '  </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '  <script src="one.js"></script> <!-- one -->\n' +
            '  <script src="two.js"></script> <!-- two-->\n' +
            '</body>');


        //============================================================
        // Tests script indent behavior - (indent_scripts = ""normal"")
        reset_options();
        set_name('Tests script indent behavior - (indent_scripts = ""normal"")');
        opts.indent_scripts = 'normal';
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '    <script>\n' +
            '        if (a == b) {\n' +
            '            test();\n' +
            '        }\n' +
            '    </script>\n' +
            '    <style>\n' +
            '        .selector {\n' +
            '            font-size: 12px;\n' +
            '        }\n' +
            '    </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '    <script src="one.js"></script> <!-- one -->\n' +
            '    <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Tests script indent behavior - (indent_scripts = ""keep"")
        reset_options();
        set_name('Tests script indent behavior - (indent_scripts = ""keep"")');
        opts.indent_scripts = 'keep';
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '    <script>\n' +
            '    if (a == b) {\n' +
            '        test();\n' +
            '    }\n' +
            '    </script>\n' +
            '    <style>\n' +
            '    .selector {\n' +
            '        font-size: 12px;\n' +
            '    }\n' +
            '    </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '    <script src="one.js"></script> <!-- one -->\n' +
            '    <script src="two.js"></script> <!-- two-->\n' +
            '</body>');

        // Tests script indent behavior - (indent_scripts = ""separate"")
        reset_options();
        set_name('Tests script indent behavior - (indent_scripts = ""separate"")');
        opts.indent_scripts = 'separate';
        test_fragment(
            '<head>\n' +
            '<script>\n' +
            'if (a == b) {\n' +
            'test();\n' +
            '}\n' +
            '</script>\n' +
            '<style>\n' +
            '.selector {\n' +
            'font-size: 12px;\n' +
            '}\n' +
            '</style>\n' +
            '</head>',
            //  -- output --
            '<head>\n' +
            '    <script>\n' +
            'if (a == b) {\n' +
            '    test();\n' +
            '}\n' +
            '    </script>\n' +
            '    <style>\n' +
            '.selector {\n' +
            '    font-size: 12px;\n' +
            '}\n' +
            '    </style>\n' +
            '</head>');
        test_fragment(
            '<body>\n' +
            '    <script src="one.js"></script> <!-- one -->\n' +
            '    <script src="two.js"></script> <!-- two-->\n' +
            '</body>');


        //============================================================
        // ASP(X) and JSP directives <%@ indent formatting
        reset_options();
        set_name('ASP(X) and JSP directives <%@ indent formatting');
        bth(
            '<%@Master language="C#"%>\n' +
            '<%@Register TagPrefix="a" Namespace="a" Assembly="a"%>\n' +
            '<%@Register TagPrefix="b" Namespace="a" Assembly="a"%>\n' +
            '<%@Register TagPrefix="c" Namespace="a" Assembly="a"%>\n' +
            '<!DOCTYPE html>\n' +
            '\n' +
            '<html>\n' +
            '\n' +
            '<some-content />\n' +
            '\n' +
            '</html>');


        //============================================================
        // Issue #1027 -- Formatting SVG files
        reset_options();
        set_name('Issue #1027 -- Formatting SVG files');
        bth(
            '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"\n' +
            '     viewBox="0 0 36 36" style="enable-background:new 0 0 36 36;" xml:space="preserve">\n' +
            '                    <rect id="XMLID_20_" x="-7"\n' +
            '                          class="st0"\n' +
            '                          width="49" height="36"/>\n' +
            '</svg>',
            //  -- output --
            '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 36 36" style="enable-background:new 0 0 36 36;" xml:space="preserve">\n' +
            '    <rect id="XMLID_20_" x="-7" class="st0" width="49" height="36" />\n' +
            '</svg>');


        //============================================================
        // Linewrap length
        reset_options();
        set_name('Linewrap length');
        opts.wrap_line_length = 80;

        // This test shows how line wrapping is still not correct.
        test_fragment(
            '<body>\n' +
            '    <div>\n' +
            '        <div>\n' +
            '            <p>Reconstruct the schematic editor the EDA system <a href="http://www.jedat.co.jp/eng/products.html"><i>AlphaSX</i></a> series</p>\n' +
            '        </div>\n' +
            '    </div>\n' +
            '</body>',
            //  -- output --
            '<body>\n' +
            '    <div>\n' +
            '        <div>\n' +
            '            <p>Reconstruct the schematic editor the EDA system <a\n' +
            '                    href="http://www.jedat.co.jp/eng/products.html"><i>AlphaSX</i></a>\n' +
            '                series</p>\n' +
            '        </div>\n' +
            '    </div>\n' +
            '</body>');
        test_fragment(
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014 0015 0016 0017 0018 0019 0020</span>',
            //  -- output --
            '<span>0 0001 0002 0003 0004 0005 0006 0007 0008 0009 0010 0011 0012 0013 0014\n' +
            '    0015 0016 0017 0018 0019 0020</span>');
        test_fragment('<div>----1---------2---------3---------4---------5---------6---------7----</div>');
        bth('<span>----1---------2---------3---------4---------5---------6---------7----</span>');
        bth('<span>----1---------2---------3---------4---------5---------6---------7----<br /></span>');
        bth(
            '<div>----1---------2---------3---------4---------5---------6---------7-----</div>',
            //  -- output --
            '<div>----1---------2---------3---------4---------5---------6---------7-----\n' +
            '</div>');
        test_fragment(
            '<div>----1---------2---------3---------4---------5---------6---------7-----<br /></div>',
            //  -- output --
            '<div>\n' +
            '    ----1---------2---------3---------4---------5---------6---------7-----<br />\n' +
            '</div>');
        test_fragment(
            '<div>----1---------2---------3---------4---------5---------6---------7-----<hr /></div>',
            //  -- output --
            '<div>----1---------2---------3---------4---------5---------6---------7-----\n' +
            '    <hr />\n' +
            '</div>');
        test_fragment(
            '<div>----1---------2---------3---------4---------5---------6---------7-----<hr />-</div>',
            //  -- output --
            '<div>----1---------2---------3---------4---------5---------6---------7-----\n' +
            '    <hr />-\n' +
            '</div>');
        bth(
            '<div>----1---------2---------3---------4---------5---------6---------7 --------81 ----2---------3---------4---------5---------6---------7-----</div>',
            //  -- output --
            '<div>----1---------2---------3---------4---------5---------6---------7\n' +
            '    --------81 ----2---------3---------4---------5---------6---------7-----\n' +
            '</div>');
        bth(
            '<span>---1---------2---------3---------4---------5---------6---------7 --------81 ----2---------3---------4---------5---------6</span>',
            //  -- output --
            '<span>---1---------2---------3---------4---------5---------6---------7\n' +
            '    --------81 ----2---------3---------4---------5---------6</span>');
        bth(
            '<p>---------1---------2---------3---------4 ---------1---------2---------3---------4</p>',
            //  -- output --
            '<p>---------1---------2---------3---------4\n' +
            '    ---------1---------2---------3---------4</p>');
        bth(
            '<div>----1---------2---------3---------4---------5---------6---------7 --------81 ----2---------3---------4---------5---------6</div>',
            //  -- output --
            '<div>----1---------2---------3---------4---------5---------6---------7\n' +
            '    --------81 ----2---------3---------4---------5---------6</div>');
        bth(
            '<div>----1---------2---------3---------4---------5---------6---------7 --------81 ----2---------3---------4---------5---------6<br /></div>',
            //  -- output --
            '<div>----1---------2---------3---------4---------5---------6---------7\n' +
            '    --------81 ----2---------3---------4---------5---------6<br /></div>');
        bth(
            '<div>----1---------2---------3---------4---------5---------6---------7 --------81 ----2---------3---------4---------5---------6<hr /></div>',
            //  -- output --
            '<div>----1---------2---------3---------4---------5---------6---------7\n' +
            '    --------81 ----2---------3---------4---------5---------6\n' +
            '    <hr />\n' +
            '</div>');

        // #1238  Fixed
        bth(
            '<span uib-tooltip="[[firstName]] [[lastName]]" tooltip-enable="showToolTip">\n' +
            '   <ng-letter-avatar charCount="2" data="[[data]]"\n' +
            '        shape="round" fontsize="[[font]]" height="[[height]]" width="[[width]]"\n' +
            '   avatarcustombgcolor="[[bgColor]]" dynamic="true"></ng-letter-avatar>\n' +
            '     </span>',
            //  -- output --
            '<span uib-tooltip="[[firstName]] [[lastName]]" tooltip-enable="showToolTip">\n' +
            '    <ng-letter-avatar charCount="2" data="[[data]]" shape="round"\n' +
            '        fontsize="[[font]]" height="[[height]]" width="[[width]]"\n' +
            '        avatarcustombgcolor="[[bgColor]]" dynamic="true"></ng-letter-avatar>\n' +
            '</span>');

        // Issue #1122
        test_fragment(
            '<div>\n' +
            '<div>\n' +
            '<p>\n' +
            '    В РАБОЧЕМ РЕЖИМЕ, после ввода параметров опыта (номер, шаг отсчетов и глубина зондирования), текущие\n' +
            '    отсчеты сохраняются в контроллере при нажатии кнопки «ПУСК». Одновременно, они распечатываются\n' +
            '    на минипринтере. Управлять контроллером для записи данных зондирования можно при помощи <link_row to="РК.05.01.01">Радиокнопки РК-11</link_row>.\n' +
            '</p>\n' +
            '</div>\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <div>\n' +
            '        <p>\n' +
            '            В РАБОЧЕМ РЕЖИМЕ, после ввода параметров опыта (номер, шаг отсчетов\n' +
            '            и глубина зондирования), текущие\n' +
            '            отсчеты сохраняются в контроллере при нажатии кнопки «ПУСК».\n' +
            '            Одновременно, они распечатываются\n' +
            '            на минипринтере. Управлять контроллером для записи данных\n' +
            '            зондирования можно при помощи <link_row to="РК.05.01.01">Радиокнопки\n' +
            '                РК-11</link_row>.\n' +
            '        </p>\n' +
            '    </div>\n' +
            '</div>');

        // Issue #1122
        test_fragment(
            '<div>\n' +
            '<div>\n' +
            '<p>\n' +
            '    В РАБОЧЕМ РЕЖИМЕ, после ввода параметров опыта (номер, шаг отсчетов и глубина зондирования), текущие отсчеты сохраняются в контроллере при нажатии кнопки «ПУСК». Одновременно, они распечатываются на минипринтере. Управлять контроллером для записи данных зондирования можно при помощи <link_row to="РК.05.01.01">Радиокнопки РК-11</link_row>.\n' +
            '</p>\n' +
            '</div>\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <div>\n' +
            '        <p>\n' +
            '            В РАБОЧЕМ РЕЖИМЕ, после ввода параметров опыта (номер, шаг отсчетов\n' +
            '            и глубина зондирования), текущие отсчеты сохраняются в контроллере\n' +
            '            при нажатии кнопки «ПУСК». Одновременно, они распечатываются на\n' +
            '            минипринтере. Управлять контроллером для записи данных зондирования\n' +
            '            можно при помощи <link_row to="РК.05.01.01">Радиокнопки РК-11\n' +
            '            </link_row>.\n' +
            '        </p>\n' +
            '    </div>\n' +
            '</div>');

        // #607 - preserve-newlines makes this look a bit odd now, but it much better
        test_fragment(
            '<p>В РАБОЧЕМ РЕЖИМЕ, после ввода параметров опыта (номер, шаг отсчетов и глубина зондирования), текущие\n' +
            '    отсчеты сохраняются в контроллере при нажатии кнопки «ПУСК». Одновременно, они распечатываются\n' +
            '    на минипринтере. Управлять контроллером для записи данных зондирования можно при помощи <link_row to="РК.05.01.01">Радиокнопки РК-11</link_row>.</p>',
            //  -- output --
            '<p>В РАБОЧЕМ РЕЖИМЕ, после ввода параметров опыта (номер, шаг отсчетов и глубина\n' +
            '    зондирования), текущие\n' +
            '    отсчеты сохраняются в контроллере при нажатии кнопки «ПУСК». Одновременно,\n' +
            '    они распечатываются\n' +
            '    на минипринтере. Управлять контроллером для записи данных зондирования можно\n' +
            '    при помощи <link_row to="РК.05.01.01">Радиокнопки РК-11</link_row>.</p>');


        //============================================================
        // Indent with tabs
        reset_options();
        set_name('Indent with tabs');
        opts.indent_with_tabs = true;
        test_fragment(
            '<div>\n' +
            '<div>\n' +
            '</div>\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '\t<div>\n' +
            '\t</div>\n' +
            '</div>');


        //============================================================
        // Indent without tabs
        reset_options();
        set_name('Indent without tabs');
        opts.indent_with_tabs = false;
        test_fragment(
            '<div>\n' +
            '<div>\n' +
            '</div>\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    <div>\n' +
            '    </div>\n' +
            '</div>');


        //============================================================
        // Do not indent html inner html by default
        reset_options();
        set_name('Do not indent html inner html by default');
        test_fragment(
            '<html>\n' +
            '<body>\n' +
            '<div></div>\n' +
            '</body>\n' +
            '\n' +
            '</html>',
            //  -- output --
            '<html>\n' +
            '<body>\n' +
            '    <div></div>\n' +
            '</body>\n' +
            '\n' +
            '</html>');


        //============================================================
        // indent_inner_html set to true indents html inner html
        reset_options();
        set_name('indent_inner_html set to true indents html inner html');
        opts.indent_inner_html = true;
        test_fragment(
            '<html>\n' +
            '    <body>\n' +
            '        <div></div>\n' +
            '    </body>\n' +
            '\n' +
            '</html>');


        //============================================================
        // Indent body inner html by default
        reset_options();
        set_name('Indent body inner html by default');
        test_fragment(
            '<html>\n' +
            '<body>\n' +
            '<div></div>\n' +
            '</body>\n' +
            '\n' +
            '</html>',
            //  -- output --
            '<html>\n' +
            '<body>\n' +
            '    <div></div>\n' +
            '</body>\n' +
            '\n' +
            '</html>');


        //============================================================
        // indent_body_inner_html set to false prevents indent of body inner html
        reset_options();
        set_name('indent_body_inner_html set to false prevents indent of body inner html');
        opts.indent_body_inner_html = false;
        test_fragment(
            '<html>\n' +
            '<body>\n' +
            '<div></div>\n' +
            '</body>\n' +
            '\n' +
            '</html>');


        //============================================================
        // Indent head inner html by default
        reset_options();
        set_name('Indent head inner html by default');
        test_fragment(
            '<html>\n' +
            '\n' +
            '<head>\n' +
            '<meta>\n' +
            '</head>\n' +
            '\n' +
            '</html>',
            //  -- output --
            '<html>\n' +
            '\n' +
            '<head>\n' +
            '    <meta>\n' +
            '</head>\n' +
            '\n' +
            '</html>');


        //============================================================
        // indent_head_inner_html set to false prevents indent of head inner html
        reset_options();
        set_name('indent_head_inner_html set to false prevents indent of head inner html');
        opts.indent_head_inner_html = false;
        test_fragment(
            '<html>\n' +
            '\n' +
            '<head>\n' +
            '<meta>\n' +
            '</head>\n' +
            '\n' +
            '</html>');


        //============================================================
        // Inline tags formatting
        reset_options();
        set_name('Inline tags formatting');
        bth(
            '<div><span></span></div><span><div></div></span>',
            //  -- output --
            '<div><span></span></div><span>\n' +
            '    <div></div>\n' +
            '</span>');
        bth(
            '<div><div><span><span>Nested spans</span></span></div></div>',
            //  -- output --
            '<div>\n' +
            '    <div><span><span>Nested spans</span></span></div>\n' +
            '</div>');
        bth(
            '<p>Should remove <span><span \n' +
            '\n' +
            'class="some-class">attribute</span></span> newlines</p>',
            //  -- output --
            '<p>Should remove <span><span class="some-class">attribute</span></span> newlines</p>');
        bth('<div><span>All</span> on <span>one</span> line</div>');
        bth('<span class="{{class_name}}">{{content}}</span>');
        bth('{{#if 1}}<span>{{content}}</span>{{/if}}');


        //============================================================
        // Preserve newlines false
        reset_options();
        set_name('Preserve newlines false');
        opts.indent_size = 2;
        opts.preserve_newlines = false;
        bth(
            '<div>\n' +
            '\tfoo\n' +
            '</div>',
            //  -- output --
            '<div> foo </div>');
        bth(
            '<div>Should not</div>\n' +
            '\n' +
            '\n' +
            '<div>preserve newlines</div>',
            //  -- output --
            '<div>Should not</div>\n' +
            '<div>preserve newlines</div>');
        bth(
            '<header>\n' +
            '  <h1>\n' +
            '\n' +
            '\n' +
            '    <ul>\n' +
            '\n' +
            '      <li class="menuactive menuparent">\n' +
            '        <a>\n' +
            '          <span>Anita Koppe</span>\n' +
            '        </a>\n' +
            '\n' +
            '\n' +
            '      </li>\n' +
            '    </ul>\n' +
            '  </h1>\n' +
            '</header>',
            //  -- output --
            '<header>\n' +
            '  <h1>\n' +
            '    <ul>\n' +
            '      <li class="menuactive menuparent">\n' +
            '        <a>\n' +
            '          <span>Anita Koppe</span>\n' +
            '        </a>\n' +
            '      </li>\n' +
            '    </ul>\n' +
            '  </h1>\n' +
            '</header>');


        //============================================================
        // Preserve newlines true
        reset_options();
        set_name('Preserve newlines true');
        opts.indent_size = 1;
        opts.indent_char = "	";
        opts.preserve_newlines = true;
        test_fragment(
            '<div>\n' +
            '\tfoo\n' +
            '</div>');


        //============================================================
        // Preserve newlines true with zero max newline
        reset_options();
        set_name('Preserve newlines true with zero max newline');
        opts.preserve_newlines = true;
        opts.max_preserve_newlines = 0;
        opts.indent_size = 2;
        bth(
            '<div>Should</div>\n' +
            '\n' +
            '\n' +
            '<div>preserve zero newlines</div>',
            //  -- output --
            '<div>Should</div>\n' +
            '<div>preserve zero newlines</div>');
        bth(
            '<header>\n' +
            '  <h1>\n' +
            '\n' +
            '\n' +
            '    <ul>\n' +
            '\n' +
            '      <li class="menuactive menuparent">\n' +
            '        <a>\n' +
            '          <span>Anita Koppe</span>\n' +
            '        </a>\n' +
            '\n' +
            '\n' +
            '      </li>\n' +
            '    </ul>\n' +
            '  </h1>\n' +
            '</header>',
            //  -- output --
            '<header>\n' +
            '  <h1>\n' +
            '    <ul>\n' +
            '      <li class="menuactive menuparent">\n' +
            '        <a>\n' +
            '          <span>Anita Koppe</span>\n' +
            '        </a>\n' +
            '      </li>\n' +
            '    </ul>\n' +
            '  </h1>\n' +
            '</header>');


        //============================================================
        // Preserve newlines true with 1 max newline
        reset_options();
        set_name('Preserve newlines true with 1 max newline');
        opts.preserve_newlines = true;
        opts.indent_size = 2;
        opts.max_preserve_newlines = 1;
        bth(
            '<div>Should</div>\n' +
            '\n' +
            '\n' +
            '<div>preserve one newline</div>',
            //  -- output --
            '<div>Should</div>\n' +
            '\n' +
            '<div>preserve one newline</div>');
        bth(
            '<header>\n' +
            '  <h1>\n' +
            '\n' +
            '\n' +
            '    <ul>\n' +
            '\n' +
            '      <li class="menuactive menuparent">\n' +
            '        <a>\n' +
            '          <span>Anita Koppe</span>\n' +
            '        </a>\n' +
            '\n' +
            '\n' +
            '      </li>\n' +
            '    </ul>\n' +
            '  </h1>\n' +
            '</header>',
            //  -- output --
            '<header>\n' +
            '  <h1>\n' +
            '\n' +
            '    <ul>\n' +
            '\n' +
            '      <li class="menuactive menuparent">\n' +
            '        <a>\n' +
            '          <span>Anita Koppe</span>\n' +
            '        </a>\n' +
            '\n' +
            '      </li>\n' +
            '    </ul>\n' +
            '  </h1>\n' +
            '</header>');


        //============================================================
        // Preserve newlines true with null max newline
        reset_options();
        set_name('Preserve newlines true with null max newline');
        opts.preserve_newlines = true;
        opts.indent_size = 2;
        opts.max_preserve_newlines = null;
        bth(
            '<div>Should</div>\n' +
            '\n' +
            '\n' +
            '<div>preserve zero newlines</div>');
        bth(
            '<header>\n' +
            '  <h1>\n' +
            '\n' +
            '\n' +
            '    <ul>\n' +
            '\n' +
            '      <li class="menuactive menuparent">\n' +
            '        <a>\n' +
            '          <span>Anita Koppe</span>\n' +
            '        </a>\n' +
            '\n' +
            '\n' +
            '      </li>\n' +
            '    </ul>\n' +
            '  </h1>\n' +
            '</header>');


        //============================================================
        // unformatted to prevent formatting changes
        reset_options();
        set_name('unformatted to prevent formatting changes');
        opts.unformatted = ['h1', 'br', 'u', 'span', 'textarea'];
        bth('<u><div><div>Ignore block tags in unformatted regions</div></div></u>');
        bth('<div><u>Do not wrap unformatted regions with extra newlines</u></div>');
        bth(
            '<div>\n' +
            '<u>Do not wrap unformatted regions with extra newlines</u></div>',
            //  -- output --
            '<div>\n' +
            '    <u>Do not wrap unformatted regions with extra newlines</u>\n' +
            '</div>');
        bth('<div><br /></div>');
        bth(
            '<div>\n' +
            '<br /></div>',
            //  -- output --
            '<div>\n' +
            '    <br />\n' +
            '</div>');
        bth('<div><h1 /></div>');
        bth(
            '<div>\n' +
            '<h1 /></div>',
            //  -- output --
            '<div>\n' +
            '    <h1 />\n' +
            '</div>');
        bth('<label><br /></label>');

        // Inline parent should not add newline unlike block
        bth(
            '<label>\n' +
            '<br /></label>',
            //  -- output --
            '<label>\n' +
            '    <br /></label>');

        // Inline parent with unformatted non-inline child
        bth('<label><h1>Unformatted non-inline</h1></label>');

        // Inline parent with unformatted non-inline child
        bth(
            '<label>\n' +
            '<h1>Unformatted non-inline</h1></label>',
            //  -- output --
            '<label>\n' +
            '    <h1>Unformatted non-inline</h1></label>');

        // Inline parent with unformatted non-inline empty child
        bth('<label><h1 /></label>');

        // Inline parent with unformatted non-inline empty child
        bth(
            '<label>\n' +
            '<h1 /></label>',
            //  -- output --
            '<label>\n' +
            '    <h1 /></label>');
        bth(
            '<u>  \n' +
            '\n' +
            '\n' +
            '  Ignore extra """whitespace mostly  \n' +
            '\n' +
            '\n' +
            '  </u>',
            //  -- output --
            '<u>\n' +
            '\n' +
            '\n' +
            '  Ignore extra """whitespace mostly  \n' +
            '\n' +
            '\n' +
            '  </u>');
        bth(
            '<u><div \n' +
            '\t\n' +
            'class=""">Ignore whitespace in attributes\t</div></u>');

        // Regression test #1534 - interaction between unformatted, content_unformatted, and inline
        bth(
            '<div>\n' +
            '    <textarea></textarea>\n' +
            '    <textarea>\n' +
            '\n' +
            '</textarea>\n' +
            '    <span></span>\n' +
            '    <span>\n' +
            '\n' +
            '</span>\n' +
            '</div>');
        bth(
            '<u \n' +
            '\n' +
            '\t\t  class="">Ignore whitespace\n' +
            'in\tattributes</u>',
            //  -- output --
            '<u\n' +
            '\n' +
            '\t\t  class="">Ignore whitespace\n' +
            'in\tattributes</u>');


        //============================================================
        // content_unformatted to prevent formatting content
        reset_options();
        set_name('content_unformatted to prevent formatting content');
        opts.content_unformatted = ['?php', 'script', 'style', 'p', 'span', 'br', 'meta', 'textarea'];
        test_fragment(
            '<html><body><h1>A</h1><script>if(1){f();}</script><style>.a{display:none;}</style></body></html>',
            //  -- output --
            '<html>\n' +
            '<body>\n' +
            '    <h1>A</h1>\n' +
            '    <script>if(1){f();}</script>\n' +
            '    <style>.a{display:none;}</style>\n' +
            '</body>\n' +
            '\n' +
            '</html>');
        bth(
            '<div><p>Beautify me</p></div><p><div>But not me</div></p>',
            //  -- output --
            '<div>\n' +
            '    <p>Beautify me</p>\n' +
            '</div>\n' +
            '<p><div>But not me</div></p>');
        bth(
            '<div><p\n' +
            '  class="beauty-me"\n' +
            '>Beautify me</p></div><p><div\n' +
            '  class="iamalreadybeauty"\n' +
            '>But not me</div></p>',
            //  -- output --
            '<div>\n' +
            '    <p class="beauty-me">Beautify me</p>\n' +
            '</div>\n' +
            '<p><div\n' +
            '  class="iamalreadybeauty"\n' +
            '>But not me</div></p>');
        bth('<div><span>blabla<div>something here</div></span></div>');
        bth('<div><br /></div>');
        bth('<div><br></div>');
        bth(
            '<div>\n' +
            '<br>\n' +
            '<br />\n' +
            '<br></div>',
            //  -- output --
            '<div>\n' +
            '    <br>\n' +
            '    <br />\n' +
            '    <br>\n' +
            '</div>');

        // Regression test #1534 - interaction between unformatted, content_unformatted, and inline
        bth(
            '<div>\n' +
            '    <textarea></textarea>\n' +
            '    <textarea>\n' +
            '\n' +
            '</textarea>\n' +
            '    <span></span>\n' +
            '    <span>\n' +
            '\n' +
            '</span>\n' +
            '</div>');
        bth(
            '<div>\n' +
            '<meta>\n' +
            '<meta />\n' +
            '<meta></div>',
            //  -- output --
            '<div>\n' +
            '    <meta>\n' +
            '    <meta />\n' +
            '    <meta>\n' +
            '</div>');
        bth(
            '<div><pre>var a=1;\n' +
            'var b=a;</pre></div>',
            //  -- output --
            '<div>\n' +
            '    <pre>var a=1;\n' +
            '        var b=a;</pre>\n' +
            '</div>');
        bth(
            '<?php\n' +
            '/**\n' +
            ' * Comment\n' +
            ' */\n' +
            '\n' +
            '?>\n' +
            '<div class="">\n' +
            '\n' +
            '</div>');
        bth(
            '<div><pre>\n' +
            'var a=1;\n' +
            'var b=a;\n' +
            '</pre></div>',
            //  -- output --
            '<div>\n' +
            '    <pre>\n' +
            '        var a=1;\n' +
            '        var b=a;\n' +
            '    </pre>\n' +
            '</div>');


        //============================================================
        // default content_unformatted and inline element test
        reset_options();
        set_name('default content_unformatted and inline element test');
        test_fragment(
            '<html><body><h1>A</h1><script>if(1){f();}</script><style>.a{display:none;}</style></body></html>',
            //  -- output --
            '<html>\n' +
            '<body>\n' +
            '    <h1>A</h1>\n' +
            '    <script>\n' +
            '        if (1) {\n' +
            '            f();\n' +
            '        }\n' +
            '    </script>\n' +
            '    <style>\n' +
            '        .a {\n' +
            '            display: none;\n' +
            '        }\n' +
            '    </style>\n' +
            '</body>\n' +
            '\n' +
            '</html>');
        bth(
            '<div><p>Beautify me</p></div><p><p>But not me</p></p>',
            //  -- output --
            '<div>\n' +
            '    <p>Beautify me</p>\n' +
            '</div>\n' +
            '<p>\n' +
            '<p>But not me</p>\n' +
            '</p>');
        bth(
            '<div><p\n' +
            '  class="beauty-me"\n' +
            '>Beautify me</p></div><p><p\n' +
            '  class="iamalreadybeauty"\n' +
            '>But not me</p></p>',
            //  -- output --
            '<div>\n' +
            '    <p class="beauty-me">Beautify me</p>\n' +
            '</div>\n' +
            '<p>\n' +
            '<p class="iamalreadybeauty">But not me</p>\n' +
            '</p>');
        bth('<div><span>blabla<div>something here</div></span></div>');
        bth('<div><br /></div>');

        // Regression test #1534 - interaction between unformatted, content_unformatted, and inline
        bth(
            '<div>\n' +
            '    <textarea></textarea>\n' +
            '    <textarea>\n' +
            '\n' +
            '</textarea>\n' +
            '    <span></span>\n' +
            '    <span>\n' +
            '\n' +
            '    </span>\n' +
            '</div>');
        bth(
            '<div><pre>var a=1;\n' +
            'var b=a;</pre></div>',
            //  -- output --
            '<div>\n' +
            '    <pre>var a=1;\n' +
            'var b=a;</pre>\n' +
            '</div>');
        bth(
            '<div><pre>\n' +
            'var a=1;\n' +
            'var b=a;\n' +
            '</pre></div>',
            //  -- output --
            '<div>\n' +
            '    <pre>\n' +
            'var a=1;\n' +
            'var b=a;\n' +
            '</pre>\n' +
            '</div>');

        // Test for #1041
        bth(
            '<p><span class="foo">foo <span class="bar">bar</span></span></p>\n' +
            '\n' +
            '<aside><p class="foo">foo <span class="bar">bar</span></p></aside>\n' +
            '<p class="foo"><span class="bar">bar</span></p>',
            //  -- output --
            '<p><span class="foo">foo <span class="bar">bar</span></span></p>\n' +
            '\n' +
            '<aside>\n' +
            '    <p class="foo">foo <span class="bar">bar</span></p>\n' +
            '</aside>\n' +
            '<p class="foo"><span class="bar">bar</span></p>');

        // Test for #869 - not exactly what the user wants but no longer horrible
        bth('<div><input type="checkbox" id="j" name="j" value="foo">&nbsp;<label for="j">Foo</label></div>');

        // Test for #1167
        bth(
            '<span>\n' +
            '    <span><img src="images/off.svg" alt=""></span>\n' +
            '    <span><img src="images/on.svg" alt=""></span>\n' +
            '</span>');

        // Test for #882
        bth(
            '<tr><th><h3>Name</h3></th><td class="full-width"></td></tr>',
            //  -- output --
            '<tr>\n' +
            '    <th>\n' +
            '        <h3>Name</h3>\n' +
            '    </th>\n' +
            '    <td class="full-width"></td>\n' +
            '</tr>');

        // Test for #1184
        bth(
            '<div><div></div>Connect</div>',
            //  -- output --
            '<div>\n' +
            '    <div></div>Connect\n' +
            '</div>');

        // Test for #1383
        bth(
            '<p class="newListItem">\n' +
            '  <svg height="40" width="40">\n' +
            '              <circle cx="20" cy="20" r="18" stroke="black" stroke-width="0" fill="#bddffa" />\n' +
            '              <text x="50%" y="50%" text-anchor="middle" stroke="#1b97f3" stroke-width="2px" dy=".3em">1</text>\n' +
            '            </svg> This is a paragraph after an SVG shape.\n' +
            '</p>',
            //  -- output --
            '<p class="newListItem">\n' +
            '    <svg height="40" width="40">\n' +
            '        <circle cx="20" cy="20" r="18" stroke="black" stroke-width="0" fill="#bddffa" />\n' +
            '        <text x="50%" y="50%" text-anchor="middle" stroke="#1b97f3" stroke-width="2px" dy=".3em">1</text>\n' +
            '    </svg> This is a paragraph after an SVG shape.\n' +
            '</p>');


        //============================================================
        // indent_empty_lines true
        reset_options();
        set_name('indent_empty_lines true');
        opts.indent_empty_lines = true;
        test_fragment(
            '<div>\n' +
            '\n' +
            '    <div>\n' +
            '\n' +
            '    </div>\n' +
            '\n' +
            '</div>',
            //  -- output --
            '<div>\n' +
            '    \n' +
            '    <div>\n' +
            '        \n' +
            '    </div>\n' +
            '    \n' +
            '</div>');


        //============================================================
        // indent_empty_lines false
        reset_options();
        set_name('indent_empty_lines false');
        opts.indent_empty_lines = false;
        test_fragment(
            '<div>\n' +
            '\n' +
            '    <div>\n' +
            '\n' +
            '    </div>\n' +
            '\n' +
            '</div>');


        //============================================================
        // Smarty tests for extra whitespace in nested quotes
        reset_options();
        set_name('Smarty tests for extra whitespace in nested quotes');
        opts.templating = 'smarty';
        bth('<div class="foo{if $bar=="1"} bar{/if}">foo</div>');
        bth('<input type="radio" name="foo" {if $bar==""}checked{/if}>');


        //============================================================
        // Smarty tests for script
        reset_options();
        set_name('Smarty tests for script');
        opts.templating = 'smarty';
        bth(
            '<script>\n' +
            '    var foo = {$bar|json_encode};\n' +
            '</script>');


        //============================================================
        // Recognize handlebars with whitespace control
        reset_options();
        set_name('Recognize handlebars with whitespace control');
        opts.indent_handlebars = true;
        bth(
            '{{#if true}}<div><div>\n' +
            '{{~#if true ~}}<p>true</p>{{/if}}\n' +
            '</div></div>{{/if}}',
            //  -- output --
            '{{#if true}}\n' +
            '    <div>\n' +
            '        <div>\n' +
            '            {{~#if true ~}}\n' +
            '                <p>true</p>\n' +
            '            {{/if}}\n' +
            '        </div>\n' +
            '    </div>\n' +
            '{{/if}}');
        bth(
            '{{~#*inline "MyInlinePartial"}}\n' +
            '{{MyIdentifier}}\n' +
            '{{/inline}}',
            //  -- output --
            '{{~#*inline "MyInlinePartial"}}\n' +
            '    {{MyIdentifier}}\n' +
            '{{/inline}}');
        bth(
            '{{~#> myPartial }}\n' +
            '<span>format correctly</span>\n' +
            '{{/myPartial}}',
            //  -- output --
            '{{~#> myPartial }}\n' +
            '    <span>format correctly</span>\n' +
            '{{/myPartial}}');
        bth(
            '{{#if callOn}}\n' +
            '    {{translate "onText"}}\n' +
            '{{~else if (eq callOn false)}}\n' +
            '    {{translate "offText"}}\n' +
            '{{/if}}');
        bth(
            '{{~#if callOn}}\n' +
            '    {{translate "onText"}}\n' +
            '{{~else if (eq callOn false)}}\n' +
            '    {{translate "offText"}}\n' +
            '{{/if}}');


        //============================================================
        // Corrects Partial Behavior Involving Whitespace
        reset_options();
        set_name('Corrects Partial Behavior Involving Whitespace');
        bth(
            '{{#>row}}\n' +
            '    {{#>column}}\n' +
            '        <span>content</span>\n' +
            '        {{/column}}\n' +
            '        {{/row}}',
            //  -- output --
            '{{#>row}}\n' +
            '    {{#>column}}\n' +
            '        <span>content</span>\n' +
            '    {{/column}}\n' +
            '{{/row}}');
        bth(
            '{{~#>row}}\n' +
            '{{#>column}}\n' +
            '<p>content</p>\n' +
            '{{/column}}\n' +
            '{{/row}}',
            //  -- output --
            '{{~#>row}}\n' +
            '    {{#>column}}\n' +
            '        <p>content</p>\n' +
            '    {{/column}}\n' +
            '{{/row}}');
        bth(
            '{{#>row}}\n' +
            '    {{#>column}}\n' +
            '        <span>content</span>\n' +
            '    {{/column}}\n' +
            '{{/row}}');
        bth(
            '{{#> row}}\n' +
            '    {{#> column}}\n' +
            '        <span>content</span>\n' +
            '    {{/column}}\n' +
            '{{/row}}');


        //============================================================
        // New Test Suite
        reset_options();
        set_name('New Test Suite');


    }

    function beautifier_unconverted_tests()
    {
        sanitytest = test_obj;

        reset_options();
        //============================================================
        test_fragment(null, '');

        reset_options();
        //============================================================
        // Test user pebkac protection, converts dash names to underscored names
        opts["end-with-newline"] = true;
        test_fragment(null, '\n');

        reset_options();
        //============================================================
        set_name('end_with_newline = true');
        opts.end_with_newline = true;

        test_fragment('', '\n');
        test_fragment('<div></div>\n');
        test_fragment('<div></div>\n\n\n', '<div></div>\n');
        test_fragment('<head>\n' +
            '    <script>\n' +
            '        mocha.setup("bdd");\n' +
            '\n' +
            '    </script>\n' +
            '</head>\n');


        reset_options();
        //============================================================
        set_name('Error cases');
        // error cases need love too
        bth('<img title="Bad food!" src="foo.jpg" alt="Evil" ">');
        bth("<!-- don't blow up if a comment is not complete"); // -->

        reset_options();
        //============================================================
        set_name('Basic beautify');

        test_fragment(
            '<head>\n' +
            '    <script>\n' +
            '        mocha.setup("bdd");\n' +
            '    </script>\n' +
            '</head>');

        test_fragment('<div></div>\n', '<div></div>');
        bth('<div></div>');
        bth('<div>content</div>');
        bth('<div><div></div></div>',
            '<div>\n' +
            '    <div></div>\n' +
            '</div>');
        bth('<div><div>content</div></div>',
            '<div>\n' +
            '    <div>content</div>\n' +
            '</div>');
        bth('<div>\n' +
            '    <span>content</span>\n' +
            '</div>');
        bth('<div>\n' +
            '</div>');
        bth('<div>\n' +
            '    content\n' +
            '</div>');
        bth('<div>\n' +
            '    </div>',
            '<div>\n' +
            '</div>');
        test_fragment('   <div>\n' +
            '    </div>',
            '   <div>\n' +
            '   </div>');
        bth('<div>\n' +
            '</div>\n' +
            '    <div>\n' +
            '    </div>',
            '<div>\n' +
            '</div>\n' +
            '<div>\n' +
            '</div>');
        test_fragment('   <div>\n' +
            '</div>',
            '   <div>\n' +
            '   </div>');
        bth('<div        >content</div>',
            '<div>content</div>');
        bth('<div     thinger="preserve  space  here"   ></div  >',
            '<div thinger="preserve  space  here"></div>');
        bth('content\n' +
            '    <div>\n' +
            '    </div>\n' +
            'content',
            'content\n' +
            '<div>\n' +
            '</div>\n' +
            'content');
        bth('<li>\n' +
            '    <div>\n' +
            '    </div>\n' +
            '</li>');
        bth('<li>\n' +
            '<div>\n' +
            '</div>\n' +
            '</li>',
            '<li>\n' +
            '    <div>\n' +
            '    </div>\n' +
            '</li>');
        bth('<li>\n' +
            '    content\n' +
            '</li>\n' +
            '<li>\n' +
            '    content\n' +
            '</li>');

        bth('<img>content');
        bth('<img> content');
        bth('<img>   content', '<img> content');

        bth('<img><img>content');
        bth('<img> <img>content');
        bth('<img>   <img>content', '<img> <img>content');

        bth('<img><b>content</b>');
        bth('<img> <b>content</b>');
        bth('<img>   <b>content</b>', '<img> <b>content</b>');

        bth('<div>content<img>content</div>');
        bth('<div> content <img> content</div>');
        bth('<div>    content <img>    content </div>',
            '<div> content <img> content </div>');
        bth('Text <a href="#">Link</a> Text');

        reset_options();
        //============================================================
        set_name('content_unformatted = ["script", "style"]');
        var content_unformatted = opts.content_unformatted;
        opts.content_unformatted = ['script', 'style'];
        bth('<script id="javascriptTemplate" type="text/x-kendo-template">\n' +
            '  <ul>\n' +
            '  # for (var i = 0; i < data.length; i++) { #\n' +
            '    <li>#= data[i] #</li>\n' +
            '  # } #\n' +
            '  </ul>\n' +
            '</script>');
        bth('<style>\n' +
            '  body {background-color:lightgrey}\n' +
            '  h1   {color:blue}\n' +
            '</style>');

        reset_options();
        //============================================================
        set_name('inline = ["custom-element"]');

        inline_tags = opts.inline;
        opts.inline = ['custom-element'];
        test_fragment('<div>should <custom-element>not</custom-element>' +
                      ' insert newlines</div>',
                      '<div>should <custom-element>not</custom-element>' +
                      ' insert newlines</div>');
        opts.inline = inline_tags;


        reset_options();
        //============================================================
        set_name('line wrap tests');

        bth('<div><span>content</span></div>');

        opts.wrap_line_length = 0;
        //...---------1---------2---------3---------4---------5---------6---------7
        //...1234567890123456789012345678901234567890123456789012345678901234567890
        bth('<div>Some text that should not wrap at all.</div>',
            /* expected */
            '<div>Some text that should not wrap at all.</div>');

        // A value of 0 means no max line length, and should not wrap.
        //...---------1---------2---------3---------4---------5---------6---------7---------8---------9--------10--------11--------12--------13--------14--------15--------16--------17--------18--------19--------20--------21--------22--------23--------24--------25--------26--------27--------28--------29
        //...12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
        bth('<div>Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all.</div>',
            /* expected */
            '<div>Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all.</div>');

        opts.wrap_line_length = "0";
        //...---------1---------2---------3---------4---------5---------6---------7
        //...1234567890123456789012345678901234567890123456789012345678901234567890
        bth('<div>Some text that should not wrap at all.</div>',
            /* expected */
            '<div>Some text that should not wrap at all.</div>');

        // A value of "0" means no max line length, and should not wrap
        //...---------1---------2---------3---------4---------5---------6---------7---------8---------9--------10--------11--------12--------13--------14--------15--------16--------17--------18--------19--------20--------21--------22--------23--------24--------25--------26--------27--------28--------29
        //...12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
        bth('<div>Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all.</div>',
            /* expected */
            '<div>Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all. Some text that should not wrap at all.</div>');

        opts.wrap_line_length = 40;
        //...---------1---------2---------3---------4---------5---------6---------7
        //...1234567890123456789012345678901234567890123456789012345678901234567890
        bth('<div>Some test text that should wrap_inside_this section here__.</div>',
            /* expected */
            '<div>Some test text that should\n' +
            '    wrap_inside_this section here__.\n' +
            '</div>');

        // Support passing string of number
        opts.wrap_line_length = "40";
        //...---------1---------2---------3---------4---------5---------6---------7
        //...1234567890123456789012345678901234567890123456789012345678901234567890
        bth('<div>Some test text that should wrap_inside_this section here__.</div>',
            /* expected */
            '<div>Some test text that should\n' +
            '    wrap_inside_this section here__.\n' +
            '</div>');

        reset_options();
        //============================================================

    }

    beautifier_tests();
    beautifier_unconverted_tests();
}

if (typeof exports !== "undefined") {
    exports.run_html_tests = run_html_tests;
}
//== js/test/generated/beautify-html-tests.js end


//== js/test/sanitytest.js
//
// simple testing interface
// written by Einar Lielmanis, einar@beautifier.io
//
// usage:
//
// var t = new SanityTest(function (x) { return x; }, 'my function');
// t.expect('input', 'output');
// t.expect('a', 'a');
// output_somewhere(t.results()); // good for <pre>, html safe-ish
// alert(t.results_raw());        // html unescaped

function SanityTest(func, name_of_test) {
  'use strict';
  var tl = new TitleLogger(WScript.ScriptName + ": ");
  tl.total = tl._(55252 /*TESTS_COUNT*/);

  var test_func = func || function(x) {
    return x;
  };

  var test_name = name_of_test || '';

  var n_failed = 0;
  var n_succeeded = 0;

  var failures = [];

  this.test_function = function(func, name) {
    test_func = func;
    test_name = name || '';
  };

  this.get_exitcode = function() {
    return n_succeeded === 0 || n_failed !== 0 ? 1 : 0;
  };

  this.expect = function(parameters, expected_value) {
    // multi-parameter calls not supported (I don't need them now).
    var result = test_func(parameters);
    // proper array checking is a pain. i'll maybe do it later, compare strings representations instead
    if ((result === expected_value) || (expected_value instanceof Array && result.join(', ') === expected_value.join(', '))) {
      n_succeeded += 1;
      if(n_succeeded % 10 == 0)
        tl.log("Test: " + tl._(n_succeeded) + (n_failed ? "+" + tl._(n_failed) : "") + "/" + tl.total);
      return true;
    } else {
      n_failed += 1;
      tl.log("Test: " + tl._(n_succeeded) + (n_failed ? "+" + tl._(n_failed) : "") + "/" + tl.total);
      failures.push([test_name, parameters, expected_value, result]);
      return false;
    }
  };


  this.results_raw = function() {
    var results = '';
    if (n_failed === 0) {
      if (n_succeeded === 0) {
        results = 'No tests run.';
      } else {
        results = _localize("All %S tests passed.").replace("%S", tl._(n_succeeded));
      }
    } else {
      for (var i = 0; i < failures.length; i++) {
        var f = failures[i];
        if (f[0]) {
          f[0] = f[0] + ' ';
        }
        results += '==== ' + f[0] + '============================================================\n';
        results += '---- input -------\n' + this.prettyprint(f[1]) + '\n';
        results += '---- expected ----\n' + this.prettyprint(f[2]) + '\n';
        results += '---- output ------\n' + this.prettyprint(f[3]) + '\n';
        results += '---- expected-ws ------\n' + this.prettyprint_whitespace(f[2]) + '\n';
        results += '---- output-ws ------\n' + this.prettyprint_whitespace(f[3]) + '\n';
        results += '================================================================\n\n';
      }
      results += tl._(n_failed) + '/' + tl._(n_succeeded + n_failed) + ' tests failed.\n';
    }
    tl.restore();
    return results;
  };


  this.results = function() {
    return this.lazy_escape(this.results_raw());
  };

  this.prettyprint_whitespace = function(something, quote_strings) {
    return (this.prettyprint(something, quote_strings)
      .replace(/\r\n/g, '\\r\n')
      .replace(/\n/g, '\\n\n')
      .replace(/\r/g, '\\r\n')
      .replace(/ /g, '_')
      .replace(/\t/g, '===|'));
  };

  this.prettyprint = function(something, quote_strings) {
    var type = typeof something;
    switch (type.toLowerCase()) {
      case 'string':
        if (quote_strings) {
          return "'" + something.replace("'", "\\'") + "'";
        }
        return something;
      case 'number':
        return '' + something;
      case 'boolean':
        return something ? 'true' : 'false';
      case 'undefined':
        return 'undefined';
      case 'object':
        if (something instanceof Array) {
          var x = [];
          var expected_index = 0;
          for (var k in something) {
            if (k === expected_index) {
              x.push(this.prettyprint(something[k], true));
              expected_index += 1;
            } else {
              x.push('\n' + k + ': ' + this.prettyprint(something[k], true));
            }
          }
          return '[' + x.join(', ') + ']';
        }
        return 'object: ' + something;
      default:
        return type + ': ' + something;
    }
  };


  this.lazy_escape = function(str) {
    return str.replace(/</g, '&lt;').replace(/\>/g, '&gt;').replace(/\n/g, '<br />');
  };


  this.log = function() {
    if (window.console) {
      if (console.firebug) {
        console.log.apply(console, Array.prototype.slice.call(arguments));
      } else {
        console.log.call(console, Array.prototype.slice.call(arguments));
      }
    }
  };

}

if (typeof module !== 'undefined' && module.exports) {
  module.exports = SanityTest;
}
//== js/test/sanitytest.js end